Package: fPortfolio
Title: Rmetrics - Portfolio Selection and Optimization
Date: 2023-04-22
Version: 4023.84
Authors@R: c( person("Diethelm", "Wuertz", role = "aut"), person("Tobias", "Setz", role = "aut"),
              person("Yohan", "Chalabi", role = "aut"), person("William", "Chen", role = "ctb"),
	      person("Stefan", "Theussl", role = c("aut", "cre"), email = "Stefan.Theussl@R-project.org") )
Description: A collection of functions to optimize portfolios and to analyze them from different points of view.
Depends: R (>= 2.15.1), timeDate, timeSeries, fBasics, fAssets
Imports: fCopulae, robustbase, MASS, Rglpk, slam, Rsolnp, quadprog,
        kernlab, rneos, methods, grDevices, graphics, stats, utils
Suggests: parma, Rsymphony, dplR, bcp, fGarch, mvoutlier
Additional_repositories: https://r-forge.r-project.org/
LazyData: yes
License: GPL (>= 2)
URL: https://r-forge.r-project.org/projects/rmetrics/
NeedsCompilation: no
Packaged: 2023-04-23 20:28:47 UTC; parallels
Author: Diethelm Wuertz [aut],
  Tobias Setz [aut],
  Yohan Chalabi [aut],
  William Chen [ctb],
  Stefan Theussl [aut, cre]
Maintainer: Stefan Theussl <Stefan.Theussl@R-project.org>
Repository: CRAN
Date/Publication: 2023-04-25 07:50:06 UTC
Built: R 4.6.0; ; 2026-04-28 03:35:26 UTC; unix

################################################################################
## Exports
################################################################################
exportPattern("^[^\\.]")

# Needed within the portfolio book
export(.fportfolio.plot.1)
export(.fportfolio.plot.2)
export(.fportfolio.plot.3)
export(.fportfolio.plot.4)
export(.fportfolio.plot.5)
export(.fportfolio.plot.6)
export(.fportfolio.plot.7)
export(.fportfolio.plot.8)

################################################################################
## Imports
################################################################################
import(methods)
import(timeDate)
import(timeSeries)
import(fBasics)
import(fAssets)

import(fCopulae)
import(robustbase)
import(MASS)
import(Rglpk)
import(slam)
import(Rsolnp)
import(quadprog)
import(kernlab)
import(rneos)

importFrom("grDevices", heat.colors, rainbow, topo.colors)
importFrom("graphics", axis, barplot, box, contour, grid, hist, image, layout, lcm,
	legend, mtext, par, pie, plot.new, plot.window, polygon, rect, text, title)
importFrom("stats", approx, cov, density, dnorm, nlminb, optim, optimize, pnorm,
	qnorm, rcauchy, rnorm, runif, sd, ts.plot, var, weights)
importFrom("utils", capture.output, data, packageDescription)

################################################################################
## S3 Exports
################################################################################
S3method("print", "solver")


S3method("getData", "fPFOLIODATA")
S3method("getSeries", "fPFOLIODATA")
S3method("getNAssets", "fPFOLIODATA")
S3method("getUnits", "fPFOLIODATA")

S3method("getStatistics", "fPFOLIODATA")
S3method("getMean", "fPFOLIODATA")
S3method("getCov", "fPFOLIODATA")
S3method("getEstimator", "fPFOLIODATA")
S3method("getMu", "fPFOLIODATA")
S3method("getSigma", "fPFOLIODATA")
S3method("getTailRisk", "fPFOLIODATA")


S3method("plot", "fPORTFOLIO")
S3method("summary", "fPORTFOLIO")

S3method("getData", "fPORTFOLIO")
S3method("getSeries", "fPORTFOLIO")
S3method("getNAssets", "fPORTFOLIO")
S3method("getUnits", "fPORTFOLIO")

S3method("getStatistics", "fPORTFOLIO")
S3method("getMean", "fPORTFOLIO")
S3method("getCov", "fPORTFOLIO")
S3method("getEstimator", "fPORTFOLIO")
S3method("getMu", "fPORTFOLIO")
S3method("getSigma", "fPORTFOLIO")

S3method("getSpec", "fPORTFOLIO")
S3method("getModel", "fPORTFOLIO")
S3method("getType", "fPORTFOLIO")
S3method("getOptimize", "fPORTFOLIO")
S3method("getEstimator", "fPORTFOLIO")
S3method("getTailRisk", "fPORTFOLIO")
S3method("getParams", "fPORTFOLIO")
S3method("getAlpha", "fPORTFOLIO")
S3method("getA", "fPORTFOLIO")

S3method("getPortfolio", "fPORTFOLIO")
S3method("getWeights", "fPORTFOLIO")
S3method("getTargetReturn", "fPORTFOLIO")
S3method("getTargetRisk", "fPORTFOLIO")
S3method("getRiskFreeRate", "fPORTFOLIO")
S3method("getNFrontierPoints", "fPORTFOLIO")
S3method("getStatus", "fPORTFOLIO")

S3method("getOptim", "fPORTFOLIO")
S3method("getSolver", "fPORTFOLIO")
S3method("getObjective", "fPORTFOLIO")
S3method("getOptions", "fPORTFOLIO")
S3method("getControl", "fPORTFOLIO")
S3method("getTrace", "fPORTFOLIO")
S3method("getCovRiskBudgets", "fPORTFOLIO")

S3method("getConstraints", "fPORTFOLIO")
S3method("getTailRiskBudgets", "fPORTFOLIO")


S3method("getPortfolio", "fPFOLIOVAL")
S3method("getWeights", "fPFOLIOVAL")
S3method("getCovRiskBudgets", "fPFOLIOVAL")
S3method("getTargetReturn", "fPFOLIOVAL")
S3method("getTargetRisk", "fPFOLIOVAL")
S3method("getAlpha", "fPFOLIOVAL")
S3method("getRiskFreeRate", "fPFOLIOVAL")
S3method("getNFrontierPoints", "fPFOLIOVAL")
S3method("getStatus", "fPFOLIOVAL")


S3method("getModel", "fPFOLIOSPEC")
S3method("getType", "fPFOLIOSPEC")
S3method("getOptimize", "fPFOLIOSPEC")
S3method("getEstimator", "fPFOLIOSPEC")
S3method("getTailRisk", "fPFOLIOSPEC")
S3method("getParams", "fPFOLIOSPEC")
S3method("getAlpha", "fPFOLIOSPEC")
S3method("getA", "fPFOLIOSPEC")

S3method("getPortfolio", "fPFOLIOSPEC")
S3method("getWeights", "fPFOLIOSPEC")
S3method("getTargetReturn", "fPFOLIOSPEC")
S3method("getTargetRisk", "fPFOLIOSPEC")
S3method("getRiskFreeRate", "fPFOLIOSPEC")
S3method("getNFrontierPoints", "fPFOLIOSPEC")
S3method("getStatus", "fPFOLIOSPEC")

S3method("getOptim", "fPFOLIOSPEC")
S3method("getSolver", "fPFOLIOSPEC")
S3method("getObjective", "fPFOLIOSPEC")
S3method("getOptions", "fPFOLIOSPEC")
S3method("getControl", "fPFOLIOSPEC")
S3method("getTrace", "fPFOLIOSPEC")
S3method("getMessages", "fPFOLIOSPEC")


S3method("getWindows", "fPFOLIOBACKTEST")
S3method("getWindowsFun", "fPFOLIOBACKTEST")
S3method("getWindowsParams", "fPFOLIOBACKTEST")
S3method("getWindowsHorizon", "fPFOLIOBACKTEST")
S3method("getSmoother", "fPFOLIOBACKTEST")
S3method("getSmootherFun", "fPFOLIOBACKTEST")
S3method("getSmootherParams", "fPFOLIOBACKTEST")
S3method("getSmootherLambda", "fPFOLIOBACKTEST")
S3method("getSmootherDoubleSmoothing", "fPFOLIOBACKTEST")
S3method("getSmootherInitialWeights", "fPFOLIOBACKTEST")
S3method("getSmootherSkip", "fPFOLIOBACKTEST")
S3method("getStrategy", "fPFOLIOBACKTEST")
S3method("getStrategyFun", "fPFOLIOBACKTEST")
S3method("getStrategyParams", "fPFOLIOBACKTEST")
S3method("getMessages", "fPFOLIOBACKTEST")

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################
# FUNCTION:                DESCRIPTION:
#  ramplNLP                 Rmetrics Interface for AMPL LP solvers 
#  amplNLP                  Convenience wrapper for AMPL LP solvers
#  amplControl              AMPL LP control parameter list         
################################################################################


ramplNLP <- 
    function(
        start, objective, lower=0, upper=1, amplCons, control=list(), ...)
{
    # A function implemented by Diethelm Wuertz
    
    # FUNCTION:
    
    # Control List:
    ctrl = amplNLPControl()
    if (length(control) > 0)
        for (name in names(control)) ctrl[name] = control[name]
    control = ctrl
    
    # General Settings:
    if (length(start) == 1) {
        n = start
    } else {
        n = length(start)
    }
    
    # Box Constraints:
    if(length(lower) == 1) {
        par.lower = rep(lower, n)
    } else {
        par.lower = lower
    }
    if(length(upper) == 1) {
        par.upper = rep(upper, n)
    } else {
        par.upper = upper
    }
       
    # Controls:
    solver = control$solver
    project = control$project
    trace = control$trace
    
    # Add dots ...
    args <- list(...)
    m1 = names(match.call(expand.dots = TRUE))
    m2 = names(match.call(expand.dots = FALSE))
    Names = m1[!(m1 %in% m2)]
    amplDataOpen(project)
    amplDataAdd("n", data = n, type = "value", project)
    amplModelOpen(project)
    amplModelAdd("param n ;", project)
    count = 0
    for (a in args) {
        count = count + 1
        if(length(a) == 1) {
            type = "value"
        } else {
            if(is.matrix(a)) {
                type = "matrix"
            } else {
                type = "vector"
            }
        }
        if (type == "value") 
            amplModelAdd(paste("param ", Names[count], ";", sep = ""), project)
        if (type == "vector") 
            amplModelAdd(paste("param ", Names[count], "{1 ..", length(a), "};", sep = ""), project)
        if (type == "matrix") 
            amplModelAdd(paste("param ", Names[count], 
                "{1..", NCOL(a), ", 1..", NROW(a), "}", ";", sep=""), project)
        amplDataAdd(Names[count], data = a, type = type, project)
    }
    amplModelAdd(paste("param lower {1..", length(par.lower), "};", sep = ""), project) 
    amplModelAdd(paste("param upper {1..", length(par.upper), "};", sep = ""), project) 
    amplDataAdd("lower", data = par.lower, type = "vector", project)
    amplDataAdd("upper", data = par.upper, type = "vector", project)
    if(trace) amplDataShow(project)
    
    # Add Objective:
    amplModelAdd("var x{1..n} ;", project)
    amplModelAdd(paste( "minimize Function: ", objective, sep = ""), project)
    amplModelAdd(amplCons, project)
    if(trace) amplModelShow(project)  
    
    # Write Run File:
    solver = "ipopt"
    amplRunOpen(project)
    run <- c(
        "reset ;",
        paste("option solver ", solver, " ;", sep = ""),
        paste("model ", project, ".mod ;", sep = ""),
        paste("data ", project, ".dat ;", sep = ""),
        "solve ;",
        paste("display x > ", project, ".txt ;", sep = ""),
        "exit ;")
    amplRunAdd(run, project)
    if(trace) amplRunShow(project)
    
    # AMPL:
    command = paste("ampl", paste(project, "run", sep="."))
    solve = system(command, show.output.on.console = TRUE)

    # Read Result:
    file <- paste(project, "txt", sep = ".")
    ans = scan(file, what = character())
    ans = matrix(as.numeric(ans[-c(1:3, length(ans))]), byrow=TRUE, ncol = 2)
    index = sort(ans[,1], index.return = TRUE)$ix
    solution = ans[index, 2]
   
    # Return Value:
    value <- list(
        opt = args, 
        solution = solution, 
        objective = NA,
        status = NA,
        message = "none",
        solver = "amplNLP")
    class(value) = c("solver", "list")
    value      
}


# -----------------------------------------------------------------------------


amplNLP <-
    function()
{
    # A function implemented by Diethelm Wuertz
    
    # FUNCTION:

    NA
}


###############################################################################


amplNLPControl <- 
    function(solver = "minos", project = "ampl", trace = FALSE)
{
    # A function implemented by Diethelm Wuertz
    
    # FUNCTION:
    
    # Control Parameters:
    control <- list(
        solver = solver, 
        project = project, 
        trace = trace)
        
    # Return Value:
    control
}


###############################################################################
        
  

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:               DESCRIPTION:
#  pfolioVaR                Computes VaR for a portfolio of assets
#  pfolioCVaR               Computes CVaR for a portfoluio of assets
#  pfolioCVaRplus           Computes CVaR-Plus for a portfolio of assets
#  lambdaCVaR               Computes CVaR's atomic split value lambda
#  pfolioCVaRoptim          Computes CVaR from mean-CVaR portfolio optimization
# FUNCTION:               DESCRIPTION:
#  pfolioMaxLoss            Computes maximum loss for a portfolio 
#  pfolioReturn             Computes return series for a portfolio
#  pfolioTargetReturn       Computes target return for a portfolio
#  pfolioTargetRisk         Computes target risk for a portfolio
#  pfolioHist               Plots a histogram of portfolio returns
################################################################################


pfolioVaR <-
  function(x, weights = NULL, alpha = 0.05)
{
  # A function implemented by Diethelm Wuertz

  # Description:
  #   Compute Value-at-Risk for a portfolio of assets

  # Arguments:
  #   x - a time series, data.frame or any other rectangular object
  #       of assets which can be written as a matrix object
  #   weights - a numeric vector of weights
  #   alpha - the confidence level

  # FUNCTION:

  # Transform:
  x <- as.matrix(x)

  # Compute Portfolio VaR:
  if (is.null(weights)) weights <- rep(1/dim(x)[[2]], dim(x)[[2]])
  n <- dim(x)[1]
  x <- apply(t(t(x) * weights), 1, sum)
  n.alpha <- max(floor(n * alpha))
  ans <- as.vector(sort(x)[n.alpha])
  names(ans) <- "VaR"

  # Return Value:
  ans
}


# ------------------------------------------------------------------------------


pfolioCVaRplus <-
  function(x, weights = NULL, alpha = 0.05)
{
  # A function implemented by Diethelm Wuertz

  # Description:
  #   Compute Value-at-Risk Plus for a portfolio of assets

  # Arguments:
  #   x - a time series, data.frame or any other rectangular object
  #       of assets which can be written as a matrix object
  #   weights - a numeric vector of weights
  #   alpha - the confidence level

  # FUNCTION:

  # Transform:
  x <- as.matrix(x)

  # Compute Portfolio CVaRplus:
  if (is.null(weights)) weights = rep(1/dim(x)[[2]], dim(x)[[2]])
  n <- dim(x)[1]
  x <- apply(t(t(x) * weights), 1, sum)
  n.alpha <- max(1, floor(n * alpha)-1)
  ans <- as.vector(mean(sort(x)[1:n.alpha]))
  names(ans) <- "CVaRplus"

  # Return Value:
  ans
}


# ------------------------------------------------------------------------------


pfolioCVaR <-
  function(x, weights = NULL, alpha = 0.05)
{
  # A function implemented by Diethelm Wuertz

  # Description:
  #   Compute Conditional Value-at-risk for a portfolio of assets

  # Arguments:
  #   x - a time series, data.frame or any other rectangular object
  #       of assets which can be written as a matrix object
  #   weights - a numeric vector of weights
  #   alpha - the confidence level
  #   lambda - split value

  # FUNCTION:

  # Transform:
  data <- as.matrix(x)

  # Input Data:
  if (is.null(weights)) weights = rep(1/dim(data)[[2]], dim(data)[[2]])
  n <- dim(data)[1]
  Rp <- apply(t(t(data)*weights), 1, sum)

  # Sort the Portfolio returns Y
  sorted <- sort(Rp)

  # Compute Portfolio VaR:
  n.alpha <- floor(n*alpha)
  VaR <- sorted[n.alpha]

  # Compute Portfolio CVaRplus:
  n.alpha <- max(1, floor(n*alpha)-1)
  CVaRplus <- mean(sorted[1:n.alpha])

  # Compute Portfolio CVaR:
  lambda <- 1 - floor(n*alpha)/(n*alpha)
  ans <- as.vector(lambda*VaR + (1-lambda)*CVaRplus)
  names(ans) <- "CVaR"
  attr(ans, "control") = c(CVaRplus = CVaRplus, lambda = lambda)

  # Return Value:
  ans
}


# ------------------------------------------------------------------------------


lambdaCVaR <-
  function(n, alpha = 0.05)
{
  # A function implemented by Diethelm Wuertz

  # Description:
  #   Computes CVaR's atomic split value lambda

  # Arguments:
  #   n - the number of oberservations
  #   alpha - the confidence interval

  # FUNCTION:

  # Compute CVaR lambda:
  lambda <- 1 - floor(alpha * n) / (alpha * n)
  names(lambda) <- "lambda"

  # Return Value:
  lambda
}


# ------------------------------------------------------------------------------


pfolioCVaRoptim <- 
  function(x, weights = NULL, alpha=0.05)
{
  # A function implemented by Diethelm Wuertz

  # Description:
  #   Compute Conditional Value-at-risk by mean-CVaR portfolio Optimization

  # Arguments:
  #   x - a time series, data.frame or any other rectangular object
  #       of assets which can be written as a matrix object
  #   weights - a numeric vector of weights
  #   alpha - the confidence level

  # FUNCTION:

  # Transform:
  data <- as.matrix(x)
    
  if (is.null(weights)) weights <- rep(1/dim(data)[[2]], dim(data)[[2]])
  Rp <- apply(t(t(data) * weights), 1, sum)
  .f <- function(VaR, data, weights, alpha) {
    S <- nrow(data)
    X <- apply(t(t(data) * weights), 1, sum) -VaR
    CVaR <- VaR + sum((X - abs(X))/2) / S / alpha
    CVaR }
    
  # Compute optimized CVaR:
  ans <- optimize(.f, interval=range(Rp), tol=.Machine$double.eps, 
    maximum=TRUE, data=data, weights=weights, alpha=alpha) 
  ans <- ans$objective
  names(ans) <- "CVaRoptim"
      
  # Return Value:
  ans
}


################################################################################


pfolioMaxLoss <-
  function(x, weights = NULL)
{
  # A function implemented by Diethelm Wuertz

  # Description:
  #   Computes maximum loss for a portfolio of assets

  # Arguments:
  #   x - a timeSeries, data.frame or any other rectangular object
  #       of assets which can be written as a matrix object
  #   weights - the vector of weights
  #   alpha - the confidence level

  # FUNCTION:

  # Transform:
  x <- as.matrix(x)

  # Compute MaxLoss [MinReturn]:
  if (is.null(weights)) {
      weights = rep(1/dim(x)[[2]], dim(x)[[2]])
  }
  x = apply(t(t(x)*weights), 1, sum)
  ans = min(x)

  # Return Value:
  ans
}


# ------------------------------------------------------------------------------


pfolioReturn <- 
  function(x, weights=NULL, geometric=FALSE)
{   
  # A function implemented by Diethelm Wuertz
  
  # Description:
  #    Returns portfolio returns
  
  # Arguments:
  #    x - a 'timeSeries' object
  
  # Details:
  #    A fast(er) reimplementation
  
  # FUNCTION:
  
  # Compute Portfolio Returns:
  weights <- as.vector(weights)
  if(geometric) {
    X <- t ( colCumprods(1+x) - 1 )
    X <- rbind(diff(  t ( X * weights ) ))
    Return <- x[, 1]
    series(Return[+1, ]) <- x[1, ] %*% weights
    series(Return[-1, ]) <- rowSums(X)
  } else {
    Return <- x[, 1]
    series(Return) <- x %*% weights
  }
  colnames(Return) <- "pfolioRet"
  
  # Return Value:
  Return
}


# ------------------------------------------------------------------------------


pfolioTargetReturn <-
  function(x, weights = NULL)
{
  # A function implemented by Diethelm Wuertz

  # Description:
  #   Computes return value of a portfolio

  # Arguments:
  #   x - a timeSeries, data.frame or any other rectangular object
  #       of assets which can be written as a matrix object
  #   weights - the vector of weights

  # FUNCTION:

  # Transform:
  x <- as.matrix(x)

  # Compute Portfolio Returns:
  ans = mean(pfolioReturn(x = x, weights = weights))

  # Return Value:
  names(ans) = "TargetReturn"
  ans
}


# ------------------------------------------------------------------------------


pfolioTargetRisk <-
  function(x, weights = NULL)
{
  # A function implemented by Diethelm Wuertz

  # Description:
  #   Computes risk from covariance matrix of a portfolio

  # Arguments:
  #   x - a timeSeries, data.frame or any other rectangular object
  #       of assets which can be written as a matrix object
  #   weights - the vector of weights

  # FUNCTION:

  # Transform:
  x <- as.matrix(x)

  # Compute Portfolio Returns:
  if (is.null(weights))
      weights = rep(1/dim(x)[[2]], dim(x)[[2]])
  ans = as.vector(sqrt(weights %*% cov(x) %*% weights))

  # Return Value:
  names(ans) = "TargetRisk"
  ans
}


# ------------------------------------------------------------------------------


pfolioHist <-
  function(x, weights = NULL, alpha = 0.05, range = NULL, details = TRUE, ...)
{
  # A function implemented by Diethelm Wuertz

  # Description:
  #   Plots a histogram of the returns of a portfolio

  # Arguments:
  #   x - a timeSeries, data.frame or any other rectangular object
  #       of assets which can be written as a matrix object
  #   weights - the vector of weights

  # FUNCTION:

  # Transform:
  x <- as.matrix(x)

  # Suppress Warnings:
  opt = options()
  options(warn = -1)

  # Plot Portfolio Returns:
  Returns = pfolioReturn(x = x, weights = weights)
  if (is.null(range)) {
      lim = 1.05 * pfolioMaxLoss(x = x, weights = weights)[[1]]
      xlim = c(lim, -lim)
  } else {
      xlim = range
  }
  Histogram = hist(Returns, xlim = xlim, xlab = "Portfolio Return %",
      probability = TRUE, col = "steelblue4", border = "white", ...)
  r = seq(xlim[1], xlim[2], length = 201)
  lines(r, dnorm(r, mean = mean(Returns), sd = sd(Returns)), ...)

  points(Returns, rep(0, length(Returns)), pch = 20,
      col = "orange", cex = 1.25)

  # Add VaR, CVaRplus and MaxLoss:
  V1 = pfolioVaR(x = x, weights = weights, alpha = alpha)[[1]]
  abline(v = V1, col = "blue", ...)
  V2 = pfolioCVaRplus(x = x, weights = weights, alpha = alpha)[[1]]
  abline(v = V2, col = "red", ...)
  V3 = pfolioMaxLoss(x = x, weights = weights)[[1]]
  abline(v = V3, col = "green", ...)
  V4 = as.vector(mean(Returns))[[1]]
  V5 = as.vector(sd(Returns))[[1]]

  yt = max(density(Returns)$y)

  text(V1, yt, as.character(round(V1, 2)), cex = 0.75, col = "orange")
  text(V2, yt, as.character(round(V2, 2)), cex = 0.75, col = "orange")
  text(V3, yt, as.character(round(V3, 2)), cex = 0.75, col = "orange")
  text(V4, yt, as.character(round(V4, 2)), cex = 0.75, col = "orange")

  yt = 0.95 * yt
  text(V1, yt, "VaR", cex = 0.75, col = "orange")
  text(V2, yt, "CVaR+", cex = 0.75, col = "orange")
  text(V3, yt, "maxLoss", cex = 0.75, col = "orange")
  text(V4, yt, "Mean", cex = 0.75, col = "orange")

  # Result:
  options(opt)
  ans = list(VaR = V1, VaRplus = V2, maxLoss = V3, mean = V4, sd = V5)
  if (details) {
      cat("\nVaR:      ", V1)
      cat("\nVaRplus:  ", V2)
      cat("\nmax Loss: ", V3)
      cat("\nMean:     ", V4)
      cat("\nStDev:    ", V5)
      cat("\n")
  }

  # Return Value:
  invisible(ans)
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the 
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General 
# Public License along with this library; if not, write to the 
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
# MA 02111-1307 USA


################################################################################
# FUNCTION:                DESCRIPTION:
#  equidistWindows          Defines default equal distant rolling windows
#  tangencyStrategy         Defines default tangency strategy portfolio
#  emaSmoother              Defines default EMA weights smoother
################################################################################


equidistWindows <-
    function(data, backtest = portfolioBacktest())
{
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    #   Defines default equidistant rolling windows
   
    # Arguments:
    #   data - portfolio assets set, an object of class 'timeSeries'
    #   backtest - an object of class 'fPFOLIOBACKTEST'
   
    # Note:
    #   This is an example for a user defined windows function ...
   
    # Example:
    #   equidistWindows(as.timeSeries(data(LPP2005REC)))
   
    # FUNCTION:
   
    # Settings:
    horizon = getWindowsHorizon(backtest)
   
    # Rolling Windows:
    ans = rollingWindows(x = data, period = horizon, by = "1m")
   
    # Return Value:
    ans
}


# ------------------------------------------------------------------------------


tangencyStrategy <-
    function(data, spec = portfolioSpec(), constraints = "LongOnly", 
    backtest = portfolioBacktest())
{ 
    # A function implemented by Diethelm Wuertz and William Chen
    
    # FUNCTION:
    
    # Strategy Portfolio:
    strategyPortfolio <- try(tangencyPortfolio(data, spec, constraints))
    
    # If tangency portfolio doesn't exist take the minimum variance portfolio:
    if (inherits(strategyPortfolio,"try-error")) {
        strategyPortfolio <- minvariancePortfolio(data, spec, constraints)
    }
    
    # Return Value:
    strategyPortfolio
}


# ------------------------------------------------------------------------------


emaSmoother <-
    function(weights, spec, backtest)
{
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    #   A user defined weights smoother for portfolio backtesting
   
    # Arguments:
    #   weights - a numeric matrix of weights
    #   spec - portfolio spec, an object of class fPFLOLIOSPEC
    #   backtest - portfolio backtest, an object of class fPFLOLIOBACKTEST
   
    # Example:
    #   ans = portfolioBacktesting( ... )
    #   emaSmoother(ans$weights, spec, backtest)
   
    # FUNCTION:
   
    # EMA Function:
    ema <- function (x, lambda) {
        x = as.vector(x)
        lambda = 2/(lambda + 1)
        xlam = x * lambda
        xlam[1] = x[1]
        ema = filter(xlam, filter = (1 - lambda), method = "rec")
        ema[is.na(ema)] <- 0
        as.numeric(ema) }
       
    # Lambda:
    lambda <- getSmootherLambda(backtest)
    lambdaLength <- as.numeric(substr(lambda, 1, nchar(lambda) - 1))
    lambdaUnit <- substr(lambda, nchar(lambda), nchar(lambda))
    stopifnot(lambdaUnit == "m")
    lambda <- lambdaLength
   
    # Initial Weights:
    nAssets <- ncol(weights)
    initialWeights = getSmootherInitialWeights(backtest)
    if (!is.null(initialWeights)) weights[1, ] = initialWeights

    # Compute Exponentially Smoothed Weights:
    smoothWeights1 = NULL
    for (i in 1:nAssets) {
        # print("first smooth")
        EMA <- ema(weights[, i], lambda = lambda)
        smoothWeights1 <- cbind(smoothWeights1, EMA)
    }
   
    # Double Smoothing ?
    doubleSmooth <- getSmootherDoubleSmoothing(backtest)
    if (doubleSmooth) {
        # print("second smooth")
        smoothWeights = NULL
        for (i in 1:nAssets) {
            EMA <- ema(smoothWeights1[, i], lambda = lambda)
            smoothWeights = cbind(smoothWeights, EMA)
        }
    } else {
        smoothWeights <- smoothWeights1
    }
   
    # Add Names:
    rownames(smoothWeights) <- rownames(weights)
    colnames(smoothWeights) <- colnames(weights)
   
    # Return Value:
    smoothWeights
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


###############################################################################
# FUNCTION:
#  print.solver
#  summary.solver
###############################################################################


print.solver <- 
    function(x, ...)
{
    # A function implemented by Diethelm Wuertz
    
    # Number of Variables:
    nSolution <- length(x$solution)
    
    # Print:
    cat("\nSolver:      ", x$solver)
    cat("\nSolution:    ", 1, ":", x$solution[1])
    for (i in 2:nSolution) 
    cat("\n             ", i, ":", x$solution[i])
    cat("\nObjective:   ", x$objective)
    cat("\nStatus:      ", x$status)
    cat("\nMessage:     ", x$message)
    cat("\n")
}


# -----------------------------------------------------------------------------


.summary.solver <- 
     function(object, ...)
{
    # A function implemented by Diethelm Wuertz
    
    # Print:
    print(object[1])
}


###############################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                     DESCRIPTION:
#  solveRquadprog                Portfolio interface to solver Rquadprog
#  .rquadprogArguments           Returns arguments for solver
#  .rquadprog                    Wrapper to solver function
################################################################################


solveRquadprog <-
    function(data, spec, constraints)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Portfolio interface to solver Rquadprog

    # FUNCTION:

    # Transform Data:
    Data <- portfolioData(data, spec)
    data <- getSeries(Data)
    nAssets <- getNAssets(Data)

    # Compile Arguments for Solver:
    args <- .rquadprogArguments(Data, spec, constraints)

    # Solve Multiassets Portfolio:
    ans <- .rquadprog(
        Dmat = args$Dmat,
        dvec = args$dvec,
        Amat = args$Amat,
        bvec = args$bvec,
        meq = args$meq)

    # Save Arguments:
    ans$optim$args <- args

    # class:
    class(ans) = c("solver", "list")

    # Return Value:
    ans
}


################################################################################


.rquadprogArguments <-
    function(data, spec, constraints)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns quadprog conform arguments for the solver

    # FUNCTION:

    # Data and Constraints as S4 Objects:
    Data <- portfolioData(data, spec)
    data <- getSeries(Data)
    Sigma <- getSigma(Data)
    nAssets <- getNAssets(Data)

    # Set up A_mat of Constraints:
    eqsumW <- eqsumWConstraints(Data, spec, constraints)
    minsumW <- minsumWConstraints(Data, spec, constraints)
    maxsumW <- maxsumWConstraints(Data, spec, constraints)
    Amat = rbind(eqsumW[, -1], diag(nAssets), -diag(nAssets))
    if(!is.null(minsumW)) Amat = rbind(Amat, minsumW[, -1])
    if(!is.null(maxsumW)) Amat = rbind(Amat, -maxsumW[, -1])

    # Set up Vector A_mat >= bvec of Constraints:
    minW <- minWConstraints(Data, spec, constraints)
    maxW <- maxWConstraints(Data, spec, constraints)
    bvec <- c(eqsumW[, 1], minW, -maxW)
    if(!is.null(minsumW)) bvec = c(bvec, minsumW[, 1])
    if(!is.null(maxsumW)) bvec = c(bvec, -maxsumW[, 1])

    # Part (meq=1) or Full (meq=2) Investment, the Default ?
    meq <- nrow(eqsumW)

    # Directions:
    dir <- c(
        rep("==", times = meq),
        rep(">=", times = length(bvec) - meq))

    # Return Value:
    list(
        Dmat = Sigma, 
        dvec = rep(0, nAssets),
        Amat = t(Amat), 
        bvec = bvec, 
        meq = meq, 
        dir = dir)
}


################################################################################


.rquadprog <-
    function(Dmat, dvec, Amat, bvec, meq)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Goldfarb and Idnani's quadprog solver function

    # Note:
    #   Requires to load contributed R package quadprog from which we use
    #   the Fortran subroutine of the quadratic solver.

    # Package: quadprog
    #   Title: Functions to solve Quadratic Programming Problems.
    #   Author: S original by Berwin A. Turlach <berwin.turlach@anu.edu.au>
    #       R port by Andreas Weingessel <Andreas.Weingessel@ci.tuwien.ac.at>
    #   Maintainer: Andreas Weingessel <Andreas.Weingessel@ci.tuwien.ac.at>
    #   Description: This package contains routines and documentation for
    #       solving quadratic programming problems.
    #   License: GPL-2

    # Value of slove.QP():
    #   solution - vector containing the solution of the quadratic
    #       programming problem.
    #   value - scalar, the value of the quadratic function at the
    #       solution
    #   unconstrained.solution - vector containing the unconstrained
    #       minimizer of the quadratic function.
    #   iterations - vector of length 2, the first component contains
    #       the number of iterations the algorithm needed, the second
    #       indicates how often constraints became inactive after
    #       becoming active first. vector with the indices of the
    #       active constraints at the solution.

    # FUNCTION:

    # Optimize:
    optim <- try(quadprog::solve.QP(Dmat, dvec, Amat, bvec, meq), silent = TRUE)
    if (inherits(optim, "try-error")) {
        weights <- rep(0, length(dvec))
        optim <- list()
        status <- 1
    } else {
        # Set Tiny Weights to Zero:
        weights <- .checkWeights(optim$solution)
        attr(weights, "invest") = sum(weights)
        status <- 0
    }

    # Compose Output List:
    ans <- list(
        type = "MV",
        solver = "solveRquadprog",
        optim = optim,
        weights = weights,
        targetReturn = bvec[1],
        targetRisk = sqrt(weights %*% Dmat %*% weights)[[1, 1]],
        objective = sqrt(weights %*% Dmat %*% weights)[[1, 1]],
        status = status, 
        message = NA)

    # Return Value:
    ans
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################
# FUNCTION:                         DESCRIPTION:
#  getWindows                        Extracts windows information             
#   getWindowsFun                     Extracts windows function
#   getWindowsParams                  Extracts windows function parameters        
#   getWindowsHorizon                 Extracts windows Horizon
# FUNCTION:                         DESCRIPTION:
#  getStrategy                       Extracts strategy information
#   getStrategyFun                    Extracts strategy function
#   getStrategyParams                 Extracts strategy function parameters
# FUNCTION:                         DESCRIPTION:
#  getSmoother                       Extracts smoother information
#   getSmootherFun                    Extracts smoother function
#   getSmootherParams                 Extracts smoother function parameters
#   getSmootherLambda                 Extracts decay parameter
#   getSmootherDoubleSmoothing        Extracts double smoothing flag
#   getSmootherInitialWeights         Extracts initial weights
#   getSmootherSkip                   Extracs skip smoothing flag
# FUNCTION:                         DESCRIPTION:
#  getMessages                       Extracts messages
################################################################################


getWindows <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getWindows")
}


# ------------------------------------------------------------------------------


getWindowsFun <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getWindowsFun")
}


# ------------------------------------------------------------------------------


getWindowsParams <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getWindowsParams")
}


# ------------------------------------------------------------------------------


getWindowsHorizon <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getWindowsHorizon")
}


# ------------------------------------------------------------------------------


getSmoother <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getSmoother")
}


# ------------------------------------------------------------------------------


getSmootherFun <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getSmootherFun")
}

# ------------------------------------------------------------------------------


getSmootherParams <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getSmootherParams")
}


# ------------------------------------------------------------------------------


getSmootherLambda <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getSmootherLambda")
}

# ------------------------------------------------------------------------------


getSmootherDoubleSmoothing <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getSmootherDoubleSmoothing")
}


# ------------------------------------------------------------------------------


getSmootherInitialWeights <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getSmootherInitialWeights")
}


# ------------------------------------------------------------------------------

getSmootherSkip <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getSmootherSkip")
}

# ------------------------------------------------------------------------------


getStrategy <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getStrategy")
}


# ------------------------------------------------------------------------------


getStrategyFun <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    UseMethod("getStrategyFun")
}


# ------------------------------------------------------------------------------


getStrategyParams <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getStrategyParams")
}


# ------------------------------------------------------------------------------


if(FALSE) {
# Already defined in fPortfolio
getMessages <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getMessages")
}
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the 
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General 
# Public License along with this library; if not, write to the 
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
# MA 02111-1307 USA


################################################################################
# FUNCTION:                    DESCRIPTION:
#  setWindowsFun<-              Sets name of rolling windows function
#  setWindowsParams<-           Sets additional parameters to windows function
#  setWindowsHorizon<-          Sets horizon of the rolling window
# FUNCTION:                    DESCRIPTION:         
#  setStrategyFun<-             Sets name of portfolio strategy function    
#  setStrategyParams<-          Sets additional parameters to strategy function
# FUNCTION:                    DESCRIPTION:
#  setSmootherFun<-             Sets name of weights smoothing function
#  setSmootherParams<-          Sets additional parameters to smoother function
#  setSmootherLambda<-          Sets lambda for EMA smoothing
#  setSmootherDoubleSmoothing<- Sets double ema setting, logical
#  setSmootherInitialWeights<-  Sets initial weights of the portfolio
#  setSmootherSkip<-            Sets number of months to skip starting
################################################################################


"setWindowsFun<-" <- 
function(backtest, value)
{
    # A function implemented by William Chen
    
    # Description:
    #   Sets name of rolling windows function
    
    # Arguments:
    
    # FUNCTION:
    
    # Set Value:
    backtest@windows$windows <- value
    
    # Return Value:
    backtest
}


# ------------------------------------------------------------------------------


"setWindowsParams<-" <- 
function(backtest, value)
{
    # A function implemented by William Chen
    
    # Description:
    #   Sets additional parameters to windows function
    
    # Arguments:
    
    # FUNCTION:
    
    # Set Value:
    backtest@windows$params <- value
    
    # Return Value:
    backtest
}


# ------------------------------------------------------------------------------


"setWindowsHorizon<-" <- 
function(backtest, value)
{
    # A function implemented by William Chen
    
    # Description:
    #   Sets horizon of the rolling window
    
    # Arguments:
    
    # FUNCTION:
    
    # Set Value:
    backtest@windows$params$horizon <- value
    
    # Return Value:
    backtest
}


# ------------------------------------------------------------------------------


"setStrategyFun<-" <- 
function(backtest, value)
{
    # A function implemented by William Chen
    
    # Description:
    #   Sets portfolio strategy function
    
    # Arguments:
    
    # FUNCTION:
    
    # Set Value:
    backtest@strategy$strategy <- value
    
    # Return Value:
    backtest
}

# ------------------------------------------------------------------------------
          

"setStrategyParams<-" <- 
function(backtest, value)
{
    # A function implemented by William Chen
    
    # Description:
    #   Sets additional parameters to strategy function
    
    # Arguments:
    
    # FUNCTION:
    
    # Set Value:
    backtest@strategy$params <- value
    
    # Return Value:
    backtest
}


# ------------------------------------------------------------------------------


"setSmootherFun<-" <- 
function(backtest, value)
{
    # A function implemented by William Chen
    
    # Description:
    #   Sets name of weights smoothing function
    
    # Arguments:
    
    # FUNCTION:
    
    # Set Value:
    backtest@smoother$smoother <- value
    
    # Return Value:
    backtest
}


# ------------------------------------------------------------------------------


"setSmootherParams<-" <- 
function(backtest, value)
{
    # A function implemented by William Chen
    
    # Description:
    #   Sets additional parameters to smoother function
    
    # Arguments:
    
    # FUNCTION:
    
    # Set Value:
    backtest@smoother$params <- value
    
    # Return Value:
    backtest
}


# ------------------------------------------------------------------------------


"setSmootherLambda<-" <- 
function(backtest, value)
{
    # A function implemented by William Chen
    
    # Description:
    #   Sets lambda parameter for EMA smoothing
    
    # Arguments:
    
    # FUNCTION:
    
    # Set Value:
    backtest@smoother$params$lambda <- value
    
    # Return Value:
    backtest
}


# ------------------------------------------------------------------------------


"setSmootherDoubleSmoothing<-" <- 
function(backtest, value)   
{
    # A function implemented by William Chen
    
    # Description:
    #   Sets double EMA setting, TRUE or FALSE, a logical
    
    # Arguments:
    
    # FUNCTION:
    
    # Set Value:
    backtest@smoother$params$doubleSmoothing <- value
    
    # Return Value:
    backtest
}


# ------------------------------------------------------------------------------


"setSmootherInitialWeights<-" <- 
function(backtest, value)
{
    # A function implemented by William Chen
    
    # Description:
    #   Sets initial weights of the portfolio
    
    # Arguments:
    
    # FUNCTION:
    
    # Set Value:
    backtest@smoother$params$initialWeights <- value
    
    # Return Value:
    backtest
}


# ------------------------------------------------------------------------------


"setSmootherSkip<-" <- 
function(backtest, value)
{
    # A function implemented by William Chen
    
    # Description:
    #   Sets number of months to skip starting values
    
    # Arguments:
    
    # FUNCTION:
    
    # Set Value:
    backtest@smoother$params$skip <- value
    
    # Return Value:
    backtest
}
   

################################################################################
  
 

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################


.onAttach <- 
function(libname, pkgname)
{
  # do whatever needs to be done when the package is loaded
  # some people use it to bombard users with 
  # messages using 
  
  # packageStartupMessage( "\n" )
  # packageStartupMessage( "Rmetrics Package fPortfolio" ) 
  # packageStartupMessage( "Portfolio Optimization" )
  # packageStartupMessage( "Copyright (C) 2005-2014 Rmetrics Association Zurich" )  
  # packageStartupMessage( "Educational Software for Financial Engineering and Computational Science" ) 
  # packageStartupMessage( "Rmetrics is free software and comes with ABSOLUTELY NO WARRANTY." ) 
  # packageStartupMessage( "https://www.rmetrics.org --- Mail to: info@rmetrics.org" ) 
}


###############################################################################


.onLoad <- 
  function(libname, pkgname)
{
    if(!is.numeric(timeDate::getRmetricsOptions("length.print"))) 
        timeDate::setRmetricsOptions(length.print = 5)
    
    timeDate::setRmetricsOptions(.x.save = NA)
    
    eval(attach <- function(what) 
      base::attach(what, warn.conflicts=FALSE), envir=.GlobalEnv)

}
    # Startup Mesage and Desription:
    # MSG <- if(getRversion() >= "2.5") packageStartupMessage else message
    # dsc <- packageDescription(pkg)
    # if(interactive() || getOption("verbose")) { 
    #    title <- paste(strsplit(dsc$Title, split = "-")[1:2])
    #    MSG(paste(
    #        "\nPackage ", pkg, " (", dsc$Version, ") loaded.\n",
    #        dsc$Title, "\n", 
    #        dsc$Copyright, ", ", dsc$License, "\n", 
    #        dsc$Author, "\n", 
    #        dsc$URL, "\n", sep="")) 
    # }


###############################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the 
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General 
# Public License along with this library; if not, write to the 
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
# MA 02111-1307 USA


################################################################################
# FUNCTION:                     DESCRIPTION:
#  getPortfolio                  Extracts portfolio from value object  
# FUNCTION:                     DESCRIPTION:   
#  getWeights                    Extracts weights from value object       
#  getCovRiskBudgets             Extracts covarisnce risk budgets value   
#  getTargetReturn               Extracts target return from value object 
#  getTargetRisk                 Extracts target risk from value object   
#  getAlpha                      Extracts CVaR alpha from value object    
#  getRiskFreeRate               Extracts risk free rate from value object
#  getNFrontierPoints            Extracts number of frontier points value 
#  getStatus                     Extracts status from value object        
################################################################################
   
    
getPortfolio.fPFOLIOVAL <- 
    function(object) object@portfolio

getWeights.fPFOLIOVAL <- 
    function(object) object@portfolio$weights
getCovRiskBudgets.fPFOLIOVAL <- 
    function(object) object@portfolio$covRiskBudgets
getTargetReturn.fPFOLIOVAL <- 
    function(object) object@portfolio$targetReturn
getTargetRisk.fPFOLIOVAL <- 
    function(object) object@portfolio$targetRisk

getAlpha.fPFOLIOVAL <- 
    function(object) object@portfolio$targetAlpha

getRiskFreeRate.fPFOLIOVAL <- 
    function(object) object@Portfolio$riskFreeRate
getNFrontierPoints.fPFOLIOVAL <- 
    function(object) object@portfolio$nFrontierPoints

getStatus.fPFOLIOVAL <- 
    function(object) object@portfolio$status


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


###############################################################################
# FUNCTION:                    DESCRIPTION:
#  rneosQP                      Rmetrics Interface for AMPL/NEOS QP solvers
#  neosQP                       Convenience wrapper for AMPL/NEOS QP solvers
#  neosQPControl                NEOS QP ontrol parameter list
###############################################################################
    

rneosQP <- 
    function(objective, lower=0, upper=1, linCons, control=list())
{   
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Rmetrics Interface for AMPL/NEOS QP solvers
    
    # Arguments:
    #   objective - list(dvec=NULL, Dmat = NULL)
    
    # FUNCTION:
    
    # Control List:
    ctrl <- neosQPControl()
    if (length(control) > 0)
        for (name in names(control)) ctrl[name] = control[name]
    control <- ctrl
    
    # Control Parameters:
    solver <- control$solver
    category <- control$category
    project <- control$project
    inf <- control$inf
    trace <- control$trace    
    
    # General Settings:
    dvec <- objective$dvec
    Dmat <- objective$Dmat
    obj <- rbind(dvec, Dmat)
    
    # Box Constraints:
    replicate <- function(x, n) if(length(x) == 1) rep(x, n) else x
    n <- ncol(obj)
    x_L <- replicate(lower, n)
    x_U <- replicate(upper, n)
    x_L[is.infinite(x_L)] <- inf*sign(x_L[is.infinite(x_L)])
    x_U[is.infinite(x_U)] <- inf*sign(x_U[is.infinite(x_U)])
      
    # Linear Constraints:  
    A <- linCons[[1]]
    m <- nrow(A)
    b_L <- replicate(linCons[[2]], m)
    b_U <- replicate(linCons[[3]], m)
    b_L[is.infinite(b_L)] <- inf*sign(b_L[is.infinite(b_L)])
    b_U[is.infinite(b_U)] <- inf*sign(b_U[is.infinite(b_U)]) 
    
    # Optimize Portfolio:
    value <- neosQP(objective, x_L, x_U, A, b_L, b_U, control)
    
    # Return Value:
    value
}


###############################################################################


neosQP <- 
    function(
    objective=list(dvec=NULL, Dmat=NULL), 
    x_L=NULL, x_U=NULL, A=NULL, b_L=NULL, b_U=NULL,
    control=list(), ...)
{   
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Convenience wrapper for AMPL/NEOS QP solvers
    
    # Arguments:
    #   objective - list(dvec=NULL, Dmat = NULL)
    
    # FUNCTION:
    
    # Control List:
    ctrl <- neosQPControl()
    if (length(control) > 0)
        for (name in names(control)) ctrl[name] = control[name]
    control <- ctrl
    
    # Control Parameters:
    solver <- control$solver
    category <- control$category
    project <- control$project
    inf <- control$inf
    trace <- control$trace
    
    # Solver Settings:
    dvec <- objective$dvec
    Dmat <- objective$Dmat
    obj <- rbind(dvec, Dmat)
    n <- ncol(obj)
    m <- nrow(A)

    # Write AMPL Model File:
    amplModelOpen(project)
    model <- c(
        "param n ;",
        "param m ;",
        "param c{1..n} ;",
        "param F{1..n, 1..n} ;",        
        "param x_L{1..n} ;",
        "param x_U{1..n} ;", 
        "param A{1..m, 1..n} ;",
        "param b_L{1..m} ;",
        "param b_U{1..m} ;",   
        "var x{1..n};",
        "minimize Risk: sum {i in 1..n} x[i]*c[i] + 0.5*sum {i in 1..n} sum{j in 1..n} x[i]*F[i,j]*x[j] ;",
        "s.t. lower {i in 1..n}: x[i] >= x_L[i] ;",
        "s.t. upper {i in 1..n}: x[i] <= x_U[i] ;",   
        "s.t. linLower {j in 1..m}: sum{i in 1..n} A[j, i]*x[i] >= b_L[j] ;", 
        "s.t. linUpper {j in 1..m}: sum{i in 1..n} A[j, i]*x[i] <= b_U[j] ;", 
        NULL)
    amplModelAdd(model, project)
    if (trace) amplModelShow(project)
        
    # Write AMPL Data File:
    amplDataOpen(project)
    amplDataAddValue (data="n", value=n, project)
    amplDataAddValue (data="m", value=m, project)
    amplDataAddVector(data="c", vector=dvec, project)
    amplDataAddMatrix(data="F", matrix=Dmat, project) 
    amplDataAddVector(data="x_L", vector=x_L, project)
    amplDataAddVector(data="x_U", vector=x_U, project)    
    amplDataAddMatrix(data="A", matrix=A, project)
    amplDataAddVector(data="b_L", vector=b_L, project)
    amplDataAddVector(data="b_U", vector=b_U, project)  
    if (trace) amplDataShow(project)
    
    # Write AMPL/NEOS RUN File:
    amplRunOpen(project)
    run <- c(
        "solve ;",
        "display x;",
        "display solve_result_num;", 
        "display solve_result;",
        "display solve_message;",
        "exit ;")
    amplRunAdd(run, project)
    if (trace) amplRunShow(project)
    
    # Get AMPL Files:
    model <- paste(readLines(
      paste(project, "mod", sep=".")), sep = " ", collapse ="\n")
    data <- paste(readLines(
      paste(project, "dat", sep=".")), sep = " ", collapse ="\n")
    run <- paste(readLines(
      paste(project, "run", sep=".")), sep = " ", collapse ="\n")
    
    # Setup NEOS and AMPL Specifications:
    amplSpec <- list(model=model, data=data, commands=run, comments="NEOS")
    solverTemplate <- rneos::NgetSolverTemplate(
      category=category, solvername=solver, inputMethod="AMPL")
    xmls <- rneos::CreateXmlString(
      neosxml=solverTemplate, cdatalist=amplSpec)
    
    # Submit and Fetch NEOS Job:
    submittedJob <- rneos::NsubmitJob(
      xmlstring=xmls, user="rneos", interface="", id=0)
    ans <- rneos::NgetFinalResults(obj=submittedJob, convert=TRUE)
    out <- strsplit(ans@ans, split="\n")[[1]]
  
    # Get Weights:
    Index <- (grep("x .*. :=", out)+1):( grep("^;$", out)-1)
    Out <- out[Index]
    splits <- strsplit(paste(Out, collapse=" "), " ")[[1]]
    solution <- as.numeric(splits[splits != ""])[seq(2, 2*n, by=2)]
    Index <- as.numeric(splits[splits != ""])[seq(1, 2*n, by=2)]
    solution[Index] <- solution
    
    # Get Status:
    status <- strsplit(out[grep("solve_result", out)], split=" ")
    statusCode <- status[[1]][3]
    statusMessage <- status[[2]][3]
    
    # Get Solver Message:
    Index <- grep("solve_message", out):length(out)
    message <- out[Index]
    
    # Neos Job Version: 
    version <- out[1]
        
    # Compute Obective Function Value:
    objval <- (dvec %*% solution + 0.5 * solution %*% Dmat %*% solution)[[1, 1]]
    
    # Return Value:
    model <- capture.output(amplModelShow(project))
    run <- capture.output(amplRunShow(project))
    value = list(
        opt = list(solve=ans, model=model, run=run, out=out),
        solution = solution, 
        objective = objval,
        status = statusCode,
        message = statusMessage,
        solver = paste("AMPL", solver),
        version = version)
    class(value) <- c("solver", "list")
    value
}


# -----------------------------------------------------------------------------


neosQPControl <- 
    function(solver="ipopt", category="nco", project="neos", 
        inf=1e12, trace=FALSE) 
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns control parameter list
    
    # FUNCTION:
    
    # Control Parameter:
    control <- list(
        solver=solver, category=category, project=project, inf=inf, trace=trace)
        
    # Return Value:
    control
}



###############################################################################

################################################################################
# FUNCTION:                     DESCRIPTION:
#  show.fPORTFOLIO               S4 Print method for 'fPPORTFOLIO' objects
#  show.fPFOLIODATA              S4 Print method for 'fPFOLIODATA' objects
#  show.fPFOLIOSPEC              S4 Print method for 'fPFOLIOSPEC' objects
#  show.fPFOLIOCON               S4 Print method for 'fPFOLIOCON' objects
################################################################################


setMethod("show", "fPORTFOLIO",
    function(object)
{
    # A function implemented by Diethelm Wuertz and Yohan Chalabi
    
    # Description:
    #   S4 Print Method for an object of class "fPORTFOLIO"

    # Arguments:
    #   object - an object of class "fPORTFOLIO"

    # FUNCTION:

    # Determine Length Out:
    nFrontierPoints <- NROW(matrix(getWeights(object@portfolio), 
        ncol = getNAssets(object)))
    length.out <- getRmetricsOptions("length.print") # from Rmetrics Options
    index <-
        if (length.out) {
            unique(trunc(seq.int(from = 1, to = nFrontierPoints,
                length.out = length.out)))
        } else {
            seq.int(from = 1, to = NROW(nFrontierPoints))
        }
        
    # Print Title:
    cat("\nTitle:\n ")
        # cat(getType(object),  getTitle(object),     "\n")
        cat(getType(object),  object@title,     "\n")
        cat(" Estimator:        ", getEstimator(object), "\n")
        cat(" Solver:           ", getSolver(object),    "\n")
        cat(" Optimize:         ", getOptimize(object),  "\n")
        cat(" Constraints:      ", getConstraintsTypes(object), "\n")
    if (object@spec@ampl$ampl) {
        cat(" AMPL Project:     ", object@spec@ampl$project, "\n")
        cat(" AMPL Solver:      ", object@spec@ampl$solver, "\n")
    }
    if (!identical(index, 1))
        cat(" Portfolio Points: ", length(index), "of", nFrontierPoints, "\n")
    if (getType(object) == "CVaR")
        cat(" VaR Alpha:        ", getAlpha(object),     "\n")
        #at(" Objective:        ", getObjective(object), "\n")

    # Assets:
    nAssets <- getNAssets(object)
    Names <- names(object@data@statistics$mu)

    # Print Target Weights:
    cat("\nPortfolio Weights:\n")
    table <- matrix(round(getWeights(object@portfolio), digits = 4),
        ncol = nAssets)
    colnames(table) = Names
    rownames(table) = 1:NROW(table)
    print.table(table[index, ])

    # Print Covariance Risk Budgets:
    cat("\nCovariance Risk Budgets:\n")
    table = matrix(round(getCovRiskBudgets(object@portfolio), digits = 4),
        ncol = nAssets)
    colnames(table) = Names
    rownames(table) = 1:NROW(table)
    print.table(table[index, ])

    # PrintCVaR Risk Budgets:
    # to do ...
    
    # Print Tail Risk Budgets:
    # to do ...

    # Print Target Return and Risks:
    # DW: Note object@targetR* is a list do not use getTargetR*()
    targetReturn <- matrix(getTargetReturn(object@portfolio), ncol = 2)
    targetRisk <- matrix(getTargetRisk(object@portfolio), ncol = 4)
    target <- round(cbind(targetReturn, targetRisk), digits = 4)
    if (inherits(getSeries(object),"logical")) {
        cat("\nTarget Return and Risk:\n")
        target = target[, c(1, 3), drop = FALSE]
        colnames(target) = c("mean", "Cov")
    } else if(inherits(getSeries(object),"timeSeries")) {
        cat("\nTarget Returns and Risks:\n")
        colnames(target) = c("mean", "mu", "Cov", "Sigma", "CVaR", "VaR")
    }
    rownames(target) = 1:NROW(target)
    # DW Only print mu and Sigma for robust estimators!
    if (getEstimator(object) == "covEstimator") {
        print.table(target[index, -c(2,4)])
    } else {
        print.table(target[index, ])
    }
    
    # Print Description:
    cat("\nDescription:\n ")
    # cat(getDescription(object), "\n")
    cat(object@description, "\n")
    
    # Return Value:
    invisible(NULL)
})


# ------------------------------------------------------------------------------


setMethod("show", "fPFOLIODATA",
    function(object)
{
    # A function implemented by Diethelm Wuertz and Yohan Chalabi
    
    # Description:
    #   S4 Print Method for an object of class "fPFOLIODATA"

    # Arguments:
    #   object - an object of class "fPFOLIOSPEC"

    # FUNCTION:

    # Series:
    cat("\nHead/Tail Series Data:\n")
    if(is.null(dim(object@data$series))) {   
        cat("  No time series data available.\n")
    } else {
        cat("\n")
        print(head(object@data$series, n = 3))
        print(tail(object@data$series, n = 3))
    }
    
    # Statistics:
    cat("\nStatistics:\n\n")
    if(is.null(dim(object@data$series))) { 
        # Print mean and Cov only ..
        print(object@statistics[1:2])
    } else {
        # Print mean, Cov, estimator, mu and Sigma ...
        print(object@statistics)
    }
    
    # Tailrisk:
    # NYI

    # Return Value:
    invisible(NULL)
})


# ------------------------------------------------------------------------------


setMethod("show", "fPFOLIOSPEC",
    function(object)
{
    # A function implemented by Diethelm Wuertz and Yohan Chalabi
    
    # Description:
    #   S4 Print Method for an object of class "fPFOLIOSPEC"

    # Arguments:
    #   object - an object of class "fPFOLIOSPEC"

    # FUNCTION:

    # Model:

    cat("\nModel List:\t")

    cat("\n Type:                     ",
        object@model$type)

    cat("\n Optimize:                 ",
        object@model$optimize)

    cat("\n Estimator:                ",
        object@model$estimator)

    if (length(object@model$tailRisk) > 0) {
        cat("\n Tail Risk:                ",
            object@model$tailRisk)
    }
    # DW:
    # } else {
    #     cat("\n Tail Risk:                ", "list()")
    # }

    cat("\n Params:                   ",
        paste(names(unlist(object@model$params)), "=",
            unlist(object@model$params)))

    # Portfolio:

    cat("\n\nPortfolio List:\t")

    if (!is.null(object@portfolio$weights)) {
        cat("\n Portfolio Weights:        ",
            object@portfolio$weights)
    } else {
        cat("\n Target Weights:           ", "NULL")
    }

    if (!is.null(object@portfolio$targetReturn)) {
        cat("\n Target Return:            ",
            object@portfolio$targetReturn)
    } else {
        cat("\n Target Return:            ", "NULL")
    }

    if (!is.null(object@portfolio$targetRisk)) {
        cat("\n Target Risk:              ",
            object@portfolio$targetRisk)
    } else {
        cat("\n Target Risk:              ", "NULL")
    }

    if (!is.null(object@portfolio$riskFreeRate)) {
        cat("\n Risk-Free Rate:           ",
        as.character(object@portfolio$riskFreeRate))
    }

    if (!is.null(object@portfolio$nFrontierPoints)) {
        cat("\n Number of Frontier Points:",
        as.character(object@portfolio$nFrontierPoints))
    }

    if (!is.na(object@portfolio$status)) {
      cat("\n Status:                   ",
        as.character(object@portfolio$status)) }


    # Optimization:

    cat("\n\nOptim List:\t")

    cat("\n Solver:                   ",
        object@optim$solver)

    if (!is.null(object@optim$objective)) {
        cat("\n Objective:                ",
        as.character(object@optim$objective))
    } else {
        cat("\n Objective:                ", "list()" )
    }

    if (substr(object@optim$solver, 1, 14) != "solveRquadprog") object@optim$options$meq <- NULL
    if (length(object@optim$options) > 0) {
        cat("\n Options:                  ",
            paste(names(unlist(object@optim$options)), "=",
                unlist(object@optim$options)))
    }

    if (length(object@optim$control) > 0) {
        cat("\n Control:                  ",
        as.character(object@optim$control))
    }
    # DW
    # } else {
    #     cat("\n Control:                  ", "list()")
    # }


    cat("\n Trace:                    ",
        object@optim$trace)


    # Messages:

    if (object@messages$messages)
    {
        cat("\n\nMessage List:\t")
    
        if (!is.null(object@messages$list)) {
            cat("\n List:                     ",
                object@messages$list)
        } else {
            cat("\n List:                     ", "NULL")
        }
    }

    # AMPL:
    
    if (object@ampl$ampl)
    {
        cat("\n\nAMPL List:\t")
            
        cat("\n Project:                  ",
            object@ampl$project)
    
        cat("\n Solver:                   ",
            object@ampl$solver)
            
        cat("\n Trace:                    ",
            object@ampl$trace)
    }

    cat("\n")


    # Return Value:
    invisible(NULL)
})


# ------------------------------------------------------------------------------


setMethod("show", "fPFOLIOCON",
    function(object)
{
    # A function implemented by Diethelm Wuertz and Yohan Chalabi
    
    # Description:
    #   S4 Print Method for an object of class "fPFOLIODATA"

    # Arguments:
    #   object - an object of class "fPFOLIOSPEC"

    # FUNCTION:

    # Print Title:
    cat("\nTitle:\n ")
    cat("Portfolio Constraints\n")

    minmaxW = rbind(object@minWConstraints, object@maxWConstraints)
    rownames(minmaxW) = c("Lower", "Upper")
    if (length(minmaxW)) {
        cat("\nLower/Upper Bounds:\n")
        print(minmaxW)
    }

    eqsumW = object@eqsumWConstraints
    if (sum(dim(eqsumW)) > 2) {
        cat("\nEqual Matrix Constraints:\n")
        print(eqsumW)
    }

    minsumW = object@minsumWConstraints
    if (sum(dim(minsumW)) > 2) {
        cat("\nLower Matrix Constraints:\n")
        print(minsumW)
    }

    maxsumW = object@maxsumWConstraints
    if (sum(dim(maxsumW)) > 2) {
        cat("\nUpper Matrix Constraints:\n")
        print(maxsumW)
    }

    minmaxB <- rbind(object@minBConstraints, object@maxBConstraints)
    if (length(minmaxB) > 0 &&
        !(all(object@minBConstraints == -Inf) && all(object@maxBConstraints == 1))) {
        cat("\nLower/Upper Cov Risk Budget Bounds:\n")
        rownames(minmaxB) = c("Lower", "Upper")
        print(minmaxB)
    }

    listF = object@listFConstraints
    minF = object@minFConstraints
    maxF = object@maxFConstraints
    if (length(listF) > 0) {
        cat("\nNon-Linear Function Constraints:\n")
        minmaxF = rbind(minF, maxF)
        colnames(minmaxF) = names(listF)
        rownames(minmaxF) = c("Lower", "Upper")
        print(minmaxF)
    }
    
    nCard = object@nCardConstraints
    if(nCard > 0) {
        minCard = object@minCardConstraints
        maxCard = object@maxCardConstraints
        minmaxCard = rbind(minCard, maxCard)
        #colnames(minmaxCard) = names(listF)
        rownames(minmaxCard) = c("Lower", "Upper")
        cat("\nCardinality Constraints:\n")
        print(minmaxCard)
    }


    # Return Value:
    invisible(NULL)
})


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the 
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General 
# Public License along with this library; if not, write to the 
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
# MA 02111-1307 USA


################################################################################
# FUNCTION:                     DESCRIPTION:                
#  weightsLinePlot               Plots staggered weights
#  weightedReturnsLinePlot       Plots staggered weighted returns
#  covRiskBudgetsLinePlot        Plots covariance risk budgets
################################################################################


weightsLinePlot <- 
function(object, labels = TRUE, col = NULL, title = TRUE, 
    box = TRUE, legend = TRUE, ...)
{   
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Plots a bar chart of weights
    
    # Arguments:
    #   object - an object of class 'fPORTFOLIO'
    #   labels - should the graph be automatically labeled?
    #   col - a color palette, by default the rainbow palette
    #   title - should the graph get default title and labels?
    #   legend - should a legend be added to the plot?
    
    # FUNCTION:
    
    # Use default color if not specified ...
    Title = "Weights"
    if (is.null(col)) col = seqPalette(getNAssets(object)+1, "Blues")[-1]
    if (sum(c(par()$mfrow, par()$mfcol)) == 4) CEX = 0.9 else CEX = 0.7
    
    # Compute Weights:
    X = weights = getWeights(object)
    
    # Define Plot Range:
    ymax = max(colMaxs(weights))
    ymin = min(colMins(weights))
    range = ymax - ymin
    ymax = ymax + 0.005 * range
    ymin = ymin - 0.005 * range
    dim = dim(weights)
    range = dim[1]
    xmin = 0
    xmax = range + 0.2 * range
    
    # Create Bar Plots:
    if (labels) {
        if(legend){
            ts.plot(X, 
                gpars = list(col = col, ann = FALSE, xaxt = "n"), 
                xlim = c(xmin, xmax), ylim = c(ymin, ymax))
            legendtext = names(getStatistics(object)$mu)
            if(is.null(legendtext)){
                for(i in 1:dim[2]){legendtext[i] = paste("Asset", i, sep = " ")}
            }
            legend("topright", legend = legendtext, bty = "n", cex = CEX,
                fill = col)
        } else {
            ts.plot(weights, gpars = list(col = col, ann = FALSE, xaxt = "n"))
        }
    } else {
        ts.plot(X, ...)
    }
    
    # Add Tailored Labels -  6 may be a good Number ...
    targetRisk = getTargetRisk(object)[, 1]
    targetReturn = getTargetReturn(object)[, 1]
    nSigma = length(targetRisk)
    nLabels = 6
    M = c(0, ( 1:(nSigma %/% nLabels) ) ) *nLabels + 1
    nSignifDigits = 3
    axis(3, at = M, labels = signif(targetRisk[M], nSignifDigits))
    axis(1, at = M, labels = signif(targetReturn[M], nSignifDigits))
    
    # Add Axis Labels and Title:
    if (title) {
        mtext("Target Risk", side = 3, line = 2, adj = 1, cex = CEX)
        mtext("Target Return", side = 1, line = 2, adj = 1, cex = CEX)
        mtext("Weight", side = 2, line = 2, adj = 1, cex = CEX)
    }
      
    # Add Weights 0 and 1 Reference Lines
    # lines(x = c(0, nSigma), c(1, 1), col = "grey", lty = 3) 
    # lines(x = c(0, nSigma), c(0, 0), col = "grey", lty = 3)   
    
    # Add vertical Line at minimum risk:
    minIndex = which.min(targetRisk)
    minRisk = signif(min(targetRisk), 3)
    abline(v = minIndex, col = "black", lty = 1, lwd = 2)
    
    # Add Info:
    if (title) {
        mtext(paste(getType(object), "|", getSolver(object), "|", "minRisk =", 
            minRisk), side = 4, adj = 0, col = "grey", cex = 0.7)
    }
        
    # Add Title:
    if (title) {
        mtext(Title, adj = 0, line = 2.5, font = 2, cex = CEX+0.1)
    }
    
    # Complete to draw box ...
    if (box) box()
    
    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------


weightedReturnsLinePlot <- 
function(object, labels = TRUE, col = NULL, title = TRUE, 
    box = TRUE, legend = TRUE, ...)
{   
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Plots a bar chart of weights
    
    # Arguments:
    #   object - an object of class 'fPORTFOLIO'
    #   labels - should the graph be automatically labeled?
    #   col - a color palette, by default the rainbow palette
    #   title - should the graph get default title and labels?
    #   legend - should a legend be added to the plot?
    
    # FUNCTION:
    
    # Use default color if not specified ...
    Title = "Weighted Returns"
    if (is.null(col)) col = seqPalette(getNAssets(object)+1, "Blues")[-1]
    if (sum(c(par()$mfrow, par()$mfcol)) == 4) CEX = 0.9 else CEX = 0.7
    
    # Compute Weighted Returns:
    weights = getWeights(object)
    dim = dim(weights)
    returns = getStatistics(object)$mu
    weightedReturns = NULL
    for(i in 1:dim[2]){
        nextWeightedReturns = weights[,i]*returns[i]
        weightedReturns = cbind(weightedReturns, nextWeightedReturns)
    }
    colnames(weightedReturns) = colnames(weights)
    X = weightedReturns
    
    # Define Plot Range:
    ymax = max(colMaxs(X))
    ymin = min(colMins(X))
    range = ymax - ymin
    ymax = ymax + 0.005 * range
    ymin = ymin - 0.005 * range
    dim = dim(weights)
    range = dim[1]
    xmin = 0
    xmax = range + 0.2 * range
    
    # Create Bar Plots:
    if (labels) {
        if(legend){
            ts.plot(X, 
                gpars = list(col = col, ann = FALSE, xaxt = "n"), 
                xlim = c(xmin, xmax), ylim = c(ymin, ymax))
            legendtext = names(getStatistics(object)$mu)
            if(is.null(legendtext)){
                for(i in 1:dim[2]){legendtext[i] = paste("Asset", i, sep = " ")}
            }
            legend("topright", legend = legendtext, bty = "n", cex = CEX,
                fill = col)
        } else {
            ts.plot(weights, gpars = list(col = col, ann = FALSE, xaxt = "n"))
        }
    } else {
        ts.plot(X, ...)
    }
    
    # Add Tailored Labels -  6 may be a good Number ...
    targetRisk = getTargetRisk(object)[, 1]
    targetReturn = getTargetReturn(object)[, 1]
    nSigma = length(targetRisk)
    nLabels = 6
    M = c(0, ( 1:(nSigma %/% nLabels) ) ) *nLabels + 1
    nSignifDigits = 3
    axis(3, at = M, labels = signif(targetRisk[M], nSignifDigits))
    axis(1, at = M, labels = signif(targetReturn[M], nSignifDigits))
    
    # Add Axis Labels and Title:
    if (title) {
        mtext("Target Risk", side = 3, line = 2, adj = 1, cex = CEX)
        mtext("Target Return", side = 1, line = 2, adj = 1, cex = CEX)
        mtext("Weight", side = 2, line = 2, adj = 1, cex = CEX)
    }
      
    # Add Weights 0 and 1 Reference Lines
    # lines(x = c(0, nSigma), c(1, 1), col = "grey", lty = 3) 
    # lines(x = c(0, nSigma), c(0, 0), col = "grey", lty = 3)   
    
    # Add vertical Line at minimum risk:
    minIndex = which.min(targetRisk)
    minRisk = signif(min(targetRisk), 3)
    abline(v = minIndex, col = "black", lty = 1, lwd = 2)
    
    # Add Info:
    if (title) {
        mtext(paste(getType(object), "|", getSolver(object), "|", "minRisk =", 
            minRisk), side = 4, adj = 0, col = "grey", cex = 0.7)
    }
        
    # Add Title:
    if (title) {
        mtext(Title, adj = 0, line = 2.5, font = 2, cex = CEX+0.1)
    }
    
    # Complete to draw box ...
    if (box) box()
    
    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------


covRiskBudgetsLinePlot <- 
function(object, labels = TRUE, col = NULL, title = TRUE, 
    box = TRUE, legend = TRUE, ...)
{   
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Plots a bar chart of weights
    
    # Arguments:
    #   object - an object of class 'fPORTFOLIO'
    #   labels - should the graph be automatically labeled?
    #   col - a color palette, by default the rainbow palette
    #   title - should the graph get default title and labels?
    #   legend - should a legend be added to the plot?
    
    # FUNCTION:
    
    # Use default color if not specified ...
    Title = "Covariance Risk Budgets"
    if (is.null(col)) col = seqPalette(getNAssets(object)+1, "Blues")[-1]
    if (sum(c(par()$mfrow, par()$mfcol)) == 4) CEX = 0.9 else CEX = 0.7
    
    # Compute Covariance Risk Budgets:
    X = getCovRiskBudgets(object)
 
    # Define Plot Range:
    ymax = max(colMaxs(X))
    ymin = min(colMins(X))
    range = ymax - ymin
    ymax = ymax + 0.005 * range
    ymin = ymin - 0.005 * range
    dim = dim(X)
    range = dim[1]
    xmin = 0
    xmax = range + 0.2 * range
    
    # Create Bar Plots:
    if (labels) {
        if(legend){
            ts.plot(X, 
                gpars = list(col = col, ann = FALSE, xaxt = "n"), 
                xlim = c(xmin, xmax), ylim = c(ymin, ymax))
            legendtext = names(getStatistics(object)$mu)
            if(is.null(legendtext)){
                for(i in 1:dim[2]){legendtext[i] = paste("Asset", i, sep = " ")}
            }
            legend("topright", legend = legendtext, bty = "n", cex = CEX,
                fill = col)
        } else {
            ts.plot(weights, gpars = list(col = col, ann = FALSE, xaxt = "n"))
        }
    } else {
        ts.plot(X, ...)
    }
    
    # Add Tailored Labels -  6 may be a good Number ...
    targetRisk = getTargetRisk(object) 
    targetReturn = getTargetReturn(object) 
    nSigma = length(targetRisk)
    nLabels = 6
    M = c(0, ( 1:(nSigma %/% nLabels) ) ) *nLabels + 1
    nSignifDigits = 3
    axis(3, at = M, labels = signif(targetRisk[M], nSignifDigits))
    axis(1, at = M, labels = signif(targetReturn[M], nSignifDigits))
    
    # Add Axis Labels and Title:
    if (title) {
        mtext("Target Risk", side = 3, line = 2, adj = 1, cex = CEX)
        mtext("Target Return", side = 1, line = 2, adj = 1, cex = CEX)
        mtext("Weight", side = 2, line = 2, adj = 1, cex = CEX)
    }
      
    # Add Weights 0 and 1 Reference Lines
    # lines(x = c(0, nSigma), c(1, 1), col = "grey", lty = 3) 
    # lines(x = c(0, nSigma), c(0, 0), col = "grey", lty = 3)   
    
    # Add vertical Line at minimum risk:
    minIndex = which.min(targetRisk[, 1])
    minRisk = signif(min(targetRisk[, 1]), 3)
    abline(v = minIndex, col = "black", lty = 1, lwd = 2)
    
    # Add Info:
    if (title) {
        mtext(paste(getType(object), "|", getSolver(object), "|", "minRisk =", 
            minRisk), side = 4, adj = 0, col = "grey", cex = 0.7)
    }
        
    # Add Title:
    if (title) {
        mtext(Title, adj = 0, line = 2.5, font = 2, cex = CEX+0.1)
    }
    
    # Complete to draw box ...
    if (box) box()
    
    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------


.tailRiskBudgetsLinePlot <- 
function(object, labels = TRUE, col = NULL, title = TRUE, 
    box = TRUE, legend = TRUE, ...)
{   
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Plots a bar chart of weights
    
    # Arguments:
    #   object - an object of class 'fPORTFOLIO'
    #   labels - should the graph be automatically labeled?
    #   col - a color palette, by default the rainbow palette
    #   title - should the graph get default title and labels?
    #   legend - should a legend be added to the plot?
    
    # FUNCTION:
    
    # Use default color if not specified ...
    Title = "Covariance Risk Budgets"
    if (is.null(col)) col = seqPalette(getNAssets(object)+1, "Blues")[-1]
    if (sum(c(par()$mfrow, par()$mfcol)) == 4) CEX = 0.9 else CEX = 0.7
    
    # Compute Covariance Risk Budgets:
    stop("Not yet implemented")
    tailRiskMatrix = getTailRisk(object) 
    X = getTailRiskBudgets(object)
 
    # Define Plot Range:
    ymax = max(colMaxs(X))
    ymin = min(colMins(X))
    range = ymax - ymin
    ymax = ymax + 0.005 * range
    ymin = ymin - 0.005 * range
    dim = dim(weights)
    range = dim[1]
    xmin = 0
    xmax = range + 0.2 * range
    
    # Create Bar Plots:
    if (labels) {
        if(legend){
            ts.plot(X, 
                gpars = list(col = col, ann = FALSE, xaxt = "n"), 
                xlim = c(xmin, xmax), ylim = c(ymin, ymax))
            legendtext = names(getStatistics(object)$mu)
            if(is.null(legendtext)){
                for(i in 1:dim[2]){legendtext[i] = paste("Asset", i, sep = " ")}
            }
            legend("topright", legend = legendtext, bty = "n", cex = CEX,
                fill = col)
        } else {
            ts.plot(weights, gpars = list(col = col, ann = FALSE, xaxt = "n"))
        }
    } else {
        ts.plot(X, ...)
    }
    
    # Add Tailored Labels -  6 may be a good Number ...
    targetRisk = getTargetRisk(object) 
    targetReturn = getTargetReturn(object) 
    nSigma = length(targetRisk)
    nLabels = 6
    M = c(0, ( 1:(nSigma %/% nLabels) ) ) *nLabels + 1
    nSignifDigits = 3
    axis(3, at = M, labels = signif(targetRisk[M], nSignifDigits))
    axis(1, at = M, labels = signif(targetReturn[M], nSignifDigits))
    
    # Add Axis Labels and Title:
    if (title) {
        mtext("Target Risk", side = 3, line = 2, adj = 1, cex = CEX)
        mtext("Target Return", side = 1, line = 2, adj = 1, cex = CEX)
        mtext("Weight", side = 2, line = 2, adj = 1, cex = CEX)
    }
      
    # Add Weights 0 and 1 Reference Lines
    # lines(x = c(0, nSigma), c(1, 1), col = "grey", lty = 3) 
    # lines(x = c(0, nSigma), c(0, 0), col = "grey", lty = 3)   
    
    # Add vertical Line at minimum risk:
    minIndex = which.min(targetRisk[, 1])
    minRisk = signif(min(targetRisk[, 1]), 3)
    abline(v = minIndex, col = "black", lty = 1, lwd = 2)
    
    # Add Info:
    if (title) {
        mtext(paste(getType(object), "|", getSolver(object), "|", "minRisk =", 
            minRisk), side = 4, adj = 0, col = "grey", cex = 0.7)
    }
        
    # Add Title:
    if (title) {
        mtext(Title, adj = 0, line = 2.5, font = 2, cex = CEX+0.1)
    }
    
    # Complete to draw box ...
    if (box) box()
    
    # Return Value:
    invisible()
}


################################################################################

################################################################################
# FUNCTION:                DESCRIPTION:
#  portfolioFrontier        Returns the efficient frontier of a portfolio
# DEPRECATED:          
#  .portfolioFrontier       old/alt Version
################################################################################


portfolioFrontier <-
function(data, spec = portfolioSpec(), constraints = "LongOnly",
    include.mvl = TRUE, title = NULL, description = NULL)
{
    # A function implemented by Rmetrics

    # Description:
    #   Computes the efficient frontier of a portfolio

    # Arguments:
    #   data - a rectangular object of assets
    #   spec - an object of class 'fPFOLIOSPEC'
    #   constraints - a character vector or NULL
    #   include.mvl - a logical flag, should the minimum variance
    #       locus be added to the plot?

    # Example:
    #   data = as.timeSeries(data(LPP2005REC))[, 1:6]
    #   spec = portfolioSpec()
    #   constraints = c("minW[3:4]=0.1", "maxW[5:6]=0.8", "minsumW[1:3]=0.2", "maxsumW[c(2,4)]=0.8")
    #   portfolioFrontier(data, spec, constraints)

    # FUNCTION:

    DEBUG <- TRUE

    # Match Spec Versus Constraints?
    .checkSpecVsConstraints(spec, constraints)

    # Transform Data and Constraints:
    Data <- portfolioData(data, spec)

    # Optimize in N Points the Portfolios along the frontier:
    nFrontierPoints = getNFrontierPoints(spec)

    # The Target Return - get problems in the first and last point for
    #   long only portfolios, just move a little bit aside ...
    mu <- getMu(Data)
    targetReturns <- seq(min(mu), max(mu), length = nFrontierPoints)
    eps <- .Machine$double.eps^0.5
    targetReturns[1] = targetReturns[1]*(1+eps)
    targetReturns[nFrontierPoints] = targetReturns[nFrontierPoints]*(1-eps)

    # How to Go Along the Frontier ?
    #   The Idea is to start from the minvariance portfolio and to explore
    #   the efficient frontier and the minimum variance locus starting from
    #   this point ...
    #   Then we stop when the status flag fails ...

    # Compute minvariancePortfolio:
    mvPortfolio <- minvariancePortfolio(Data, spec, constraints)
    mvReturn <- getTargetReturn(mvPortfolio@portfolio)["mean"]
    minIndex <- which.min(abs(mvReturn-targetReturns))

    # Upper Frontier Part:
    Status <- 0
    IDX <- minIndex
    weights <- targetReturn <- targetRisk <- covRiskBudgets <- maxDD <- NULL
    while (Status == 0 & IDX <= nFrontierPoints) {
        # Add Target Return to Specification:
        setTargetReturn(spec) = targetReturns[IDX]
        # Optimize Efficient Portfolio:
        ans = try(efficientPortfolio(Data, spec, constraints), silent = TRUE)
        if (inherits(ans,"try-error")) {
            Status = 1
        } else {
            portfolio = ans
            Status = getStatus(portfolio)
        }
        if (Status == 0) {
            Weights = getWeights(portfolio)
            weights = rbind(weights, Weights)
            targetReturn = rbind(targetReturn, getTargetReturn(portfolio@portfolio))
            targetRisk = rbind(targetRisk, getTargetRisk(portfolio@portfolio))
            covRiskBudgets = rbind(covRiskBudgets, getCovRiskBudgets(portfolio@portfolio))
            ### maxDD = rbind(maxDD,
            ###    min(drawdowns(pfolioReturn(data/100, as.vector(Weights)))) )
        }
        IDX = IDX + 1
    }

    # Lower Min Variance Locus:
    if (include.mvl) {
        if (minIndex > 1) {
            weights2 = targetReturn2 = targetRisk2 = covRiskBudgets2 = maxDD2 = NULL
            Status = 0
            IDX = minIndex - 1
            while (Status == 0 & IDX > 0) {
                # Add Target Return to Specification:
                setTargetReturn(spec) = targetReturns[IDX]
                # Optimize Efficient Portfolio:
                ans = try(efficientPortfolio(Data, spec, constraints), silent = TRUE)
                if (inherits(ans,"try-error")) {
                    Status = 1
                } else {
                    portfolio = ans
                    Status = getStatus(portfolio)
                }
                if (Status == 0) {
                    Weights2 = getWeights(portfolio)
                    weights2 = rbind(Weights2, weights2)
                    targetReturn2 =
                        rbind(getTargetReturn(portfolio@portfolio), targetReturn2)
                    targetRisk2 =
                        rbind(getTargetRisk(portfolio@portfolio), targetRisk2)
                    covRiskBudgets2 =
                        rbind(getCovRiskBudgets(portfolio@portfolio), covRiskBudgets2)
                    ### maxDD2 = rbind(maxDD2, min(drawdowns(
                    ###    pfolioReturn(data/100, as.vector(Weights2)))) )
                }
                IDX = IDX - 1
            }
            weights = rbind(weights2, weights)
            targetReturn = rbind(targetReturn2, targetReturn)
            targetRisk = rbind(targetRisk2, targetRisk)
            covRiskBudgets = rbind(covRiskBudgets2, covRiskBudgets)
            ### maxDD = rbind(maxDD2, maxDD)
        }
    }
    colnames(weights) <- names(getMu(Data))
    rownames(weights) <- NULL
    rownames(covRiskBudgets) <- NULL
    rownames(targetReturn) <- NULL
    rownames(targetRisk) <- NULL

    # Check: Did we find points on the frontier?
    if (is.null(weights)) {
        portfolio <- mvPortfolio
        ### portfolio@portfolio$maxDD = min(drawdowns(
        ### pfolioReturn(data/100, as.vector(getWeights(mvPortfolio)))))
        return(portfolio)
    }

    # Reset Target Return:
    setTargetReturn(spec) <- NULL

    # Call:
    portfolio@call <- match.call()

    # Compose Portfolio:
    portfolio@portfolio <- new("fPFOLIOVAL",
        portfolio = list(
            weights = weights,
            covRiskBudgets = covRiskBudgets,
            targetReturn = targetReturn,
            targetRisk = targetRisk,
            targetAlpha = getAlpha(spec),
            minriskPortfolio = mvPortfolio,
            status = 0))

    ### portfolio@portfolio$maxDD = maxDD

    # Update Title
    portfolio@title = "Portfolio Frontier"

    # Return Value:
    portfolio
}


################################################################################
# DEPRECATED


.portfolioFrontier <-
function(data, spec = portfolioSpec(), constraints = "LongOnly",
    include.mvl = TRUE, title = NULL, description = NULL)
{
    # A function implemented by Rmetrics

    # Description:
    #   Computes the efficient frontier of a portfolio

    # Arguments:
    #   data - a rectangular object of assets
    #   spec - an object of class 'fPFOLIOSPEC'
    #   constraints - a character vector or NULL

    # Example:
    #   data = as.timeSeries(data(LPP2005REC))[, 1:6]
    #   spec = portfolioSpec()
    #   constraints = c("minW[3:4]=0.1", "maxW[5:6]=0.8", "minsumW[1:3]=0.2", "maxsumW[c(2,4)]=0.8")
    #   portfolioFrontier(data, spec, constraints)

    # FUNCTION:

    DEBUG = TRUE

    # Match Spec Versus Constraints?
    .checkSpecVsConstraints(spec, constraints)

    # Transform Data and Constraints:
    Data = portfolioData(data, spec)
    data <- getSeries(Data)

    # Optimize in N Points the Portfolios along the frontier:
    nFrontierPoints = getNFrontierPoints(spec)

    # The Target Return - get problems in the first and last point for
    #   long only portfolios, just move a little bit aside ...
    mu = getMu(Data)
    targetReturns <- seq(min(mu), max(mu), length = nFrontierPoints)
    eps = .Machine$double.eps^0.5
    targetReturns[1] = targetReturns[1]*(1+eps)
    targetReturns[nFrontierPoints] = targetReturns[nFrontierPoints]*(1-eps)

    # How to Go Along the Frontier ?
    #   The Idea is to start from the minvariance portfolio and to explore
    #   the efficient frontier and the minimum variance locus starting from
    #   this point ...
    #   Then we stop when the status flag fails ...

    # Compute minvariancePortfolio:
    mvPortfolio = minvariancePortfolio(Data, spec, constraints)
    mvReturn = getTargetReturn(mvPortfolio)[, "mean"]
    minIndex = which.min(abs(mvReturn-targetReturns))

    # Upper Frontier Part:
    Status = 0
    IDX = minIndex
    weights = targetReturn = targetRisk = covRiskBudgets = maxDD = NULL
    while (Status == 0 & IDX <= nFrontierPoints) {
        # Add Target Return to Specification:
        setTargetReturn(spec) = targetReturns[IDX]
        # Optimize Efficient Portfolio:
        ans = try(efficientPortfolio(Data, spec, constraints), silent = TRUE)
        if (inherits(ans,"try-error")) {
            Status = 1
        } else {
            portfolio = ans
            Status = getStatus(portfolio)
        }
        if (Status == 0) {
            Weights = getWeights(portfolio)
            weights = rbind(weights, Weights)
            targetReturn = rbind(targetReturn, getTargetReturn(portfolio))
            targetRisk = rbind(targetRisk, getTargetRisk(portfolio))
            covRiskBudgets = rbind(covRiskBudgets, getCovRiskBudgets(portfolio))
            maxDD = rbind(maxDD,
                min(drawdowns(pfolioReturn(data/100, as.vector(Weights)))) )
        }
        IDX = IDX + 1
    }

    # Lower Min Variance Locus:
    if (include.mvl) {
        if (minIndex > 1) {
            weights2 = targetReturn2 = targetRisk2 = covRiskBudgets2 = maxDD2 = NULL
            Status = 0
            IDX = minIndex - 1
            while (Status == 0 & IDX > 0) {
                # Add Target Return to Specification:
                setTargetReturn(spec) = targetReturns[IDX]
                # Optimize Efficient Portfolio:
                ans = try(efficientPortfolio(Data, spec, constraints), silent = TRUE)
                if (inherits(ans,"try-error")) {
                    Status = 1
                } else {
                    portfolio = ans
                    Status = getStatus(portfolio)
                }
                if (Status == 0) {
                    Weights2 = getWeights(portfolio)
                    weights2 = rbind(Weights2, weights2)
                    targetReturn2 =
                        rbind(getTargetReturn(portfolio), targetReturn2)
                    targetRisk2 =
                        rbind(getTargetRisk(portfolio), targetRisk2)
                    covRiskBudgets2 =
                        rbind(getCovRiskBudgets(portfolio), covRiskBudgets2)
                    maxDD2 = rbind(maxDD2, min(drawdowns(
                        pfolioReturn(data/100, as.vector(Weights2)))) )
                }
                IDX = IDX - 1
            }
            weights = rbind(weights2, weights)
            targetReturn = rbind(targetReturn2, targetReturn)
            targetRisk = rbind(targetRisk2, targetRisk)
            covRiskBudgets = rbind(covRiskBudgets2, covRiskBudgets)
            maxDD = rbind(maxDD2, maxDD)
        }
    }

    # Check: Did we find points on the frontier?
    if (is.null(weights)) {
        portfolio = mvPortfolio
        portfolio@portfolio$maxDD = min(drawdowns(
            pfolioReturn(data/100, as.vector(getWeights(mvPortfolio)))))
        return(portfolio)
    }

    # Reset Target Return:
    setTargetReturn(spec) <- NULL

    # Compose Frontier:
    portfolio@call = match.call()
    portfolio@portfolio$weights  = weights
    portfolio@portfolio$targetReturn = targetReturn
    portfolio@portfolio$targetRisk = targetRisk
    portfolio@portfolio$covRiskBudgets = covRiskBudgets
    portfolio@portfolio$maxDD = maxDD
    portfolio@portfolio$status = 0
    portfolio@portfolio$minriskPortfolio = mvPortfolio
    portfolio@title = "Portfolio Frontier"

    # Return Value:
    portfolio
}


###############################################################################

################################################################################
# FUNCTION:                 DESCRIPTION:
#  covEstimator              Uses sample covariance estimation
#  mveEstimator              Uses robust estimation "cov.mve" from [MASS]
#  mcdEstimator              Uses robust estimation "cov.mcd" from [MASS]
# FUNCTION:                 DESCRIPTION:
#  lpmEstimator              Returns Lower Partial Moment Estimator
#  slpmEstimator             Returns Symmetric Lower Partial Moment Estimator
# FUNCTION:                 DESCRIPTION:
#  kendallEstimator          Returns Kendall's Covariance Estimator
#  spearmanEstimator         Returns Spearman's Covariance Estimator
# FUNCTION:                 DESCRIPTION:
#  covMcdEstimator           Requires "covMcd" from [robustbase]
#  covOGKEstimator           Requires "covOGK" from [robustbase]
#  shrinkEstimator           Requires "cov.shrink" from [corpcor]
#  nnveEstimator             Requires "cov.nnve" from [covRobust]
# FUNCTION:                 DESCRIPTION:
#  .studentEstimator         uses "cov.trob" from [MASS]
#  .baggedEstimator          uses builtin from [corpcor]
#  .donostahEstimator        uses builtin from [robust]
#  .bayesSteinEstimator      copy from Alexios Ghalanos
#  .ledoitWolfEstimator      uses builtin from [tawny]
#  .rmtEstimator             uses builtin from [tawny]
# FUNCTION:                 DESCRIPTION:
#  .mveEstimator2            Uses robust estimation "cov.mve" from [MASS]
#  .mcdEstimator2            Uses robust estimation "cov.mcd" from [MASS]
#  .covMcdEstimator2         Requires "covMcd" from [robustbase]
#  .covOGKEstimator2         Requires "covOGK" from [robustbase]
#  .arwEstimator2            Uses robust estimation ".cov.arw"from [mvoutlier]
################################################################################


covEstimator <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Uses sample covariance estimation

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Example:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; covEstimator(x)

    # FUNCTION:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    mu = colMeans(x.mat)
    Sigma = cov(x.mat)

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


# ------------------------------------------------------------------------------


mveEstimator <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Uses robust estimation "cov.mve" from [MASS]

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Example:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; mveEstimator(x)

    # FUNCTION:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate: {
    mu = colMeans(x.mat)
    Sigma = MASS::cov.rob(x = x.mat, method = "mve")$cov

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


# ------------------------------------------------------------------------------


mcdEstimator <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Uses robust estimation "cov.mve" from [MASS]

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Example:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; mcdEstimator(x)

    # FUNCTION:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    mu = colMeans(x.mat)
    Sigma = MASS::cov.rob(x = x.mat, method = "mcd")$cov

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


################################################################################


lpmEstimator <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Returns lower partial moment estimator

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Example:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; lpmEstimator(x)

    # FUNCTION:

    # Check Arguments:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    mu <- colMeans(x)
    if (is.null(spec)) {
        FUN = colMeans
        a = 2
    } else {
        FUN = match.fun(spec@model$param$tau)
        a = spec@model$param$a
    }
    Sigma <- assetsLPM(x, tau = FUN(x), a = a)$Sigma
    colnames(Sigma) <- rownames(Sigma) <- names(mu)

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


################################################################################


slpmEstimator <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Returns symmetric lower partial moment estimator

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Example:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; slpmEstimator(x)

    # FUNCTION:

    # Check Arguments:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    mu <- colMeans(x)
    if (is.null(spec)) {
        FUN = colMeans
        a = 2
    } else {
        FUN = match.fun(spec@model$param$tau)
        a = spec@model$param$a
    }
    Sigma <- assetsSLPM(x, tau = FUN(x), a = a)$Sigma
    colnames(Sigma) <- rownames(Sigma) <- names(mu)

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


################################################################################


kendallEstimator <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Uses Kendall's rank covariance estimation

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Example:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; covEstimator(x)

    # FUNCTION:

    # Check Arguments:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    mu = colMeans(x.mat)
    Sigma = cov(x.mat, method = "kendall")

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


# ------------------------------------------------------------------------------


spearmanEstimator <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Uses Spearman's rank covariance estimation

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Example:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; covEstimator(x)

    # FUNCTION:

    # Check Arguments:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    mu = colMeans(x.mat)
    Sigma = cov(x.mat, method = "spearman")

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


################################################################################


covMcdEstimator <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Example:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; covMcdEstimator(x)

    # FUNCTION:

    # Check Arguments:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    mu = colMeans(x.mat)
    Sigma = robustbase::covMcd(x.mat, alpha = 1/2, ...)$cov

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


# ------------------------------------------------------------------------------


covOGKEstimator <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Example:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; covOGKEstimator(x)

    # FUNCTION:

    # Check Arguments:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    mu = colMeans(x.mat)
    Sigma = robustbase::covOGK(x.mat, sigmamu = robustbase::scaleTau2, ...)$cov
    colnames(Sigma) <- rownames(Sigma) <- names(mu)

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


# ------------------------------------------------------------------------------


shrinkEstimator <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Eample:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; shrinkEstimator(x)

    # FUNCTION:

    # Check Arguments:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    mu = colMeans(x.mat)
    Sigma = fAssets::.cov.shrink(x = x.mat, verbose = FALSE, ...)
    attr(Sigma, "lambda.var") <- NULL
    attr(Sigma, "lambda.var.estimated") <- NULL

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


# ------------------------------------------------------------------------------


nnveEstimator <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Eample:
    #   x  = as.timeSeries(data(LPP2005REC))[, 1:6]; nnveEstimator(x)

    # FUNCTION:

    # Check Arguments:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    mu = colMeans(x.mat)
    Sigma = fAssets::.cov.nnve(datamat = x.mat, ...)$cov
    colnames(Sigma) <- rownames(Sigma) <- names(mu)

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


################################################################################


.studentEstimator <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Uses mean/student-d covariance estimation

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Note:
    #   Source from package MASS

    # Eample:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; .studentEstimator(x)

    # FUNCTION:

    # Check Arguments:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    robust = fAssets::.studentMeanCov(x.mat, ...)
    mu = robust$center
    Sigma = robust$cov

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


# ------------------------------------------------------------------------------


.baggedEstimator <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Uses bagged mean/covariance estimation

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Note:
    #   Source from package corpcor

    # Eample:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; .baggedEstimator(x)

    # FUNCTION:

    # Check Arguments:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    robust = fAssets::.baggedMeanCov(x, ...)
    mu = robust$center
    Sigma = robust$cov

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


# ------------------------------------------------------------------------------


.donostahEstimator <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Description:
    #   Uses Donostah's mean/covariance estimation

    # Note:
    #   Source from package robust

    # Eample:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; .donostahEstimator(x)

    # FUNCTION:

    # Check Arguments:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    robust = fAssets::.donostahMeanCov(x, ...)
    mu = robust$center
    Sigma = robust$cov

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


# ------------------------------------------------------------------------------


.bayesSteinEstimator <-
    function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Uses Bayes Stein mean/covariance estimation

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Note:
    #   Source from Alexios Ghalanos

    # Eample:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; .bayesSteinEstimator(x)

    # FUNCTION:

    # Check Arguments:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    robust = fAssets::.bayesSteinMeanCov(x, ...)
    mu = robust$center
    Sigma = robust$cov

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


# ------------------------------------------------------------------------------


.ledoitWolfEstimator <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Uses Ledoit-Wolf mean/covariance estimation

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Note:
    #   Source from package tawny

    # Eample:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; .ledoitWolfEstimator(x)

    # FUNCTION:

    # Check Arguments:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    robust = fAssets::.ledoitWolfMeanCov(x, ...)
    mu = robust$center
    Sigma = robust$cov

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


# ------------------------------------------------------------------------------


.rmtEstimator <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Uses Random Matrix Theory correlation estimation

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Note:
    #   Source from package tawny

    # Eample:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; .rmtEstimator(x)

    # FUNCTION:

    # Check Arguments:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    robust = fAssets::.rmtMeanCov(x, ...)
    mu = robust$center
    Sigma = robust$cov

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


################################################################################


.mveEstimator2 <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Uses robust estimation "cov.mve" from [MASS]

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Example:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; mveEstimator(x)

    # FUNCTION:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    ans = MASS::cov.rob(x = x.mat, method = "mve")
    mu = ans$center
    Sigma = ans$cov

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


# ------------------------------------------------------------------------------


.mcdEstimator2 <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Uses robust estimation "cov.mve" from [MASS]

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Example:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; mcdEstimator(x)

    # FUNCTION:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    ans = MASS::cov.rob(x = x.mat, method = "mcd")
    mu = ans$center
    Sigma = ans$cov

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


################################################################################


.covMcdEstimator2 <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Example:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; covMcdEstimator(x)

    # FUNCTION:

    # Check Arguments:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    ans = robustbase::covMcd(x.mat, alpha = 1/2, ...)
    mu = ans$center
    Sigma = ans$cov

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


# ------------------------------------------------------------------------------


.covOGKEstimator2 <-
function(x, spec = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:

    # Arguments:
    #   x - an object of class timeSeries
    #   spec - a portfolio specification of class fPFOLIOSPEC

    # Example:
    #   x = as.timeSeries(data(LPP2005REC))[, 1:6]; covOGKEstimator(x)

    # FUNCTION:

    # Check Arguments:
    stopifnot(inherits(x, "timeSeries"))

    # Extract Matrix:
    x.mat = getDataPart(x)

    # Estimate:
    ans = robustbase::covOGK(x.mat, sigmamu = robustbase::scaleTau2, ...)
    mu = ans$center
    Sigma = ans$cov
    colnames(Sigma) <- rownames(Sigma) <- names(mu)

    # Return Value:
    list(mu = mu, Sigma = Sigma)
}


# ------------------------------------------------------------------------------


.arwEstimator2 <-
function (x, spec = NULL, ...)
{
    x.mat <- as.matrix(x)
    N <- ncol(x)
    assetNames <- colnames(x)
    fit <- fAssets::.cov.arw(x = x.mat, center = colMeans(x.mat), cov = cov(x) , ...)

    # Estimate:
    mu <- fit$center
    Sigma <- fit$cov
    names(mu) <- assetNames
    rownames(Sigma) <- colnames(Sigma) <- assetNames

    # Return Value:
    list(mu = fit$center, Sigma = fit$cov)
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################
# FUNCTION:                    DESCRIPTION:
#  getWindows                   Extracts windows slot
#   getWindowsFun               Extracts name of windows function
#   getWindowsParams            Extracts a list of windows specific parameters
#   getWindowsHorizon           Extracts windows horizon
# FUNCTION:                    DESCRIPTION:
#  getStrategy                  Extracts strategy slot
#   getStrategyFun              Extracts the name of portfolio strategy function
#   getStrategyParams           Extracts a list of strategy specific parameters
# FUNCTION:                    DESCRIPTION:
#  getSmoother                  Extracts the smoother slot
#   getSmootherFun              Extracts the name of the moother function
#   getSmootherParams           Extracts a list of smoothing specific parameters
#   getSmootherLambda           Extracts the smoothing parameter Lambda
#   getSmootherDoubleSmoothing  Extracts setting for double smoothing
#   getSmootherInitialWeights   Extracts the initial weights in the smoothing
#   getSmootherSkip             Extracts the number of skipped months
# FUNCTION:                    DESCRIPTION:
#  getMessages                  Extracts the message slot
################################################################################


getWindows.fPFOLIOBACKTEST <- 
    function(object)
{   
    # A function implemented by William Chen and Diethelm Wuertz
    
    # Description:
    #   Extracts Windows slot from an object of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - an object of class fPFOLIOBACKTEST
    
    # FUNCTION:
        
    # Description:
    #   gets the "model" slot from an object of class 4
   
    # Arguments:
    #   object - an object of class S4
   
    # FUNCTION:
   
    # Return Value:
    getSlot(object, "windows")
}
            

# ------------------------------------------------------------------------------
    

getWindowsFun.fPFOLIOBACKTEST <- 
function(object) 
{
    # A function implemented by William Chen and Diethelm Wuertz
    
    # Description:
    #   Extracts name of windows function from an object of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - an object of class fPFOLIOBACKTEST
    
    # FUNCTION:
    
    # Return Value:
    getWindows(object)$windows
}


# ------------------------------------------------------------------------------


getWindowsParams.fPFOLIOBACKTEST <- 
    function(object) 
{
    # A function implemented by William Chen and Diethelm Wuertz
    
    # Description:
    #   Extracts a list of windows specific parameters from an object 
    #   of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - an object of class fPFOLIOBACKTEST
    
    # FUNCTION:
    
    # Return Value:
    getWindows(object)$params
}

   
# ------------------------------------------------------------------------------


getWindowsHorizon.fPFOLIOBACKTEST <- 
    function(object) 
{
    # A function implemented by William Chen and Diethelm Wuertz
    
    # Description:
    #   Extracts windows horizon from an object of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - an object of class fPFOLIOBACKTEST
    
    # FUNCTION:
    
    # Return Value:
    getWindowsParams(object)$horizon
}
    
    
# ------------------------------------------------------------------------------


getSmoother.fPFOLIOBACKTEST <- 
    function(object)
{   
    # A function implemented by William Chen and Diethelm Wuertz
        
    # Description:
    #   Extracts the smoother slot from an object of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - an object of class fPFOLIOBACKTEST
    
    # FUNCTION:
   
    # Return Value:
    getSlot(object, "smoother")
}


# ------------------------------------------------------------------------------


getSmootherFun.fPFOLIOBACKTEST <- 
    function(object) 
{
    # A function implemented by William Chen and Diethelm Wuertz
    
    # Description:
    #   Extracts the name of the moother function from an object 
    #   of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - an object of class fPFOLIOBACKTEST
    
    # FUNCTION:
    
    # Return Value:
    getSmoother(object)$smoother
}

    
# ------------------------------------------------------------------------------


getSmootherParams.fPFOLIOBACKTEST <-  
    function(object) 
{
    # A function implemented by William Chen and Diethelm Wuertz
    
    # Description:
    #   Extracts a list of strategy specific parameters
    
    # Arguments:
    #   object - an object of class fPFOLIOBACKTEST
    
    # FUNCTION:
    
    # Return Value:
    getSmoother(object)$params
}

    
# ------------------------------------------------------------------------------


getSmootherLambda.fPFOLIOBACKTEST <- 
    function(object) 
{
    # A function implemented by William Chen and Diethelm Wuertz
    
    # Description:
    #   Extracts the smoothing parameter Lambda from an object 
    #   of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - an object of class fPFOLIOBACKTEST
    
    # FUNCTION:
    
    # Return Value:
    getSmootherParams(object)$lambda
}

    
# ------------------------------------------------------------------------------


getSmootherDoubleSmoothing.fPFOLIOBACKTEST <- 
    function(object) 
{
    # A function implemented by William Chen and Diethelm Wuertz
    
    # Description:
    #   Extracts setting for double smoothing from an object 
    #   of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - an object of class fPFOLIOBACKTEST
    
    # FUNCTION:
    
    # Return Value:
    getSmootherParams(object)$doubleSmoothing
}

    
# ------------------------------------------------------------------------------


getSmootherInitialWeights.fPFOLIOBACKTEST <- 
    function(object) 
{
    # A function implemented by William Chen and Diethelm Wuertz
    
    # Description:
    #   Extracts the initial weights in the smoothing from an object 
    #   of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - an object of class fPFOLIOBACKTEST
    
    # FUNCTION:
    
    # Return Value:
    getSmootherParams(object)$initialWeights
}

    
# ------------------------------------------------------------------------------


getSmootherSkip.fPFOLIOBACKTEST <- 
    function(object) 
{
    # A function implemented by William Chen and Diethelm Wuertz
    
    # Description:
    #   Extracts the number of skipped months from an object 
    #   of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - an object of class fPFOLIOBACKTEST
    
    # FUNCTION:
    
    # Return Value:
    getSmootherParams(object)$skip
}
 
       
# ------------------------------------------------------------------------------


getStrategy.fPFOLIOBACKTEST <- 
    function(object)
{   
    # A function implemented by William Chen and Diethelm Wuertz
        
    # Description:
    #   Extracts strategy slot from an object of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - an object of class fPFOLIOBACKTEST
    
    # FUNCTION:
   
    # Return Value:
    getSlot(object, "strategy")
}

    
# ------------------------------------------------------------------------------


getStrategyFun.fPFOLIOBACKTEST <-  
    function(object) 
{
    # A function implemented by William Chen and Diethelm Wuertz
    
    # Description:
    #   Extracts the name of portfolio strategy function from an object 
    #   of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - an object of class fPFOLIOBACKTEST
    
    # FUNCTION:
    
    # Return Value:
    getStrategy(object)$strategy
}

    
# ------------------------------------------------------------------------------


getStrategyParams.fPFOLIOBACKTEST <- 
    function(object) 
{
    # A function implemented by William Chen and Diethelm Wuertz
    
    # Description:
    #   Extracts a list of strategy specific parameters from an object 
    #   of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - an object of class fPFOLIOBACKTEST
    
    # FUNCTION:
    
    # Return Value:
    getStrategy(object)$params
}
   

# ------------------------------------------------------------------------------
                

getMessages.fPFOLIOBACKTEST <- 
    function(object)
{   
    # A function implemented by William Chen and Diethelm Wuertz
    
    # Description:
    #   Extracts the message slot from an object of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - an object of class fPFOLIOBACKTEST
    
    # FUNCTION:
   
    # Return Value:
    getSlot(object, "messages")
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                     DESCRIPTION:
#  solveRquadprog.CLA            Portfolio interface to solver Rquadprog
#  .claRquadprogArguments        Returns arguments for solver
# FUNCTION:                     DESCRIPTION:
#  .quadprog.CLA                 Wrapper to solver function
################################################################################


solveRquadprog.CLA <-
    function(data, spec, constraints)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Portfolio interface to solver Rquadprog

    # Example:
    #   solveRquadprogCLA(data, spec, constraints)[-3]
   
    # FUNCTION:   

    # Update Specification:
    setTargetReturn(spec) <- NA
    
    # Transform Data:
    Data <- portfolioData(data, spec)
    nAssets <- getNAssets(Data)
        
    # Compile Arguments for Solver:
    args <- .claRquadprogArguments(data, spec, constraints)
    
    # Solve Multiassets Portfolio:
    ans <- .quadprog.CLA(
        Dmat = args$Dmat, 
        dvec = args$dvec, 
        Amat = args$Amat, 
        bvec = args$bvec, 
        meq = args$meq,
        lambda = args$lambda)
     
    # Save Arguments:
    ans$optim$args <- args
    class(ans) <- c("solver", "list")

    # Return Value:
    ans
}


################################################################################


.claRquadprogArguments <-
    function(data, spec, constraints)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns quadprog conform arguments for the solver
    
    # FUNCTION:
    
    # Set up the default quadprog QP
    ans <- .rquadprogArguments(data, spec, constraints)
    
    # Optimize:
    # min(-d^T x + 1/2 x^T D x) 
    
    # Start from it and modify for CLA:
    lambda <- spec@model$param$lambda
    ans$Dmat <- lambda * getSigma(portfolioData(data)) / 2
    ans$dvec <- getMu(portfolioData(data)) 
    ans$Amat <- ans$Amat[, -1]
    ans$bvec <- ans$bvec[-1]
    ans$meq <- ans$meq - 1
    ans$dir <- ans$dir[-1]
    ans$lambda <- lambda

    # Return Value:
    ans
}


################################################################################


.quadprog.CLA <-
    function(Dmat, dvec, Amat, bvec, meq, lambda)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Goldfarb and Idnani's quadprog solver function
    
    # Note:
    #   Requires to load contributed R package quadprog from which we use
    #   the Fortran subroutine of the quadratic solver.
    
    # Package: quadprog
    #   Title: Functions to solve Quadratic Programming Problems.
    #   Author: S original by Berwin A. Turlach <berwin.turlach@anu.edu.au>
    #       R port by Andreas Weingessel <Andreas.Weingessel@ci.tuwien.ac.at>
    #   Maintainer: Andreas Weingessel <Andreas.Weingessel@ci.tuwien.ac.at>
    #   Description: This package contains routines and documentation for
    #       solving quadratic programming problems.
    #   License: GPL-2
    
    # Value of slove.QP():
    #   solution - vector containing the solution of the quadratic
    #       programming problem.
    #   value - scalar, the value of the quadratic function at the
    #       solution
    #   unconstrained.solution - vector containing the unconstrained
    #       minimizer of the quadratic function.
    #   iterations - vector of length 2, the first component contains
    #       the number of iterations the algorithm needed, the second
    #       indicates how often constraints became inactive after
    #       becoming active first. vector with the indices of the
    #       active constraints at the solution.

    # FUNCION:

    # Optimize:
    optim <- quadprog::solve.QP(
        Dmat = Dmat, 
        dvec = dvec, 
        Amat = Amat, 
        bvec = bvec, 
        meq = meq, 
        factorized = FALSE) 
    
    # Set Tiny Weights to Zero:
    weights <- .checkWeights(optim$solution)
    attr(weights, "invest") <- sum(weights) 
    
    # Compose Output List:
    Sigma <- Dmat * 2 / lambda
    mu <- dvec
    ans <- list(
        type = "MV",
        solver = "solveRquadprog.CLA",
        optim = optim,
        weights = weights,
        solution = weights,
        targetReturn = (optim$solution %*% mu)[[1,1]],
        targetRisk = sqrt(optim$solution %*% Sigma %*% optim$solution)[[1,1]],
        objective = optim$crval,
        # To do: Add status information
        status = 0,
        message = "minRisk")
            
    # Return Value:
    ans
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


###############################################################################
# FUNCTION:            DESCRIPTION:
#  ternaryMap           Displays a risk map for ternary portfolios     
#  ternaryFrontier      Plots the efficient frontier of a ternary portfolio
# FUNCTION:            DESCRIPTION:
#  riskMap              normalVaR risk map function called from ternaryMap()
#  maxddMap             max Drawdown risk map function called from ternaryMap()
# FUNCTION:            DESCRIPTION:
#  ternaryWeights       Creates a set of ternary weights
#  ternaryCoord         Computes x, y coordinates from weights
#  ternaryPoints        Adds points to a ternary map plot
###############################################################################


ternaryMap <- 
    function(data, FUN=NULL, ..., 
    locator=FALSE, N=41, palette=topo.colors, nlevels=11)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #    Displays a risk map for ternary portfolios
    
    # Arguments:
    #    data - a ternary 'timeSeries' object of financial returns
    #    FUN - the map function
    #    ... - optional arguments passed to function FUN
    #    locator - a logical flag to activate the locator
    #    N - number of bins
    #    palette - color palette
    #    nlevels - number of contour levels
    
    # Examples:
    #    ternaryMap(data=SWX.RET[, 1:3])
    #    ternaryMap(data=SWX.RET[, 1:3], FUN=.riskTest, locator=TRUE)
    #    ternaryMap(data=SWX.RET[, 1:3], FUN=.maxddTest)
    
    # FUNCTION:
    
    # N=41; palette=topo.colors; nlevels=11; FUN <- NULL; ... <- NULL
    
    # Surface Function:
    if (is.null(FUN)) FUN <- function(data, weights, ...) var(weights)
    fun <- match.fun(FUN)
    
    # Grid Points:
    s <- sqrt(3)/2
    x <- seq(0, 1, length=N)
    y <- s * x 
    
    # Surface Values:
    G <- matrix(NA, N, N)
    xy <- W <- NULL
    for (j in 1:N) {
        w3 <- y[j] / s
        for (i in 1:N) {
            w2 <- x[i] - w3/2
            w1 <- 1 - w2 - w3
            if (w1 >= -1/N && w2 >= -1/N) {
            # if (w1 >= 0 && w2 >= 0) {
               w <- c(w1, w2, w3)
               xy <- rbind(xy, c(x[i], y[j]))
               W <- rbind(W, w)
               G[i, j] <- ans <- fun(data, w, ...)
            }
        }
    }
    surface <- list(x=x, y=y/s, z=G)   
    x <- surface$x
    y <- surface$y
    z <- surface$z

    # Color Settings:
    colors <- .scaledColors(surface, palette = palette, nlevels = nlevels)
    levels <- colors$levels
    palette <- colors$palette
    
    # Image Ranges:
    yOffset <- 0.025 * diff(range(y))
    yLim <- c(min(y) - yOffset, max(y) + yOffset)
    xOffset <- 0.1 * diff(range(x))
    xLim <- c(min(x) - xOffset/4, max(x) + xOffset)
    
    # Filled Contour Plot:
    image(x, y, z, xlim = xLim, ylim = yLim, xlab = "", ylab = "", 
          col = "white")
    grid()
    
    # DW
    # .Internal(filledcontour()) no longer works on 3.0.
    # .Internal(filledcontour(
    #     as.double(x), as.double(y), z, 
    #     as.double(levels), col = palette))

    # Use instead:
    graphics::.filled.contour(
      x = as.double(x), 
      y = as.double(y), 
      z = z,
      levels = as.double(levels), 
      col = palette)
    
    contour(x, y, z, add = TRUE, levels = signif(levels, 3))
    d <- 3/N
    polygon(c(1, 1+d, 0.5+d, 0.5, 1), c(0, 0, s, s, 0)/s, col="white", 
            border="white")
    polygon(c(0, 0.5, 0.5-d,  -d, 0), c(0, s, s, 0, 0)/s, col="white", 
            border="white")
    box(col = "white")
    
    # Please do not Remove:
    mtext("Rmetrics", 4, col="grey", adj=0, cex=0.7)

    # Grid Lines:
    for(k in 1:10) 
      lines(c(k*0.1, k*0.05), c(0, k*0.1), col="grey", lty=3)
    for(k in 1:10) 
      lines(c(1-k*0.1, 1-k*0.05), c(0, k*0.1), col="grey", lty=3)
    for(k in 1:9) 
      lines(c(k*0.05, 1-k*0.05), c(k*0.1, k*0.1), col="grey", lty=3)

    # Add Legend:
    cs <- cumsum(levels)
    css <- (cs - min(cs))/diff(range(cs))
    css <- 0.95 * css + 0.025
    cy <- min(y) + css * diff(range(y))
    cx <- rep(xLim[2] - 0.1 * xOffset, length(cy))
    lines(cx, cy, lwd = 3)
    for (i in 1:(nlevels - 1)) lines(c(cx[i], cx[i + 1]), c(cy[i], 
        cy[i + 1]), lwd = 3, col = palette[i])
    for (i in 1:nlevels) points(cx[i], cy[i], pch = 16, cex = 1.1, 
        col = "black")
    textOffset <- c(-5e-04, 5e-04, 8e-04, 8e-04, rep(0, 7))
    text(cx, cy + textOffset, as.character(signif(levels, 2)), 
        pos = 2, cex = 0.8)

    # Decoration:
    pointCex <- 2.5
    textCex <- 0.5
    
    # EfficientFrontier:
    frontier <- portfolioFrontier(data)
    weights <- getWeights(frontier@portfolio)
    xy <- cbind(x=weights[, 2]+weights[, 3]/2, y=weights[, 3])
    lines(xy, col="brown", lwd=3)
    
    # Global Minimum Variance Portfolio:
    weights<- getWeights(minvariancePortfolio(data))
    xy <- c(x=weights[2]+weights[3]/2, y=weights[3])
    points(xy[1], xy[2], pch=19, cex=pointCex, col="red")
    text(xy[1], xy[2], "MVP", font=2, col="white",  cex=textCex)
    
    # Tangency Portfolio:
    weights <- getWeights(tangencyPortfolio(data))
    xy <- c(x=weights[2]+weights[3]/2, y=weights[3])
    points(xy[1], xy[2], pch=19, cex=pointCex, col="orange")
    text(xy[1], xy[2], "TGP", font=2, col="white", cex=textCex)
    
    # Equal Weights:
    xy <- rbind(c(1/3+1/6, 1/3))
    points(xy, pch=19, cex=pointCex, col="brown")
    text(xy, "EWP", font=2, col="white", cex = textCex)
     
    # Individual Assets:   
    xy = rbind(c(0,0), c(1,0), c(1/2, 1))
    points(xy, pch=19, cex=pointCex, col="black")
    text(xy, colnames(data), font = 2, col = "white", cex = textCex)

    # Locator:
    if (locator) {
        for (i in 1:512) {
            ans <- locator(n = 1, type = "p", pch=10) 
            w3 <- ans$y
            w2 <- ans$x - w3/2
            w <- round(c(w1=1-w2-w3, w2, w3), 2)
            names(w) <- colnames(data)
            total <- sum(w)
            names(total) <- "Total"
            z <- signif(fun(data, w, ...), 3)
            ans <- data.frame(rbind(c(w, total, z)))
            rownames(ans) <- "Composition"
            print(ans)
        }
    }
    
    # Return Value:
    invisible()
}


# -----------------------------------------------------------------------------


ternaryFrontier <-
    function(data, locator=FALSE)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #    Plots the efficient frontier of a ternary map
    
    # Arguments:
    #    data - a ternary 'timeSeries' object of financial returns
    #    locator - a logical flag to activate the locator
    
    # Example:
    #    ternaryFrontier(SWX.RET[, 1:3], locator=TRUE)
    
    # FUNCTION:
    
    # Long Only Markowitz Portfolio:
    polygon <- markowitzHull(data)
    object <- attr(polygon, "frontier")
      
    # Plot Range:
    offset <- 0.1
    xlim <- c(0, max(sqrt(diag(getCov(object)))))
    Xlim <- c(xlim[1] - diff(xlim) * offset, xlim[2] + diff(xlim) * offset)
    ylim <- range(getMean(object))
    Ylim <- c(ylim[1] - diff(ylim) * offset, ylim[2] + diff(ylim) * offset)

    # Get Points and Add Frontier:
    
    frontierPlot(object, auto=FALSE, xlim=Xlim, ylim=Ylim, pch=19, 
                 labels=FALSE)
    polygon(polygon, col="grey", border="grey")
    points(frontierPoints(object, frontier="upper"), pch=19)
    box(col="white")
    grid()
    
    # Please do not Remove:
    mtext("Rmetrics", 4, col="grey", adj=0, cex=0.7)

    # Zero  Axis Lines:
    abline(h = 0, col = "grey")
    abline(v = 0, col = "grey")
           
    # Decoration Settings:
    pointCex <- 2.5
    textCex <- 0.5

    # Add Global Minimum Variance Portfolio:
    xy <- minvariancePoints(
      object, auto=FALSE, pch=19, col="red", cex=pointCex)
    text(xy[1], xy[2], "MVP", font=2, col="white",  cex=textCex)
      
    # Add Tangency Portfolio for zero risk free Rate:
    tangencyLines(object, auto=FALSE, col="blue")  
    xy <- tangencyPoints(
      object, auto=FALSE, pch=19, col="blue", cex=pointCex) 
    text(xy[1], xy[2], "TGP", font=2, col="white",  cex=textCex)
    
    # Add Equal Weights Portfolio:   
    xy <- equalWeightsPoints(
      object, auto=FALSE, pch=19, col="brown", cex=pointCex)
    text(xy[1], xy[2], "EWP", font=2, col="white", cex=textCex)
    
    # Add Two Assets Portfolios: 
    xy <- singleAssetPoints(
      object, auto=FALSE, pch=19, col="black", cex=pointCex)
    text(xy[,1], xy[,2], colnames(data), font=2, col="white", cex=textCex)

    # Add Sharpe Ratio:
    sharpeRatioLines(object, auto=FALSE, col="orange", lwd=2, pch=19)
    
    # Locator:
    if (locator) {
        for (i in 1:512) {
            ans <- locator(n = 1, type = "p", pch=10) 
            Risk <- ans$x
            Return <- ans$y
            SharpeRatio <- ans$y/ans$x
            ans <- data.frame(rbind(c(Risk, Return, SharpeRatio)))
            colnames(ans) <- c("Risk", "Return", "SharpeRatio")
            rownames(ans) <- "Portfolio"
            print(signif(ans, 3))
        }
    }
     
    # Retirn Value:
    invisible(object)
}


# #############################################################################


riskMap <- 
    function(data, weights) 
{  
    # Description:
    #    normalVaR risk map function called from ternaryMap()
    
    # FUNCTION:
    
    # Map:
    tS <- pfolioReturn(data, weights=as.vector(weights))
    ans <- normalVaR(tS) 
    names(ans) <- "normalVaR"
    
    # Return Value:
    ans
}


# -----------------------------------------------------------------------------


maxddMap <- 
    function(data, weights) 
{  
    # Description:
    #    Max Drawdown map function called from ternaryMap()
    
    # FUNCTION:
    
    # Map:
    tS <- pfolioReturn(data, weights=as.vector(weights))
    ans <- colMins(tS) 
    names(ans) <- "maxdd"
    
    # Return Value:
    ans
}
    

# #############################################################################
# Utility Functions:


ternaryWeights <- 
    function(n=21)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #    Returns a set of ternary weights
    
    # Arguments:
    #    n - number of bins
    
    # Example:
    #    ternaryWeights()
    
    # FUNCTION:
    
    # Settings:
    eps <- sqrt(.Machine$double.eps)
    
    # Creates a set of ternary weights
    W <- seq(0, 1, length=n)
    W1 <- rep(W, times = length(W))
    W2 <- rep(W, each = length(W))
    W3 <- 1 - W1 - W2
    W3[abs(W3) < eps] <- 0
    W <- cbind(W1, W2, W3)
    weights <- W[W1 + W2 <= 1, ]
    
    # Return Value:
    weights
}


# -----------------------------------------------------------------------------


ternaryCoord <- 
    function(weights)  
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #    Returns x,y Coordinates of weights triangle
    
    # Example:
    #    ternaryCoord(ternaryWeights())
    
    # FUNCTION:
    
    # Computexs x, y coordinates from weights
    x <- 1 - weights[,1] - weights[, 2]/2
    y <- sqrt(3) * weights[, 2] /2
    
    # Return Value:
    cbind(x=x, y=y)
}


# -----------------------------------------------------------------------------

  
ternaryPoints <- 
    function(weights, ...)  
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #    Adds points to a ternary map plot
    
    # Example:
    #    ternaryPoints(ternaryWeights())
    
    # FUNCTION:
    
    # Transpose in the case of a single weight
    if(is.null(dim(weights))) weights <- t(weights)
    
    # Adds points to a ternary map
    coord <- ternaryCoord(weights)
    points(coord, ...)
    
    # Return Value:
    invisible()
}


###############################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                     DESCRIPTION:
#  getA                          Defines Use Method for A                
#  getAlpha                      Defines Use Method for Alpha            
#  getConstraints                Defines Use Method for Constraints      
#  getControl                    Defines Use Method for Control          
#  getCov                        Defines Use Method for Cov              
#  getCovRiskBudgets             Defines Use Method for CovRiskBudgets   
#  getData                       Defines Use Method for Data             
#  getEstimator                  Defines Use Method for Estimator        
#  getMean                       Defines Use Method for Mean             
#  getMu                         Defines Use Method for Mu               
#  getNAssets                    Defines Use Method for NAssets          
# .getNames                      Defines Use Method for Names            
#  getNFrontierPoints            Defines Use Method for NFrontierPoints  
#  getMessages                   Defines Use Method for Messages         
#  getObjective                  Defines Use Method for Objective        
#  getOptim                      Defines Use Method for Optim            
#  getOptimize                   Defines Use Method for Optimize         
#  getOptions                    Defines Use Method for Options          
#  getPortfolio                  Defines Use Method for Portfolio        
#  getParams                     Defines Use Method for Params           
#  getRiskFreeRates              Defines Use Method for RiskFreeRates    
#  getSeries                     Defines Use Method for Series           
#  getSigma                      Defines Use Method for Sigma            
#  getSolver                     Defines Use Method for Solver           
#  getSpec                       Defines Use Method for Spec             
#  getStatistics                 Defines Use Method for Statistics       
#  getStatus                     Defines Use Method for Status           
#  getTailRisk                   Defines Use Method for TailRisk         
#  getTailRiskBudgets            Defines Use Method for TailRiskBudgets  
#  getTargetReturn               Defines Use Method for TargetReturn     
#  getTargetRisk                 Defines Use Method for TargetRisk       
#  getTrace                      Defines Use Method for Trace            
#  getType                       Defines Use Method for Type  
#  getUnits                      Defines Use Method for Units [Asset Names]           
#  getWeights                    Defines Use Method for Weights          
################################################################################


getA <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getA")
}


# ------------------------------------------------------------------------------


getAlpha <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getAlpha")
}


# ------------------------------------------------------------------------------


getConstraints <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getConstraints")
}


# ------------------------------------------------------------------------------


getControl <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getControl")
}


# ------------------------------------------------------------------------------


getCov <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getCov")
}


# ------------------------------------------------------------------------------


getData <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getData")
}


# ------------------------------------------------------------------------------


getCovRiskBudgets <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getCovRiskBudgets")
}


# ------------------------------------------------------------------------------


getEstimator <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getEstimator")
}


# ------------------------------------------------------------------------------


getMean <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # Return Value:
    UseMethod("getMean")
}


# ------------------------------------------------------------------------------

getMu <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getMu")
}


# ------------------------------------------------------------------------------


getNAssets <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # Return Value:
    UseMethod("getNAssets")
}


# ------------------------------------------------------------------------------


.getNames <-
    function(object)
{
    # A function implemented by Diethelm Wuertz
    
    # REPLACED BY getUnits.

    # Return Value:
    UseMethod("getNames")
}


# ------------------------------------------------------------------------------


getNFrontierPoints <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getNFrontierPoints")
}


# ------------------------------------------------------------------------------


getMessages <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getMessages")
}


# ------------------------------------------------------------------------------


getObjective <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    UseMethod("getObjective")
}


# ------------------------------------------------------------------------------


getOptim <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getOptim")
}


# ------------------------------------------------------------------------------


getOptimize <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    UseMethod("getOptimize")
}


# ------------------------------------------------------------------------------


getOptions <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getOptions")
}


# ------------------------------------------------------------------------------


getPortfolio <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # Return Value:
    UseMethod("getPortfolio")
}


# ------------------------------------------------------------------------------


getParams <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    UseMethod("getParams")
}


# ------------------------------------------------------------------------------


getRiskFreeRate <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getRiskFreeRate")
}


# ------------------------------------------------------------------------------
# DW: Take care of getSeries in package timeSeries


getSeries <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getSeries")
}


# ------------------------------------------------------------------------------


getSigma <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getSigma")
}


# ------------------------------------------------------------------------------


getSolver <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getSolver")
}


# ------------------------------------------------------------------------------


getSpec <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getSpec")
}


# ------------------------------------------------------------------------------


getStatistics <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getStatistics")
}


# ------------------------------------------------------------------------------


getStatus <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getStatus")
}


# ------------------------------------------------------------------------------


getTailRisk <-
    function(object)
{   # A function implemented by Diethelm Wuertz

    # Return Value:
    UseMethod("getTailRisk")
}


# ------------------------------------------------------------------------------


getTailRiskBudgets <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getTailRiskBudgets")
}


# ------------------------------------------------------------------------------


getTargetReturn <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getTargetReturn")
}


# ------------------------------------------------------------------------------


getTargetRisk <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getTargetRisk")
}


# ------------------------------------------------------------------------------


getTrace <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getTrace")
}


# ------------------------------------------------------------------------------


getType <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getType")
}


# ------------------------------------------------------------------------------
# DW: already defined in package timeSeries

# getUnits <-
#     function(object)
# {
#     # A function implemented by Diethelm Wuertz
# 
#     # FUNCTION: 
#     
#     # Return Value:
#     UseMethod("getUnits")
# }


# ------------------------------------------------------------------------------
# TS: already defined in package fBasics

# getModel <-
#   function(object)
#   {
#     # A function implemented by Tobias Setz
#     
#     # FUNCTION: 
#     
#     # Return Value:
#     UseMethod("getModel")
#   }


# ------------------------------------------------------------------------------


getWeights <-
    function(object)
{
    # A function implemented by Diethelm Wuertz

    # FUNCTION: 
    
    # Return Value:
    UseMethod("getWeights")
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################
# FUNCTION:              DESCRIPTION:
#  .lpAssign              Assigns linear programming model
#  .qpAssign              Assigns quadratic programming model
################################################################################


.lpAssign <-
    function(project, c, x_L, x_U, A, b_L, b_U, solver="ipopt", trace=FALSE)
{
    # A function written by Diethelm Wuertz
    
    # Description:
    #   Assigns LP Model
    
    # Arguments:
    #   project - project name
    
    # FUNCTION:
    
    # Settings:
    n <- length(c)
    m <- nrow(A)
    
    # Write AMPL Model File:
    amplModelOpen(project)
    model <- c(
        "param n ;",
        "param m ;",
        "param c{1..n} ;",      
        "param x_L{1..n} ;",
        "param x_U{1..n} ;", 
        "param A{1..m, 1..n} ;",
        "param b_L{1..m} ;",
        "param b_U{1..m} ;",   
        "var x{1..n};",
        "minimize Objective: sum {i in 1..n} x[i]*c[i] ;",
        "s.t. lower {i in 1..n}: x[i] >= x_L[i] ;",
        "s.t. upper {i in 1..n}: x[i] <= x_U[i] ;",   
        "s.t. linLower {j in 1..m}: sum{i in 1..n} A[j, i]*x[i] >= b_L[j] ;", 
        "s.t. linUpper {j in 1..m}: sum{i in 1..n} A[j, i]*x[i] <= b_U[j] ;", 
        NULL)
    amplModelAdd(model, project)
    if (trace) amplModelShow(project)
        
    # Write AMPL Data File:
    amplDataOpen(project)
    amplDataAddValue (data="n", value=n, project)
    amplDataAddValue (data="m", value=m, project)
    amplDataAddVector(data="c", vector=c, project)
    amplDataAddVector(data="x_L", vector=x_L, project)
    amplDataAddVector(data="x_U", vector=x_U, project)    
    amplDataAddMatrix(data="A", matrix=A, project)
    amplDataAddVector(data="b_L", vector=b_L, project)
    amplDataAddVector(data="b_U", vector=b_U, project)  
    if (trace) amplDataShow(project)
       
    # Write AMPL RUN File:
    amplRunOpen(project)
    run <- c(
        paste("reset ;"),
        paste("option solver ", solver, " ;", sep = ""),
        paste("model ", project, ".mod ;", sep = ""),
        paste("data ", project, ".dat ;", sep = ""),
        paste("solve ;"),
        paste("display x > ", project, ".txt ;", sep = ""),
        paste("display solve_result_num > ", project, ".txt ;", sep = ""),
        paste("display solve_result > ", project, ".txt ;", sep = ""),
        paste("display solve_message > ", project, ".txt ;", sep = ""),
        paste("exit ;") )
    amplRunAdd(run, project)
    if (trace) amplRunShow(project)
    
    # Return Value:
    invisible()  
}


# -----------------------------------------------------------------------------


.qpAssign <-
    function(project, c, F, x_L, x_U, A, b_L, b_U, solver="ipopt", trace=FALSE)
{
    # A function written by Diethelm Wuertz
    
    # Description:
    #   Assigns LP Model
    
    # Arguments:
    #   project - project name
    
    # FUNCTION:
    
    # Settings:
    n <- length(c)
    m <- nrow(A)
    
    # Write AMPL Model File:
    amplModelOpen(project)
    model <- c(
        "param n ;",
        "param m ;",
        "param c{1..n} ;",
        "param F{1..n, 1..n} ;",        
        "param x_L{1..n} ;",
        "param x_U{1..n} ;", 
        "param A{1..m, 1..n} ;",
        "param b_L{1..m} ;",
        "param b_U{1..m} ;",   
        "var x{1..n};",
        "minimize Risk: sum {i in 1..n} x[i]*c[i] + 0.5*sum {i in 1..n} sum{j in 1..n} x[i]*F[i,j]*x[j] ;",
        "s.t. lower {i in 1..n}: x[i] >= x_L[i] ;",
        "s.t. upper {i in 1..n}: x[i] <= x_U[i] ;",   
        "s.t. linLower {j in 1..m}: sum{i in 1..n} A[j, i]*x[i] >= b_L[j] ;", 
        "s.t. linUpper {j in 1..m}: sum{i in 1..n} A[j, i]*x[i] <= b_U[j] ;", 
        NULL)
    amplModelAdd(model, project)
    if (trace) amplModelShow(project)
        
    # Write AMPL Data File:
    amplDataOpen(project)
    amplDataAddValue (data="n", value=n, project)
    amplDataAddValue (data="m", value=m, project)
    amplDataAddVector(data="c", vector=c, project)
    amplDataAddMatrix(data="F", matrix=F, project) 
    amplDataAddVector(data="x_L", vector=x_L, project)
    amplDataAddVector(data="x_U", vector=x_U, project)    
    amplDataAddMatrix(data="A", matrix=A, project)
    amplDataAddVector(data="b_L", vector=b_L, project)
    amplDataAddVector(data="b_U", vector=b_U, project)  
    if (trace) amplDataShow(project)
       
    # Write AMPL RUN File:
    amplRunOpen(project)
    run <- c(
        paste("reset ;"),
        paste("option solver ", solver, " ;", sep = ""),
        paste("model ", project, ".mod ;", sep = ""),
        paste("data ", project, ".dat ;", sep = ""),
        paste("solve ;"),
        paste("display x > ", project, ".txt ;", sep = ""),
        paste("display solve_result_num > ", project, ".txt ;", sep = ""),
        paste("display solve_result > ", project, ".txt ;", sep = ""),
        paste("display solve_message > ", project, ".txt ;", sep = ""),
        paste("exit ;") )
    amplRunAdd(run, project)
    if (trace) amplRunShow(project)
    
    # Return Value:
    invisible()
}
    

################################################################################
    
       

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################
# FUNCTION:                DESCRIPTION:
#  rsolnpNLP                NLP wrapper for solver solnpNLP()
#  solnpNLP                 NLP wrapper for solver solnp()
#  solnpNLPControl          Control parameter list 
# REQUIRES:
#  solnp                    Solver in package Rsolnp
################################################################################


rsolnpNLP <- 
    function(start, objective, lower=0, upper=1, linCons, funCons, 
    control=list())
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Function wrapper for solver solnp()
    
    # FUNCTION:
    
    # Load:
    # require(Rsolnp)
    
    # Update Control List:
    ctrl <- solnpNLPControl()
    if (length(control) > 0)
        for (name in names(control)) ctrl[name] = control[name]
    control <- ctrl
    BIG <- 1e6
    N <- length(start)
    
    # Box Constraints:
    if(length(lower) == 1) {
        par.lower <- rep(lower, N)
    } else {
        par.lower <- lower
    }
    if(length(upper) == 1) {
        par.upper <- rep(upper, N)
    } else {
        par.upper <- upper
    }
    par.lower[is.infinite(par.lower)] <- BIG*sign(par.lower[is.infinite(par.lower)])
    par.upper[is.infinite(par.upper)] <- BIG*sign(par.upper[is.infinite(par.upper)])
    
    # Linear Constraints:  
    if(missing(linCons)) {
        eqA <- ineqA <- NULL
        eqA.bound <- ineqA.lower <- ineqA.upper <- NULL
    } else {
        mat <- linCons[[1]]
        M <- nrow(mat)
        lower <- linCons[[2]]
        upper <- linCons[[3]]
        if(length(lower) == 1) lower <- rep(lower, M)
        if(length(upper) == 1) upper <- rep(upper, M)
        lower[is.infinite(lower)] <- BIG*sign(lower[is.infinite(lower)])
        upper[is.infinite(upper)] <- BIG*sign(upper[is.infinite(upper)])
        
        eqIndex <- which(lower == upper)
        ineqIndex <- which(lower != upper)
        
        if (length(eqIndex) == 0) {
            eqA <- NULL
            eqA.bound <- NULL
        } else {
            eqA <- mat[eqIndex, ]
            eqA.bound <- lower[eqIndex]
        }
        if (length(ineqIndex) == 0) {
            ineqA <- NULL
            ineqA.lower <- NULL
            ineqA.upper <- NULL
        } else {
            ineqA <- mat[ineqIndex, ]
            ineqA.lower <- lower[ineqIndex]
            ineqA.upper <- upper[ineqIndex]
        }
    }   
    
    # Nonlinear Constraints:
    if(missing(funCons)) {
        eqFun <- ineqFun <- list()
        eqFun.bound <- ineqFun.lower <- ineqFun.upper <- NULL
    } else {
        fun <- funCons[[1]]
        lower <- funCons[[2]]
        upper <- funCons[[3]]
        eqIndex <- which(lower == upper)
        ineqIndex <- which(lower != upper)
        if (length(eqIndex) == 0) {
            eqFun <- list()
            eqFun.boud <- NULL
        } else {
            eqFun = fun[eqIndex]
            eqFun.bound = lower[eqIndex]
        }
        if (length(ineqIndex) == 0) {
            ineqFun <- list()
            ineqFun.lower <- NULL
            ineqFun.upper <- NULL
        } else {
            ineqFun = fun[ineqIndex]
            ineqFun.lower <- lower[ineqIndex]
            ineqFun.upper <- upper[ineqIndex]
        }
    }
    
    # Optimize Portfolio:
    elapsed <- Sys.time()
    optim <- solnpNLP(
        start = start, 
        objective = objective, 
        par.lower = par.lower, 
        par.upper = par.upper,
        eqA = eqA, 
        eqA.bound = eqA.bound,
        ineqA = ineqA, 
        ineqA.lower = ineqA.lower, 
        ineqA.upper = ineqA.upper,
        eqFun = eqFun, 
        eqFun.bound = eqFun.bound,
        ineqFun = ineqFun, 
        ineqFun.lower = ineqFun.lower, 
        ineqFun.upper = ineqFun.upper,
        control = control) 
    elapsed <- Sys.time() - elapsed
    
    # Version:
    package <- packageDescription(pkg="Rsolnp")
    version <- paste(package$Package, package$Version, package$Date)
          
    # Return Value:
    value <- list(
        opt = optim,
        solution = optim$solution, 
        objective = objective(optim$solution), 
        status = optim$status,
        message = optim $message,
        solver = "solnpNLP",
        elapsed = elapsed,
        version = version)
    class(value) <- c("solver", "list")
    
    # Return Value:
    value
}


################################################################################


solnpNLP <- 
    function(
    start, objective,
    par.lower = NULL, par.upper = NULL,
    eqA = NULL, eqA.bound = NULL,
    ineqA = NULL, ineqA.lower = NULL, ineqA.upper = NULL,
    eqFun = list(), eqFun.bound = NULL,
    ineqFun = list(), ineqFun.lower = NULL, ineqFun.upper = NULL,
    control = list())
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Universal function wrapper for solver solnp(). 
    
    # FUNCTION:
    
    # Load:
    # require(Rsolnp)
    fun <- objective
    
    # Control List:
    ctrl <- solnpNLPControl()
    if (length(control) > 0)
        for (name in names(control)) ctrl[name] = control[name]
    control <- ctrl
    
    # Environment Setting:
    env <- .GlobalEnv
    
    # Box Constraints:
    BIG <- 1e8 # inf does not works here, DW.
    if (is.null(par.lower)) par.lower <- rep(-BIG, length(start))
    if (is.null(par.upper)) par.upper <- rep(+BIG, length(start))
    
    # Linear and Function Equality Constraints:
    if (length(eqA) > 0 || length(eqFun) > 0 ) {
        eqfun <- function(x) {
            ans <- NULL
            if(!is.null(eqA)) ans <- c(ans, as.vector(eqA %*% x))
            if (length(eqFun) > 0)
                for (i in 1:length(eqFun)) ans <- c(ans, eqFun[[i]](x))
            ans }
    } else {
        eqfun <- NULL
    }      
    eqB <- c(eqA.bound, eqFun.bound)
    
    # Linear and Function Inequality Constraints:
    if (length(ineqA) > 0 || length(ineqFun) > 0) {
        ineqfun <- function(x) {
            ans <- NULL
            if(!is.null(ineqA)) ans <- c(ans, as.vector(ineqA %*% x))
            if (length(ineqFun) > 0)
                for (i in 1:length(ineqFun)) ans <- c(ans, ineqFun[[i]](x))
            ans }
    } else {
        ineqfun <- NULL
    }  
    ineqLB <- c(ineqA.lower, ineqFun.lower)
    ineqUB <- c(ineqA.upper, ineqFun.upper)
        
    # Optimize Portfolio:
    elapsed <- Sys.time()
    optim <- Rsolnp::solnp(
        pars = start, 
        fun = fun, 
        eqfun = eqfun, 
        eqB = eqB, 
        ineqfun = ineqfun, 
        ineqLB = ineqLB, 
        ineqUB = ineqUB,  
        LB = par.lower, 
        UB = par.upper, 
        control = control) 
    elapsed <- Sys.time() - elapsed
    names(optim$pars) <- names(start)
    
    # Version:
    package <- packageDescription(pkg="Rsolnp")
    version <- paste(package$Package, package$Version, package$Date)
       
    # Return Value:
    value <- list(
        opt = optim,
        solution = optim$pars, 
        objective = fun(optim$pars), 
        status = optim$convergence,
        message = "not available",
        solver = "solnpNLP",
        elapsed = elapsed,
        version = version)
    class(value) <- c("solver", "list")
    value
}


###############################################################################


solnpNLPControl <-
    function(
    rho = 1, 
    outer.iter = 400, 
    inner.iter = 800, 
    delta = 1.0e-7, 
    tol = 1.0e-8, 
    trace = 0)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns control list
    
    # Arguments:
    #   rho - a numeric value. The penalty parameter 
    #   majit - an integer value. The maximum number of major iterations 
    #   minit - an integer value. The maximum number of minor iterations 
    #   delta - a numeric value. The relative step size in forward 
    #       difference evaluation 
    #   tol - a numeric value. The tolerance on feasibility and optimality 
    
    # Notes:
    #   DW: default trace=1 changed to trace=0
    
    # FUNCTION:
    
    # Control Parameters:
    control <- list(
        rho = rho,
        outer.iter = outer.iter,
        inner.iter = inner.iter,
        delta = delta,
        tol = tol,
        trace = trace)
       
    # Return Value:
    control
}


###############################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


###############################################################################
# FUNCTION:                     DESCRIPTION:
#  .amplObjval                   Extracts objective function value
#  .amplSolution                 Extracts solution vector
#  .amplModel                    Extracts model file information
#  .amplRun                      Extracts model file information
#  .amplSolver                   Extracts solver name
#  .amplVersion                  Extracts version number
#  .amplPresolve                 Extracts presolve information
###############################################################################


.amplObjval <- 
    function(ampl) 
{  
    # A function Implemented by Diethelm Wuertz
    
    # Description:
    #    Extracts objective function value.
    
    # Arguments:
    #    ampl - an object as returned by the the rampl[OPT] solver
    #         functions.
    
    # FUNCTION:
    
    # Return Value:
    ampl$objective
}


# -----------------------------------------------------------------------------


.amplSolution <- 
    function(ampl) 
{
    # A function Implemented by Diethelm Wuertz
    
    # Description:
    #    Extracts solution vector.
    
    # Arguments:
    #    ampl - an object as returned by the the rampl[OPT] solver
    #         functions.
    
    # FUNCTION:
    
    # Return Value:
    ampl$solution
}


# -----------------------------------------------------------------------------


.amplModel <- 
    function(ampl) 
{
    # A function Implemented by Diethelm Wuertz
    
    # Description:
    #    Extracts model file information
    
    # Arguments:
    #    ampl - an object as returned by the the rampl[OPT] solver
    #         functions.
    
    # FUNCTION:
    
    model <- ampl$opt$model
    cat(model, sep="\n")
    
    # Return Value:
    invisible(model) 
}


# -----------------------------------------------------------------------------


.amplRun <- 
    function(ampl) 
{ 
    # A function Implemented by Diethelm Wuertz
    
    # Description:
    #    Extracts run file information.
    
    # Arguments:
    #    ampl - an object as returned by the the rampl[OPT] solver
    #         functions.
    
    # FUNCTION:
    
    # Get run file:
    run <- ampl$opt$run
    cat(, sep="\n")
    
    # Return Value:
    invisible(run)
}


# -----------------------------------------------------------------------------


.amplSolver <- 
    function(ampl) 
{   
    # A function Implemented by Diethelm Wuertz
    
    # Description:
    #    Extracts solver information
    
    # Arguments:
    #    ampl - an object as returned by the the rampl[OPT] solver
    #         functions.
    
    # FUNCTION:
    
    # Get solver vector:
    solver <- ampl$solver
    cat(solver, sep="\n")
    
    # Return Value:
    invisible(solver)
}


# -----------------------------------------------------------------------------


.amplVersion <- 
    function(ampl) 
{
    # A function Implemented by Diethelm Wuertz
    
    # Description:
    #    Extracts solver version.
    
    # Arguments:
    #    ampl - an object as returned by the the rampl[OPT] solver
    #         functions.
    
    # FUNCTION:
    
    # Get Version Number:
    version <- ampl$version
    cat(version, sep="\n")
    
    # Return Value:
    invisible(version)
}


# -----------------------------------------------------------------------------


.amplPresolve <- 
    function(ampl)
{
    # A function Implemented by Diethelm Wuertz
    
    # Description:
    #   Extracts presolver results
    
    # Arguments:
    #    ampl - an object as returned by the the rampl[OPT] solver
    #         functions.
    
    # FUNCTION:
    
    # Get presolve information:
    solve <- ampl$opt$solve
    Index <- grep("^Presolve", solve)
    solve <- solve[-(1:(Index-1))]
    Index <- grep("^$", solve)[1]
    solve <- solve[1:(Index-1)]
    cat(solve, sep="\n")
    
    # Return Value:
    invisible(solve)
}


###############################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:             DESCRIPTION:
#  frontierPoints        Extracts the efficient frontier from a 'fPORTFOLO' object
################################################################################


frontierPoints <-
    function(object, frontier = c("both", "lower", "upper"),
        return = c("mean", "mu"), risk = c("Cov", "Sigma", "CVaR", "VaR"),
        auto = TRUE)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Extracts the efficient frontier from a 'fPORTFOLO' object

    # Arguments:
    #   object - an object of S4 class fPORTFOLIO as returned by the
    #       functions fooPortfolio().
    #   frontier - a character string, which part of the frontier
    #       should be extracted
    #   return, rrisk - a character string, which return or risk
    #       measures should be selected
    #   auto - a logical, allows auto selection for the return and
    #       risk measure depending on

    # FUNCTION:

    # Settings:
    frontier = match.arg(frontier)
    
    # Match Arguments:
    return = match.arg(return)
    risk = match.arg(risk)

    # Get Efficient Frontier:
    if (auto) {
        Type = getType(object)
        Estimator = getEstimator(object)
        if (Type == "MV") risk = "Cov"
        if (Type == "MV" & Estimator != "covEstimator") risk = "Sigma"
        if (Type == "QLPM") risk = "Sigma"
        if (Type == "CVaR") risk = "CVaR" 
    }
    
    if (is.vector(getTargetRisk(object@portfolio))) { 
        targetRisk = getTargetRisk(object@portfolio)[risk]
        targetReturn = getTargetReturn(object@portfolio)[return]
    } else {       
        targetRisk = getTargetRisk(object@portfolio)[, risk]
        targetReturn = getTargetReturn(object@portfolio)[, return]
    }
    
    # Extract Whole Frontier:
    ans = cbind(Risk = targetRisk, Return = targetReturn)

    # Extract "upper|lower" Part of Frontier:
    if(frontier == "upper"){
        index = 1:length(ans[, 1])
        test = c(-1, diff(ans[, 1]))
        index = index[test > 0]
        ans = ans[index, ]
    } else if(frontier == "lower"){
        index = 1:length(ans[, 1])
        test = c(-1, diff(ans[, 1]))
        index = index[test < 0]
        if (length(index) == 1) {
            ans = matrix(ans[index, ], ncol = 2)
        } else {
            ans = ans[index, ]
        }
    }

    # Add Colnames:
    colnames(ans) = c("targetRisk", "targetReturn")
    rownames(ans) = as.character(1:NROW(ans))
    attr(ans, "control") <- 
        c(targetRisk = risk, targetReturn = return, auto = as.character(auto)) 

    # Return Value:
    ans
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the 
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General 
# Public License along with this library; if not, write to the 
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
# MA 02111-1307 USA


################################################################################
# FUNCTION:                     DESCRIPTION:
#  'fPORTFOLIOCON'               S4 Portfolio Constraints Class
################################################################################


setClass("fPFOLIOCON", 
    representation(
    
        # Function Implemented by Diethelm Wuertz
    
        # Constraints expressed by strings
        #   LongOnly, Short, Full, ...
        stringConstraints = "character",
        
        # BoxConstraints:
        minWConstraints = "numeric",
        maxWConstraints = "numeric",
        
        # Group Constraints:
        eqsumWConstraints = "matrix",
        minsumWConstraints = "matrix",
        maxsumWConstraints = "matrix",
        
        # Covariance Risk Budget Constraints:
        minBConstraints = "numeric", 
        maxBConstraints = "numeric",
        
        # Nonlinear Constraints:
        listFConstraints = "list",
        minFConstraints = "numeric", 
        maxFConstraints = "numeric",
        
        # Buyin Constraints:
        minBuyinConstraints = "numeric", 
        maxBuyinConstraints = "numeric",
        
        # Cardinality Constraints:
        nCardConstraints = "integer",
        minCardConstraints = "numeric", 
        maxCardConstraints = "numeric")
        
) 
        

################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                     DESCRIPTION:
#  weightsSlider                 Graphical Weights Slider
#  .counterWeightsSlider
################################################################################


.counterWeightsSlider <-  NA


# ------------------------------------------------------------------------------


weightsSlider <-
    function(object, control = list(), ...)
{
    # A function implemented by Rmetrics

    # Description:
    #    Interactive view of Portfolio Weights

    # FUNCTION:

    # Global Variables:
    object <<- object
    nFrontierPoints <- length(getTargetRisk(object)[ ,1])
    dim = dim(getWeights(object))[2]

    # Use default, if xlim and ylim is not specified ...
    mu = getStatistics(object)$mu
    Sigma = getStatistics(object)$Sigma
    yLim = range(mu) + 0.25*c(-diff(range(mu)), diff(range(mu)))

    # First, take care that all assets appear on the plot ...
    sqrtSig = sqrt(diag(Sigma))
    xLimAssets = c(min(sqrtSig), max(sqrtSig))+
         c(-0.4*diff(range(sqrtSig)), 0.1*diff(range(sqrtSig)))

    # ... second take care that the whole frontier appears on the plot:
    fullFrontier = frontierPoints(object)
    xLimFrontier = range(fullFrontier[, 1])
    xLim = range(c(xLimAssets, xLimFrontier))
    xLim[1] = xLim[1]-diff(xLim)/5

    # Control Parameters:
    con <<- list(
        sliderResolution = 1,
        sliderFlag = "weights",
        runningPoint.col  = "red",
        minvariance.col = "red",
        tangency.col = "steelblue",
        singleAsset.col = rainbow(dim),
        minvariance.pch = 19,
        singleAsset.pch = 19,
        tangency.pch = 17,
        runningPoint.cex = 1.5,
        minvariance.cex = 1,
        tangency.cex = 1.25,
        singleAsset.cex = 1,
        xlim = xLim,
        ylim = yLim
        )
    con[(Names <- names(control))] <- control

    # Internal Function:
    refresh.code = function(...)
    {
        # Startup Counter:
        .counterWeightsSlider <- getRmetricsOptions(".counterWeightsSlider") + 1
        setRmetricsOptions(.counterWeightsSlider = .counterWeightsSlider)
        if (.counterWeightsSlider < 1) return ()

        # Sliders:
        N = .sliderMenu(no = 1)

        # Reset Frame:
        par(mfrow = c(2, 2))

        # Plot 1 - Frontier Plot:

        frontier = frontierPoints(object)

        fPoint = frontier[N, ]

        frontierPlot(object, xlim = con$xlim, ylim = con$ylim,
            xlab = "", ylab = "", pch = 19, cex = 0.7, title = FALSE)

        mtext("Target Risk", side = 1, line = 2, adj = 1, cex = 0.7)
        mtext("Target Return", side = 2, line = 2, adj = 1, cex = 0.7)

        points(fPoint[1], fPoint[2], col = con$runningPoint.col, pch = 19,
            cex = con$runningPoint.cex)

        tangencyLines(object, col = con$tangency.col, pch = con$tangency.pch)
        tangencyPoints(object, col = con$tangency.col)

        singleAssetPoints(object, col = con$singleAsset.col,
            cex = con$singleAsset.cex, pch = con$singleAsset.pch)

        minvariancePoints(object, col = con$minvariance.col,
            cex = con$minvariancePlot.cex, pch = con$minvariance.pch)

        Title = paste(
            "Return =", signif(fPoint[2], 2), "|",
            "Risk = ", signif(fPoint[1], 2))

        Title = "Efficient Frontier"
        mtext(Title, adj = 0, line = 2.5, font = 2, cex = 0.7)

        grid()


        # Plot 2 - Weights Pie:
        weightsPie(object, pos = N)

        # Plot 3 - Weights Plot:
        weightsPlot(object)
        abline(v = N, col = "black")

        # Plot 4 - Single Weights Plot:
        weightsLinePlot(object)
        abline(v = N, col = "black")

    }

    # Open Slider Menu:
    setRmetricsOptions(.counterWeightsSlider = 0)
    Start <- which.min(getTargetRisk(object)[ , 1])
    .sliderMenu(refresh.code, title = "Weights Slider",
       names =       c(                 "N"),
       minima =      c(                   1),
       maxima =      c(     nFrontierPoints),
       resolutions = c(con$sliderResolution),
       starts =      c(               Start))

    # Return Value:
    invisible()
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                DESCRIPTION:
#  portfolioData            Returns an object of class fPFOLIODATA
################################################################################


portfolioData <-
    function(data, spec=portfolioSpec())
{
    # A function implemented by Rmetrics

    # Description:
    #   Creates portfolio data list

    # Arguments:
    #   data - a multivariate 'timeSeries' object
    #   spec -  a portfolio specification structure, from which
    #       the mean and covariance type of estimator will be extracted
    
    # Details:
    #   The first argument can be either:
    #   1) an object of class "fPFOLIODATA"
    #   2) an object of class "timeSeries"
    #   3) a "list: with portfolio mean and covariance

    # FUNCTION:

    # Data, if we have already an object of class "fPFOLIODATA":
    if (is(data, "fPFOLIODATA")) return(data)

    # Data, if we have a "timeSeries" or a "list":
    if (inherits(data,"timeSeries")) {
        series = data = sort(data)
        assetsNames = colnames(data)
    } else if (inherits(data,"list")) {
        series = rep(NA, times = length(data[[1]]))
        assetsNames = names(series) = names(data[[1]])
    }
    nAssets = length(assetsNames)
    names = assetsNames
    if(is.null(names)) names = paste("A", 1:nAssets, sep = "")
    .data = list(
        series = series,
        nAssets = nAssets,
        names = assetsNames)
        
    # Statistics:
    if (inherits(data,"timeSeries")) {
        estimator = getEstimator(spec)
        estimatorFun = match.fun(estimator)
        muSigma = estimatorFun(data, spec)
        Cov = cov(data)
        rownames(Cov) <- colnames(Cov) <- names
        .statistics = list(
            mean = colMeans(data),
            Cov = Cov,
            estimator = estimator,
            mu = muSigma$mu,
            Sigma = muSigma$Sigma)
    } else if (inherits(data,"list")) {
        .statistics = list(
            mean = data[[1]],
            Cov = data[[2]],
            estimator = NA,
            mu = data[[1]],
            Sigma = data[[2]])
    }

    # Tail Risk:
    .tailRisk = spec@model$tailRisk

    # Return Value:
    new("fPFOLIODATA",
        data = .data,
        statistics = .statistics,
        tailRisk = .tailRisk)
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                DESCRIPTION:
#  solveRsolnp              Portfolio interface to solver Rsolnp
#  .rsolnpArguments         Returns arguments for solver
################################################################################


solveRsolnp <-
    function(data, spec, constraints)
{
    # Description:
    #   Portfolio interface to solver Rsolnp
    
    # Arguments;
    #   data - an object of class timeSeries
    #   spec - an object of class fPFOLIOSPEC
    #   constraints - an object of class character
    
    # FUNCTION:   

    # Settings:
    Data <- portfolioData(data, spec)
    nAssets <- getNAssets(Data)
    mu <- getMu(Data)
    Sigma <- getSigma(Data)
        
    # Compose Arguments for Solver:
    args <- .rsolnpArguments(data, spec, constraints)
    
     optim <- Rsolnp::solnp(
        pars = args$par, 
        fun = args$fun, 
        grad = NULL, 
        eqfun = args$eqfun, 
        eqB = args$B, 
        eqgrad = NULL, 
        ineqfun = args$ineqfun, 
        ineqLB = args$ineqLB, 
        ineqUB = args$ineqUB, 
        ineqgrad = NULL, 
        LB = args$LB, 
        UB = args$UB,
        control = list())   
    
    # Extract Weights:
    weights <- .checkWeights(optim$par)
    attr(weights, "invest") <- sum(weights)
    
    # Check Messages and Get Status:
    #   ... unfortunately solnp has no status variable, 
    #       so we have to analyze the messages
    Status = 1
    
    # Result:
    ans <- list(
        type = "MV",
        solver = "solveRsolnp",
        optim = optim,
        weights = weights, 
        targetReturn = NA,
        targetRisk = NA,
        objective = optim$fx, 
        status = Status, 
        message = optim$message)    
    returnFun <- match.fun(getObjective(spec)[2])
    ans$targetReturn <- returnFun(ans$weights)
    riskFun <- match.fun(getObjective(spec)[3])
    ans$targetRisk <- riskFun(ans$weights)
        
    # Return Value:
    ans
}
 
  
# ------------------------------------------------------------------------------


.rsolnpArguments <-
    function(data, spec, constraints)
{
    # Description:
    #   Create Arguments for Rsolnp
    
    # Details:
    #       min:                    fun(x)
    #       subject to: 
    #                        g_i(x) = eqB
    #                  ineqLB <= h_i(x) <= ineqUB
    #                       LB <= x <= UB  
 
    # FUNCTION:  

    DEBUG = FALSE
    
    # Settings:
    Data <- portfolioData(data)
    nAssets <- getNAssets(Data)  
    mu <- getMu(Data)
    Sigma <- getSigma(Data)
    fn <- match.fun(getObjective(spec)[1])
     
    # Box Constrains:
    LB <- minWConstraints(data, spec, constraints)
    UB <- maxWConstraints(data, spec, constraints)
    if(DEBUG) print(rbind(LB, UB))
    
    # Linear / Group Constraints:
    # ... targetReturn may be not defined,then set it to NA
    if (is.null(getTargetReturn(spec))) setTargetReturn(spec) <- NA
    # ... has in the first line the return constraint, if NA then ignore  it
    eqsumW <- eqsumWConstraints(data, spec, constraints)
    if (is.na(eqsumW[1, 1])) eqsumW = eqsumW[-1, , drop= FALSE]
    Aeqsum <- eqsumW[, -1]
    aeqsum <- eqsumW[, 1]
    minsumW <- minsumWConstraints(data, spec, constraints)
    if (is.null(minsumW)) {
        Aminsum <- aminsum <- NULL
    } else {
        Aminsum <- minsumW[, -1]
        aminsum <- minsumW[, 1]
    }      
    maxsumW = maxsumWConstraints(data, spec, constraints)
    if (is.null(maxsumW)) {
        Amaxsum <- amaxsum <- NULL
    } else {
        Amaxsum <- maxsumW[, -1]
        amaxsum <- maxsumW[, 1]
    }      
    A <- rbind(Aeqsum, Aminsum, Amaxsum)
    lin.lower <- c(aeqsum, aminsum, rep(-Inf, length(amaxsum)))
    lin.upper <- c(aeqsum, rep(Inf, length(aminsum)), amaxsum)
    if(DEBUG) print(cbind(lin.lower, A, lin.upper))
   
    # Nonlinear Constraints - Here Covariance Risk Budgets:
    nlin <- list()
    nlin.lower <- NULL
    nlin.upper <- NULL
    # Check Constraints Strings for Risk Budgets:
    # Example: constraints = c("minB[2:3]=0.1", "maxB[3:5]=0.9")
    validStrings <- c("minB", "maxB")
    usedStrings <- unique(sort(sub("\\[.*", "", constraints)))
    checkStrings <- sum(usedStrings %in% validStrings)
    includeRiskBudgeting <- as.logical(checkStrings)
    if (DEBUG) print(includeRiskBudgeting)
    if (includeRiskBudgeting) {
        # Compose Non-Linear (Cov Risk Budget) Constraints Functions:
        nlcon <- function(x) {
            B1 = as.vector(x %*% Sigma %*% x)
            B2 = as.vector(x * Sigma %*% x)
            B = B2/B1
            B
        }
        if(DEBUG) print(nlcon)
        # Compose non-linear functions now for each asset ...
        for (I in 1:nAssets)
            eval( parse(text = paste(
                "nlcon", I, " = function(x) { nlcon(x)[", I, "] }", sep = "")) )
        nlinFunctions = paste("nlcon", 1:nAssets, sep = "", collapse = ",")
        nlinFunctions = paste("list(", nlinFunctions, ")")
        nlin = eval( parse(text = nlinFunctions) )
        if(DEBUG) print(nlin)
        # ... and finally Compose Constraints Vectors:
        nlin.lower = minBConstraints(data, spec, constraints)
        nlin.upper = maxBConstraints(data, spec, constraints)
        if(DEBUG) print(rbind(nlin.lower, nlin.upper))
    }
    
    # General non-lin Portfolio Constraints:
    # ... todo: currently overwrites previous selection
    nlin = listFConstraints(data, spec, constraints)
    if(DEBUG) print(nlin)
    nlin.lower = minFConstraints(data, spec, constraints)
    nlin.upper = maxFConstraints(data, spec, constraints)
    if(DEBUG) print(cbind(nlin.lower, nlin.upper))
 
    # Return Value:
    list(
        pars = rep(1/nAssets, nAssets), 
        fun = fn,
        grad = NULL,
        eqfun = NULL,
        eqB = NULL,
        eqgrad = NULL,
        ineqfun = nlinFunctions,
        ineqLB = nlin.lower, 
        ineqUB = nlin.upper, 
        ineqgrad = NULL, 
        LB = LB, 
        UB = UB)
}
    

################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


###############################################################################
# FUNCTION:                    DESCRIPTION:
#  rsolveQP                     General Interface for QP solvers
#  .solveQP.MV.demo
###############################################################################


rsolveQP <- 
  function(objective, lower=0, upper=1, linCons, 
           control=list(solver="quadprog", invoke=c("R", "AMPL", "NEOS")))
  {    
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Implements general function wrapper for QP solvers
    
    # Arguments:
    #   objective - list(dvec=NULL, Dmat=NULL)
    #   lower - lower box constraints
    #   upper - upper box constraints
    #   linCons - linear constraints, list with entries:
    #       mat, lower, upper.
    #   control - control list
    
    # FUNCTION:
    
    # Control:
    solver <- control$solver
    invoke <- control$invoke[1]
    
    # Solve Linear Problem:
    if (invoke == "R") {
      rfooLP <- match.fun ( paste("r", solver, "QP", sep=""))
      ans <- rfooLP(objective, lower, upper, linCons, control)
    }
    if (invoke == "AMPL" ) {
      ans <- ramplQP(objective, lower, upper, linCons, control)
    }
    if (invoke == "NEOS" ) {
      ans <- rneosQP(objective, lower, upper, linCons, control)
    }  
    ans$solver <- paste(invoke, ans$solver) 
    
    # Return Value:
    ans  
  }


###############################################################################


.solveQP.MV.demo <- 
  function()
  {
    # Solve Mean-Variance Portfolio:
    
    # Load Dataset
    dataSet <- data("LPP2005REC", package="timeSeries", envir=environment())
    LPP2005REC <- get(dataSet, envir=environment())
    
    # Load Swiss Pension Fund Data:
    nAssets <- 6
    data <- 100 * LPP2005REC[, 1:nAssets]     
    
    # Arguments: 
    objective <- list(dvec=rep(0, nAssets), Dmat=cov(data))
    lower <- 0
    upper <- 1
    mat <- rbind(
      budget = rep(1, times=nAssets), 
      returns = colMeans(data))
    matLower <- c(
      budget = 1, 
      return = mean(data))
    matUpper <- matLower
    linCons <- list(mat, matLower, matUpper)
    control <- list()
    
    # R Contributed Solvers:
    rquadprogQP(objective, lower, upper, linCons)
    ripopQP(objective, lower, upper, linCons)
    
    # Default - AMPL Interface:
    ampl <- ramplQP(objective, lower, upper, linCons)
    ampl
    
    # All AMPL:
    for (solver in c(
      "cplex", "donlp2", "loqo", "lpsolve", "minos", "snopt",
      "ipopt", "bonmin", "couenne")) {
      ans <- ramplQP(objective, lower, upper, linCons, 
                     control=list(solver=solver))
      print(ans)
    }
    
    # NEOS:
    # require(rneos)
    neos <- rneosQP(objective, lower, upper, linCons, 
                    control=list(solver="ipopt", category="nco"))
    neos
    
    # nco: Using Nonlinear Constrained Optimization Solver:    
    for (solver in c(
      "conopt", "filter", "knitro", "lancelot", "loqo", "minos", "mosek",
      "pennon", "snopt")) 
    {   
      ans <- rneosQP(objective, lower, upper, linCons, 
                     control=list(solver=solver, category="nco"))
      print(ans)
    }
    
    # KRESTREL:
    kestrel <- rkestrelQP(objective, lower, upper, linCons, 
                          control=list(solver="loqo"))
    kestrel
  }


###############################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################
# FUNCTION:                     DESCRIPTION:
#  setType<-                     Sets type of portfolio optimization
#  setOptimize<-                 Sets what to optimze, minRisk or maxRetururn
#  setEstimator<-                Sets name of mean-covariance estimator
#  setTailRisk<-                 Sets tail dependency matrix
#  setParams<-                   Sets optional model parameters
# FUNCTION:                     DESCRIPTION:
#  setWeights<-                  Sets weights vector
#  setTargetReturn<-             Sets target return value
#  setTargetRisk<-               Sets target return value
#  setRiskFreeRate<-             Sets risk-free rate value
#  setNFrontierPoints<-          Sets number of frontier points
#  setStatus<-                   Sets portfolio status information
# FUNCTION:                     DESCRIPTION:
#  setSolver<-                   Sets name of desired solver
#  setObjective<-                Sets objective function name
#  setTrace<-                    Sets solver's trace flag
################################################################################


"setType<-" <- 
    function(spec, value)
{   
    # A function implemented by Diethelm Wuertz

    # Description:                
    #   Sets the portfolio type for a portfolio structure
      
    # Notes:
    #   What types are valid?
    #   MV     Standard Mean-Variance Portfolio
    #          Objective   = Covariance Risk
    #          Constraints = Budget 1'*W = 1
    #          Allowed Constraints: Box + Group
    #   CVaR   Standard Mean-CVaR portfolio
    #          Objective   = Linearized CVaR Risk
    #          Constraints = Budget 1'*W = 1
    #          Allowed Constraints: Box + Group
      
    # Arguments:
    
    # FUNCTION:
    
    # Type ?
    spec@model$type <- value
    message("Solver set to solveRquadprog")
      
    # Set Solver:
    if (value == "MV") {
      setSolver(spec) <- "solveRquadprog"
      message("setSolver: solveRquadprog") }
    if (value == "CVaR") {
      setSolver(spec) <- "solveRglpk" 
      message("setSolver: solveRglpk") }
     
    # Return Value:
    spec
}


# ------------------------------------------------------------------------------


"setOptimize<-" <- 
    function(spec, value)
{   
    # A function implemented by Diethelm Wuertz

    # Description:                
    #   Sets the portfolio type for a portfolio structure
     
    # Arguments:
    
    # FUNCTION:
    
    # Type ?
    spec@model$optimize <- value
    
    # Return Value:
    spec
}


# ------------------------------------------------------------------------------


"setEstimator<-" <- 
    function(spec, value)
{   
    # A function implemented by Diethelm Wuertz

    # Description:                  
    #   Sets the type of mean-cov estimator for a portfolio structure
    
    # Arguments:
    
    # FUNCTION:
    
    # Estimator ?
    spec@model$estimator <- value 
    
    # Return Value:
    spec
}


# ------------------------------------------------------------------------------


"setParams<-" <- 
    function(spec, name, value)
{   
    # A function implemented by Diethelm Wuertz

    # Description:                  
    #   Sets optional parameters for a portfolio structure
    
    # Arguments:
    
    # FUNCTION:

    # Extend Parameter List:
    spec@model$params[name] <- value 
    
    # Return Value:
    spec
}


################################################################################


"setWeights<-" <- 
    function(spec, value)
{   
    # A function implemented by Diethelm Wuertz

    # Description:                    
    #   Sets the weights vector for a portfolio structure
    
    # Arguments:
    
    # FUNCTION:
    
    # Weights ?
    spec@portfolio$weights <- value
    spec@portfolio$targetReturn <- NA
    spec@portfolio$targetRisk <- NA
    
    # Return Value:
    spec
}


# ------------------------------------------------------------------------------


"setTargetReturn<-" <- 
    function(spec, value)
{   
    # A function implemented by Diethelm Wuertz

    # Description:                                   
    #   Sets the target return value for a portfolio structure
    
    # Arguments:
    
    # FUNCTION:

    # Target Return ?
    spec@portfolio$targetReturn <- value
    spec@portfolio$weights <- NA
    spec@portfolio$targetRisk <- NA
    
    # What to optimize ?
    spec@model$optimize <- "minRisk"
    
    # Return Value:
    spec
}


# ------------------------------------------------------------------------------


"setTargetRisk<-" <- 
    function(spec, value)
{   
    # A function implemented by Diethelm Wuertz

    # Description:                                   
    #   Sets the target return value for a portfolio structure
  
    # Arguments:
    
    # FUNCTION:
 
    # Target Risk ?
    spec@portfolio$targetRisk <- value
    spec@portfolio$weights <- NA
    spec@portfolio$targetReturn <- NA
     
    # What to optimize ?
    spec@model$optimize <- "maxReturn"
    
    # Return Value:
    spec
}


# ------------------------------------------------------------------------------


"setAlpha<-" <-
    function(spec, value)
{   
    # A function implemented by Diethelm Wuertz

    # Description:                  
    #   Sets the CVaR alpha significance value for a portfolio structure
    
    # Arguments:
    
    # FUNCTION:
 
    # Estimator ?
    spec@model$params$alpha <- value 
    
    # Return Value:
    spec
}


# ------------------------------------------------------------------------------


"setRiskFreeRate<-" <- 
    function(spec, value)
{   
    # A function implemented by Diethelm Wuertz

    # Description:                                   
    #   Sets the risk free rate for a portfolio structure
    
    # Arguments:
    
    # FUNCTION:
    
    # Check Validity:
    stopifnot(is.numeric(value))
    stopifnot(length(value) == 1)
    
    # Risk-Free Rate ?
    spec@portfolio$riskFreeRate <- value
    
    # Return Value:
    spec
}


# ------------------------------------------------------------------------------


"setNFrontierPoints<-" <- 
    function(spec, value)
{   
    # A function implemented by Diethelm Wuertz

    # Description:                                   
    #   Sets the number of frontier points for a portfolio structure
    
    # Arguments:
    
    # FUNCTION:
    
    # Check Validity:
    stopifnot(is.numeric(value))
    stopifnot(length(value) == 1)
    stopifnot(value > 0)
    
    # Risk-Free Rate ?
    spec@portfolio$nFrontierPoints <- value
    
    # Return Value:
    spec
}


# ------------------------------------------------------------------------------


"setStatus<-" <- 
    function(spec, value)
{   
    # A function implemented by Diethelm Wuertz

    # Description:                                   
    #   Sets portfolio status information
    
    # Arguments:
    
    # FUNCTION:
    
    # Check Validity:
    stopifnot(is.numeric(value))
    stopifnot(length(value) == 1)
    
    # Risk-Free Rate ?
    spec@portfolio$status <- value
    
    # Return Value:
    spec
}


# ------------------------------------------------------------------------------


"setTailRisk<-" <- 
    function(spec, value)
{   
    # A function implemented by Diethelm Wuertz

    # Description:                    
    #   Sets the tail risk value for a portfolio structure
    
    # Arguments:
    #   value - a list with two matrix elements, $lower and $upper, 
    #       with the pairwise tail dependence coefficints.
            
    # Example:
    #   LPP = as.timeSeries(data(LPP2005REC))[, 1:6]
    #   setTailRisk <- .nigDependencyFit(LPP)
    
    # Arguments:
    
    # FUNCTION:
    
    # Tail Risk ?
    spec@model$tailRisk <- value  
    
    # Return Value:
    spec
}


################################################################################
 

"setSolver<-" <- 
    function(spec, value)
{   
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Sets the solver value for a portfolio structure
    
    # Arguments:
    
    # FUNCTION:
      
    # Set Solver:
    spec@optim$solver <- value
    
    # Return Value:
    spec
}


# ------------------------------------------------------------------------------


"setObjective<-" <- 
    function(spec, value)
{   
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Sets the solver objective function name for a portfolio structure
    
    # Arguments:
    
    # FUNCTION:
      
    # Set Solver:
    spec@optim$objective <- value
    
    # Return Value:
    spec
}


# ------------------------------------------------------------------------------


"setTrace<-" <- 
    function(spec, value)
{   
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Sets the trace value for a portfolio structure
    
    # Arguments:
    
    # FUNCTION:
    
    # Set Trace:
    spec@optim$trace <- value
    
    # Return Value:
    spec
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                    DESCRIPTION:
#  .weightsWheel                Adds a pie of weights to frontier plot
#  .attributesWheel             Adds a pie of attributes to frontier plot
# FUNCTION:                    DESCRIPTION:
#  .notStackedWeightsPlot       Plots the not stacked weights of potfolio
#  .addlegend                   Adds legend to sliders
################################################################################


.weightsWheel <-
    function(object, piePos = NULL, pieR = NULL, pieOffset = NULL, ...)
{
    # A function implemented by Oliver Greshake and Diethelm Wuertz

    # Description:
    #   Adds a pie plot of weights for MV and CVaR Portfolios

    # Arguments:
    
    # Details:
    #   The default settings are:
    #   piePos - Position of tangency Portfolio
    #   pieR - 10% of the Risk Range: diff(range(targetRisk(object)))/10

    # FUNCTION:

    # Extract Coordinates:
    p = par()$usr/15
    dx = p[2]-p[1]
    dy = p[4]-p[3]

    # Pie Position:
    if(is.null(piePos)) {
        Data = getSeries(object)
        Spec = getSpec(object)
        Constraints = getConstraints(object)
        tg = getTargetReturn(tangencyPortfolio(Data, Spec, Constraints))
        ef = as.vector(getTargetReturn(object))
        piePos = which(diff(sign(ef-tg)) > 0)
    }

    # Pie Radius:
    if(is.null(pieR)) {
        pieR = c(1, 1)
    }

    # Pie Offset:
    if(is.null(pieOffset)) {
        pieOffset = c(-2*dx, 0)
    }

    # Plot Circle:
    weights = getWeights(object)[piePos, ]
    nWeights = length(weights)
    Sign = rep("+", nWeights)
    Sign[(1:nWeights)[weights < 0]] = "-"
    x = getTargetRisk(object)[piePos]
    y = getTargetReturn(object)[piePos]
    phi =  seq(0, 2*pi, length = 360)
    X = x + pieOffset[1] + pieR[1] * sin(phi) * dx
    Y = y + pieOffset[2] + pieR[2] * cos(phi) * dy
    lines(X, Y)

    # Add Center Point:
    points(x, y, col = "red", pch = 19, cex = 1.5)

    # Add Arrow:
    lines(c(x, x+pieOffset[1]), c(y, y+pieOffset[2]))

    # Add Color Wheel:
    psi = 2*pi*c(0, cumsum(abs(weights)/sum(abs(weights))))
    for (i in 1 : length(weights) ) {
        # Plotting Only Pie pieces with Weights > 5%
        if(psi[i+1]-psi[i] > 0.05 * 2*pi) {
            Psi = psi[i] + (0:100) * (psi[i+1]-psi[i])/100
            polyX = x + pieOffset[1] + pieR[1]*c(0, sin(Psi), 0) * dx
            polyY = y + pieOffset[2] + pieR[2]*c(0, cos(Psi), 0) * dy
            polygon(polyX, polyY, col = rainbow(nWeights)[i])
            # Adding the Asset Signs:
            text(x + pieOffset[1] + 0.75*pieR[1]* sin(Psi[51]) * dx,
                y + pieOffset[2] + 0.75*pieR[2]* cos(Psi[51]) * dy,
                col = "white", Sign[i])
         }
    }

    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------


.attributesWheel <-
    function(object, piePos = NULL, pieR = NULL, pieOffset = NULL, ...)
{
    # A function implemented by Oliver Greshake and Diethelm Wuertz

    # Description:
    #   Adds a pie plot of the weights

    # Arguments:
    
    # Details:
    #   The default settings are:
    #   piePos - Position of tangency Portfolio
    #   pieR - 10% of the Risk Range: diff(range(targetRisk(object)))/10

    # FUNCTION:

    # Extraction coordinates
    p = par()$usr/15
    dx = p[2]-p[1]
    dy = p[4]-p[3]

    # Pie Position:
    if(is.null(piePos)) {
        Data = getSeries(object)
        Spec = getSpec(object)
        Constraints = getConstraints(object)
        tg = getTargetReturn(tangencyPortfolio(Data, Spec, Constraints))
        ef = as.vector(getTargetReturn(object))
        piePos = which(diff(sign(ef-tg)) > 0)
    }

    # Pie Radius:
    if(is.null(pieR)) {
        pieR = c(1, 1)
    }

    # Pie Offset:
    if(is.null(pieOffset)) {
        pieOffset = c(2*dx, 0)
    }

    # Plot Circle - Get weighted Returns:
    weights = getWeights(object)
    dim = dim(weights)
    returns = getStatistics(object)$mu
    weightedReturns = NULL
    for(i in 1:dim[2]){
        nextWeightedReturns = weights[,i]*returns[i]
        weightedReturns = cbind(weightedReturns, nextWeightedReturns)
    }
    colnames(weightedReturns) = colnames(weights)
    weightedReturns = weightedReturns[piePos, ]
    nWeights = length(weightedReturns)
    Sign = rep("+", times =  nWeights)
    Sign[(1:nWeights)[weightedReturns < 0]] = "-"
    x = getTargetRisk(object)[piePos]
    y = getTargetReturn(object)[piePos]
    phi =  seq(0, 2*pi, length = 360)
    X = x + pieOffset[1] + pieR[1] * sin(phi) * dx
    Y = y + pieOffset[2] + pieR[2] * cos(phi) * dy
    lines(X, Y)

    # Add Center Point:
    points(x, y, col = "red", pch = 19, cex = 1.5)

    # Add Arrow:
    lines(c(x, x+pieOffset[1]), c(y, y+pieOffset[2]))

    # Add Color Wheel:
    psi = 2*pi*c(0, cumsum(abs(weightedReturns)/sum(abs(weightedReturns))))
    for (i in 1 : nWeights) {
        # Plotting Only Pie pieces with Weights > 5%
        if(psi[i+1]-psi[i] > 0.05 * 2*pi) {
            Psi = psi[i] + (0:100) * (psi[i+1]-psi[i])/100
            polyX = x + pieOffset[1] + pieR[1]*c(0, sin(Psi), 0) * dx
            polyY = y + pieOffset[2] + pieR[2]*c(0, cos(Psi), 0) * dy
            polygon(polyX, polyY, col = rainbow(nWeights)[i])
            # Adding the Asset Signs:
            text(x + pieOffset[1] + 0.75*pieR[1]* sin(Psi[51]) * dx,
                y + pieOffset[2] + 0.75*pieR[2]* cos(Psi[51]) * dy,
                col = "white", Sign[i])
         }
    }

    # Return Value:
    invisible()
}


#-------------------------------------------------------------------------------


.notStackedWeightsPlot <-
    function(object, col = NULL)
{
    # A function implemented by Oliver Greshake

    # Description:

    # Arguments:
    #   object - an object of class 'fPORTFOLIO'
    #   col - a color palette, by default the rainbow palette

    # FUNCTION:

    # Settings:
    weights = getWeights(object)
    N = ncol(weights)
    targetRisk = getTargetRisk(object)[, 1]
    targetReturn = getTargetReturn(object)[, 1]
    nSigma = length(targetRisk)

    # Select Colors if not specified ...
    if (is.null(col)) col = rainbow(N)

    # Plot first asset ...
    plot(weights[, 1], col = col[1], type = "l", ylim = c(min(weights),
        max(weights)), xaxt = "n", xlab = "", ylab = "")

    # Add vertical Line at minimum risk:
    minIndex = which.min(targetRisk)
    minRisk = min(targetRisk)

    # Big Point at minimum risk for first asset ...
    points(x = minIndex, y = weights[minIndex, 1], col = col[1], pch = 19,
        xaxt = "n", yaxt = "n", cex = 2)

    # ... and all other assets
    for(i in 1:(N-1)){
        points(weights[, i+1], col = col[i+1], type = "l", xaxt = "n",
        yaxt = "n")
        points(x = minIndex, y = weights[minIndex, i+1], col = col[i+1],
            pch = 19, xaxt = "n", yaxt = "n", cex = 2)
    }
    grid()
    abline(h = 0, col = "grey", lty = 3)
    lines(x = c(minIndex, minIndex), y = c(0, 1), col = "black", lwd = 2)

    # Add Tailored Labels -  6 may be a good Number ...
    nLabels = 6
    M = c(0, ( 1: (nSigma %/% nLabels) ) ) * nLabels + 1
    text(minIndex, 1, "Min Risk", pos = 4)
    minRiskValue = as.character(signif(minRisk, 3))
    minReturnValue = as.character(signif(targetReturn[minIndex], 3))
    mtext(minRiskValue, side = 1, at = minIndex, cex = 0.7)
    mtext(minReturnValue, side = 3, line = 0.5, at = minIndex, cex = 0.7)

    # Take a reasonable number of significant digits to plot, e.g. 2 ...
    nPrecision = 3
    axis(1, at = M, labels = signif(targetRisk[M], nPrecision))
    axis(3, at = M, labels = signif(targetReturn[M], nPrecision))

    # Add Axis Labels and Title:
    mtext("Target Risk", side = 1, line = 2, cex = 0.7)
    mtext("Target Return", side = 3, line = 2, cex = 0.7)
    mtext("Weight", side = 2, line = 2, cex = 0.7)

    # Add Info:
    mtext(paste(getType(object), "|", getSolver(object)),
        side = 4, adj = 0, col = "grey", cex = 0.7)

    # Add Title:
    mtext("Weights", adj = 0, line = 2.5, font = 2, cex = 0.8)

    # Return Value:
    invisible()
}


#-------------------------------------------------------------------------------


.addlegend <-
    function(object, control = list())
{
    # A function implemented by Oliver Greshake

    # Description:
    #   Adds a perdefined legend to sliders

    # Arguments:
    #   object - an object of class 'fPORTFOLIO'
    #   control - control list for colors and symbols

    # FUNCTION:

    # Settings:
    dim = getNAssets(object)
    namesSingleAsset = names(object@data$statistics$mu)
    # Check if polt is used for forntierSlider...
    if(control$sliderFlag == "frontier"){
        legendtext = c("Efficient Frontier", "Sharpe Ratio", "Minimum Variance",
            "Tangency Portfolio", "Market Portfolio", "Equal Weights",
            namesSingleAsset)
        color = c("black", control$sharpeRatio.col, control$minvariance.col,
            control$tangency.col, control$cml.col, control$equalWeights.col,
            control$singleAsset.col)
        sym = c(19, 19, control$minvariance.pch, control$tangency.pch,
            control$cml.pch, control$equalWeights.pch,
            rep(control$singleAsset.pch, times = dim))
    # ... else is the weightsSlider case
    } else {
            legendtext = c("Efficient Frontier", "Minimum Variance",
            "Tangency Portfolio", namesSingleAsset)
        color = c("black", control$minvariance.col,
            control$tangency.col, control$singleAsset.col)
        sym = c(19, control$minvariance.pch, control$tangency.pch,
            rep(control$singleAsset.pch, times = dim))
    }

    # Adding Legend:
    legend("topleft", legend = legendtext, col = color, pch = sym, cex = .8,
        bty = "n")

    # Return Value:
    invisible()
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                     DESCRIPTION:
#  weightsPie                    Plots a pie of portfolio weights
#  weightedReturnsPie            Plots a pie of weighted means
#  covRiskBudgetsPie             Plots a pie of covariance risk budgets
#  tailRiskBudgetsPie            Plots a pie of copulae tail risk budgets
################################################################################


weightsPie <-
function(object, pos = NULL, labels = TRUE, col = NULL,
    box = TRUE, legend = TRUE, radius = 0.8, ...)
{
    # A function implemented by Diethelm Wuertz and Oliver Greshake

    # Description:
    #   Plots a Pie Chart of Weigths

    # Arguments:
    #   object - an object of class 'fPORTFOLIO'.
    #   pos - a numeric value, determining the position on the efficient
    #       frontier plotting the pie, by default NULL, i.e. expecting
    #       an object having only one set of weights like the tangency
    #       portfolio.
    #   box - a logical value, determining whether a frame (box) should
    #       be plotted around the pie, by default TRUE.
    #   col - a color palette, by default the rainbow palette.
    #   legend - a logical value, determining whether a legend with
    #       the names of the assets should be plotted, by default TRUE.

    # Example:
    #   weightsPie(tangencyPortfolio(dutchPortfolioData(), portfolioSpec()))
    #   title(main = "Tangency Portfolio Weights")

    # FUNCTION:

    # Default Settings:
    Title <- "Weights"
    if (is.null(col)) col <- seqPalette(getNAssets(object), "Blues")
    if (sum(c(par()$mfrow, par()$mfcol)) == 4) CEX = 0.9 else CEX = 0.7

    # Get Weights:
    if (is.null(pos)) {
        Weights <- getWeights(object@portfolio)
    } else {
        Weights <- getWeights(object@portfolio)[pos, ]   
    }  
    X <- Weights
    
    # Check for Negative Pie Segments:
    nX <- getNAssets(object)
    Sign <- rep("+", nX)
    Sign[(1:nX)[X < 0]] <- "-"
    absX <- abs(X)
    Index <- (1:nX)[X > 0]

    # Take care of labels, they are also used by the function pie():
    if (!is.logical(labels)) {
        Names <- pieLabels <- labels
        labels <- FALSE
    } else  {
        Names <- pieLabels <- object@data@data$names
    }

    # Pie Chart:
    col <- col[Index]
    legendAssets <- Names[Index]
    Labels <- paste(Names, Sign)
    Labels = Labels[X > 0]
    Y <- X[X > 0]

    # Plot:
    if (labels) {
        pie(Y, labels = Labels, col = col, radius = radius, cex = CEX)
    } else {
        pie(Y, labels = pieLabels, col = col, radius = radius, ...)
    }

    # Add Title:
    if (labels) 
        mtext(Title, adj = 0, line = 2.5, font = 2, cex = CEX+0.1)

    # Add Info:
    if (labels) {
        mtext(paste(getType(object), "|", getSolver(object)),
            side = 4, adj = 0, col = "grey", cex = 0.7)
    }

    # Add Legend:
    if (legend) {
        legend("topleft", legend = legendAssets, bty = "n", cex = CEX,
            fill = col)
        legendY <- as.character(round(100*Y, digits = 1))
        legendY <- paste(Sign[Index], legendY, sep = "")
        legendY <- paste(legendY, "%")
        legend("topright", legend = legendY, bty = "n", cex = CEX,
            fill = col)
    }

    # Add Box:
    if (box) box()

    # Return Value:
    invisible(Y)
}


# ------------------------------------------------------------------------------


weightedReturnsPie <-
function(object, pos = NULL, labels = TRUE, col = NULL,
    box = TRUE, legend = TRUE, radius = 0.8, ...)
{
    # A function implemented by Diethelm Wuertz and Oliver Greshake

    # Description:
    #   Adds a pie plot of the weights

    # Arguments:
    #   object - an object of class 'fPORTFOLIO'.
    #   pos - a numeric value, determining the position on the efficient
    #       frontier plotting the pie, by default NULL, i.e. expecting
    #       an object having only one set of weights like the tangency
    #       portfolio.
    #   box - a logical value, determining whether a frame (box) should
    #       be plotted around the pie, by default TRUE.
    #   col - a color palette, by default the rainbow palette.
    #   legend - a logical value, determining whether a legend with
    #       the names of the assets should be plotted, by default TRUE.

    # Example:
    #   attributesPie(tangencyPortfolio(dutchPortfolioData(), portfolioSpec()))
    #   title(main = "Tangency Portfolio Weights")

    # FUNCTION:

    # Default Settings:
    Title <- "Weighted Returns"
    if (is.null(col)) col <- seqPalette(getNAssets(object), "Blues")
    if (sum(c(par()$mfrow, par()$mfcol)) == 4) CEX = 0.9 else CEX = 0.7


    # Get Weights:
    if (is.null(pos)) {
        Weights <- getWeights(object@portfolio)
    } else {
        Weights <- getWeights(object@portfolio)[pos, ]   
    }  
    Returns = getStatistics(object)$mu
    X <- Weights * Returns

    # Check for Negative Pie Segments:
    nX <- getNAssets(object)
    Sign <- rep("+", nX)
    Sign[(1:nX)[X < 0]] <- "-"
    absX <- abs(X)
    Index <- (1:nX)[X > 0]

    # Take care of labels, they are also used by the function pie():
    if (!is.logical(labels)) {
        Names <- pieLabels <- labels
        labels <- FALSE
    } else  {
        Names <- pieLabels <- object@data@data$names
    }

    # Pie Chart:
    col <- col[Index]
    legendAssets <- Names[Index]
    Labels <- paste(Names, Sign)
    Labels <- Labels[X > 0]
    Y <- X[X > 0]

    # Plot:
    if (labels) {
        pie(Y, labels = Labels, col = col, radius = radius, cex = CEX)
    } else {
        pie(Y, labels = pieLabels, col = col, radius = radius, ...)
    }

    # Add Title:
    if (labels) 
        mtext(Title, adj = 0, line = 2.5, font = 2, cex = CEX+0.1)

    # Add Info:
    if (labels) {
        mtext(paste(getType(object), "|", getSolver(object)),
            side = 4, adj = 0, col = "grey", cex = 0.7)
    }

    # Add Legend:
    if (legend) {
        legend("topleft", legend = legendAssets, bty = "n", cex = CEX, 
        	fill = col)
        legendY = as.character(round(100*Y, digits = 1))
        legendY = paste(Sign[Index], legendY, sep = "")
        legendY = paste(legendY, "%")
        legend("topright", legend = legendY, bty = "n", cex = CEX,
            fill = col)
    }

    # Add Box:
    if (box) box()

    # Return Value:
    invisible(Y)
}


# ------------------------------------------------------------------------------


covRiskBudgetsPie <-
function(object, pos = NULL, labels = TRUE, col = NULL,
    box = TRUE, legend = TRUE, radius = 0.8, ...)
{
    # A function implemented by Diethelm Wuertz and Oliver Greshake

    # Arguments:
    #   object - an object of class 'fPORTFOLIO'.
    #   pos - a numeric value, determining the position on the efficient
    #       frontier plotting the pie, by default NULL, i.e. expecting
    #       an object having only one set of weights like the tangency
    #       portfolio.
    #   box - a logical value, determining whether a frame (box) should
    #       be plotted around the pie, by default TRUE.
    #   col - a color palette, by default the rainbow palette.
    #   legend - a logical value, determining whether a legend with
    #       the names of the assets should be plotted, by default TRUE.

    # Description:
    #   Plots a Pie Chart of Risk Budgets

    # Arguments:
    #   object - an object of class 'fPORTFOLIO'
    #   col - a color palette, by default the rainbow palette

    # Example:
    #   riskBudgetsPie(tangencyPortfolio(dutchPortfolioData(), portfolioSpec()))
    #   title(main = "Tangency Portfolio Weights")

    # FUNCTION:

    # Default Settings:
    Title = "Covariance Risk Budgets"
    if (is.null(col)) col <- seqPalette(getNAssets(object), "Blues")
    if (sum(c(par()$mfrow, par()$mfcol)) == 4) CEX <- 0.9 else CEX <- 0.7

    # Get Cov Risk Budgets:
    if (is.null(pos)) {
        X <- getCovRiskBudgets(object@portfolio)
    } else {
        X <- getCovRiskBudgets(object@portfolio)[pos, ]   
    }  

    # Check for Negative Pie Segments:
    nX <- getNAssets(object)
    Sign <- rep("+", nX)
    Sign[(1:nX)[X < 0]] <- "-"
    absX <- abs(X)
    Index <- (1:nX)[X > 0]

    # Take care of labels, they are also used by the function pie():
    if (!is.logical(labels)) {
        Names <- pieLabels <- labels
        labels <- FALSE
    } else  {
        Names <- pieLabels <- object@data@data$names
    }

    # Legend Labels:
    col <- col[Index]
    legendAssets <- Names[Index]
    Labels <- paste(Names, Sign)
    Labels <- Labels[X > 0]
    Y <- X[X > 0]

    # Plot:
    if (labels) {
        pie(Y, labels = Labels, col = col, radius = radius, cex = CEX)
    } else {
        pie(Y, labels = pieLabels, col = col, radius = radius, ...)
    }

    # Add Title:
    if (labels) 
         mtext(Title, adj = 0, line = 2.5, font = 2, cex = CEX+0.1)

    # Add Info:
    if (labels) {
        mtext(paste(getType(object), "|", getSolver(object)),
            side = 4, adj = 0, col = "grey", cex = 0.7)
    }

    # Add Legend:
    if (legend) {
        legend("topleft", legend = legendAssets, bty = "n", cex = CEX,
            fill = col)
        legendY <- as.character(round(100*Y, digits = 1))
        legendY <- paste(Sign[Index], legendY, sep = "")
        legendY <- paste(legendY, "%")
        legend("topright", legend = legendY, bty = "n", cex = CEX,
            fill = col)
    }

    # Add Box:
    if (box) box()

    # Return Value:
    invisible(Y)
}


# ------------------------------------------------------------------------------


tailRiskBudgetsPie <-
function(object, pos = NULL, labels = TRUE, col = NULL,
    box = TRUE, legend = TRUE, radius = 0.8, ...)
{
    ### todo: take care of @portfolio slot ...
 
    # A function implemented by Diethelm Wuertz and Oliver Greshake

    # Arguments:
    #   object - an object of class 'fPORTFOLIO'.
    #   pos - a numeric value, determining the position on the efficient
    #       frontier plotting the pie, by default NULL, i.e. expecting
    #       an object having only one set of weights like the tangency
    #       portfolio.
    #   box - a logical value, determining whether a frame (box) should
    #       be plotted around the pie, by default TRUE.
    #   col - a color palette, by default the rainbow palette.
    #   legend - a logical value, determining whether a legend with
    #       the names of the assets should be plotted, by default TRUE.

    # Description:
    #   Plots a Pie Chart of Tail Risk Budgets

    # Arguments:
    #   object - an object of class 'fPORTFOLIO'
    #   col - a color palette, by default the rainbow palette

    # Example:
    #   riskBudgetsPie(tangencyPortfolio(dutchPortfolioData(), portfolioSpec()))
    #   title(main = "Tangency Portfolio Weights")

    # FUNCTION:

    # Default Settings:
    Title <- "Tail Risk Budgets"
    if (is.null(col)) col <- seqPalette(getNAssets(object), "Blues")
    if (sum(c(par()$mfrow, par()$mfcol)) == 4) CEX <- 0.9 else CEX <- 0.7

    # Extracting weights position, if specified
    if(!is.null(pos)){
        object = object
        object@portfolio$weights = getWeights(object@portfolio)[pos, ]
    }

    # Check:
    stop("Not yet implemented")
    tailRiskMatrix = getTailRisk(object)
    X <- getCovRiskBudgets(object)

    # Check for Negative Pie Segments:
    nX <- getNAssets(object)
    Sign <- rep("+", nX)
    Sign[(1:nX)[X < 0]] <- "-"
    absX <- abs(X)
    Index <- (1:nX)[X > 0]

    # Take care of labels, they are also used by the function pie():
    if (!is.logical(labels)) {
        Names <- pieLabels <- labels
        labels <- FALSE
    } else  {
        Names <- pieLabels <- object@data@data$names    
    }

    # Legend Labels:
    col <- col[Index]
    legendAssets <- Names[Index]
    Labels <- paste(Names, Sign)
    Labels <- Labels[X > 0]
    Y <- X[X > 0]

    # Plot:
    if (labels) {
        pie(Y, labels = Labels, col = col, radius = radius, cex = CEX)
    } else {
        pie(Y, labels = pieLabels, col = col, radius = radius, ...)
    }

    # Add Title:
    if (labels) 
        mtext(Title, adj = 0, line = 2.5, font = 2, cex = CEX+0.1)

    # Add Info:
    if (labels) {
        mtext(paste(getType(object), "|", getSolver(object)),
            side = 4, adj = 0, col = "grey", cex = 0.7)
    }

    # Add Legend:
    if (legend) {
        legend("topleft", legend = legendAssets, bty = "n", cex = CEX,
            fill = col)
        legendY = as.character(round(100*Y, digits = 1))
        legendY = paste(Sign[Index], legendY, sep = "")
        legendY = paste(legendY, "%")
        legend("topright", legend = legendY, bty = "n", cex = CEX,
            fill = col)
    }

    # Add Box:
    if (box) box()

    # Return Value:
    invisible(Y)
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                 DESCRIPTION:
#  solveRsocp                Portfolio interface to solver Rsocp (in parma)
#  .rsocpArguments           Returns arguments for solver
#  .rsocp                    Wrapper to solver function
#  .rsocpControl             Returns default controls for solver
################################################################################


solveRsocp <-
  function(data, spec, constraints)
  {
    # Description:
    #   Portfolio interface to solver Rsocp (ported to and currently located in package parma)
    
    # Example:
    #   ans = solveRquadprog(.lppData, .mvSpec, "LongOnly")[-3]
    #   .mvSpec2 = .mvSpec; setTargetRisk(.mvSpec2) = ans$targetRisk
    #   solveRsocp(.lppData, .mvSpec2, "LongOnly")[-3]; ans
    
    #   efficientortfolio
    
    # FUNCTION:
    
    # Transform Data and Constraints:
    Data = portfolioData(data, spec)
    
    # Trace:
    trace <- getTrace(spec)
    if(trace) cat("\nPortfolio Optimiziation:\n Using Rsocp ...\n\n")
    
    # Get Specifications:
    nAssets = getNAssets(Data)
    
    # Create '.rsocp' conform arguments:
    args <- .rsocpArguments(data, spec, constraints)
    
    # Optimize:
    ans <- .rsocp(
      f = args$f, 
      A = args$A, 
      b = args$b, 
      C = args$C, 
      d = args$d, 
      N = args$N, 
      targetRisk = args$targetRisk, 
      mu = args$mu,
      Scale = args$Scale)
    
    # Return Value:
    class(ans) = c("solver", "list")
    ans
  }


################################################################################


.rsocpArguments <-
  function(data, spec, constraints)
  {
    # Description:
    #   Returns socp conform arguments for the solver
    
    # Example:
    #   .rsocpArguments(data, spec, constraints)
    
    # FUNCTION:
    
    # Settings:
    Data = portfolioData(data, spec)
    nAssets = getNAssets(Data)
    Scale = 1.0e6 * sd(as.vector(data))
    mu = getMu(Data) / Scale
    Sigma = getSigma(Data) / Scale^2
    targetRisk = getTargetRisk(spec) / Scale
    
    # Objective Function:
    f <- -mu
    
    # Constraints:
    eqsumW = eqsumWConstraints(data, spec, constraints)
    
    # C - Cone Constraints:
    C1 <- rep(0, nAssets)                                     # xCx
    C2 <- eqsumW[2, -1]                                       # sum(x)
    C3 <- rbind(diag(nAssets), -diag(nAssets) )               # x[i]>0
    
    # d - Cone Constraints:
    d1 <- targetRisk                                          # xCx = risk
    d2 <- eqsumW[2, 1]                                        # sum(x) <= 1
    d3 <- c(rep(0, nAssets), rep(-1, nAssets))                # x[i] > 0
    
    # A - Cone Constraints:
    A1 <- .SqrtMatrix(Sigma)
    A2 <- matrix(0, ncol = nAssets)
    A3 <- matrix(0, nrow = nrow(C3), ncol = nAssets)
    
    # b - Cone Constraints:
    b1 <- rep(0, nAssets)                                     # xCx
    b2 <- 0                                                   # sum(x)
    b3 <- rep(0, nrow(C3))                                    # x[i]>0
    
    # N - Cone Constraints:
    N1 <- nAssets                                             # dim(C)
    N2 <- 1                                                   # Full Investment
    N3 <- rep(1, nrow(C3))                                    # Long
    
    # Combine Constraints for SOCP:
    A <- rbind(A1, A2, A3)
    b <- c(b1, b2, b3)
    C <- rbind(C1, C2, C3)  
    d <- c(d1, -d2, -d3)  
    N <- c(N1, N2, N3)      
    
    # Return Value:
    list(f = f, A = A, b = b, C = C, d = d, N = N, 
         targetRisk = targetRisk * Scale, mu = mu * Scale, Scale = Scale)
  }


################################################################################


.rsocp  <-
  function(f, A, b, C, d, N, x = NULL, z = NULL, w = NULL, 
           targetRisk, mu = mu, Scale = Scale, control = .rsocpControl())
  {
    # Description:
    #   SOCP solver function for portfolios
    
    # Details:
    #   Package: fPortfolio
    #   Title: An R extenstion library to use SOCP from R.
    #   Version: 0.1 as of 2008-31-01
    #   Author: Yohan Chalabi and Diethelm Wuertz
    #   Description: Second-order cone programming solver 
    #       written by M. Lobo, L. Vandenberghe, and S. Boyd. 
    #       R.socp is a wrapper library to use it from R.
    #   License: R.socp - GPL
    
    # FUNCTION
    
    # Solve Portfolio:
    optim <- parma::Socp(f, A, b, C, d, N, x, z, w, control)
    
    # Extract Weights:
    weights = .checkWeights(optim$x)
    attr(weights, "invest") = sum(weights)
    
    # Prepare Output List: 
    ans <- list(
      type = "MV",
      solver = "solveRsocp",
      optim = optim,
      weights = weights,
      targetReturn = (weights %*% mu)[[1]],
      targetRisk = targetRisk,
      objective = (weights %*% mu)[[1]],
      status = as.integer(!optim$convergence),
      message = optim$message)
    
    # Return Value: 
    ans
  }


################################################################################


.rsocpControl <- 
  function(abs.tol = 1.0e-18, rel.tol = 1.0e-16, target = 0, 
           max.iter = 1000, Nu = 10, out.mode = 0, BigM.K = 2, BigM.iter = 5)
  { 
    # Description:
    #   Control list for portfolio SOCP optimization
    
    # FUNCTION:
    
    # Return Value:
    list(
      abs.tol = abs.tol, 
      rel.tol = rel.tol,
      target = target,
      max.iter = max.iter,
      Nu = Nu,
      out.mode = out.mode,
      BigM.K = BigM.K,
      BigM.iter = BigM.iter)
  }


################################################################################

## Imported from package Rsocp
.SqrtMatrix <-
function(x)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Square Root of a quadratic Matrix:

    # Example:
    #   A = matrix(c(1,.2,.2,.2,1,.2,.2,.2,1), ncol = 3)
    #   round(Sqrt(A) %*% Sqrt(A) - A, digits = 12)

    # FUNCTION:

    # Check if matrix is square:
    stopifnot(NCOL(x) == NROW(x))

    # One-dimensional ?
    if (NCOL(x) == 1) return(sqrt(as.vector(x)))

    # Square Root of a matrix:
    e <- eigen(x)
    V <- e$vectors
    ans <- V %*% diag(sqrt(e$values)) %*% t(V)

    # Return Value:
    ans
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################
# FUNCTION:
#  require(Rsolnp)
#  ### REMOVED ### require(Rnlminb2)
#  ### REMOVED ### require(Rdonlp2)
# FUNCTION:  
#  .powellTestNLP
#  .wright4TestNLP
#  .boxTestNLP 
#  .wright9TestNLP
#  .alkylationTestNLP
#  .entropyTestNLP
#  .rosensuzukiTestNLP
#  .sharperatioTestNLP
#  .rachevratioTestNLP
#  .boxmarkowitzTestNLP 
################################################################################


.powellTestNLP <-
  function()
  {
    # A function implemented by Diethelm Wuertz
    
    # FUNCTION:
    
    start <- c(-2, 2, 2, -1, -1)
    
    fun <- function(x) { exp(x[1]*x[2]*x[3]*x[4]*x[5]) }
    
    eqFun <- list(
      function(x) x[1]*x[1]+x[2]*x[2]+x[3]*x[3]+x[4]*x[4]+x[5]*x[5],
      function(x) x[2]*x[3]-5*x[4]*x[5],
      function(x) x[1]*x[1]*x[1]+x[2]*x[2]*x[2])
    eqFun.bound <- c(10, 0, -1)
    
    ## ans.donlp = donlp2NLP(start, fun, 
    ##                       eqFun = eqFun, eqFun.bound = eqFun.bound) 
    ans.solnp = solnpNLP(start, fun, 
                         eqFun = eqFun, eqFun.bound = eqFun.bound)
    ans.nlminb = nlminb2NLP(start, fun, 
                            eqFun = eqFun, eqFun.bound = eqFun.bound)
    
    result.par = round(rbind(
        ##ans.donlp$solution,
        ans.solnp$solution, ans.nlminb$solution), 3)
    result.fun = c(
        ##fun(ans.donlp$solution),
        fun(ans.solnp$solution), fun(ans.nlminb$solution))
    cbind(result.par, result.fun)   
  }


# ------------------------------------------------------------------------------


.wright4TestNLP <-
  function()
  {
    # A function implemented by Diethelm Wuertz
    
    # nlminb2 does not converge
    
    # FUNCTION:
    
    start <- c(1, 1, 1, 1, 1)
    
    fun <- function(x){
      (x[1]-1)^2+(x[1]-x[2])^2+(x[2]-x[3])^3+(x[3]-x[4])^4+(x[4]-x[5])^4}
    
    eqFun = list(
      function(x) x[1]+x[2]*x[2]+x[3]*x[3]*x[3],
      function(x) x[2]-x[3]*x[3]+x[4],
      function(x) x[1]*x[5] ) 
    eqFun.bound = c(2+3*sqrt(2), -2+2*sqrt(2), 2)
    
    ## ans.donlp = donlp2NLP(start, fun, 
    ##                       eqFun = eqFun, eqFun.bound = eqFun.bound)
    ans.solnp = solnpNLP(start, fun, 
                         eqFun = eqFun, eqFun.bound = eqFun.bound) 
    ans.nlminb = nlminb2NLP(start, fun, 
                            eqFun = eqFun, eqFun.bound = eqFun.bound)
    
    result.par = round(rbind(
        ##ans.donlp$solution,
        ans.solnp$solution, ans.nlminb$solution), 3)
    result.fun = c(
        ##fun(ans.donlp$solution),
        fun(ans.solnp$solution), fun(ans.nlminb$solution))
    cbind(result.par, result.fun)   
  }


# ------------------------------------------------------------------------------


.boxTestNLP <-
  function()
  {
    # A function implemented by Diethelm Wuertz
    
    # FUNCTION:
    
    start <- c(1.1, 1.1, 9)
    
    fun <- function(x) { -x[1]*x[2]*x[3] }
    par.lower = rep(1, 3)
    par.upper = rep(10, 3)
    
    eqFun <- list(
      function(x) 4*x[1]*x[2]+2*x[2]*x[3]+2*x[3]*x[1] )
    eqFun.bound = 100
    
    ## ans.donlp = donlp2NLP(start, fun, 
    ##                       par.lower = par.lower, par.upper = par.upper, 
    ##                       eqFun = eqFun, eqFun.bound = eqFun.bound)  
    ans.solnp = solnpNLP(start, fun, 
                         par.lower = par.lower, par.upper = par.upper, 
                         eqFun = eqFun, eqFun.bound = eqFun.bound)        
    ans.nlminb = nlminb2NLP(start, fun, 
                            par.lower = par.lower, par.upper = par.upper, 
                            eqFun = eqFun, eqFun.bound = eqFun.bound)  
    
    result.par = round(rbind(
        ##ans.donlp$solution,
        ans.solnp$solution, ans.nlminb$solution), 3)
    result.fun = c(
        ##fun(ans.donlp$solution),
        fun(ans.solnp$solution), fun(ans.nlminb$solution))
    cbind(result.par, result.fun) 
  }


# ------------------------------------------------------------------------------ 


.wright9TestNLP <-
  function()
  {
    # A function implemented by Diethelm Wuertz
    
    # FUNCTION:
    
    start <- c(1, 1, 1, 1, 1)
    
    fun <- function(x){
      10*x[1]*x[4]-6*x[3]*x[2]*x[2]+x[2]*(x[1]*x[1]*x[1])+
        9*sin(x[5]-x[3])+x[5]^4*x[4]*x[4]*x[2]*x[2]*x[2] }
    
    ineqFun <- list(
      function(x) x[1]*x[1]+x[2]*x[2]+x[3]*x[3]+x[4]*x[4]+x[5]*x[5],
      function(x) x[1]*x[1]*x[3]-x[4]*x[5],
      function(x) x[2]*x[2]*x[4]+10*x[1]*x[5]) 
    ineqFun.lower = c(-100,  -2,   5)
    ineqFun.upper = c(  20, 100, 100)
    
    ## ans.donlp = donlp2NLP(start, fun, 
    ##                       ineqFun = ineqFun, 
    ##                       ineqFun.lower = ineqFun.lower, ineqFun.upper = ineqFun.upper)  
    ans.solnp = solnpNLP(start, fun, 
                         ineqFun = ineqFun, 
                         ineqFun.lower = ineqFun.lower, ineqFun.upper = ineqFun.upper)   
    ans.nlminb = nlminb2NLP(start, fun, 
                            ineqFun = ineqFun, 
                            ineqFun.lower = ineqFun.lower, ineqFun.upper = ineqFun.upper)
    
    result.par = round(rbind(
        ##ans.donlp$solution,
        ans.solnp$solution, ans.nlminb$solution), 3)
    result.fun = c(
        ##fun(ans.donlp$solution),
        fun(ans.solnp$solution), fun(ans.nlminb$solution))
    cbind(result.par, result.fun)    
  }


# ------------------------------------------------------------------------------ 


.alkylationTestNLP <-
  function()
  {
    # A function implemented by Diethelm Wuertz
    
    # solnp does not converge
    
    # FUNCTION:
    
    start <- c(17.45, 12, 110, 30.5, 19.74, 89.2, 92.8, 8, 3.6, 145.2)
    
    fun <- function(x) { -0.63*x[4]*x[7]+50.4*x[1]+3.5*x[2]+x[3]+33.6*x[5] }
    par.lower <- c( 0,  0,   0, 10,  0, 85, 10, 3, 1, 145)
    par.upper <- c(20, 16, 120, 50, 20, 93, 95,12, 4, 162)
    
    eqFun <- list(
      function(x) 98*x[3]-0.1*x[4]*x[6]*x[9]-x[3]*x[6],
      function(x) 1000*x[2]+100*x[5]-100*x[1]*x[8],
      function(x) 122*x[4]-100*x[1]-100*x[5])
    eqFun.bound = c(0, 0, 0)
    
    ineqFun <- list(
      function(x) (1.12*x[1]+0.13167*x[1]*x[8]-0.00667*x[1]*x[8]*x[8])/x[4],
      function(x) (1.098*x[8]-0.038*x[8]*x[8]+0.325*x[6]+57.25)/x[7],
      function(x) (-0.222*x[10]+35.82)/x[9],
      function(x) (3*x[7]-133)/x[10])
    ineqFun.lower = c(  0.99,   0.99,  0.9,   0.99)
    ineqFun.upper = c(100/99, 100/99, 10/9, 100/99)
    
    ## ans.donlp = donlp2NLP(start, fun, 
    ##                       par.lower = par.lower, par.upper = par.upper,
    ##                       eqFun = eqFun, eqFun.bound = eqFun.bound,
    ##                       ineqFun = ineqFun, 
    ##                       ineqFun.lower = ineqFun.lower, ineqFun.upper = ineqFun.upper)    
    ans.solnp = solnpNLP(start, fun, 
                         par.lower = par.lower, par.upper = par.upper,
                         eqFun = eqFun, eqFun.bound = eqFun.bound,
                         ineqFun = ineqFun, 
                         ineqFun.lower = ineqFun.lower, ineqFun.upper = ineqFun.upper)
    ans.nlminb = nlminb2NLP(start, fun, 
                            par.lower = par.lower, par.upper = par.upper,
                            eqFun = eqFun, eqFun.bound = eqFun.bound,
                            ineqFun = ineqFun, 
                            ineqFun.lower = ineqFun.lower, ineqFun.upper = ineqFun.upper)
    
    result.par = round(rbind(
        ##ans.donlp$solution,
        ans.solnp$solution, ans.nlminb$solution), 3)
    result.fun = c(
        ##fun(ans.donlp$solution),
        fun(ans.solnp$solution), fun(ans.nlminb$solution))
    cbind(result.par, result.fun)     
  }


# ------------------------------------------------------------------------------ 


.entropyTestNLP <-
  function()
  {
    # A function implemented by Diethelm Wuertz
    
    # FUNCTION:
    
    set.seed(1953)
    start <- runif(10, 0, 1)
    
    fun <- function(x) {
      m = length(x)
      f = -sum(log(x))
      vnorm = sum((x-1)^2)^(1/2) 
      f - log(vnorm + 0.1) 
    }
    par.lower <- rep(0, 10)
    
    eqFun <- list(
      function(x) sum(x) )
    eqFun.bound <- 10
    
    ## ans.donlp = donlp2NLP(start, fun, 
    ##                       par.lower = par.lower,  
    ##                       eqFun = eqFun, eqFun.bound = eqFun.bound)     
    ans.solnp = solnpNLP(start, fun, 
                         par.lower = par.lower,  
                         eqFun = eqFun, eqFun.bound = eqFun.bound)    
    ans.nlminb = nlminb2NLP(start, fun, 
                            par.lower = par.lower,  
                            eqFun = eqFun, eqFun.bound = eqFun.bound)   
    
    result.par = round(rbind(
        ##ans.donlp$solution,
        ans.solnp$solution, ans.nlminb$solution), 3)
    result.fun = c(
        ##fun(ans.donlp$solution),
        fun(ans.solnp$solution), fun(ans.nlminb$solution))
    cbind(result.par, result.fun)        
  }


# ------------------------------------------------------------------------------


.rosensuzukiTestNLP <-
  function()
  {
    # A function implemented by Diethelm Wuertz
    
    # FUNCTION:
    
    start <- c(1, 1, 1, 1)
    
    fun <- function(x) 
      x[1]*x[1]+x[2]*x[2]+2*x[3]*x[3]+x[4]*x[4]-5*x[1]-5*x[2]-21*x[3]+7*x[4]
    
    ineqFun <- list(
      function(x) 8-x[1]*x[1]-x[2]*x[2]-x[3]*x[3]-x[4]*x[4]-x[1]+x[2]-x[3]+x[4],
      function(x) 10-x[1]*x[1]-2*x[2]*x[2]-x[3]*x[3]-2*x[4]*x[4]+x[1]+x[4],
      function(x) 5-2*x[1]*x[1]-x[2]*x[2]-x[3]*x[3]-2*x[1]+x[2]+x[4] )
    ineqFun.lower <- rep(   0, 3)
    ineqFun.upper <- rep(10, 3)
    
    ## ans.donlp = donlp2NLP(start, fun,  
    ##                       ineqFun = ineqFun, 
    ##                       ineqFun.lower = ineqFun.lower, ineqFun.upper = ineqFun.upper)     
    ans.solnp = solnpNLP(start, fun,  
                         ineqFun = ineqFun, 
                         ineqFun.lower = ineqFun.lower, ineqFun.upper = ineqFun.upper)     
    ans.nlminb = nlminb2NLP(start, fun,  
                            ineqFun = ineqFun, 
                            ineqFun.lower = ineqFun.lower, ineqFun.upper = ineqFun.upper)     
    
    result.par = round(rbind(
        ##ans.donlp$solution,
        ans.solnp$solution, ans.nlminb$solution), 3)
    result.fun = c(
        ##fun(ans.donlp$solution),
        fun(ans.solnp$solution), fun(ans.nlminb$solution))
    cbind(result.par, result.fun)   
  }


#-------------------------------------------------------------------------------


.sharperatioTestNLP <-
  function()
  { 
    # A function implemented by Diethelm Wuertz
    
    # FUNCTION:
    
    dataSet <- data("LPP2005REC", package="timeSeries", envir=environment())
    LPP2005REC <- get(dataSet, envir=environment())
    ret <- as.matrix(LPP2005REC[,2:7])
    
    Mean <- colMeans(ret)
    Cov <- cov(ret)
    
    start <- rep(1/6, times = 6)
    
    fun <- function(x) {
      return = (Mean %*% x)[[1]]
      risk = sqrt ( (t(x) %*% Cov %*% x)[[1]] )
      -return/risk }
    par.lower <- rep(0, 6)
    par.upper <- rep(1, 6)  
    
    eqFun <- list(
      function(x) sum(x) )
    eqFun.bound = 1
    
    ## ans.donlp <- donlp2NLP(start, fun,  
    ##                        par.lower = par.lower, par.upper = par.upper,
    ##                        eqFun = eqFun, eqFun.bound = eqFun.bound ) 
    ans.solnp <- solnpNLP(start, fun,  
                          par.lower = par.lower, par.upper = par.upper,
                          eqFun = eqFun, eqFun.bound = eqFun.bound)      
    ans.nlminb <- fPortfolio::nlminb2NLP(start, fun,  
                                       par.lower = par.lower, par.upper = par.upper,
                                       eqFun = eqFun, eqFun.bound = eqFun.bound)  
    
    result.par = round(rbind(
      ##ans.donlp$solution, 
      ans.solnp$solution, 
      ans.nlminb$solution), 3)
    result.fun = c(
      ##fun(ans.donlp$solution), 
      fun(ans.solnp$solution), 
      fun(ans.nlminb$solution))
    cbind(result.par, result.fun)     
    
    result.par <- 100*round(rbind(
                          ##ans.donlp$solution,
                          ans.solnp$solution, ans.nlminb$solution), 4)
    result.fun <- c(
        ##fun(ans.donlp$solution),
        fun(ans.solnp$solution), fun(ans.nlminb$solution))
    cbind(result.par, result.fun)  
  }


# ------------------------------------------------------------------------------


.rachevratioTestNLP <-
  function()
  {
    # A function implemented by Diethelm Wuertz
    
    # Note:
    #    sonlp2 and nlminb do not converge!
    
    # FUNCTION:
    
    dataSet <- data("LPP2005REC", package="timeSeries", envir=environment())
    LPP2005REC <- get(dataSet, envir=environment())
    ret = as.matrix(LPP2005REC[, 2:7])
    
    Mean = colMeans(ret)
    Cov = cov(ret)
    
    set.seed(1953)
    r = runif(6)
    start <- r/sum(r)
    start <- rep(1/6, times = 6)
    
    .VaR <- function(x) { 
      quantile(x, probs = 0.05, type = 1) }
    .CVaR <- function(x) {   
      VaR = .VaR(x)
      VaR - 0.5 * mean(((VaR-ret) + abs(VaR-ret))) / 0.05 }     
    fun <- function(x) {
      port = as.vector(ret %*% x)
      ans = (-.CVaR(-port) / .CVaR(port))[[1]] 
      ans}
    par.lower = rep(0, 6)
    par.upper = rep(1, 6)  
    
    eqFun <- list(
      function(x) sum(x) )
    eqFun.bound = 1
    
    ## ans.donlp = donlp2NLP(start, fun,  
    ##                       par.lower = par.lower, par.upper = par.upper,
    ##                       eqFun = eqFun, eqFun.bound = eqFun.bound) 
    ans.solnp = solnpNLP(start, fun,  
                         par.lower = par.lower, par.upper = par.upper,
                         eqFun = eqFun, eqFun.bound = eqFun.bound)   
    ans.nlminb = nlminb2NLP(start, fun,  
                            par.lower = par.lower, par.upper = par.upper,
                            eqFun = eqFun, eqFun.bound = eqFun.bound)    
    
    result.par = 100*round(rbind(
                         ##ans.donlp$solution,
                         ans.solnp$solution, ans.nlminb$solution), 4)
    result.fun = c(
        ##fun(ans.donlp$solution),
        fun(ans.solnp$solution), fun(ans.nlminb$solution))
    cbind(result.par, result.fun)  
  }


#-------------------------------------------------------------------------------


.boxmarkowitzTestNLP <-
  function()
  {
    # A function implemented by Diethelm Wuertz
    
    # FUNCTION:
    
    dataSet <- data("LPP2005REC", package="timeSeries", envir=environment())
    LPP2005REC <- get(dataSet, envir=environment())
    ret <- 100 * as.matrix(LPP2005REC[, 2:7])
    
    Mean <- colMeans(ret)
    Cov <- cov(ret)
    targetReturn <- mean(Mean)
    
    start <- rep(1/6, times = 6)
    
    objective <- function(x) {
      risk <- (t(x) %*% Cov %*% x)[[1]]
      risk }
    fun <- objective
    
    par.lower <- rep(0, 6)
    par.upper <- rep(1, 6)  
    
    eqFun <- list(
      function(x) sum(x), 
      function(x) (Mean %*% x)[[1]] )
    eqFun.bound = c(1, targetReturn)
    
    ## ans.donlp = donlp2NLP(start, objective, 
    ##                       par.lower = par.lower, par.upper = par.upper,
    ##                       eqFun = eqFun, eqFun.bound = eqFun.bound)
    ans.solnp = solnpNLP(start, objective, 
                         par.lower = par.lower, par.upper = par.upper,
                         eqFun = eqFun, eqFun.bound = eqFun.bound)
    ans.nlminb = nlminb2NLP(start, objective, 
                            par.lower = par.lower, par.upper = par.upper,
                            eqFun = eqFun, eqFun.bound = eqFun.bound)
    
    result.par <- 100*round(rbind(
                          ##ans.donlp$solution,
                          ans.solnp$solution, ans.nlminb$solution), 4)
    result.fun <- c(
        ##fun(ans.donlp$solution),
        fun(ans.solnp$solution), fun(ans.nlminb$solution))
    cbind(result.par, result.fun)  
  }


#-------------------------------------------------------------------------------


.groupmarkowitzTestNLP <-
  function()
  {
    # A function implemented by Diethelm Wuertz
    
    # FUNCTION:
    
    dataSet <- data("LPP2005REC", package="timeSeries", envir=environment())
    LPP2005REC <- get(dataSet, envir=environment())
    ret <- 100 * as.matrix(LPP2005REC[, 2:7])
    
    Mean <- colMeans(ret)
    Cov <- cov(ret)
    targetReturn <- mean(Mean)
    
    start <- rep(1/6, times = 6)
    start[1]+start[4]
    start[2]+start[5]+start[6]
    
    # Must be feasible for nlminb !!!
    start2 <- c(0, 0, 0.3, 0.3, 0, 0.4)
    start2 <- start/sum(start2)
    sum(start2)
    start2[1]+start2[4]
    start2[2]+start2[5]+start2[6]
    
    fun <- function(x) {
      risk = (t(x) %*% Cov %*% x)[[1]]
      risk }
    par.lower <- rep(0, 6)
    par.upper <- rep(1, 6)  
    
    eqFun <- list(
      function(x) sum(x), 
      function(x) (Mean %*% x)[[1]] )
    eqFun.bound = c(1, targetReturn)
    
    ineqFun <- list(
      function(x) x[1]+x[4],
      function(x) x[2]+x[5]+x[6])
    ineqFun.lower <- c( 0.3, 0.0)
    ineqFun.upper <- c( 1.0, 0.6)
    
    ## ans.donlp = donlp2NLP(start, fun, 
    ##                       par.lower = par.lower, par.upper = par.upper,
    ##                       eqFun = eqFun, eqFun.bound = eqFun.bound,
    ##                       ineqFun = ineqFun, 
    ##                       ineqFun.lower = ineqFun.lower, ineqFun.upper = ineqFun.upper)
    ans.solnp = solnpNLP(start, fun, 
                         par.lower = par.lower, par.upper = par.upper,
                         eqFun = eqFun, eqFun.bound = eqFun.bound,
                         ineqFun = ineqFun, 
                         ineqFun.lower = ineqFun.lower, ineqFun.upper = ineqFun.upper)
    ans.nlminb = nlminb2NLP(start, fun, 
                            par.lower = par.lower, par.upper = par.upper,
                            eqFun = eqFun, eqFun.bound = eqFun.bound,
                            ineqFun = ineqFun, 
                            ineqFun.lower = ineqFun.lower, ineqFun.upper = ineqFun.upper)
    
    result.par = 100*round(rbind(
                         ##ans.donlp$solution,
                         ans.solnp$solution, ans.nlminb$solution), 4)
    result.fun = c(
        ##fun(ans.donlp$solution),
        fun(ans.solnp$solution), fun(ans.nlminb$solution))
    cbind(result.par, result.fun) 
  }


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


###############################################################################
# FUNCTION:                    DESCRIPTION:
#  rquadprogQP                  Rmetrics Interface for QUADPROG QP solvers 
#  quadprogQP                   Convenience wrapper for QUADPROG QP solvers
#  quadprogQPControl            QUADPROG QP control parameter list 
#  rquadprog                    Synonyme name for quadprog::solveLP function        
###############################################################################


rquadprogQP <- 
    function(objective, lower=0, upper=1, linCons, control=list())
{   
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Implements Goldberg-Idnani Algorithm
    
    # Arguments:
    #   objective - list(dvec=NULL, Dmat=NULL)
    #   lower - lower box constraints
    #   upper - upper box constraints
    #   linCons - linear constraints, list with entries:
    #       mat, lower, upper.
    #   control - control list
    
    # FUNCTION:
    
    # Control List:
    ctrl <- quadprogQPControl()
    if (length(control) > 0)
        for (name in names(control)) ctrl[name] = control[name]
    control <- ctrl
    
    # General Settings:
    dvec <- objective$dvec
    Dmat <- objective$Dmat
    Names <- colnames(rbind(dvec, Dmat))
    N <- ncol(rbind(dvec, Dmat))
    
    # Box Constraints:
    if(length(lower) == 1) {
        par.lower <- rep(lower, N)
    } else {
        par.lower <- lower
    }
    if(length(upper) == 1) {
        par.upper <- rep(upper, N)
    } else {
        par.upper <- upper
    }
      
    # Linear Constraints:  
    if(missing(linCons)) {
        eqA <- ineqA <- NULL
        eqA.bound <- ineqA.lower <- ineqA.upper <- NULL
    } else {
        mat <- linCons[[1]]
        M <- nrow(mat)
        lower <- linCons[[2]]
        upper <- linCons[[3]]
        if(length(lower) == 1) lower <- rep(lower, M)
        if(length(upper) == 1) upper <- rep(upper, M)
        
        eqIndex <- which(lower == upper)
        ineqIndex <- which(lower != upper)
        
        if (length(eqIndex) == 0) {
            eqA <- NULL
            eqA.bound <- NULL
        } else {
            eqA <- mat[eqIndex, ]
            eqA.bound <- lower[eqIndex]
        }
        if (length(ineqIndex) == 0) {
            ineqA <- NULL
            ineqA.lower <- NULL
            ineqA.upper <- NULL
        } else {
            ineqA <- mat[ineqIndex, ]
            ineqA.lower <- lower[ineqIndex]
            ineqA.upper <- upper[ineqIndex]
        }
    }   
    
    # Optimize Portfolio:
    optim <- quadprogQP(
        objective, 
        par.lower, par.upper, 
        eqA, eqA.bound,  
        ineqA, ineqA.lower, ineqA.upper,
        control)
    
    # Return Value:
    value <- list(
        opt = optim, 
        solution = optim$solution, 
        objective = optim$objective,
        status = optim$status,
        message = optim$message,
        solver = "quadprog",
        version = optim$version)
    class(value) = c("solver", "list")
    value
}


###############################################################################


quadprogQP <- 
    function(
        objective=list(dvec=NULL, Dmat=NULL), 
        par.lower=NULL, par.upper=NULL, 
        eqA=NULL, eqA.bound=NULL,  
        ineqA=NULL, ineqA.lower=NULL, ineqA.upper=NULL,
        control=list())
{   
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Implements Goldberg-Idnani Algorithm
    
    # FUNCTION:
    
    # Control List:
    ctrl <- quadprogQPControl()
    if (length(control) > 0)
        for (name in names(control)) ctrl[name] = control[name]
    control <- ctrl
    
    # General Settings:
    dvec <- -objective$dvec
    Dmat <- objective$Dmat
    Names <- colnames(rbind(dvec, Dmat))
    N <- ncol(rbind(dvec, Dmat))
    
    # Box Constraints:
    if (length(par.lower) == 1) par.lower <- rep(par.lower, N)
    if (length(par.upper) == 1) par.upper <- rep(par.upper, N)

    # Constraints Settings:
    Amat <- eqA
    if (!is.null(ineqA)) Amat <- rbind(eqA, ineqA, -ineqA)
    Amat <- rbind(Amat, diag(N), -diag(N))  
    Amat <- t(Amat) 
    bvec <- eqA.bound
    if (!is.null(ineqA.lower)) bvec <- c(bvec, ineqA.lower)
    if (!is.null(ineqA.upper)) bvec <- c(bvec, -ineqA.upper)
    bvec <-  c(bvec, par.lower)
    if (!is.null(par.upper)) bvec <- c(bvec, -par.upper)
    if (is.null(eqA)) meq <- 0 else meq <- nrow(eqA)
    Amat <- Amat[, is.finite(bvec)]
    bvec <- bvec[is.finite(bvec)]
    
    # Optimize:
    elapsed <- Sys.time()
    optim <- quadprog::solve.QP(
        Dmat = Dmat, 
        dvec = dvec, 
        Amat = Amat, 
        bvec = bvec, 
        meq = meq)
    elapsed <- Sys.time() - elapsed
        
    # Note:
    #   DW: if quadprog::solve.QP fails with non-zero status optim$ierr=
    #   =1: stop("constraints are inconsistent, no solution!")
    #   =2: stop("matrix D in quadratic function is not positive definite!")
    #   this is ugly!
    #   Thus:
    Status <- 0
    Message <- "solution found"
    
    # Add:  
    names(optim$solution) <- Names
    
    # Version:
    package <- packageDescription(pkg="quadprog")
    version <- paste(package$Package, package$Version, package$Date)
        
    # Return Value:
    value <- list(
        opt = optim, 
        solution = optim$solution, 
        objective = optim$value,
        status = Status,
        message = Message,
        solver = "quadprog",
        elapsed <- elapsed,
        version = version)
    class(value) = c("solver", "list")
    value
}


###############################################################################


rquadprog <- quadprog::solve.QP


###############################################################################


quadprogQPControl <-
    function(solver="quadprog", trace=FALSE)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns control parameter list
    
    # Arguments:
    #   trace - al logical flag, should the function be traced?
    
    # Details:
    #   Note there are no control paramters supported in 
    #   quadprog::solve.QP
    
    # FUNCTION:
    
    # Control Parameter:
    control <- list(trace = trace)
    
    # Return Value:
    control
}


###############################################################################

################################################################################
# FUNCTION:                         DESCRIPTION:
#  rollingWindows                    Returns a list of rolling window frames
# FUNCTION:                         DESCRIPTION:
#  rollingCmlPortfolio               Rolls a CML portfolio
#  rollingTangencyPortfolio          Rolls a tangency portfolio
#  rollingMinvariancePortfolio       Rolls a minimum risk portfolio
# FUNCTION:                         DESCRIPTION:
#  rollingPortfolioFrontier          Rolls a portfolio frontier
################################################################################


rollingWindows <-
    function(x, period = "12m", by = "1m")
{
    # A function implemented by Diethelm Wuertz and Yohan Chalabi

    # Description:
    #   Returns vectors of start and end dates for a rolling time series

    # Arguments:
    #   x - a timeSeries object of asset returns
    #   period - a character string denoting the length of the rolling
    #       window, e.g. "24m" means 24 months
    #   by - a character string denoting the shift of the rolling window,
    #       e.g. "1m" means one month

    # Note:
    #   Only "monthly" frequencies are currently supported.

    # Example:
    #   x = sort(as.timeSeries(data(smallcap.ts))); rollingWindows(x)

    # FUNCTION:

    # Get Window Parameter:
    periodLength = as.numeric(substr(period, 1, nchar(period)-1))
    periodUnit = substr(period, nchar(period), nchar(period))
    byLength = as.numeric(substr(by, 1, nchar(by)-1))
    byUnit = substr(by, nchar(by), nchar(by))
    stopifnot(periodUnit == "m")
    stopifnot(byUnit == "m")

    # Make Windows - expand series x to a monthly series:
    positions = time(x)
    startPositions = unique(timeFirstDayInMonth(positions))
    # for non monthly data
    # series(startPositions)[1] <- as.vector(start(x))
    endPositions = unique(timeLastDayInMonth(positions))
    # for non monthly data
    # series(endPositions)[length(endPositions)] <- as.vector(end(x))
    numberOfPositions = length(startPositions)
    startSeq <- seq(
        from = 1,
        to = (numberOfPositions-periodLength + 1),
        by = byLength)
    startDates = startPositions[startSeq]
    endSeq <- seq(from = periodLength,
        to = numberOfPositions,
        by = byLength)
    endDates = endPositions[endSeq]

    # Windows:
    windows = list(
        from = startDates, 
        to = endDates)
    attr(windows, "control") = list(
        start = start(positions), 
        end = end(positions),
        period = period,
        by = by)

    # Return Value:
    windows
}


# ------------------------------------------------------------------------------


rollingCmlPortfolio <-
    function(data, spec, constraints, from, to, action = NULL,
    title = NULL, description = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Computes EF on a rolling timeSeries Windows

    # Arguments:

    # FUNCTION:

    # Roll the Frontier and return it in a list:
    roll = list()
    for (i in 1:length(from)) {

        # Data must be a multivariate timeSeries object ...
        series = cut(data, from = from[i], to = to[i])

        # Calculation efficient frontiers and save them all in a list:
        portfolio = tangencyPortfolio(data = series, spec, constraints)
        roll[[i]] = portfolio

        # Now you can do any "action" you want to do with the EFs:
        if (!is.null(action)) {
            fun = match.fun(action)
            fun(roll, from, to, ...)
        }
    }

    # Return Value:
    invisible(roll)
}


# ------------------------------------------------------------------------------


rollingTangencyPortfolio <-
    function(data, spec, constraints, from, to, action = NULL,
    title = NULL, description = NULL, ...)
{   
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Computes EF on a rolling timeSeries Windows

    # Arguments:
    #   windows - a list with two named 'timeDate' entries, "from" and
    #       "to", defining the start and end dates of your windows.
    #   ... - optional parameters which can be directed to the optional
    #       function action().

    # FUNCTION:

    # Roll the Frontier and return it in a list:
    roll = list()
    for (i in 1:length(from)) {

        # Data must be a multivariate timeSeries object ...
        series = cut(data, from = from[i], to = to[i])

        # Calculation efficient frontiers and save them all in a list:
        portfolio = tangencyPortfolio(data = series, spec, constraints)
        roll[i] = portfolio

        # Now you can do any "action" you want to do with the EFs:
        if (!is.null(action)) {
            fun = match.fun(action)
            fun(roll, from, to, ...)
        }
    }

    # Return Value:
    invisible(roll)
}


# ------------------------------------------------------------------------------


rollingMinvariancePortfolio <-
    function(data, spec, constraints, from, to, action = NULL,
    title = NULL, description = NULL, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Computes EF on a rolling timeSeries Windows

    # Arguments:
    #   windows - a list with two named 'timeDate' entries, "from" and
    #       "to", defining the start and end dates of your windows.
    #   ... - optional parameters which can be directed to the optional
    #       function action().

    # FUNCTION:

    # Roll the Frontier and return it in a list:
    roll = list()
    for (i in 1:length(from)) {

        # Data must be a multivariate timeSeries object ...
        series = cut(data, from = from[i], to = to[i])

        # Calculation efficient frontiers and save them all in a list:
        portfolio = minvariancePortfolio(data = series, spec, constraints)
        roll[i] = portfolio

        # Now you can do any "action" you want to do with the EFs:
        if (!is.null(action)) {
            fun = match.fun(action)
            fun(roll, from, to, ...)
        }
    }

    # Return Value:
    invisible(roll)
}


################################################################################


rollingPortfolioFrontier <-
    function(data, spec, constraints, from, to, action = NULL,
    title = NULL, description = NULL, ...)
{   
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Computes EF on a rolling timeSeries Windows

    # Arguments:
    #   windows - a list with two named 'timeDate' entries, "from" and
    #       "to", defining the start and end dates of your windows.
    #   ... - optional parameters which can be directed to the optional
    #       function action().

    # FUNCTION:

    # Roll the Frontier and return it in a list:
    roll = list()
    for (i in 1:length(from)) {

        # Data must be a multivariate timeSeries object ...
        series = cut(data, from = from[i], to = to[i])

        # Calculation efficient frontiers and save them all in a list:
        frontier = portfolioFrontier(data = series, spec, constraints,
            title = title, description = description)
        roll[i] = frontier

        # Now you can do any "action" you want to do with the EFs:
        if (!is.null(action)) {
            fun = match.fun(action)
            fun(roll, from, to, ...)
        }
    }

    # Return Value:
    invisible(roll)
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                DESCRIPTION:
#  solveRdonlp2             Portfolio interface to solver Rdonlp2
#  .rdonlp2Arguments        Returns arguments for solver
################################################################################


solveRdonlp2 <-
  function(data, spec, constraints)
  {
    # Description:
    #   Portfolio interface to solver Rdonlp2
    
    # Arguments;
    #   data - an object of class timeSeries
    #   spec - an object of class fPFOLIOSPEC
    #   constraints - an object of class character
    
    # Example:
    #   
    
    # FUNCTION:   
    
    # Settings:
    Data <- portfolioData(data, spec)
    nAssets <- getNAssets(Data)
    mu <- getMu(Data)
    Sigma <- getSigma(Data)
    
    # Compose Arguments for Solver:
    args <- .rdonlp2Arguments(data, spec, constraints)
    
    # Solve Multiassets Portfolio:
    ans <- Rdonlp2::donlp2(
      par = args$par,
      fn = args$fn,
      par.lower = args$par.lower, 
      par.upper = args$par.upper, 
      A = args$A, 
      lin.lower = args$lin.lower, 
      lin.upper = args$lin.upper,
      nlin = args$nlin, 
      nlin.lower = args$nlin.lower, 
      nlin.upper = args$nlin.upper,
      control = Rdonlp2::donlp2Control(),
      control.fun = function(lst) {return(TRUE)}, 
      env = .GlobalEnv, 
      name = NULL)      
    returnFun <- match.fun(getObjective(spec)[2])
    ans$targetReturn <- returnFun(ans$par)
    riskFun <- match.fun(getObjective(spec)[3])
    ans$targetRisk <- riskFun(ans$par)
    ans$ans <- ans
    
    ans$solver <- "solveRdonlp2"
    ans$objective <- args$fn(ans$par)
    names(ans$par) <- names(start)
    ans$weights <- .checkWeights(ans$par)
    ans$solution <- ans$weights
    attr(ans$weights, "invest") <- sum(ans$weights)
    ans$status <- 1
    message11 <- "KT-conditions satisfied, "
    message12 <- "computed correction small"
    message13 <- "stepsizeselection: x almo"
    if (substr(ans$message, 1, 25) == message11) 
      ans$status <- 0
    if (substr(ans$message, 1, 25) == message12) 
      ans$status <- 0
    if (substr(ans$message, 1, 25) == message13) 
      ans$status <- 0
    
    # Return Value:
    class(ans) <- c("solver", "list")
    ans
  }


# ------------------------------------------------------------------------------


.rdonlp2Arguments <-
  function(data, spec, constraints)
  {
    # Description:
    #   Create Arguments for Rdonlp2
    
    # Details:
    #       min:                    fn(x)
    #       subject to: 
    #                     par.lower <= x <= par.upper
    #                  lin.lower <= A %*% x <= lin.upper
    #                 nlin.lower <= nlin(x) <= nlin.upper  
    
    # FUNCTION:  
    
    DEBUG = FALSE
    
    # Settings:
    Data <- portfolioData(data)
    nAssets <- getNAssets(Data)  
    mu <- getMu(Data)
    Sigma <- getSigma(Data)
    fn <- match.fun(getObjective(spec)[1])
    
    # Box Constrains:
    par.lower <- minWConstraints(data, spec, constraints)
    par.upper <- maxWConstraints(data, spec, constraints)
    if(DEBUG) print(rbind(par.lower, par.upper))
    
    # Linear / Group Constraints:
    # ... targetReturn may be not defined,then set it to NA
    if (is.null(getTargetReturn(spec))) setTargetReturn(spec) <- NA
    # ... has in the first line the return constraint, if NA then ignore  it
    eqsumW <- eqsumWConstraints(data, spec, constraints)
    if (is.na(eqsumW[1, 1])) eqsumW = eqsumW[-1, , drop= FALSE]
    Aeqsum <- eqsumW[, -1]
    aeqsum <- eqsumW[, 1]
    minsumW <- minsumWConstraints(data, spec, constraints)
    if (is.null(minsumW)) {
      Aminsum <- aminsum <- NULL
    } else {
      Aminsum <- minsumW[, -1]
      aminsum <- minsumW[, 1]
    }      
    maxsumW <- maxsumWConstraints(data, spec, constraints)
    if (is.null(maxsumW)) {
      Amaxsum <- amaxsum <- NULL
    } else {
      Amaxsum <- maxsumW[, -1]
      amaxsum <- maxsumW[, 1]
    }      
    A <- rbind(Aeqsum, Aminsum, Amaxsum)
    lin.lower <- c(aeqsum, aminsum, rep(-Inf, length(amaxsum)))
    lin.upper <- c(aeqsum, rep(Inf, length(aminsum)), amaxsum)
    if(DEBUG) print(cbind(lin.lower, A, lin.upper))
    
    # Nonlinear Constraints - Here Covariance Risk Budgets:
    nlin <- list()
    nlin.lower <- NULL
    nlin.upper <- NULL
    # Check Constraints Strings for Risk Budgets:
    # Example: constraints = c("minB[2:3]=0.1", "maxB[3:5]=0.9")
    validStrings <- c("minB", "maxB")
    usedStrings <- unique(sort(sub("\\[.*", "", constraints)))
    checkStrings <- sum(usedStrings %in% validStrings)
    includeRiskBudgeting <- as.logical(checkStrings)
    if (DEBUG) print(includeRiskBudgeting)
    if (includeRiskBudgeting) {
      # Compose Non-Linear (Cov Risk Budget) Constraints Functions:
      nlcon <- function(x) {
        B1 <- as.vector(x %*% Sigma %*% x)
        B2 <- as.vector(x * Sigma %*% x)
        B <- B2/B1
        B
      }
      if(DEBUG) print(nlcon)
      # Compose non-linear functions now for each asset ...
      for (I in 1:nAssets)
        eval( parse(text = paste(
          "nlcon", I, " = function(x) { nlcon(x)[", I, "] }", sep = "")) )
      nlinFunctions <- paste("nlcon", 1:nAssets, sep = "", collapse = ",")
      nlinFunctions <- paste("list(", nlinFunctions, ")")
      nlin <- eval( parse(text = nlinFunctions) )
      if(DEBUG) print(nlin)
      # ... and finally Compose Constraints Vectors:
      nlin.lower <- minBConstraints(data, spec, constraints)
      nlin.upper <- maxBConstraints(data, spec, constraints)
      if(DEBUG) print(rbind(nlin.lower, nlin.upper))
    }
    
    # General non-lin Portfolio Constraints:
    # ... todo: currently overwrites previous selection
    nlin <- listFConstraints(data, spec, constraints)
    if(DEBUG) print(nlin)
    nlin.lower <- minFConstraints(data, spec, constraints)
    nlin.upper <- maxFConstraints(data, spec, constraints)
    if(DEBUG) print(cbind(nlin.lower, nlin.upper))
    
    # Return Value:
    list(
      par = rep(1/nAssets, nAssets), fn = fn,
      par.lower = par.lower, par.upper = par.upper, 
      A = A, lin.lower = lin.lower, lin.upper = lin.upper,
      nlin = nlin, nlin.lower = nlin.lower, nlin.upper = nlin.upper)
  }


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


############################################################################### 
# FUNCTION:              DESCRIPTION:   
#  .emaIndicator           Exponential moving average indicator
#  .macdIndicator          MACD Indicator
#  .drawdownsIndicator     Maximum drawdowns Indicator
# FUNCTION:              DESCRIPTION:   
#  .rebalancingStats       Rebalancing statistics
############################################################################### 


.emaIndicator <- 
    function(series, lambda)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #    Exponential moving average indicator    
    
    # FUNCTION:
    
    # EMA:
    x <- rep(mean(series[1:10,]), times=nrow(series))
    for (i in 2:nrow(series))
        x[i] <- (1-lambda)*series[i] + lambda*x[i-1]
    x <- as.timeSeries(data=x, charvec=time(series), units=colnames(series))
    
    # Return Value:
    x
}


# -----------------------------------------------------------------------------


.macdIndicator <- 
    function(index, spar=0.5, lambda=c(0.80, 0.85, 0.90), 
        trace = TRUE, doplot=TRUE)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   MACD price/index indicator
    
    # FUNCTION:
    
    # Series:
    rets <- returns(index)
    Index <- log(index)[-1, ]
    
    # Turning Points:
    tps <- turnsAnalytics(index=index, spar=spar, 
        main = "MACD Analytics", trace=TRUE, doplot=FALSE)
    ablines <- tps$ablines
    
    # MACD Analytics:
    ema1 <- .emaIndicator(Index, lambda=lambda[1])
    ema2 <- .emaIndicator(Index, lambda=lambda[2])
    macd <- ema1 - ema2
    signal <- .emaIndicator(macd, lambda[3])
    histogram <- macd - signal
    
    # Indicator:
    indicator <- sign(histogram)
    rebalancing <- .rebalancingStats(index, indicator, trace=trace)
      
    # Plot Turning Points:
    if(doplot) {
        turnsAnalytics(index=index, spar=spar, 
            main="MACD Index Indicator", 
            trace=FALSE, doplot=doplot)
        tradePositions <- as.vector(indicator)
        tradeForecasts <- c(0, tradePositions[-length(tradePositions)])
        outSample <- Index[1] + log(cumulated(rets*tradeForecasts))
        Ups <- Index[as.vector(indicator) == 1, ]
        if(nrow(Ups) > 0) points(Ups, pch=19, cex=0.33, col="green")   
        Downs <- Index[as.vector(indicator) == 0, ]
        if(nrow(Downs) > 0) points(Downs, pch=19, cex=0.33, col="blue")   
        lines(outSample, col="magenta")
        box(col="white")
        box(bty="l")
    }
    
    # Plot Indicator:
    if(doplot) {
        plot(macd, col="green", ylab=paste("MACD", colnames(index)))
        abline(v=ablines, lty=3, lwd=2, col="grey")
        lines(histogram, type="h", col="black")
        lines(macd, col="red")
        mtext(paste("lambda: ", lambda[1], lambda[2], lambda[2], sep=" "), 
            adj=0, side=4, cex=0.7, col="darkgrey")    
        box(col="white")
        box(bty="l") 
    }
    
    # Return Value:
    invisible(list(index=index, macd=macd, histogram=histogram, 
        rebalancing=rebalancing))
}


# -----------------------------------------------------------------------------


.drawdownsIndicator <- 
    function(index, spar=0.5, lambda=c(0.80, 0.85, 0.10),  
        trace=TRUE, doplot=TRUE)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Drawdown analytics
    
    # Arguments:
    #   index - an index or price S4 'timeSeries' object
    
    # FUNCTION:
    
    # Series:
    rets <- returns(index)
    Index <- log(index)[-1, ]
    
    # Turning Points:
    tps <- turnsAnalytics(index=index, spar=spar, 
        main = "MACD Drawdown Analytics", trace=TRUE, doplot=FALSE)
    ablines <- tps$ablines
    
    # Returns and Drawdowns:
    dd <- drawdowns(rets)
       
    # Long/Short Drawdowns EMA:
    mdd1 <- .emaIndicator(dd, lambda[1])
    mdd2 <- .emaIndicator(dd, lambda[2])
        
    # MACD/Signal/Histogram Line:
    macd <- mdd1 - mdd2
    signal <- .emaIndicator(macd, lambda[3])
    histogram <- macd - signal
        
    # Indicator:
    indicator <- rets
    series(indicator) <- 1-sign(as.integer(macd < 0 & histogram < 0 )) 
    rebalancing <- .rebalancingStats(index, indicator, trace=trace)
    
    # Plot Turning Points:
    if(doplot) {
        tps <- turnsAnalytics(index=index, spar=spar, 
            main="MACD Drawdown Indicator", trace=FALSE, doplot=doplot)
        tradePositions <- as.vector(indicator)
        tradeForecasts <- c(0, tradePositions[-length(tradePositions)])
        outSample <- Index[1] + log(cumulated(rets*tradeForecasts))
        Ups <- Index[as.vector(indicator) == 1,]
        if(nrow(Ups) > 0) points(Ups, pch=19, cex=0.33, col="green")   
        Downs <- Index[as.vector(indicator) == 0, ]
        if(nrow(Downs) > 0) points(Downs, pch=19, cex=0.33, col="blue")   
        lines(outSample, col="magenta")
        box(col="white")
        box(bty="l")
    }
       
    # Plot Indicator:
    if(doplot) {
        plot(mdd1, ylim=c(min(dd), max(macd)), ylab=colnames(index))
        positions <- tps$positions
        ablines <- tps$ablines
        abline(v=ablines, lty=1, lwd=2, col="lightgrey")    
        Time <- time(indicator)
        Time <- Time[!as.logical(indicator)] 
        abline(v=Time, lty=3, lwd=2, col="steelblue")         
        lines(mdd1, col="black")
        lines(mdd2, col="red")
        lines(max(abs(macd))*histogram/max(abs(histogram)), type="h", col="orange")      
        lines(max(abs(macd))*histogram/max(abs(histogram)), type="l", col="orange")      
        abline(h=0, col="grey") 
        mtext(paste("lambda: ", lambda[1], lambda[2], lambda[2], sep=" "), 
            adj=0, side=4, cex=0.7, col="darkgrey")            
        box(col="white")
        box(bty="l")
    }
    
    # Return Value:
    invisible(list(indicator=indicator, index=index, returns=rets, 
        drawdowns=dd, macd=macd, signal=signal, histogram=histogram, 
        rebalancing=rebalancing))
}


###############################################################################    


.rebalancingStats <- 
    function(index, indicator, trace=TRUE)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #    Simple rebalancing statistics
    
    # Arguments:
    #   index - an index or price S4 'timeSeries' object
    
    # FUNCTION:
    
    # Returns:
    rets <- returns(index)
    
    # Rebalancing:
    tradePositions <- as.vector(indicator)
    tradeForecasts <- c(0, tradePositions[-length(tradePositions)])
    rebalancing <- c(
        max=sum(abs(rets)),
        insample=sum(rets*tradePositions),
        forecasts=sum(rets*tradeForecasts),
        rets=sum(rets))      
        
    # Trace:
    if (trace) {
        cat("Rebalancing:\n")
        print(rebalancing)
    }
    
    # Return Value:
    invisible(rebalancing)
}


############################################################################### 

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA
    

####################################################################################################
# FUNCTION:
####################################################################################################
                                                  
                     
# Arguments:          
#                       start objective lower upper linCons funCons amplCons             control ...
#                                          =0    =1                                      =list() 
#                               class()              list()  list()   list()              list()                   
#                                                                                                
# LP Solver                                                                                      
# glpkLP      default       .   numeric     x     x    yes       .        .       glpkLPControl    .
# symphonyLP        .       .   numeric     x     x    yes       .        .   symphonyLPControl    .
# amplLP            .       .   numeric     x     x    yes       .        .       amplLPControl    .
#                                                                                                
# QP Solver                                                                                      
# quadprogQP  default       .      list     x     x    yes       .        .   quadprogQPControl    .
# ipoptQP           .       .      list     x     x    yes       .        .      ipoptQPControl    .
# amplQP            .       .      list     x     x    yes       .        .       amplQPControl    .
#                                                                                                                                                        
# NLP Solver                                                                                     
# solnpNLP    default     yes  function     x     x    yes     yes        .     solnpNLPControl    .
# nlminb2NLP        .     yes  function     x     x    yes     yes        .   nlminb2NLPControl    .        
# amplNLP           .  length character     x     x      .       .      yes      amplNLPControl  yes      
# donlp2NLP   removed     yes  function     x     x    yes     yes        .    donlp2NLPControl    .
#

# AMPL Solver tested: ipopt, donlp2, mosek, ...


####################################################################################################


# Value:
#   
#   value <- list(
#       call, control, dots, 
#       opt,  
#       solution, objective, status, message, solver,
#   class(value) <- c("solver", "list")


####################################################################################################


# Methods:

# print.solver <- function(x, ...) x
# summary.solver <- function(x, ...) x
  
# getSolution <- function(x) x$solution
# getObjective <- function(x) x$objective
# getStatus <- function(x) x$status
# getMessage <- function(x) x$message

  
####################################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                    DESCRIPTION:
#  solveRglpk.CVAR              Portfolio interface to solver Rglpk
#  .cvarRglpkArguments          Returns CVAR arguments for solver
# FUNCTION:                    DESCRIPTION:
#  .rglpk.CVAR                  Wrapper to solver function
################################################################################


solveRglpk.CVAR <-
    function(data, spec, constraints)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Portfolio interface to solver Rglpk

    # FUNCTION:

    # Settings:
    Data <- portfolioData(data, spec)
    data <- getSeries(Data)
    nAssets <- getNAssets(Data)
    type <- getType(spec)

    # Compile Arguments for Solver:
    args <- .cvarRglpkArguments(Data, spec, constraints)

    # Solve Multiassets Portfolio:
    ans <- .rglpk.CVAR(
        obj = args$obj,
        mat = args$mat,
        dir = args$dir,
        rhs = args$rhs,
        types = args$types,
        max = args$max,
        bounds = args$bounds,
        verbose = args$verbose,
        nScenarios = args$nScenarios,
        nAssets = args$nAssets,
        targetReturn = args$targetReturn,
        Alpha = args$Alpha,
        Type = args$Type)
    ans$solver <- "solveRglpk.CVAR"

    # Return Value:
    class(ans) = c("solver", "list")
    ans
}


################################################################################


.cvarRglpkArguments <-
    function(data, spec, constraints)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns glpk conform CVaR arguments for the solver

    # Details:
    #       max/min:                   obj %*% x
    #       subject to:
    #                            mat %*% x ?=  rhs
    #                                  dir  =  "?="
    #                               upper/lower bounds
    #
    #   .Rglpk_solve_LP(obj, mat, dir, rhs, types = NULL, max = FALSE,
    #       bounds = NULL, verbose = FALSE)

    # FUNCTION:

    # Settings:
    Data <- portfolioData(data, spec)
    data <- getSeries(Data)
    nAssets <- ncol(data)
    nScenarios <- nrow(data)
    targetReturn <- getTargetReturn(spec)
    Alpha <- getAlpha(spec)
    Type <- getType(spec)

    # Objective Function:
    objNames <- c("VaR", paste("e", 1:nScenarios, sep = ""), colnames(data))
    obj <- c(1, -rep(1/(Alpha*nScenarios), nScenarios), rep(0, nAssets))
    names(obj) <- objNames

    # The A_equal Equation Constraints: A_eq %*% x == a_eq
    eqsumW <- eqsumWConstraints(Data, spec, constraints)
    Aeq <- cbind(
        matrix(0, ncol=1+nScenarios, nrow=nrow(eqsumW)), 
        matrix(eqsumW[, -1], ncol=nAssets) )   
    aeq <- eqsumW[, 1]
    deq <- rep("==", nrow(eqsumW))

    # The VaR Equation Constraints:
    #   (-1 + diag + Returns) %*% (VaR, es, W)  >= 0
    Avar <- cbind(
        matrix(rep(-1, nScenarios), ncol = 1),
        diag(nScenarios),
        getDataPart(getSeries(Data)) )
    avar <- rep(0, nrow(Avar))
    dvar <- rep(">=", nrow(Avar))

    # The e_s > = 0 Equation Constraints:
    Aes <- cbind(
        matrix(rep(0, nScenarios), ncol = 1),
        diag(nScenarios),
        matrix(0, nrow = nScenarios, ncol = nAssets) )
    aes <- rep(0, nrow(Aes))
    des <- rep(">=", nrow(Aes))

    # Group Constraints: A W >= a
    minsumW <- minsumWConstraints(Data, spec, constraints)
    if (is.null(minsumW)){
        Aminsum <- aminsum <- dminsum <- NULL
    } else {
        Aminsum <- cbind(
            matrix(0, nrow = nrow(minsumW), ncol = 1+nScenarios),
            minsumW[, -1, drop = FALSE] )
        aminsum <- minsumW[, 1]
        dminsum <- rep(">=", nrow(minsumW))
    }

    # Group Constraints: A W <= b
    maxsumW <- maxsumWConstraints(Data, spec, constraints)
    if (is.null(maxsumW)){
        Amaxsum <- amaxsum <- dmaxsum <- NULL
    } else {
        Amaxsum <- cbind(
            matrix(0, nrow = nrow(maxsumW), ncol = 1+nScenarios),
            maxsumW[, -1, drop = FALSE] )
        amaxsum <- maxsumW[, 1]
        dmaxsum <- rep("<=", nrow(maxsumW))
    }

    # Putting all together:
    mat <- rbind(Aeq, Avar, Aes, Aminsum, Amaxsum)
    rhs <- c(aeq, avar, aes, aminsum, amaxsum)
    dir <- c(deq, dvar, des, dminsum, dmaxsum)

    # Box Constraints: Upper and Lower Bounds as listn required ...
    minW <- minWConstraints(Data, spec, constraints)
    maxW <- maxWConstraints(Data, spec, constraints)
    nInd <- 1:(1+nScenarios+nAssets)
    bounds <- list(
        lower = list(ind = nInd, val = c(rep(-Inf, 1+nScenarios), minW)),
        upper = list(ind = nInd, val = c(rep( Inf, 1+nScenarios), maxW)) )

    # What variable Types, All Continuous:
    types <- NULL

    # Should I minimize or maximize ?
    max <- TRUE

    # Return Value:
    list(
        obj = obj, mat = mat, dir = dir, rhs = rhs,
        types = types, max = max, bounds = bounds, verbose = FALSE,
        nScenarios = nScenarios, nAssets = nAssets,
        targetReturn = targetReturn, Alpha = Alpha, Type = Type)
}


################################################################################


.rglpk.CVAR <-
    function(obj, mat, dir, rhs, types, max, bounds, verbose,
    nScenarios, nAssets, targetReturn, Alpha, Type)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Rglpk CVAR Solver

    # FUNCTION:

    # Solve - use Rglpk_solve_LP:
    optim <- Rglpk::Rglpk_solve_LP(
        obj = obj,
        mat = mat,
        dir = dir,
        rhs = rhs,
        types = types,
        max = max,
        bounds = bounds,
        verbose = verbose)

    # Extract Weights:
    weights <- .checkWeights(rev(rev(optim$solution)[1:nAssets]))
    attr(weights, "invest") = sum(weights)

    # Result:
    ans <- list(
        type = Type,
        solver = "Rglpk.CVAR",
        optim = optim,
        weights = weights,
        solution = weights,
        targetReturn = targetReturn,
        targetRisk = -optim$optimum,
        objective = -optim$optimum,
        status = optim$status[[1]],
        message = "NA")

    # Return Value:
    ans
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


###############################################################################
# FUNCTION:                DESCRIPTION:
#  pfolioReturn             Returns portfolio returns
# FUNCTION:                DESCRIPTION:
#  sampleCOV                Returns sample covariance risk
#  normalVaR                Returns normal Value at Risk
#  modifiedVaR              Returns modified Cornish Fisher VaR
#  sampleVaR                Returns sammple VaR from historical quantiles
# FUNCTION:                DESCRIPTION:
#  budgetsSampleCOV         Covariance risk contribution and budgets
#  budgetsNormalVAR         Normal VaR risk contribution and budgets
#  budgetsModifiedVAR       Modified VaR risk contribution and budgets
#  budgetsNormalES          Normal ES (CVaR) risk contribution and budgets
#  budgetsModifiedES        Modified ES (CVaR) risk contribution and budgets
# UTILITIES:               DESCRIPTION:
#  .M34.MM                  Internal fast computing of M3 and M4
#  .run                     Returns execution time information
#  .Ipower                  Internal utility function to compute M3
#  .derIpower               Internal utility function to compute M4
#  .myVaR                   ... to do
# DEPRECATED:              DESCRIPTION:
#  .covarRisk               Computes covariance portfolio risk
#  .mcr                     Computes marginal contribution to covariance risk
#  .mcrBeta                 Computes beta, the rescaled mcr to covariance risk
#  .riskContributions       Computes covariance risk contributions
#  .riskBudgets             Computes covariance risk budgets
###############################################################################


sampleCOV <- 
    function(x)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #    Returns sample covariance risk
    
    # Arguments:
    #    x - a 'timeSeries' object
    
    # FUNCTION:
    
    # Return Value:
    cov(x)
}


# -----------------------------------------------------------------------------


normalVaR <- 
    function(x, alpha=0.05)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #    Returns normal Value at Risk
    
    # Arguments:
    #    x - a 'timeSeries' object
    
    # FUNCTION:
    
    # Mean and Centered 2nd Moment:
    x.mean <- colMeans(x)
    x.centered <- t(t(x) - x.mean)
    m2 <- colMeans(x.centered^2)
    
    # Gaussian:
    q <- qnorm(alpha)
    
    # Return Value:
    x.mean + q * sqrt(m2)
}


# -----------------------------------------------------------------------------


modifiedVaR <- 
    function(x, alpha=0.05)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #    Returns modified Cornish Fisher VaR
    
    # Arguments:
    #    x - a 'timeSeries' object
    
    # Details:
    #     Includes Code Borrowed from Peterson and Boudt, GPL
    
    # FUNCTION:
    
    # Mean and Centered Moments:
    x.mean <- colMeans(x)
    x.centered <- t(t(x) - x.mean)
    m2 <- colMeans(x.centered^2)
    m3 <- colMeans(x.centered^3)
    m4 <- colMeans(x.centered^4)
    skew <- m3 / sqrt(m2^3)
    kurt <- m4 / (m2*m2) - 3
    
    # Cornish Fisher:
    z <- qnorm(alpha)
    q <- z + (z*z-1)*skew/6 + z*(z*z-3)*kurt/24 - z*(2*z*z-5)*skew*skew/36
    
    # Return Value:
    x.mean + q * sqrt(m2)
}


# -----------------------------------------------------------------------------


sampleVaR <- 
    function(x, alpha=0.05)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #    Returns sammple VaR from historical quantiles
    
    # Arguments:
    #    x - a 'timeSeries' object
    
    # FUNCTION:
    
    # Return Value:
    colQuantiles(x, alpha)
}


# -----------------------------------------------------------------------------


budgetsSampleCOV <- 
    function(x, weights, mu=NULL, Sigma=NULL)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    
    # Arguments:
    #    x - a 'timeSeries' object
    
    # Details:
    #     Includes Code Borrowed from Peterson and Boudt, GPL
    #     Rmetrics Re-Implementation
    
    # FUNCTION:
    
    # Risk:
    if(is.null(mu)) mu <- colMeans(x)
    if(is.null(Sigma)) Sigma <- cov(x)
    risk <- sqrt( t(weights) %*% Sigma %*% weights )[[1]]
    attr(risk, "estimator") <- substitute(FUN)
    
    # Risk Contributions:
    m2.pfolio <- (t(weights) %*% Sigma %*% weights)[[1]]
    dm2.pfolio <- as.vector(Sigma %*% weights)
    contribution <- dm2.pfolio/sqrt(m2.pfolio) * weights
    names(contribution) <- colnames(x)
    
    # Risk Budgets:
    budgets <- contribution/risk
    names(budgets) <- colnames(x)
    attr(budgets, "control") <- sum(budgets)
    
    # Return Value:
    list(riskCOV=risk, contribution=contribution, budgets=budgets)
}


# -----------------------------------------------------------------------------


budgetsNormalVAR <- 
    function(x, weights, alpha=0.05, mu=NULL, Sigma=NULL)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    
    # Arguments:
    #    x - a 'timeSeries' object
    
    # Details:
    #     Includes Code Borrowed from Peterson and Boudt, GPL
    
    # FUNCTION:
    
    # Risk:
    if(is.null(mu)) mu <- colMeans(x)
    if(is.null(Sigma)) Sigma <- cov(x)
    risk <- -(t(weights) %*% mu + qnorm(alpha) * 
                sqrt( t(weights) %*% Sigma %*% weights))[[1]]
    attr(risk, "estimator") <- substitute(FUN)
    attr(risk, "alpha") <- alpha
    
    # Risk Contributions:
    m2.pfolio <- (t(weights) %*% Sigma %*% weights)
    dm2.pfolio <- as.vector(Sigma %*% weights)
    contribution <- - (mu + qnorm(alpha)* dm2.pfolio/sqrt(m2.pfolio)) * 
      weights
    names(contribution) <- colnames(x)
    
    # Risk Budgets:
    budgets <- contribution/risk
    names(budgets) <- colnames(x)
    attr(budgets, "sumBudgets") <- sum(budgets)
    
    # Return Value:
    list(riskVAR=risk, contributionVAR=contribution, budgetsVAR=budgets)
}


# -----------------------------------------------------------------------------
    

budgetsModifiedVAR  <- 
    function(x, weights, alpha=0.05, mu=NULL, Sigma=NULL, M3=NULL, M4=NULL) 
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    
    # Arguments:
    #    x - a 'timeSeries' object
    
    # Details:
    #     Includes code borrowed from Peterson and Boudt, GPL
    
    # FUNCTION:
    
    # Compute Moments:
    if(is.null(mu)) mu <- colMeans(x)
    if(is.null(Sigma)) Sigma <- cov(x)
    if(is.null(M3) || is.null(M4)) {
        MM <- .M34.MM(x, mu=mu) 
        M3 <- MM$M3
        M4 <- MM$M4
    }
    
    # Risk:
    z <- qnorm(alpha)
    location <- t(weights) %*% mu
    pm2 <- t(weights) %*% Sigma %*% weights 
    dpm2 <- as.vector(2 * Sigma %*% weights)
    pm3 <- weights %*% M3 %*% (weights %x% weights)
    dpm3 <- as.vector(3 * M3 %*% (weights %x% weights))
    pm4 <- t(weights) %*% M4 %*% (weights %x% weights %x% weights) 
    dpm4 <- as.vector(4 * M4 %*% (weights %x% weights %x% weights))
    skew <- (pm3/pm2^(3/2))[[1]]
    exkurt <- (pm4/pm2^(2) - 3)[[1]]
    derskew <- (2 * (pm2^(3/2)) * dpm3 - 3 * pm3 * sqrt(pm2) * dpm2) / 
      (2 * pm2^3)
    derexkurt <- ((pm2) * dpm4 - 2 * pm4 * dpm2)/(pm2^3)  
    h <- z + (1/6) * (z^2 - 1) * skew
    h <- h + (1/24) * (z^3 - 3 * z) * exkurt - (1/36) * (2 * z^3 - 5 * z) * 
      skew^2 
    risk <- -(location + h * sqrt(pm2))
    
    # Risk Contribution:
    derGausVaR <- -as.vector(mu) - qnorm(alpha) * 
        (0.5 * as.vector(dpm2))/sqrt(pm2)
    derMVaR <- derGausVaR + (0.5 * dpm2/sqrt(pm2)) * (-(1/6) * 
        (z^2 - 1) * skew - (1/24) * (z^3 - 3 * z) * exkurt + 
        (1/36) * (2 * z^3 - 5 * z) * skew^2)
    derMVaR <- derMVaR + sqrt(pm2) * (-(1/6) * (z^2 - 1) * derskew - 
        (1/24) * (z^3 - 3 * z) * derexkurt + (1/36) * (2 * z^3 - 
        5 * z) * 2 * skew * derskew)
    contribution <- as.vector(weights) * as.vector(derMVaR)
    names(contribution) <- colnames(x)
    
    # Risk Budgets:
    budgets <- contribution/risk
    names(budgets) <- colnames(x)
    budgets
    attr(budgets, "sum(contribution)-risk") <- sum(contribution)-risk
    attr(budgets, "sum(budgets)") <- sum(budgets)
    
    # Return Value:
    list(modifiedVAR=risk, contribution=contribution, budgets=budgets)
}


# -----------------------------------------------------------------------------


budgetsNormalES <- 
    function(x, weights, alpha=0.05, mu=NULL, Sigma=NULL)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #    x - a 'timeSeries' object
    
    # Arguments:
    
    # Details:
    #     Includes Code Borrowed from Peterson and Boudt, GPL
    
    # FUNCTION:
    
    # Risk:
    if(is.null(mu)) mu <- colMeans(x)
    if(is.null(Sigma)) Sigma <- cov(x)
    location <- t(weights) %*% mu
    pm2 <- t(weights) %*% Sigma %*% weights 
    dpm2 <- as.vector(2 * Sigma %*% weights)
    risk <- -location + dnorm(qnorm(alpha)) * sqrt(pm2)/alpha
    attr(risk, "estimator") <- substitute(FUN)
    attr(risk, "alpha") <- alpha
    
    # Contribution:
    derES <- -mu + (1/alpha) * dnorm(qnorm(alpha)) * (0.5 * dpm2)/sqrt(pm2)
    contribution <- weights * derES
    names(contribution) <- colnames(x)
    
    # Budgets:
    budgets <- contribution/risk
    names(budgets) <- colnames(x)
    attr(budgets, "sumBudgets") <- sum(budgets)
    
    # Return Value:
    list(normalES=risk, contribution=contribution, budgets=budgets)
}


# -----------------------------------------------------------------------------


budgetsModifiedES <- 
    function(x, weights, alpha=0.05, mu=NULL, Sigma=NULL, M3=NULL, M4=NULL)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    
    # Arguments:
    #    x - a 'timeSeries' object
    
    # Details:
    #     Includes code borrowed from Peterson and Boudt, GPL
    
    # FUNCTION:
    
    # Settings:
    if(is.null(mu)) mu <- colMeans(x)
    if(is.null(Sigma)) Sigma <- cov(x)
    if(is.null(M3) || is.null(M4)) {
        MM <- .M34.MM(x, mu=mu) 
        M3 <- MM$M3
        M4 <- MM$M4
    }
    
    # Risk:
    z <- qnorm(alpha)
    location <- t(weights) %*% mu
    pm2 <- (t(weights) %*% Sigma %*% weights)[[1]]
    dpm2 <- as.vector(2 * Sigma %*% weights)
    pm3 <- (weights %*% M3 %*% (weights %x% weights))[[1]]
    dpm3 <- as.vector(3 * M3 %*% (weights %x% weights))
    pm4 <- (t(weights) %*% M4 %*% (weights %x% weights %x% weights))[[1]] 
    dpm4 <- as.vector(4 * M4 %*% (weights %x% weights %x% weights))
    skew <- (pm3/pm2^(3/2))[[1]]
    exkurt <- (pm4/pm2^(2) - 3)[[1]]
    derskew <- (2 * (pm2^(3/2)) * dpm3 - 
        3 * pm3 * sqrt(pm2) * dpm2)/(2 * pm2^3)
    derexkurt <- ((pm2) * dpm4 - 2 * pm4 * dpm2)/(pm2^3)
    h <- z + 
        (1/6) * (z^2 - 1) * skew
    h <- h + 
        (1/24) * (z^3 - 3 * z) * exkurt - 
        (1/36) * (2 * z^3 - 5 * z) * skew^2
    derh <- (1/6) * (z^2 - 1) * derskew + 
        (1/24) * (z^3 - 3 * z) * derexkurt - 
        (1/18) * (2 * z^3 - 5 * z) * skew * derskew
    E <- dnorm(h)
    E <- E + (1/24) * (.Ipower(4, h) - 6 * .Ipower(2, h) + 
        3 * dnorm(h)) * exkurt
    E <- E + (1/6) * (.Ipower(3, h) - 3 * .Ipower(1, h)) * skew
    E <- E + (1/72) * (.Ipower(6, h) - 15 * .Ipower(4, h) + 
        45 * .Ipower(2, h) - 15 * dnorm(h)) * (skew^2)
    E <- E/alpha
    risk <- MES <- -location + sqrt(pm2) * E
    
    # Risk Contributions:
    derMES <- -mu + 0.5 * (dpm2/sqrt(pm2)) * E
    derE <- (1/24) * (.Ipower(4, h) - 6 * .Ipower(2, h) + 
        3 * dnorm(h)) * derexkurt
    derE <- derE + (1/6) * (.Ipower(3, h) - 3 * .Ipower(1, h)) * derskew
    derE <- derE + (1/36) * (.Ipower(6, h) - 15 * .Ipower(4, h) + 
        45 * .Ipower(2, h) - 15 * dnorm(h)) * skew * derskew
    X <- -h * dnorm(h) + (1/24) * (.derIpower(4, h) - 6 * .derIpower(2, h) - 
        3 * h * dnorm(h)) * exkurt
    X <- X + (1/6) * (.derIpower(3, h) - 3 * .derIpower(1, h)) * skew
    X <- X + (1/72) * (.derIpower(6, h) - 15 * .derIpower(4, h) + 
        45 * .derIpower(2, h) + 15 * h * dnorm(h)) * skew^2
    derE <- derE + derh * X
    derE <- derE/alpha
    derMES <- derMES + sqrt(pm2) * derE
    contribution <- as.vector(weights) * as.vector(derMES)
    names(contribution) <- colnames(x)
    
    # Risk Budgets:
    budgets <- contribution/risk
    names(budgets) <- colnames(x)
    attr(budgets, "sumBudgets") <- sum(budgets)
    
    # Return Value:
    list(modifedES=risk, contribution=contribution, budgets=budgets)
}


###############################################################################


.M34.MM <- 
    function (x, mu=NULL)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    
    # Arguments:
    #    x - a 'timeSeries' object
    
    # Details:
    #     Includes Code Borrowed from Peterson and Boudt, GPL
    
    # Fast Rmetrics Implementation:
    n <- ncol(x)
    m <- nrow(x)
    if(is.null(mu)) mu <- colMeans(x)
    M3 <- matrix(rep(0, n^3), nrow=n, ncol=n^2)
    M4 <- matrix(rep(0, n^4), nrow=n, ncol=n^3)
    centret <- series(x) - matrix(rep(mu, each=m), ncol=n)
    for (i in c(1:m)) {
        cent <- centret[i, ]
        tcent <- t(cent)
        M <- (cent %*% tcent) %x% tcent
        M3 <- M3 + M
        M4 <- M4 + M %x% tcent
    }
    
    # Return Value:
    list(M3=M3/m, M4=M4/m, mu=mu)
}


# -----------------------------------------------------------------------------


.run <- 
    function(FUN, times=10, mult=100, ...)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Timing:
    fun <- match.fun(FUN)
    now <- Sys.time()
    for (i in 1:as.integer(times)) ans <- fun(...)
    done <- Sys.time()
    time <- mult * as.numeric(done - now)
    
    # Print Timing Results:
    cat("Timing:\n")
    print(c(sec=round(time,0), times=times, mult=mult, runs=times*mult))
    cat("\nResults:\n\n")
    
    # Return Value:
    ans
}


# -----------------------------------------------------------------------------


.Ipower <- 
    function (power, h) 
{
    # Description:
    
    # Arguments:
    
    # Details:
    #     A function borrowed from PerformanceAnalytics, GPL
    
    fullprod <- 1
    if ((power%%2) == 0) {
        pstar <- power/2
        for (j in c(1:pstar)) fullprod <- fullprod * (2 * j)
        I <- fullprod * dnorm(h)
        for (i in c(1:pstar)) {
            prod <- 1
            for (j in c(1:i)) prod <- prod * (2 * j)
            I <- I + (fullprod/prod) * (h^(2 * i)) * dnorm(h)
        }
    } else {
        pstar <- (power - 1)/2
        for (j in c(0:pstar)) {
            fullprod = fullprod * ((2 * j) + 1)
        }
        I <- -fullprod * pnorm(h)
        for (i in c(0:pstar)) {
            prod = 1
            for (j in c(0:i)) prod = prod * ((2 * j) + 1)
            I <- I + (fullprod/prod) * (h^((2 * i) + 1)) * dnorm(h)
        }
    }
    return(I)
}


# -----------------------------------------------------------------------------


.derIpower <- 
    function (power, h) 
{
    # Description:
    
    # Arguments:
    
    # Details:
    #     A function borrowed from PerformanceAnalytics, GPL
    
    fullprod <- 1
    if ((power%%2) == 0) {
        pstar <- power/2
        for (j in c(1:pstar)) fullprod = fullprod * (2 * j)
        I <- -fullprod * h * dnorm(h)
        for (i in c(1:pstar)) {
            prod = 1
            for (j in c(1:i)) prod = prod * (2 * j)
            I <- I + (fullprod/prod) * (h^(2 * i - 1)) * 
                (2 * i - h^2) * dnorm(h)
        }
    } else {
        pstar = (power - 1)/2
        for (j in c(0:pstar)) fullprod <- fullprod * ((2 * j) + 1)
        I <- -fullprod * dnorm(h)
        for (i in c(0:pstar)) {
            prod = 1
            for (j in c(0:i)) prod <- prod * ((2 * j) + 1)
            I <- I + (fullprod/prod) * (h^(2 * i) * 
                (2 * i + 1 - h^2)) * dnorm(h)
        }
    }
    return(I)
}


# -----------------------------------------------------------------------------


.myVaR <- 
    function(x, alpha=0.05, method=c("normal", "modified", "sample"))
{
    # A function implemented by Diethelm Wuertz
    
    # todo ...
    
    # Funcion Selection:
    fun <- match.fun(paste(match.arg(method), "VaR", sep=""))
    
    # Return Value:
    fun(x, alpha)
}


###############################################################################
# DEPRECATED - DO NOT REMOVE - REQUIRED BY PACKAGE appRmetricsHandbook

.covarRisk <-
    function(data, weights=NULL, FUN="cov", ...)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Computes covariance portfolio risk
    
    # Arguments:
    #   data - a multivariate timeSeries object of financial returns
    #   weights - numeric vector of portfolio weights
    #   FUN - a covariance estimator, which returns a matrix of 
    #       covariance estimates, by default the sample covariance
    #   ... - Optional arguments passed to the function FUN
    
    # Example:
    #   covarRisk(data)
    
    # FUNCTION:
    
    # Covariance Risk:
    covFun <- match.fun(FUN)
    COV <- covFun(data)
    
    # Portfolio Weights:
    N <- ncol(COV)
    if (is.null(weights)) weights = rep(1/N, N)
    names(weights) <- colnames(COV)
    
    # Covariance Portfolio Risk:
    covarRisk <- sqrt( t(weights) %*% COV %*% weights )[[1, 1]]
    
    # Return Value:
    covarRisk
}


# -----------------------------------------------------------------------------


.mcr <- 
    function(data, weights=NULL, FUN="cov", ...)
{
    # A function implemented by Diethelm Wuertz
    
    # Description
    #   Computes marginal contribution to covariance risk
    
    # Arguments:
    #   data - a multivariate timeSeries object of financial returns
    #   weights - numeric vector of portfolio weights
    #   FUN - a covariance estimator, which returns a matrix of 
    #       covariance estimates, by default the sample covariance
    #   ... - Optional arguments passed to the function FUN
    
    # Details:
    #   The formula are implemented according to Goldberg et al., 
    #   see also R script assetsPfolio.R
    
    # References:
    #   Lisa Goldberg et al., Extreme Risk Management, 2009
    #   Scherer and Martin, Introduction to modern portfolio Optimimization
    
    # Example:
    #   data <- assetsSim(100, 6); mcr(data)
    
    # FUNCTION:
    
    # Covariance Risk:
    covFun <- match.fun(FUN)
    COV <- covFun(data)
    N <- ncol(data)
    if (is.null(weights)) weights <- rep(1/N, N)
    
    # Marginal Contribution to Risk  
    mcr <- (COV %*% weights)[, 1] / .covarRisk(data, weights, FUN, ...) 
    names(mcr) <- colnames(data) 
    
    # Return Value:
    mcr
}


# -----------------------------------------------------------------------------


.mcrBeta <- 
    function(data, weights=NULL, FUN="cov", ...)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Computes beta, the rescaled marginal contribution to covariance risk
    
    # Arguments:
    #   data - a multivariate timeSeries object of financial returns
    #   weights - numeric vector of portfolio weights
    #   FUN - a covariance estimator, which returns a matrix of 
    #       covariance estimates, by default the sample covariance
    #   ... - Optional arguments passed to the function FUN
    
    # Example:
    #    .mcrBeta(data)
   
    # FUNCTION:
    
    # Portfolio Beta:
    beta <- .mcr(data, weights, FUN = FUN, ...) / 
        .covarRisk(data, weights, FUN = FUN, ...)
   
    # Return Value:
    beta
}


# -----------------------------------------------------------------------------


.riskContributions <-
    function(data, weights=NULL, FUN="cov", ...)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Computes covariance risk contributions
    
    # Arguments:
    #   data - a multivariate timeSeries object of financial returns
    #   weights - numeric vector of portfolio weights
    #   FUN - a covariance estimator, which returns a matrix of 
    #       covariance estimates, by default the sample covariance
    #   ... - Optional arguments passed to the function FUN
    
    # Example:
    #    .riskContributions(data)
    
    # FUNCTION:
    
    # Risk Contributions:
    if (is.null(weights)) {
        N <- ncol(data)
        weights <- rep(1/N, times = N)
    }
    riskContributions <- weights * .mcr(data, weights, FUN, ...)
    
    # Return Value:
    riskContributions
}


# -----------------------------------------------------------------------------


.riskBudgets <-
    function(data, weights=NULL, FUN="cov", ...)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Computes covariance risk budgets
    
    # Arguments:
    #   data - a multivariate timeSeries object of financial returns
    #   weights - numeric vector of portfolio weights
    #   FUN - a covariance estimator, which returns a matrix of 
    #       covariance estimates, by default the sample covariance
    #   ... - Optional arguments passed to the function FUN
    
    # Example:
    #    data <- 100*LPP2005.RET[, 1:6]; .riskBudgets(data)
    
    # FUNCTION:
    
    # Risk Budgets:
    riskBudgets <- .riskContributions(data, weights, FUN, ...) /
        .covarRisk(data, weights, FUN, ...)
    
    # Return Value:
    riskBudgets
} 


###############################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                     DESCRIPTION:
#  .vaniniFig                    Creates Vinini's Figure in Portfolio eBook
################################################################################


.vaniniFig <- 
  function()
  {
    # A function implemented by Diethelm Wuertz
    
    # FUNCTION:
    
    # Load Dataset
    dataSet <- data("LPP2005.RET", package="fPortfolio", envir=environment())
    LPP2005.RET <- get(dataSet, envir=environment())
    
    # Example Data:
    data = 100 * LPP2005.RET[, 1:6]
    mu = colMeans(data)
    Sigma = cov(data)
    one = rep(1, times = 6)
    
    # Short Selling Solution :
    invSigma = solve(Sigma)
    A = (mu  %*% invSigma %*% one)[[1,1]]     
    C = (one %*% invSigma %*% one)[[1,1]]  
    B = (mu  %*% invSigma %*%  mu)[[1,1]]  
    E = (one %*% invSigma %*%  mu)[[1,1]]
    D = B*C - A*A
    
    # Minimum Variance Point:
    xMV = 1/sqrt(C)
    yMV = A/C
    
    # Frontier Points:
    x = seq(xMV, 4*xMV, length = 500)
    a = C
    b = -2 * A
    c = B - D * x^2
    yp = (-b + sqrt(b^2 - 4 * a * c))/(2 * a)
    ym = (-b - sqrt(b^2 - 4 * a * c))/(2 * a)
    
    # Asymptotic Slopes:
    slope = sqrt(D/C)
    intercept = A/C
    
    # Tangency Line:
    x.tg = sqrt(B)/E
    y.tg = B/E
    slope.tg = y.tg/x.tg
    
    # Plot:
    plot(x,  yp, type = "l", xlim = c(0, max(x)), ylim = c(-0.08, 0.08),
         axes = FALSE, xlab = "Covariance Risk", ylab = "Mean Retun")
    lines(x, ym)
    points(xMV, yMV, col = "orange", cex = 2, pch = 19)
    abline(intercept, slope, col = "blue", lty = 2)
    abline(intercept, -slope, col = "blue", lty = 2)
    abline(h = yMV, col = "grey", lty = 3)
    abline(v = 0, col = "grey", lty = 3)
    points(x.tg, y.tg, col = "red", cex = 2, pch = 19)
    abline(0, slope.tg, col = "brown", lty = 2)
    
    # Return Value:
    invisible()
  }


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


###############################################################################
# FUNCTION:                    DESCRIPTION:
#  rsolveLP                     General function wrapper for LP solvers
#  .solveLP.MAD.demo            MAD portfolio example
#  .solveLP.GLPK.demo           GLPK help page example
###############################################################################


rsolveLP <- 
  function(objective, lower=0, upper=1, linCons, 
           control=list(solver="glpk", invoke=c("R", "AMPL", "NEOS")))
  {    
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Rmetrics Interface for AMPL/NEOS LP solvers
    
    # Argments:
    #    objective - numeric vector.
    #    lwer, upper - box constraints
    #    linCons - linear constraints: mat, lower and upper
    #    control - control list
    
    # Control:
    solver <- control$solver
    invoke <- control$invoke[1]
    
    # Solve Linear Problem:
    if (invoke == "R") {
      rfooLP <- match.fun ( paste("r", solver, "LP", sep=""))
      ans <- rfooLP(objective, lower, upper, linCons, control)
    }
    if (invoke == "AMPL" ) {
      ans <- ramplLP(objective, lower, upper, linCons, control)
    }
    if (invoke == "NEOS" ) {
      ans <- rneosLP(objective, lower, upper, linCons, control)
    }  
    ans$solver <- paste(invoke, ans$solver) 
    
    # Return Value:
    ans  
  }


###############################################################################


.solveLP.MAD.demo <- 
  function()
  {
    # Solve MAD Portfolio:
    
    # Load Dataset
    dataSet <- data("LPP2005REC", package="timeSeries", envir=environment())
    LPP2005REC <- get(dataSet, envir=environment())
    
    # Load Swiss Pension Fund Data:
    nAssets <- 6
    nScenarios <- 100   
    data <- 100 * LPP2005REC[1:nScenarios, 1:nAssets] 
    Mean <- colMeans(data)
    Data <- as.matrix(data)
    targetReturn <- mean(data)
    
    # Objective Function:
    vec <- c(weights=rep(0, nAssets), scenarios=rep(1/nScenarios, nScenarios))
    
    # Set up Constraints Matrix:
    mat <- rbind(
      MAD.LE = cbind(Data, -diag(nScenarios)),
      MAD.GE = cbind(Data, +diag(nScenarios)),
      RETURN = t(c(Mean, rep(0, nScenarios))),
      BUDGET = t(c(rep(1, nAssets), rep(0, nScenarios))),
      X = cbind(matrix(rep(0, nAssets*nScenarios),ncol=nAssets), diag(nScenarios)),
      WEIGHTS = cbind(diag(nAssets), matrix(rep(0, nScenarios*nAssets), nrow=nAssets)))
    
    # Set up Right Hand Side of Constraints Equations:
    rhs <- c(
      MAD.LE = rep(0, nScenarios),
      MAD.GE = rep(0, nScenarios),
      RETURN = targetReturn,
      BUDGET = 1,
      X = rep(0, nScenarios),
      WEIGHTS = rep(0, nAssets))
    
    # Set up Vector of Directions:
    dir <- c(
      MAD.LE = rep("<=", nScenarios),
      MAS.GE = rep(">=", nScenarios),
      RETURN = "==",
      BUDGET = "==",
      X = rep(">=", nScenarios),
      WEIGHTS = rep(">=", nAssets))
    
    # Conversions:
    RHS <- rep(Inf, times=length(dir))
    RHS[dir == "<="] <- rhs[dir == "<="]
    RHS[dir == "=="] <- rhs[dir == "=="] 
    LHS <- rep(-Inf, times=length(dir))
    LHS[dir == ">="] <- rhs[dir == ">="]
    LHS[dir == "=="] <- rhs[dir == "=="]
    
    # Arguments: 
    objective <- vec
    lower <- 0
    upper <- 1
    linCons <- list(mat, LHS, RHS)
    control <- list()
    
    # Contributed R Solver - Original Function Calls:
    Rglpk::Rglpk_solve_LP(vec, mat, dir, rhs)
    
    # Contributed R Solver - Interfaced  
    rglpkLP(objective, lower, upper, linCons)
    
    # AMPL:
    ramplLP(objective, lower, upper, linCons)
    
    # All AMPL:
    for (solver in c(
      "cplex", "donlp2", "gurobi", "loqo", "lpsolve", "minos", "snopt",
      "ipopt", "bonmin", "couenne")) {
      ans <- ramplLP(objective=vec, lower, upper, linCons, control=list(solver=solver))
      print(ans)
    }
    
    # NEOS:
    # require(rneos)
    # lp: Linear Programming Solver:
    for (solver in c("gurobi", "mosek", "ooqp"))   
      print(rneosLP(objective=vec, lower, upper, linCons, 
                    control=list(solver=solver, category="lp"))) 
    # nco: Using Nonlinear Constrained Optimization Solver:    
    for (solver in c(
      "conopt", "filter", "knitro", "lancelot", "loqo", "minos", "mosek",
      "pennon", "snopt"))    
      print(rneosLP(objective=vec, lower, upper, linCons, 
                    control=list(solver=solver, category="nco")))
  }


# -----------------------------------------------------------------------------


.solveLP.GLPK.demo <- 
  function()
  {
    # GLPK Demo from Help Rglpk Page:
    vec <- -c(2, 4, 3)
    mat <- matrix(c(
      3, 2, 1, 
      4, 1, 3, 
      2, 2, 2), 3, 3)
    dir <- c("<=", "<=", "<=")
    rhs <- c(60, 40, 80) 
    
    # For Testing:
    # mat <- rbind(mat, c(0,0,0))
    # dir <- c(dir, "<=")
    # rhs <- c(rhs, 1000)
    
    # Arguments:
    objective <- vec
    lower <- 0
    upper <- Inf
    linCons <- list(mat, lower=-Inf, upper=rhs)
    control <- list()
    
    # Contributed R Solver - Original Function Calls:
    Rglpk::Rglpk_solve_LP(vec, mat, dir, rhs)
    
    # Contributed R Solver - Interfaced  
    rglpkLP(objective, lower, upper, linCons)
    
    # AMPL:
    ramplLP(objective, lower, upper, linCons)
    
    # All AMPL:
    for (solver in c(
      "cplex", "donlp2", "gurobi", "loqo", "lpsolve", "minos", "snopt",
      "ipopt", "bonmin", "couenne")) {
      ans <- ramplLP(objective=vec, lower, upper, linCons, control=list(solver=solver))
      print(ans)
    }
    
    # NEOS:
    # require(rneos)
    # lp: Linear Programming Solver:
    for (solver in c("gurobi", "mosek", "ooqp"))   
      print(rneosLP(objective=vec, lower, upper, linCons, 
                    control=list(solver=solver, category="lp"))) 
    # nco: Using Nonlinear Constrained Optimization Solver:    
    for (solver in c(
      "conopt", "filter", "knitro", "lancelot", "loqo", "minos", "mosek",
      "pennon", "snopt"))    
      print(rneosLP(objective=vec, lower, upper, linCons, 
                    control=list(solver=solver, category="nco")))
  }


###############################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                    DESCRIPTION:
#  .solveRtwoAssets             Two Assets LongOnly MV Portfolio
#  .cvarSolveTwoAssets          Two Assets LongOnly CVaR Portfolio
#  .madSolveTwoAssets           Two Assets LongOnly MAD Portfolio
################################################################################


.solveRtwoAssets <-
    function(data, spec, constraints)
{
    # Description:
    #   Two Assets LongOnly MV Portfolio

    # Details:
    # ... this is only thohgt for 'unlimited' LongOnly
    # box and group constraints are discarded here.

    # FUNCTION:

    # Solver:
    # print(".mvSolveTwoAssets")

    # Convert Data and Constraints to S4 Objects:
    Data = portfolioData(data, spec)
    data <- getSeries(Data)
    Constraints = portfolioConstraints(Data, spec, constraints)

    # Stop if the Target Return is not Defined!
    targetReturn = getTargetReturn(spec)
    stopifnot(is.numeric(targetReturn))

    # Optimize Portfolio:
    nAssets = getNAssets(Data)

    # Solve the two Assets Case Analytically:
    mu <- getMu(Data)
    Sigma <- getSigma(Data)
    stopifnot(targetReturn >= min(mu))
    stopifnot(targetReturn <= max(mu))
    weights <- (targetReturn-mu[2]) / (mu[1]-mu[2])
    weights <- c(weights, 1 - weights)

    # Output List:
    ans = list(
        solver = "MVTwoAssets",
        optim = NA,
        weights = weights,
        targetReturn = targetReturn,
        targetRisk = NA,
        objective = sqrt(weights %*% Sigma %*% weights),
        status = 0,
        message = NA)

    # Return Value:
    ans
}


# ------------------------------------------------------------------------------


.cvarSolveTwoAssets <-
    function(data, spec, constraints)
{
    # Description:
    #   Two Assets LongOnly CVaR Portfolio

    # Details:
    # ... this is only thohgt for 'unlimited' LongOnly
    # box and group constraints are discarded here.

    # FUNCTION:

    # Solver:
    # print(".cvarSolveTwoAssets")

    # Convert Data and Constraints to S4 Objects:
    Data <- portfolioData(data, spec)
    data <- getSeries(Data)
    Constraints = portfolioConstraints(Data, spec, constraints)

    # Stop if the Target Return is not Defined!
    targetReturn = getTargetReturn(spec)
    stopifnot(is.numeric(targetReturn))
    targetAlpha = getAlpha(spec)

    # Optimize Portfolio:
    nAssets <- getNAssets(Data)

    # Solve the two Assets Case Analytically:
    mu <- getMu(Data)
    stopifnot(targetReturn >= min(mu))
    stopifnot(targetReturn <= max(mu))
    weights <- (targetReturn-mu[2]) / (mu[1]-mu[2])
    weights <- c(weights, 1 - weights)

    optim <- list(
        VaR = .varRisk(data, weights, targetAlpha),
        CVaR = -.cvarRisk(data, weights, targetAlpha),
        targetAlpha = targetAlpha)

    ans <- list(
        solver = "CVaRTwoAssets",
        optim = optim,
        weights = weights,
        targetReturn = targetReturn,
        targetRisk = NA,
        objective = optim$CVaR,
        status = 0,
        message = NA)

    # Return Value:
    ans
}


# ------------------------------------------------------------------------------


.madSolveTwoAssets <-
    function(data, spec, constraints)
{
    # Description:
    #   Two Assets LongOnly MAD Portfolio

    # Details:
    # ... this is only thohgt for 'unlimited' LongOnly
    # box and group constraints are discarded here.

    # FUNCTION:

    # Convert Data and Constraints to S4 Objects:
    Data = portfolioData(data, spec)
    data <- getSeries(Data)
    Constraints <- portfolioConstraints(Data, spec, constraints)

    # Stop if the Target Return is not Defined!
    targetReturn <- getTargetReturn(spec)
    stopifnot(is.numeric(targetReturn))

    # Optimize Portfolio:
    nAssets <- getNAssets(Data)

    # Solve the two Assets Case Analytically:
    mu = getMu(Data)
    stopifnot(targetReturn >= min(mu))
    stopifnot(targetReturn <= max(mu))
    weights = (targetReturn-mu[2]) / (mu[1]-mu[2])
    weights = c(weights, 1 - weights)
    targetRisk = mean( abs( (data - colMeans(data)) %*% weights ) )

    # Output List:
    ans = list(
        solver = "MADTwoAssets",
        optim = NA,
        weights = weights,
        targetReturn = targetReturn,
        targetRisk = targetRisk,
        objective = targetRisk,
        status = 0,
        message = NA)

    # Return Value:
    ans
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                     DESCRIPTION:
#  plot.fPORTFOLIO               S3 Plot method for 'fPORTFOLIO' objects
# FUNCTION:                     DESCRIPTION:
#  .fPortfolio.plot1..8          Internal plot functions
################################################################################


plot.fPORTFOLIO <-
    function(x, which = "ask", control = list(), ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Plot method for an object of class 'fPORTFOLIO'

    # FUNCTION:

    # Control Parameters:
    Statistics = getStatistics(x)

    # Use default, if xlim and ylim is not specified ...
    mu = Statistics$mu
    Sigma = Statistics$Sigma
    N = length(mu)
    yLim = range(mu) + 0.25*c(-diff(range(mu)), diff(range(mu)))

    # First, take care that all assets appear on the plot ...
    # sqrtSig = sqrt(diag(Sigma))
    # xLimAssets = c(
    #    min(sqrtSig),
    #    max(sqrtSig))+ c(-0.4*diff(range(sqrtSig)), 0.1*diff(range(sqrtSig)))
    xRange = range(frontierPoints(x)[, 1])
    xDiff = diff(xRange)
    xLimAssets = c(xRange[1] - 2.5*xDiff/10, xRange[2] + xDiff/10)

    # ... second take care that the whole frontier appears on the plot:
    fullFrontier = frontierPoints(x)
    xLimFrontier = range(fullFrontier[, 1])
    xLim = range(c(xLimAssets, xLimFrontier))

    # Control List:
    # YC: merge with user control list
    con <- c(control, frontierPlotControl())
    # YC: remove double entries and keep user args
    con <- con[unique(names(con))]
    attr(x, "control") <- con

    # Plot:
    interactivePlot(
        x,
        choices = c(
            "Plot Efficient Frontier",
            "Add Minimum Risk Portfolio",
            "Add Tangency Portfolio",
            "Add Risk/Return of Single Assets",
            "Add Equal Weights Portfolio",
            "Add Two Asset Frontiers [LongOnly Only]",
            "Add Monte Carlo Portfolios",
            "Add Sharpe Ratio [Markowitz PF Only]"),
        plotFUN = c(
            ".fportfolio.plot.1", ".fportfolio.plot.2", ".fportfolio.plot.3",
            ".fportfolio.plot.4", ".fportfolio.plot.5", ".fportfolio.plot.6", 
            ".fportfolio.plot.7", ".fportfolio.plot.8"),
        which = which)

    # Return Value:
    invisible(x)
}


# ------------------------------------------------------------------------------


.fportfolio.plot.1 <-
    function(x)
{
    # Description:
    #   Plot Efficient Frontier

    # FUNCTION:

    # Control:
    con = attr(x, "control")
    Type = getType(x)
    if (Type == "MV") {
        xLab = "Mean-Var Target Risk"
    } else if (Type == "CVaR") {
        xLab = "-CVaR Target Risk"
    }

    # Plot:
    frontierPlot(
        object = x, xlim = con$xlim, ylim = con$ylim, 
        pch = 19, cex = 0.75, title = FALSE,
        las = ifelse(is.null(con$las), 0, con$las))
    title(
        main = ifelse(is.null(con$main), "Efficient Frontier", con$main),
        xlab = ifelse(is.null(con$xlab), xLab, con$xlab),
        ylab = ifelse(is.null(con$ylab), "Target Return", con$ylab))

}


# ------------------------------------------------------------------------------


.fportfolio.plot.2 <-
    function(x)
{
    # Description:
    #   Add Minimum Risk Portfolio

    # FUNCTION:

    # Control:
    con = attr(x, "control")

    # Plot:
    minvariancePoints(
        object = x,
        col = con$minvariance.col,
        cex = con$minvariance.cex,
        pch = 19)
}


# ------------------------------------------------------------------------------


.fportfolio.plot.3 <-
    function(x)
{
    # Description:
    #   Add Tangency Portfolio

    # FUNCTION:

    # Control:
    con = attr(x, "control")

    # Plot:
    tangencyPoints(
        object = x,
        col = con$tangency.col,
        cex = con$tangency.cex,
        pch = 17)
    tangencyLines(
        object = x,
        col = con$tangency.col,
        cex = con$tangency.cex)
}


# ------------------------------------------------------------------------------


.fportfolio.plot.4 <-
    function(x)
{
    # Description:
    #   Add Risk/Return of Single Assets

    # FUNCTION:

    # Control:
    con = attr(x, "control")

    # Plot:
    Palette = match.fun(con$singleAsset.col)
    col = Palette(getNAssets(x))
    singleAssetPoints(
        object = x,
        col = col,
        cex = con$singleAsset.cex,
        pch = 18)
}


# ------------------------------------------------------------------------------


.fportfolio.plot.5 <-
    function(x)
{
    # Description:
    #   Add Equal Weights Portfolio

    # FUNCTION:

    # Control:
    con = attr(x, "control")

    # Plot:
    equalWeightsPoints(
        object = x,
        col = con$equalWeights.col,
        cex = con$equalWeights.cex,
        pch = 15)
}


# ------------------------------------------------------------------------------


.fportfolio.plot.6 <-
    function(x)
{
    # Description:
    #   Add Two Asset Frontiers [0-1 PF Only]

    # FUNCTION:

    # Control:
    con = attr(x, "control")

    # Lines:
    lines(frontierPoints(object = x), col = "grey")
    twoAssetsLines(object = x, col = con$twoAssets.col)

    # Points:
    Palette = match.fun(con$singleAsset.col)
    col = Palette(getNAssets(x))
    singleAssetPoints(
        object = x,
        col = col,
        cex = con$singleAsset.cex,
        pch = 18)
}


# ------------------------------------------------------------------------------


.fportfolio.plot.7 <-
    function(x)
{
    # Description:
    #   Add Monte Carlo Portfolios

    # FUNCTION:

    # Control:
    con = attr(x, "control")

    # Plot:
    monteCarloPoints(
        object = x,
        col = con$monteCarlo.col,
        cex = con$monteCarlo.cex,
        mcSteps = con$mcSteps)
}


# ------------------------------------------------------------------------------


.fportfolio.plot.8 <-
    function(x)
{
    # Description:
    #   Add Sharpe Ratio [MV PF Only]
    # FUNCTION:

    # Control:
    con = attr(x, "control")

    # Plot:
    sharpeRatioLines(
        object = x,
        col = con$sharpeRatio.col,
        cex = con$sharpeRatio.cex,
        lty = 3)
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the 
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General 
# Public License along with this library; if not, write to the 
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
# MA 02111-1307 USA


################################################################################
# FUNCTION:                DESCRIPTION:
#  getData                  Extracts data slot
#   getSeries                Extracts assets series data 
#   getNAssets               Extracts number of assets from data
#   getUnits                 Extracts assets names from data
# FUNCTION:                DESCRIPTION:
#  getStatistics            Extracts statistics slot
#   getMean                  Extracs mean from statistics
#   getCov                   Extracs covariance Sigma from statistics
#   getMu                    Extracs mu from statistics
#   getSigma                 Extracs Sigma from statistics
#   getEstimator             Extracts estimator from 
# FUNCTION:                DESCRIPTION:
#  getTailRisk               Extracts tailRisk slot
################################################################################


# fPFOLIODATA:

# data = list(
#   series
#   nAssets
#   names)

# statistics = list(
#   mean,
#   Cov,
#   mu,
#   Sigma,
#   estimator) 

# tailRisk = list()  


# ------------------------------------------------------------------------------t


getData.fPFOLIODATA <- function(object) object@data
# Extracts the @data slot from a fPFOLIODATA object
getSeries.fPFOLIODATA <- function(object) object@data$series   
getNAssets.fPFOLIODATA <- function(object) object@data$nAssets
getUnits.fPFOLIODATA <- function(x) x@data$names


# ------------------------------------------------------------------------------


getStatistics.fPFOLIODATA <- function(object) object@statistics
# Extracts the @statistics slot from a fPFOLIODATA object
getMean.fPFOLIODATA <- function(object) object@statistics$mean
getCov.fPFOLIODATA <- function(object) object@statistics$Cov
getEstimator.fPFOLIODATA <- function(object) object@statistics$estimator
getMu.fPFOLIODATA <- function(object) object@statistics$mu
getSigma.fPFOLIODATA <- function(object) object@statistics$Sigma


# ------------------------------------------------------------------------------


getTailRisk.fPFOLIODATA <- function(object) object@tailRisk
# Extracts the @tailRisk slot from a fPFOLIODATA object 

################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the 
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General 
# Public License along with this library; if not, write to the 
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
# MA 02111-1307 USA


################################################################################
# FUNCTION:                     DESCRIPTION:
#  getModel                      Extract whole model slot
#   getType                       Extract portfolio type from specification 
#   getOptimize                   Extract what to optimize from specification
#   getEstimator                  Extract type of covariance estimator
#   getTailRisk                   Extract list of tail dependency risk matrixes
#   getParams                     Extract parameters from specification
#    getAlpha                      Extracts target VaR-alpha specification
#    getA                          Extracts quadratic LPM Exponent
# FUNCTION:                     DESCRIPTION:
#  getPortfolio                  Extract whole portfolio slot
#   getWeights                    Extracts weights from a portfolio object
#   getTargetReturn               Extracts target return from specification
#   getTargetRisk                 Extracts target riks from specification
#   getRiskFreeRate               Extracts risk free rate from specification 
#   getNFrontierPoints            Extracts number of frontier points 
#   getStatus                     Extracts portfolio status information
# FUNCTION:                     DESCRIPTION:
#  getOptim                       Extract whole optim slot
#   getSolver                     Extracts solver from specification
#   getObjective                  Extracs name of objective function
#   getOptions                    Extracs options              
#   getControl                    Extracs control list parameters
#   getTrace                      Extracts solver's trace flag
# FUNCTION:                     DESCRIPTION:
#  getMessages                    Extract whole messages slot
################################################################################


# fPFOLIOSPEC:

# model = list(
#   type = "MV",
#   optimize = "minRisk",
#   estimator = "covEstimator",
#   tailRisk = NULL,
#   params = list(alpha = 0.05, a = 1))

# portfolio = list(
#   weights = NULL, 
#   targetReturn = NULL, 
#   targetRisk = NULL, 
#   targetAlpha = NULL,
#   riskFreeRate = 0, 
#   nFrontierPoints = 50,
#   status = 0)

# optim = list(
#   solver = "solveRquadprog",
#   objective = NULL,
#   options = list(meq=2), 
#   control = list(),
#   trace = FALSE)

# messages = list(NULL) 


# ------------------------------------------------------------------------------


getModel.fPFOLIOSPEC <- function(object) object@model
getType.fPFOLIOSPEC <- function(object) object@model$type[1]
getOptimize.fPFOLIOSPEC <- function(object) object@model$optimize
getEstimator.fPFOLIOSPEC <- function(object) object@model$estimator
getTailRisk.fPFOLIOSPEC <- function(object) object@model$tailRisk
getParams.fPFOLIOSPEC <- function(object) object@model$params
getAlpha.fPFOLIOSPEC <- function(object) object@model$params$alpha
getA.fPFOLIOSPEC <- function(object) object@model$params$a 

.getEstimatorFun <- function(object) match.fun(getEstimator(object))


# ------------------------------------------------------------------------------


getPortfolio.fPFOLIOSPEC <- function(object) object@portfolio
getWeights.fPFOLIOSPEC <- function(object) object@portfolio$weights
getTargetReturn.fPFOLIOSPEC <- function(object) object@portfolio$targetReturn
getTargetRisk.fPFOLIOSPEC <- function(object) object@portfolio$targetRisk
getRiskFreeRate.fPFOLIOSPEC <- function(object) object@portfolio$riskFreeRate
getNFrontierPoints.fPFOLIOSPEC <- function(object) object@portfolio$nFrontierPoints
getStatus.fPFOLIOSPEC <-  function(object) object@portfolio$status


# ------------------------------------------------------------------------------


getOptim.fPFOLIOSPEC <- function(object) object@optim
getSolver.fPFOLIOSPEC <- function(object) object@optim$solver 
getObjective.fPFOLIOSPEC <- function(object) object@optim$objective 
getOptions.fPFOLIOSPEC <- function(object) object@optim$options  
getControl.fPFOLIOSPEC <- function(object) object@optim$control
getTrace.fPFOLIOSPEC <- function(object) object@optim$trace


# ------------------------------------------------------------------------------


getMessages.fPFOLIOSPEC <- function(object) object@messages


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the 
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General 
# Public License along with this library; if not, write to the 
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
# MA 02111-1307 USA


################################################################################
# FUNCTION:                DESCRIPTION:                                         
#  getData                  Extracts data slot                                  
#   getSeries                Extracts assets series data                        
#   getNAssets               Extracts number of assets from data                
#   getUnits                 Extracts assets names from data                    
#  getStatistics            Extracts statistics slot                            
#   getMean                  Extracs mean from statistics                       
#   getCov                   Extracs covariance Sigma from statistics           
#   getMu                    Extracs mu from statistics                         
#   getSigma                 Extracs Sigma from statistics                      
#   getEstimator             Extracts estimator from                            
#  getTailRisk              Extracts tailRisk slot                              
# FUNCTION:                DESCRIPTION:                                         
#  getSpec                  Extracs specification Slot                          
#   getType                  Extracts type of portfolio                         
#   getOptimize              Extracts what to optimize of portfolio             
#   getEstimator             Extracts mean-covariance estimator                 
#   getParams                Extracts optional parameter list                   
#    getAlpha                 Extracts target VaR-alpha specification           
#    getA                     Extracts quadratic LPM exponent specification     
#  getPortfolio             Extract portfolio slot                              
#   getWeights               Extracts weights from a portfolio object           
#   getTargetReturn          Extracts target return from specification          
#   getTargetRisk            Extracts target riks from specification            
#   getRiskFreeRate          Extracts risk free rate from specification         
#   getNFrontierPoints       Extracts number of frontier points                 
#   getStatus                Extracts portfolio status information              
#  getOptim                 Extract optim slot                                  
#   getSolver                Extracts solver from specification                 
#   getObjective             Extracts objective
#   getOptions               Extracts optimization options
#   getControl               Extracts solver control options
#   getTrace                 Extracts solver's trace flag
# FUNCTION:                 DESCRIPTION:
#  getConstraints            Extracts weight constraints
# FUNCTION:                 DESCRIPTION:
#  getCovRiskBudgets         Extracts covariance risk budgets
#  getTailRiskBudgets        Extracts tail risk budgets
################################################################################


# Extract from data slot of an object of class fPORTFOLIO:

        
getData.fPORTFOLIO <- 
    function(object) object@data

getSeries.fPORTFOLIO <- 
    function(object) object@data@data$series
getNAssets.fPORTFOLIO <- 
    function(object) object@data@data$nAssets
getUnits.fPORTFOLIO <- 
  function(x) x@data@data$names
    
    
getStatistics.fPORTFOLIO <- 
    function(object) object@data@statistics 
getMean.fPORTFOLIO <- 
    function(object) object@data@statistics$mean
getCov.fPORTFOLIO <- 
    function(object) object@data@statistics$Cov
getEstimator.fPORTFOLIO <- 
    function(object) object@data@statistics$estimator
getMu.fPORTFOLIO <- 
    function(object) object@data@statistics$mu
getSigma.fPORTFOLIO <- 
    function(object) object@data@statistics$Sigma
 
    
# ------------------------------------------------------------------------------


# Extract from spec slot of an object of class fPORTFOLIO:


getSpec.fPORTFOLIO <- 
    function(object) object@spec 
getModel.fPORTFOLIO <- 
    function(object) object@spec@model
getType.fPORTFOLIO <- 
    function(object) object@spec@model$type
getOptimize.fPORTFOLIO <- 
    function(object) object@spec@model$optimize
getEstimator.fPORTFOLIO <- 
    function(object) object@spec@model$estimator
getTailRisk.fPORTFOLIO <- 
    function(object) object@spec@model$tailRisk
getParams.fPORTFOLIO <- 
    function(object) object@spec@model$params
getAlpha.fPORTFOLIO <- 
    function(object) object@spec@model$params$alpha
getA.fPORTFOLIO <- 
    function(object) object@spec@model$params$a


# DW object@spec renamed to object@portfolio
getPortfolio.fPORTFOLIO <-     
    function(object) object@portfolio@portfolio
getWeights.fPORTFOLIO <-      
    function(object) object@portfolio@portfolio$weights  
getTargetReturn.fPORTFOLIO <- 
    function(object) object@portfolio@portfolio$targetReturn
getTargetRisk.fPORTFOLIO <-   
    function(object) object@portfolio@portfolio$targetRisk
getRiskFreeRate.fPORTFOLIO <- 
    function(object) object@spec@portfolio$riskFreeRate
getNFrontierPoints.fPORTFOLIO <- 
    function(object) object@spec@portfolio$nFrontierPoints
getStatus.fPORTFOLIO <-  
    function(object) object@spec@portfolio$status

    
getOptim.fPORTFOLIO <-      
    function(object) object@spec@optim
getSolver.fPORTFOLIO <-    
    function(object) object@spec@optim$solver
getObjective.fPORTFOLIO <- 
    function(object) object@spec@optim$objective 
getOptions.fPORTFOLIO <-   
    function(object) object@spec@optim$options 
getControl.fPORTFOLIO <-   
    function(object) object@spec@optim$control 
getTrace.fPORTFOLIO <-     
    function(object) object@spec@optim$trace

getCovRiskBudgets.fPORTFOLIO <- 
    function(object) object@portfolio@portfolio$covRiskBudgets


# ------------------------------------------------------------------------------


# Extract from constraints slot of an object of class fPORTFOLIO:


getConstraints.fPORTFOLIO <- 
    function(object) object@constraints@stringConstraints
    
    
getConstraintsTypes <- 
    function(object) 
{
    Constraints = getConstraints(object)
    Types = NULL
    if(!is.na(pmatch("LongOnly", Constraints))) Types = c(Types, "LongOnly") 
    if(!is.na(pmatch("Short", Constraints))) Types = c(Types, "Short") 
    if(!is.na(pmatch("minW", Constraints))) Types = c(Types, "minW") 
    if(!is.na(pmatch("maxW", Constraints))) Types = c(Types, "maxW") 
    if(!is.na(pmatch("minsumW", Constraints))) Types = c(Types, "minsumW") 
    if(!is.na(pmatch("maxsumW", Constraints))) Types = c(Types, "maxsumW") 
    if(!is.na(pmatch("minB", Constraints))) Types = c(Types, "minB") 
    if(!is.na(pmatch("maxB", Constraints))) Types = c(Types, "maxB") 
    Types
}


################################################################################


.getCovRiskBudgets.fPORTFOLIO <-  
function (object) 
{   # A function implemented by Rmetrics

    # Description:
    #   Extracts risk budgets from a portfolio object
    
    # FUNCTION:
    
    # Covariance Risk Budgets:
    weights = object@portfolio$weights
    ans = NA
    Sigma = object@data$data@statistics$Sigma
    if (is.null(dim(weights))) {
        # Single Portfolio ...
        ans1 = as.vector(weights %*% Sigma %*% weights)
        ans2 = as.vector(weights * Sigma %*% weights)
        ans = round(ans2/ans1, digits = 4)
        names(ans) = names(weights)
    } else {
        # Frontier ...
        Names = colnames(weights)
        ans = NULL
        for (i in 1:(dim(weights)[1])) {
            ans1 = as.vector(weights[i, ] %*% Sigma %*% weights[i, ])
            ans2 = as.vector(weights[i, ] * Sigma %*% weights[i, ])
            ans = rbind(ans, ans2/ans1)
        }
        colnames(ans) = Names
    }
    
    # Return Value:
    ans
}


# ------------------------------------------------------------------------------


getTailRiskBudgets.fPORTFOLIO <-  
function (object) 
{   # A function implemented by Rmetrics

    # Description:
    #   Extracts tail risk budgets from a portfolio object
    
    # Arguments:
    #   object - an object of S4 class fPORTFOLIO as returned by the
    #       functions *Portfolio().
    
    # FUNCTION:
    
    # Check if available:
    Lambda = object@spec@model$tailRisk$lower
    if (is.null(Lambda)) return(NA)
    
    # Tail Risk Budgets:
    weights = getWeights(object)
    ans = NA
    if (is.null(dim(weights))) {
        ans1 = as.vector(weights %*% Lambda %*% weights)
        ans2 = as.vector(weights * Lambda %*% weights)
        ans1 = 1
        ans = round(ans2/ans1, digits = 4)
        names(ans) = names(weights)
    }
    else {
        Names = colnames(weights)
        ans = NULL
        for (i in 1:(dim(weights)[1])) {
            ans1 = as.vector(weights[i, ] %*% Lambda %*% weights[i, ])
            ans2 = as.vector(weights[i, ] * Lambda %*% weights[i, ])
            ans1 = 1
            ans = rbind(ans, ans2/ans1)
        }
        colnames(ans) = Names
    }
    
    # Return Value:
    ans
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General 
# Public License along with this library; if not, write to the 
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
# MA  02111-1307  USA


###############################################################################
# FUNCTION:                DESCRIPTION:
#  markowitzHull            Hull for a long-only Markowitz portfolio
#  feasibleGrid             Square grid on top of the feasible set
# FUNCTION:                DESCRIPTION:
#  .scaledColors            Quantile color scaling 
################################################################################


markowitzHull <-
    function (data, nFrontierPoints=50)
{
    # Description:
    #    Returns the Hull for a long-only Markowitz portfolio
    
    # Arguments:
    #    data - an object of class 'timeSeries'
   
    # Example:
    #    hull <- markowitzHull(100*LPP2005.RET[, 1:6], nFrontierPoints=11) 
    #    plot(hull[[1]], type="n"); polygon(hull[[1]], col="grey")
    
    # FUNCTION:
    
    # Check:
    stopifnot(is.timeSeries(data))   
    
    # Compute Frontier and Minimum Variance Locus:
    Spec <- portfolioSpec()
    setNFrontierPoints(Spec) <- nFrontierPoints
    frontier <- portfolioFrontier(data, spec=Spec)
    Risks <- risks <- frontierPoints(frontier)[, 1]
    Returns <- frontierPoints(frontier)[, 2]
    
    # Compute Maximum Variance Locus - Pairwise Assets Approach:
    N <- ncol(data)
    for (i in 1:(N - 1)) for (j in (i + 1):N) {
        Data <- data[, c(i, j)]
        ans <- portfolioFrontier(Data, spec=Spec)
        coord <- frontierPoints(ans)
        nextFrontier <- approx(coord[, 2], coord[, 1], xout = Returns)$y
        naIndex <- which(is.na(nextFrontier))
        nextFrontier[naIndex] <- Risks[naIndex]
        risks <- rbind(risks, nextFrontier)
    }
    
    # Hull:
    targetReturn <- Returns
    minTargetRisk <- Risks
    maxTargetRisk <- colMaxs(risks)
    hull <- cbind(
        targetReturn = Returns, 
        minTargetRisk = Risks,
        maxTargetRisk = colMaxs(risks))
    
    # Polygon:
    polygon <- cbind(
        c(minTargetRisk, rev(maxTargetRisk)[-1]), 
        c(targetReturn, rev(targetReturn)[-1]) )
    rownames(polygon) <- 1:nrow(polygon)
    colnames(polygon) <- c("targetRisk", "targetReturn")
    
    # Return Value:
    ans <- polygon 
    attr(ans, "data") <- data
    attr(ans, "hull") <- hull
    attr(ans, "frontier") <- frontier
    invisible(ans)
}


# -----------------------------------------------------------------------------


feasibleGrid <-
    function(hull, trace=FALSE)
{
    # Description:
    #    Returns best diversified portfolios on top of the feasible Set
    
    # Arguments:
    #    hull - an object as returned from the function markowitzHull
    #    trace - a logical, should the function be traced ?
    
    # Example:
    #    hull <- markowitzHull(100*LPP2005.RET[, 1:6], nFrontierPoints=21) 
    #    grid <- feasibleGrid(hull, TRUE) 
    
    # FUNCTION:
    
    # Data:
    polygon <- hull
    data <- attr(hull, "data")
    hull <- attr(hull, "hull")
    
    # Trace Hull:
    if (trace) {
        plot(polygon)
        box(col="white")
        polygon(polygon, col="grey")
        grid()
    }
    
    # Settings:
    minRisks <- as.vector(hull[, 2])
    maxRisks <- as.vector(hull[, 3])
    minRisk <- min(minRisks)
    maxRisk <- max(maxRisks)
    targetRisks <- seq(minRisk, maxRisk, length = length(minRisks))
    targetReturns <- as.vector(hull[, 1])
    N <- length(targetReturns)
    
    # Get Weights on Grid:
    Grid <- matrix(NA, ncol=N, nrow=N)
    offset <- diff(range(targetRisks[1:2]))/2
    for (i in 1:N) {
        targetReturn <- targetReturns[i]
        for (j in 1:N) {
            targetRisk <- targetRisks[j] + offset
            if (targetRisk >= minRisks[i] && targetRisk <= maxRisks[i]) {
                Grid[j, i] <- 1
                if (trace) points(targetRisk, targetReturn, pch=19)
            }
       }
    }
    
    # Return Value:
    ans <- list(x=targetRisks, y=targetReturns, z=Grid)
    attr(ans, "data") <- data
    attr(ans, "polygon") <- polygon
    attr(ans, "hull") <- hull
    class(ans) <- c("feasibleGrid", "list")
    invisible(ans)
}


# ------------------------------------------------------------------------------


.scaledColors <- 
    function(surface, palette=topo.colors, nlevels=11)
{
    # Description:
    #   scales a color palette
    
    # Arguments:
    #    surface - a list with x,y positions and z values
    #    palette - color palette function
    #    bin - quantile bin width of contour levels
    
    # FUNCTION:
    
    # Extract Surface Risk Values:
    Z <- as.vector(surface$z)
  
    # Scale by Equidistant Quantiles:
    levels <- quantile(Z, probs=seq(from=0, to=1, length=nlevels), na.rm=TRUE)
    
    # Compose Color Palette:
    palette <- palette(nlevels-1)
  
    # Return Value:
    list(palette=palette, levels=levels)
}


###############################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################
# FUNCTION:                    DESCRIPTION:
#   show.fPFOLIOBACKTEST        Print method for 'fPFOLIOBACKTEST' objects
################################################################################


setMethod("show", "fPFOLIOBACKTEST",
function(object)
{
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    #   S4 Print Method for an object of class "fPFOLIODATA"

    # Arguments:
    #   object - an object of class "fPFOLIOSPEC"

    # FUNCTION:

    # Windows:
    cat("\nBacktest Specification:\t")
    cat("\n\n Windows Function:           ",
        object@windows$windows)
    cat("\n Windows Params:\t")
    winParams <- object@windows$params
    paramNames <- names(winParams)
    chars <- nchar(paramNames)
    for (i in seq(along = winParams)){
        cat("\n  -", paramNames[i], 
            paste(rep(" ", 24-nchar(paramNames[i])),collapse = ""), 
            as.character(winParams[[i]]))
    }
 
    # Strategy:
    cat("\n\n Strategy Function:          ",
        object@strategy$strategy)
    cat("\n Strategy Params:\t")
    strategyParams <- object@strategy$params
    paramNames <- names(strategyParams)
    for (i in seq(along = strategyParams)){
        cat("\n  -", paramNames[i], 
        paste(rep(" ", 24-nchar(paramNames[i])),collapse = ""), 
        as.character(strategyParams[[i]]))      
    }     
    
    # Smoother:
    cat("\n\n Smoother Function:          ",
        object@smoother$smoother)
    cat("\n Smoother Params:\t")
    smootherParams <- object@smoother$params
    paramNames <- names(smootherParams)
    for (i in seq(along = smootherParams)){
        cat("\n  -", paramNames[i], 
        paste(rep(" ", 24-nchar(paramNames[i])),collapse = ""), 
        substr(as.character(smootherParams[[i]]), 1, 5))      
    }     
    
    # Messages:
    cat("\n\n Messages:                   ",
        unlist(object@messages), "\n\n")
   
    # Return Value:
    invisible(object)
})


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################
# FUNCTION:                       DESCRIPTION:
#  "fPFOLIOBACKTEST"               S4 Portfolio Backtest Class
################################################################################


setClass("fPFOLIOBACKTEST",
   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    #   Represens S4 fPFOLIOBACKTEST Class
     
    representation(
        windows = "list",
        strategy = "list",
        smoother = "list",
        messages = "list")
    )

    
################################################################################

   

################################################################################
# FUNCTION:                DESCRIPTION:
#  portfolioSpec            Returns an object of class fPFOLIOSPEC
#  .checkWeights            Checks and forces tiny weights to zero
#  .checkSpecVsConstraints  Checks if spec and constraints do match
#  .checkTargetReturn       Checks if target Return is defined
################################################################################


portfolioSpec <-
function(
    model = list(
        type = "MV",                   # Alt: "LPM", "CVaR", "LPM"
        optimize = "minRisk",          # Alt: "maxReturn"
        estimator = "covEstimator",    # Alt: "shrinkEstimator", 
        tailRisk = list(),
        ## I think the LPM a=1 we don't need.
        ## params = list(alpha = 0.05, a = 1)),
        ## This is CVaR's alpha it should not be removed here, it is
        ## ... required in any situation when it comes to the calculation
        ## ... of VaR and CVaR Risk
        params = list(alpha = 0.05)),
    portfolio = list(
        ## What is the here the meaning of NULL and what makes the 
        ## difference here between NULL and NA?
        ## ... NULL means, None of the three
        weights = NULL,
        targetReturn = NULL,
        targetRisk = NULL,
        # Risk Free rate ...
        riskFreeRate = 0,
        nFrontierPoints = 50,
        ## Should not be prined if NA
        status = NA),
    optim = list(
        solver = "solveRquadprog",  # Alt: "solveRsolnp"
                                    #      "solveRglpk", 
                                    #      "solveRsymphony"
                                    #      "solveRsocp" ...
       
        objective = c(
             "portfolioObjective", 
             "portfolioReturn", 
             "portfolioRisk"),
        ## Should not be printed if type != MV and solver != quadprog
        options = list(meq = 2),
        control = list(),
        trace = FALSE),
    messages = list(
        messages = FALSE,
        note = ""),
    ampl = list(
        ampl = FALSE,
        project = "ampl",
        solver = "ipopt",
        protocol = FALSE,
        trace = FALSE)
    )
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Specifies a portfolio to be optimized

    # Example:
    #   portfolioSpec(portfolio = list(targetReturn = 1.5))

    # FUNCTION:

    # Compose Checklists:
    # model.type = c("MV", "CVaR")
    # model.estimator.mean = "mean"
    # model.estimator.cov = c("cov", "mcd", "Mcd", "shrink")
    # optim.solver = c("solveRquadprog", "solveRglpk")
    # optim.trace = FALSE

    # Check Arguments:
    # stopifnot(model$type %in% model.type)
    # stopifnot(model$estimator[1] %in% model.estimator.mean)
    # stopifnot(model$estimator[2] %in% model.estimator.cov)
    # stopifnot(optim$solver %in% optim.solver)

    # Model Slot:
    Model = list(
        type = "MV",
        optimize = "minRisk",
        estimator = "covEstimator",
        tailRisk = NULL,
        params = list())
    model$type = model$type[1]
    Model[(Names <- names(model))] <- model

    # Portfolio Slot:
    Portfolio = list(
        weights = NULL,
        targetReturn = NULL,
        targetRisk = NULL,
        riskFreeRate = 0,
        nFrontierPoints = 50,
        status = 0)
    Portfolio[(Names <- names(portfolio))] <- portfolio

    # Check Portfolio - weights, targetReturn, targetRisk:
    # ... at least two of them must be set to NULL!
    checkPortfolio = 0
    if(!is.null(portfolio$weights)) checkPortfolio = checkPortfolio + 1
    if(!is.null(portfolio$targetReturn)) checkPortfolio = checkPortfolio + 1
    stopifnot(checkPortfolio <= 1)

    # Optim Slot:
    Optim = list(
        solver = "solveRquadprog",
        objective = NULL,
        options = list(meq = 2),
        control = list(),
        trace = FALSE)
    Optim[(Names <- names(optim))] <- optim
    
    # Messages Slot:
    Messages = list(
        list = NULL)
    Messages[(Names <- names(messages))] <- messages

    # Return Value:
    new("fPFOLIOSPEC",
        model = Model,
        portfolio = Portfolio,
        optim = Optim,
        messages = messages,
        ampl = ampl)
}


# ------------------------------------------------------------------------------


.checkSpec <-
function(spec)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Checks for specification conflicts
    
    # FUNCTION:
    
    if (getSolver(spec) == "solveRglpk" && getType(spec) == "MV") {
        # Error Message:
        cat("\nExecution stopped:")   
        cat("\n  Specification conflict for portfolio solver and type.")
        cat("\nSpec Information:")
        cat("\n  Solver=", getSolver(spec), ",", " type = ", getType(spec), 
            ".", sep = "")
        cat("\n")
        stop(call. = FALSE, show.error.messages = "\n  returned from Rmetrics")
     }
     
     if (getSolver(spec) == "solveRsymphony" && getType(spec) == "MV") {
        # Error Message:
        cat("\nExecution stopped:")   
        cat("\n  Specification conflict for portfolio solver and type.")
        cat("\nSpec Information:")
        cat("\n  Solver=", getSolver(spec), ",", "type = ", getType(spec), 
            ".", sep = "")
        cat("\n")
        stop(call. = FALSE, show.error.messages = "\n  returned from Rmetrics")
     }
     
     # Return Value:
     "ok"
}


# ------------------------------------------------------------------------------


.checkWeights <-
function(weights, eps = sqrt(.Machine$double.eps))
{    
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Sets tiny weights to zero
    
    # Arguments:
    #   weights - a numeric vector of portfolio weights
    #   eps - a numeric value, lower bounds of weigths
    
    # FUNCTOION:
    
    # Check:
    for(i in 1:length(weights)) {
        if(abs(weights[i]) < eps) weights[i] = 0
    }
    
    # Return Value:
    weights
}


# ------------------------------------------------------------------------------


.checkSpecVsConstraints <-
function(spec, constraints)
{    
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Check if spec and constraints do match
    
    # Arguments:
    #   spec - portfolio specification as fPFOLIOSPEC object
    #   constraints - as charvec or as fPFOLIOSPEC object
    
    # FUNCTOION:
    
    # Check:
    if(inherits(constraints,"fPFOLIOCON"))
        constraints = constraints@stringConstraints
    if(any(constraints == "Short")) {
        stopifnot(getSolver(spec) == "solveRshortExact")
    }
    
    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------


.checkTargetReturn <-
function(spec)
{    
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Check if target Return is defined
    
    # Arguments:
    #   spec - specification object
    
    # FUNCTOION:
    
    # Check:
    targetReturn = getTargetReturn(spec)
    if(is.null(targetReturn))
        stop("The target return is not available")
    
    # Return Value:
    invisible(targetReturn)
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################
# FUNCTION:                    DESCRIPTION:
#  rnlminb2NLP                  Rmetrics Interface for NLMINB2 LP solvers 
#  nlminb2NLP                   Convenience wrapper for NLMINB2 LP solvers
#  nlminb2Control               NLMINB2 LP control parameter list
#  nlminb2                      Code imported from Rnlminb2 on R-Forge
#  rnlminb2                     Synonym for nlminb2 function
################################################################################


rnlminb2NLP <-
    function(start, objective, lower=0, upper=1, linCons, funCons, 
        control=list())
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Function wrapper for solver nlminb2()

    # FUNCTION:

    # Update Control List:
    ctrl <- nlminb2NLPControl()
    if (length(control) > 0)
        for (name in names(control)) ctrl[name] = control[name]
    control <- ctrl
    BIG <- 1e6
    N <- length(start)

    # Box Constraints:
    if(length(lower) == 1) {
        par.lower <- rep(lower, N)
    } else {
        par.lower <- lower
    }
    if(length(upper) == 1) {
        par.upper <- rep(upper, N)
    } else {
        par.upper <- upper
    }
    par.lower[is.infinite(par.lower)] <- BIG*sign(par.lower[is.infinite(par.lower)])
    par.upper[is.infinite(par.upper)] <- BIG*sign(par.upper[is.infinite(par.upper)])
  
    # Linear Constraints:
    if(missing(linCons)) {
        eqA <- ineqA <- NULL
        eqA.bound <- ineqA.lower <- ineqA.upper <- NULL
    } else {
        mat <- linCons[[1]]
        lower <- linCons[[2]]
        upper <- linCons[[3]]
        
        if(length(lower) == 1) {
        lower <- rep(lower, N)
        } else {
            lower <- lower
        }
        if(length(upper) == 1) {
            upper <- rep(upper, N)
        } else {
            upper <- upper
        }
        lower[is.infinite(lower)] <- BIG*sign(lower[is.infinite(lower)])
        upper[is.infinite(upper)] <- BIG*sign(upper[is.infinite(upper)])
        
        eqIndex <- which(lower == upper)
        ineqIndex <- which(lower != upper)
        
        if (length(eqIndex) == 0) {
            eqA <- NULL
            eqA.bound <- NULL
        } else {
            eqA <- mat[eqIndex, ]
            eqA.bound <- lower[eqIndex]
        }
        if (length(ineqIndex) == 0) {
            ineqA <- NULL
            ineqA.lower <- NULL
            ineqA.upper <- NULL
        } else {
            ineqA <- mat[ineqIndex, ]
            ineqA.lower <- lower[ineqIndex]
            ineqA.upper <- upper[ineqIndex]
        }
    }

    # Nonlinear Constraints:
    if(missing(funCons)) {
        eqFun <- ineqFun <- list()
        eqFun.bound <- ineqFun.lower <- ineqFun.upper <- NULL
    } else {
        fun <- funCons[[1]]
        lower <- funCons[[2]]
        upper <- funCons[[3]]
        eqIndex <- which(lower == upper)
        ineqIndex <- which(lower != upper)
        if (length(eqIndex) == 0) {
            eqFun <- list()
            eqFun.boud <- NULL
        } else {
            eqFun <- fun[eqIndex]
            eqFun.bound <- lower[eqIndex]
        }
        if (length(ineqIndex) == 0) {
            ineqFun <- list()
            ineqFun.lower <- NULL
            ineqFun.upper <- NULL
        } else {
            ineqFun <- fun[ineqIndex]
            ineqFun.lower <- lower[ineqIndex]
            ineqFun.upper <- upper[ineqIndex]
        }
    }

    # Optimize Portfolio:
    optim <- nlminb2NLP(
        start = start,
        objective = objective,
        par.lower = par.lower,
        par.upper = par.upper,
        eqA = eqA,
        eqA.bound = eqA.bound,
        ineqA = ineqA,
        ineqA.lower = ineqA.lower,
        ineqA.upper = ineqA.upper,
        eqFun = eqFun,
        eqFun.bound = eqFun.bound,
        ineqFun = ineqFun,
        ineqFun.lower = ineqFun.lower,
        ineqFun.upper = ineqFun.upper,
        control = control)

    # Return Value:
    value <- list(
        opt = optim,
        solution = optim$solution,
        objective = objective(optim$solution),
        status = optim$status,
        message = optim$message,
        solver = "nlminb2NLP")
    class(value) <- c("solver", "list")
    value
}


###############################################################################


nlminb2NLP <-
    function(
        start, objective,  
        par.lower = NULL, par.upper = NULL,
        eqA = NULL, eqA.bound = NULL,
        ineqA = NULL, ineqA.lower = NULL, ineqA.upper = NULL,
        eqFun = list(), eqFun.bound = NULL,
        ineqFun = list(), ineqFun.lower = NULL, ineqFun.upper = NULL,
        control = list())
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Function wrapper for solver nlminb2()

    # FUNCTION:

    # Environment:
    env <- .GlobalEnv

    # Update Control List:
    ctrl <- nlminb2NLPControl()
    if (length(control) > 0)
        for (name in names(control)) ctrl[name] = control[name]
    control <- ctrl
    N <- length(start)
    
    # Set Box Constraints:
    if (is.null(par.lower)) par.lower <- -Inf
    if (is.null(par.upper)) par.upper <- +Inf
    if (length(par.lower) == 1) par.lower <- rep(par.lower, N)
    if (length(par.upper) == 1) par.upper <- rep(par.upper, N)

    # Set Linear and Function Equality Constraints:
    if (!is.null(eqA) || length(eqFun) > 0) {
        eqfun <- function(x) {
            ans <- NULL
            if(!is.null(eqA)) {
                ans <- c(ans, eqA %*% x - eqA.bound)
            }
            if (length(eqFun) > 0)
                for (i in 1:length(eqFun))
                    ans <- c(ans, eqFun[[i]](x) - eqFun.bound[i])
            return(as.double(eval(ans, env)))
        }
    } else {
        eqfun <- NULL
    }

    # Set Linear and Function Inequality Constraints:
    if (!is.null(ineqA) || length(ineqFun) > 0) {
        leqfun <- function(x) {
            ans <- NULL
            if(!is.null(ineqA)) 
                ans <- c(ans, +ineqA %*% x - ineqA.upper)
            if(!is.null(ineqA)) 
                ans <- c(ans, -ineqA %*% x + ineqA.lower)
            if (length(ineqFun) > 0)
                for (i in 1:length(ineqFun))
                    ans <- c(ans, +ineqFun[[i]](x) - ineqFun.upper[i])
            if (length(ineqFun) > 0)
                for (i in 1:length(ineqFun))
                    ans <- c(ans, -ineqFun[[i]](x) + ineqFun.lower[i])
            return(as.double(eval(ans, env))) }
    } else {
        leqfun <- NULL
    }

    # Optimize Portfolio:
    optim <- rnlminb2(
        start = start,
        objective = objective,
        eqFun = eqfun,
        leqFun = leqfun,
        lower = par.lower,
        upper = par.upper,
        gradient = NULL,
        hessian = NULL,
        control = control,
        env = env)
    names(optim$par) <- names(start)

    # Return Value:
    value <- list(
        opt = optim,
        solution = optim$par,
        objective = objective(optim$par)[[1]],
        convergence = optim$convergence,
        message = optim$message,
        solver = "nlminb2NLP")
    class(value) <- c("solver", "list")
    
    # Return Value:
    value
}


################################################################################

## Description: 
##     NLMINB2 is an interior point nonlinear constrained programming 
##     problem interface written by Dietgelm Wuertz. The version here 
##     calls the R function nlminb from R's base environment. 
## Author: 
##     Douglas Bates and Deepayan Sarkar have written the nlminb R Port.
##     David M. Gay has written the underlying Fortran code.
##     Diethelm Wuertz has added nonlinear constraints functionality nlminb2.
##     To facilitate CRAN checking, Stefan Theussl integrated the code into
##     fPortfolio.


rnlminb2 <- function(...) {
 fPortfolio::nlminb2(...)
}

nlminb2 <- function(
    start, objective, 
    
    eqFun = NULL, 
    leqFun = NULL,
     
    lower = -Inf, 
    upper = Inf,
    
    gradient = NULL, 
    hessian = NULL, 
    
    control = list(),
    
    env = .GlobalEnv)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Nonlinear programming with nonlinear constraints
    
    # Details:
    #                        min f(x)
    #
    #                 lower_i < x_i < upper_i
    #    s/t                h_i(x)  = 0
    #                       g_i(x) <= 0
    
    # Arguments:
    #   start - numeric vector of start values
    #   objective - objective function to be minimized f(x)
    #   eqFun - equal constraint functions h_i(x) = 0
    #   leqFun - less equal constraint functions g_i(x) <= 0
    #   lower, upper - lower and upper bounds
    #   gradient - optional gradient of f(x)
    #   hessian - optional hessian of f(x)
    #   scale - control parameter
    #   control - control list
    #       eval.max - maximum number of evaluations (200)
    #       iter.max - maximum number of iterations (150) 
    #       trace - value of the objective function and the parameters 
    #           is printed every trace'th iteration (0)
    #       abs.tol - absolute tolerance (1e-20)
    #       rel.tol - relative tolerance (1e-10) 
    #       x.tol - X tolerance (1.5e-8)
    #       step.min - minimum step size (2.2e-14)
    
    # Todo:
    #   R, N and alpha should become part of the control list.
   
    # FUNCTION:
    
    # Debug:
    DEBUG = FALSE
    
    # Control List:
    ctrl = nlminb2Control()
    if (length(control) > 0)
        for (name in names(control)) ctrl[name] = control[name]
    control = ctrl
    
    # Arg Functions:
    if (DEBUG) {
        print(eqFun)
        print(eqFun(start))
        print(leqFun)
        print(leqFun(start))
    }
    
    # Composed Objective Function:
    if (is.null(eqFun(start))) {
        type = "leq"
        fun <- function(x, r) { 
            objective(x) - 
                r * sum(.Log(-leqFun(x))) }  
    } else if (is.null(leqFun(start))) {
        type = "eq"
        fun <- function(x, r) { 
            objective(x) +
                sum((eqFun(x))^2 / r) } 
    } else {
        type = "both"
        fun <- function(x, r) { 
            objective(x) +
                sum((eqFun(x))^2 / r) - 
                r * sum(.Log(-leqFun(x))) }  
    }
    
    # Compute in global environment:
    fun2 = function(x, r) {
        return(as.double(eval(fun(x, r), env)))
    }
       
    # Debug: 
    if (DEBUG) {
        print(fun)
        print(fun(start, 1))
    }
       
    # Minimization:
    steps.tol <- control$steps.tol
    R <- control$R
    beta <- control$beta
    scale <- control$scale
    
    trace = control$trace
    if (trace > 0) TRACE = TRUE else TRACE = FALSE
    
    control2 = control
    control2[["R"]] <- NULL
    control2[["beta"]] <- NULL
    control2[["steps.max"]] <- NULL
    control2[["steps.tol"]] <- NULL
    control2[["scale"]] <- NULL
    
    counts <- 0
    test <- 0
    while (counts < control$steps.max && test == 0) {
        counts = counts + 1
        ans = nlminb(
            start = start, objective = fun2, 
            gradient = gradient, hessian = hessian, 
            scale = scale, control = control2, lower = lower, upper = upper,
            r = R)
        start = ans$par
        tol = abs((fun(ans$par, R)-objective(ans$par))/objective(ans$par))
        if (!is.na(tol)) 
            if (tol < steps.tol) test = 1
        if (TRACE) {
            print(paste("counts:", counts, "R:", R))
            print(paste("   ", ans$convergence))
            print(paste("   ", ans$message))
            print(ans$par)
            print(fun(ans$par, R))
            print(objective(ans$par))
            print(tol)
        }
        R = beta * R
    } 
    
    if (TRACE) {
        print(paste("type:", type))
        cat("\n\n") 
    } 
    
    # Return Value:
    ans
} 


# ------------------------------------------------------------------------------


.Log <-
function(x) 
{
    # Description:
    #   Returns log taking care of negative values
    
    # FUNCTION:
    
    # Check for negative values:
    x[x < 0] <- 0
    
    # Return Value:
    log(x)
}


################################################################################


nlminb2Control <- 
function(
    eval.max = 500, 
    iter.max = 400,
    trace = 0,
    abs.tol = 1e-20, 
    rel.tol = 1e-10,
    x.tol = 1.5e-8, 
    step.min = 2.2e-14,
    scale = 1,
    R = 1,
    beta = 0.01,
    steps.max = 10,
    steps.tol = 1e-6)         
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns Control list
    
    # Arguments:
    #   none
    
    # FUNCTION:
    
    # Control list:
    optim <- list(
        eval.max = eval.max, 
        iter.max = iter.max,
        trace = trace,
        abs.tol = abs.tol, 
        rel.tol = rel.tol,
        x.tol = x.tol, 
        step.min = step.min,
        scale = scale,
        R = R,
        beta = beta,
        steps.max = steps.max,
        steps.tol = steps.tol)
        
   # Return Value:
   optim
}


nlminb2NLPControl <-
    function(eval.max = 500, iter.max = 400, trace = 0, abs.tol = 1e-20,
    rel.tol = 1e-10, x.tol = 1.5e-08, step.min = 2.2e-14, scale = 1,
    R = 1, beta.tol = 1e-20) #, step.beta = 20)
{
    # A function implemented by Diethelm Wuertz
    
    # FUNCTION:
    
    # Control Parameters:
    ans <- list(eval.max = eval.max, iter.max = iter.max, trace = trace,
        abs.tol = abs.tol, rel.tol = rel.tol, x.tol = x.tol,
        step.min = step.min, scale = scale, R = R, 
        beta.tol = beta.tol) #, step.beta = step.beta)
        
    # Return Value:
    ans
}


###############################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################
# FUNCTION:                    DESCRIPTION:
#  rdonlp2NLP                   Rmetrics Interface for DONLP2 LP sol
#  donlp2NLP                    Convenience wrapper for DONLP2 LP so    
#  donlp2NLPControl             DONLP2 LP control parameter list
#  rdonlp2                      Synonyme name for Rdonlp2::donlp2 function
################################################################################


rdonlp2NLP <- 
    function(
        start, objective, lower=0, upper=1, linCons, funCons, control=list())
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Function wrapper for solver donlp2()
    
    # FUNCTION:
    
    # Update Control List:
    ctrl <- donlp2NLPControl()
    if (length(control) > 0)
        for (name in names(control)) ctrl[name] = control[name]
    control <- ctrl
    N <- length(start)
    
    # Box Constraints:
    if(length(lower) == 1) {
        par.lower <- rep(lower, N)
    } else {
        par.lower <- lower
    }
    if(length(upper) == 1) {
        par.upper <- rep(upper, N)
    } else {
        par.upper <- upper
    }
    
    # Linear Constraints:  
    if(missing(linCons)) {
        eqA <- ineqA <- NULL
        eqA.bound <- ineqA.lower <- ineqA.upper <- NULL
    } else {
        mat <- linCons[[1]]
        lower <- linCons[[2]]
        upper <- linCons[[3]]
        eqIndex <- which(lower == upper)
        ineqIndex <- which(lower != upper)
        if (length(eqIndex) == 0) {
            eqA <- NULL
            eqA.bound <- NULL
        } else {
            eqA <- mat[eqIndex, ]
            eqA.bound <- lower[eqIndex]
        }
        if (length(ineqIndex) == 0) {
            ineqA <- NULL
            ineqA.lower <- NULL
            ineqA.upper <- NULL
        } else {
            ineqA <- mat[ineqIndex, ]
            ineqA.lower <- lower[ineqIndex]
            ineqA.upper <- upper[ineqIndex]
        }
    }   
    
    # Nonlinear Constraints:
    if(missing(funCons)) {
        eqFun <- ineqFun <- list()
        eqFun.bound <- ineqFun.lower <- ineqFun.upper <- NULL
    } else {
        fun <- funCons[[1]]
        lower <- funCons[[2]]
        upper <- funCons[[3]]
        eqIndex <- which(lower == upper)
        ineqIndex <- which(lower != upper)
        if (length(eqIndex) == 0) {
            eqFun <- list()
            eqFun.boud <- NULL
        } else {
            eqFun <- fun[eqIndex]
            eqFun.bound <- lower[eqIndex]
        }
        if (length(ineqIndex) == 0) {
            ineqFun <- list()
            ineqFun.lower <- NULL
            ineqFun.upper <- NULL
        } else {
            ineqFun <- fun[ineqIndex]
            ineqFun.lower <- lower[ineqIndex]
            ineqFun.upper <- upper[ineqIndex]
        }
    }
    
    # Optimize Portfolio:
    optim <- donlp2NLP(
        start = start, 
        objective = objective, 
        par.lower = par.lower, 
        par.upper = par.upper,
        eqA = eqA, 
        eqA.bound = eqA.bound,
        ineqA = ineqA, 
        ineqA.lower = ineqA.lower, 
        ineqA.upper = ineqA.upper,
        eqFun = eqFun, 
        eqFun.bound = eqFun.bound,
        ineqFun = ineqFun, 
        ineqFun.lower = ineqFun.lower, 
        ineqFun.upper = ineqFun.upper,
        control = control) 
          
    # Return Value:
    value = list(
        opt = optim,
        solution = optim$solution, 
        objective = objective(optim$solution)[[1]], 
        status = optim$status,
        message = optim$message,
        solver = "donlp2NLP")
    class(value) <- c("solver", "list")
    value
}


################################################################################


donlp2NLP <- 
    function(
        start, objective, 
        par.lower = NULL, par.upper = NULL,
        eqA = NULL, eqA.bound = NULL,
        ineqA = NULL, ineqA.lower = NULL, ineqA.upper = NULL,
        eqFun = list(), eqFun.bound = NULL,
        ineqFun = list(), ineqFun.lower = NULL, ineqFun.upper = NULL,
        control = list())
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   NLP wrapper for solver donlp2
    
    # FUNCTION:
    
    # Environment:
    env <- .GlobalEnv
    
    # Update Control List:
    ctrl <- donlp2NLPControl()
    if (length(control) > 0)
        for (name in names(control)) ctrl[name] <- control[name]
    control <- ctrl
    
    # Set Box Constraints:
    if (is.null(par.lower)) par.lower <- rep(-Inf, length(start))
    if (is.null(par.upper)) par.upper <- rep(+Inf, length(start))
    if (length(par.lower) == 1) par.lower <- rep(par.lower, length(start))
    if (length(par.upper) == 1) par.upper <- rep(par.upper, length(start))
    
    # Set Linear Equality and Inequality Constraints:
    A <- rbind(eqA, ineqA)
    lin.lower <- c(eqA.bound, ineqA.lower)
    lin.upper <- c(eqA.bound, ineqA.upper)
    
    # Set Nonlinear Equality and Inequality Constraints:
    if ((length(eqFun) + length(ineqFun)) == 0) {
        nlin <- list()
        nlin.lower <- rep(-Inf, length(nlin))
        nlin.upper <- rep(+Inf, length(nlin))
    } else {
        nlin <- list()
        if (length(eqFun) > 0) nlin = c(nlin, eqFun)
        if (length(ineqFun) > 0) nlin = c(nlin, ineqFun)
        nlin.lower <- c(eqFun.bound, ineqFun.lower)
        nlin.upper <- c(eqFun.bound, ineqFun.upper)
    }
    
    # Optimize Portfolio:
    optim <- rdonlp2(
        par = start, 
        fn = objective, 
        par.upper = par.upper, 
        par.lower = par.lower, 
        A = A, 
        lin.upper = lin.upper, 
        lin.lower = lin.lower, 
        nlin = nlin, 
        nlin.upper = nlin.upper, 
        nlin.lower = nlin.lower, 
        control = control, 
        control.fun = function(lst) {return(TRUE)}, 
        env = .GlobalEnv, 
        name = NULL)
    names(optim$par) <- names(start) 
    
    # Extract Weights:
    weights <- .checkWeights(optim$par)
    attr(weights, "invest") <- sum(weights)
    
    # Check Messages and Get Status:
    #   ... unfortunately donlp2 has no status vaqriable, 
    #       so we have to analyze the messages
    Status <- 1
    # Message <- "1234567890123456789012345"
    message11 <- "KT-conditions satisfied, " # no further correction computed"
    message12 <- "computed correction small" # , regular case"
    message13 <- "stepsizeselection: x almo" # st feasible, dir. deriv. very small"
    if (substr(optim$message, 1, 25) == message11) Status <- 0 
    if (substr(optim$message, 1, 25) == message12) Status <- 0 
    if (substr(optim$message, 1, 25) == message13) Status <- 0 
       
    # Return Value:
    value <- list(
        opt = optim,
        solution = optim$par, 
        objective = objective(optim$par)[[1]], 
        status = Status,
        message = optim$message,
        solver = "donlp2NLP")
    class(value) <- c("solver", "list")
    value
}


################################################################################


rdonlp2 <- function(...) {
  Rdonlp2::donlp2(...)
}


################################################################################


donlp2NLPControl <-
function (
    iterma = 4000, nstep = 20, fnscale = 1, report = FALSE, 
    rep.freq = 1, tau0 = 1, tau = 0.1, del0 = 1, epsx = 1e-05, 
    delmin = 0.1 * del0, epsdif = 1e-08, nreset.multiplier = 1, 
    difftype = 3, epsfcn = 1e-16, taubnd = 1, hessian = FALSE, 
    te0 = TRUE, te1 = FALSE, te2 = FALSE, te3 = FALSE, silent = TRUE, 
    intakt = TRUE) 
{
    # A function implemented by Diethelm Wuertz
    
    # FUNCTION:
    
    # Control Parameters:
    control <- list(
        iterma = iterma, nstep = nstep, fnscale = fnscale, 
        report = report, rep.freq = rep.freq, tau0 = tau0, tau = tau, 
        del0 = del0, epsx = 1e-05, delmin = delmin, epsdif = epsdif, 
        nreset.multiplier = nreset.multiplier, difftype = difftype, 
        epsfcn = epsfcn, taubnd = taubnd, hessian = hessian, 
        te0 = te0, te1 = te1, te2 = te2, te3 = te3, silent = silent, 
        intakt = intakt)
        
    # Return Value:
    control
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                    DESCRIPTION:
#  solveRshortExact             Portfolio interface to solver RshortExact
#  .rshortExactArguments        Returns arguments for solver
#  .rshortExact                 Wrapper to solver function
################################################################################


solveRshortExact <-
    function(data, spec, constraints)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Portfolio interface to solver RshortExact

    # Details:
    #   If getTargetReturn() is specified we minimze the risk,
    #   if getTargetRisk() is pecified we maximize the risk.

    # FUNCTION:

    # Convert Data and Constraints to S4 Objects:
    Data <- portfolioData(data, spec)
    data <- getSeries(Data)
    Sigma <- getSigma(Data)
    Constraints <- portfolioConstraints(Data, spec, constraints)

    # Stop if the Target Return is not Defined!
    optimize <- getOptimize(spec)
    targetReturn <- getTargetReturn(spec)
    targetRisk <- getTargetRisk(spec)

    # Get '.rshortexact' conform arguments:
    args <- .rshortExactArguments(Data, spec, Constraints)

    # Solve Portfolio:
    ans <- .rshortExact(
        optimize = optimize,
        C0 = args$C0, 
        a = args$a, 
        b = args$b, 
        c = args$c, 
        d = args$d,
        Sigma = Sigma,
        invSigma = args$invSigma, 
        mu = args$mu,
        targetReturn, 
        targetRisk)

    # Return Value:
    class(ans) <- c("solver", "list")
    ans
}


# ------------------------------------------------------------------------------


.rshortExactArguments <-
function(data, spec, constraints)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns 'shortexact' conform arguments for the solver

    # FUNCTION:

    # Data as S4 Objects:
    Data <- portfolioData(data, spec)
    data <- getSeries(Data)

    # Get Specifications:
    mu <- getMu(Data)
    Sigma <- getSigma(Data)
    weights <- getWeights(spec)
    targetReturn <- getTargetReturn(spec)
    targetRisk <- getTargetRisk(spec)

    # Parameter Settings:
    C0 <- 1
    one <- rep(1, times = length(mu))
    invSigma <- solve(Sigma)
    a <- as.numeric(mu %*% invSigma %*% mu)
    b <- as.numeric(mu %*% invSigma %*% one)
    c <- as.numeric(one %*% invSigma %*% one)
    d <- as.numeric(a*c - b^2)

    # Return Value:
    list(C0 = C0, a = a, b = b, c = c, d = d, mu = mu, invSigma = invSigma)
}


################################################################################


.rshortExact <-
    function(optimize, C0, a, b, c, d, Sigma, invSigma, mu, 
    targetReturn, targetRisk)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Analytical 'shortexact' solver function

    # FUNCTION:

    # Optimize:
    if (!is.null(targetReturn)) {
        if (optimize == "minRisk") {
            # Compute Target Risk:
            objective <- targetRisk <-
                sqrt((c*targetReturn^2 - 2*b*C0*targetReturn + a*C0^2) / d)
        } else if (optimize == "maxReturn")  {
            # Compute Target Return:
            aq <- c
            bq <- -2*b*C0
            cq <- a*C0^2 - d*targetRisk^2
            objective <- targetReturn <- (-bq + sqrt(bq^2 - 4*aq*cq)) / (2*aq)
        }
    }
    
    # Compute Weights:
    if (is.null(targetReturn)) {
        # global minimum Variance Portfolio:
        optimize <- "minvariancePortfolio"
        one <- rep(1, times=length(mu))
        weights <- as.vector(invSigma %*% one) / ( one %*% invSigma %*% one )
        objective <- t(weights) %*% Sigma %*% weights
    } else {
        weights <- as.vector(invSigma %*% ((a-b*mu)*C0 + (c*mu-b)*targetReturn )/d)
    }
    weights <- .checkWeights(weights)

    # Return Value:
    list(
        type = "MV",
        solver = "solveRshortExact",
        optim = NA,
        solution = weights,
        weights = weights,
        targetReturn = targetReturn,
        targetRisk = targetRisk,
        objective = objective,
        status = 0,
        message = optimize)
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


###############################################################################
# FUNCTION:                    DESCRIPTION:
#  ramplLP                      Rmetrics Interface for AMPL LP solvers
#  amplLP                       Convenience wrapper for AMPL LP solvers
#  amplLPControl                AMPL LP control parameter list
###############################################################################


ramplLP <- 
    function(objective, lower=0, upper=1, linCons, control=list())
{   
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Implements AMPL LP Interface
    
    # Arguments:
    #   objective - vec
    
    # FUNCTION:
    
    # Control List:
    ctrl <- amplLPControl()
    if (length(control) > 0) 
        for (name in names(control)) ctrl[name] = control[name]
    control <- ctrl
    
    # Controls:
    solver <- control$solver
    project <- control$project
    inf <- control$inf
    trace <- control$trace
    
    # Objective:
    vec <- objective
    
    # Box Constraints:
    replicate <- function(x, n) if(length(x) == 1) rep(x, n) else x
    n <- length(vec)
    x_L <- replicate(lower, n)
    x_U <- replicate(upper, n)
    x_L[is.infinite(x_L)] <- inf*sign(x_L[is.infinite(x_L)])
    x_U[is.infinite(x_U)] <- inf*sign(x_U[is.infinite(x_U)])
      
    # Linear Constraints:  
    A <- linCons[[1]]
    m <- nrow(A)
    b_L <- replicate(linCons[[2]], m)
    b_U <- replicate(linCons[[3]], m)
    b_L[is.infinite(b_L)] <- inf*sign(b_L[is.infinite(b_L)])
    b_U[is.infinite(b_U)] <- inf*sign(b_U[is.infinite(b_U)]) 
    
    # Optimize Portfolio:
    value <- amplLP(vec, x_L, x_U, A, b_L, b_U, control)
    
    # Return Value:
    value
}


# -------------------------------------------------------------------------


amplLP <- 
function(
    objective, x_L=NULL, x_U=NULL, A=NULL, b_L=NULL, b_U=NULL,
    control=list())
{   
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Universal function wrapper for AMPL LP solvers
    
    # Arguments:
    #   objective - vec
    
    # FUNCTION:
    
    # Control List:
    ctrl <- amplLPControl()
    if (length(control) > 0)
        for (name in names(control)) ctrl[name] = control[name]
    control <- ctrl
    
    # Control Parameters:
    project <- control$project
    solver <- control$solver
    inf <- control$inf
    trace <- control$trace
    
    # Objective:
    c <- objective
    n <- length(vec)
    m <- nrow(A) 

    # Assign LP Model:
    .lpAssign(project, c, x_L, x_U, A, b_L, b_U , trace=FALSE)
    
    # Run AMPL:
    command <- paste("ampl -t -vs", paste(project, "run", sep="."))
    solve <- system(command, intern=TRUE)

    # Read AMPL Output File: 
    file <- paste(project, "txt", sep = ".")
    out <- scan(file, what = character(0), sep="\n", quiet=TRUE)
  
    # Get Weights:
    Index <- (grep(";", out) - 1)[1]
    splits <- strsplit(paste(out[2:Index], collapse=" "), " ")[[1]]
    solution <- as.numeric(splits[splits != ""])[seq(2, 2*n, by=2)]
    Index <- as.numeric(splits[splits != ""])[seq(1, 2*n, by=2)]
    solution[Index] <- solution
    
    # Get Status:
    status <- strsplit(out[grep("solve_result", out)], split=" ")
    statusCode <- status[[1]][3]
    statusMessage <- status[[2]][3]
    
    # Get Solver Message:
    Index <- grep("solve_message", out):length(out)
    message <- out[Index]
    
    # Version:
    version <- system(paste(solver, "-v"), intern=TRUE) 
    
    # Compose Results into a List:
    objective <- (c %*% solution)[[1, 1]]
    
    # Return Value:
    model <- capture.output(amplModelShow(project))
    run <- capture.output(amplModelShow(project))
    value = list(
        opt = list(solve=solve, model=model, run=run, out=out),
        solution = solution, 
        objective = objective,
        status = statusCode,
        message = statusMessage,
        solver = paste("AMPL", solver),
        version = version)
    class(value) <- c("solver", "list")
    value
}

    
# -------------------------------------------------------------------------


amplLPControl <- 
    function(solver="ipopt", project="ampl", inf=1e12, trace=FALSE) 
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns AMPL LP control parameter list
    
    # FUNCTION:
    
    # Return Value:
    list(solver=solver, project=project, inf=inf, trace=trace)
}


###############################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


###############################################################################


.exampleData <-
  function()
  {
    
    dataSet <- data("LPP2005REC", package="timeSeries", envir=environment())
    LPP2005REC <- get(dataSet, envir=environment())
    
    nAssets = 6
    lppData = 100 * (as.matrix(LPP2005REC[,-1]))[1:20, 1:nAssets]
    nScenarios = nrow(lppData)
    
    
    targetReturn = mean(lppData)
    Mean = colMeans(lppData)
    Cov = cov(lppData)
    
    start = rep(1/nAssets, nAssets)
    names(start) = colnames(lppData)
    
    madObjective = c( rep(1, nScenarios), rep(0, nAssets)) / nScenarios
    markowitzObjective = list(dvec = rep(0, nAssets), Dmat = Cov)
    sharpeObjective = function(x)  { ( -Mean %*% x / sqrt ( x %*% Cov %*% x )[[1]] ) }
    
    
    markowitzFun = function(x) { ( x %*% Cov %*% x)[[1]] }
    sharpeFun = function(x)  { ( -Mean %*% x / sqrt ( x %*% Cov %*% x )[[1]] ) }
    
    
    lower = 0
    upper = 1
    
    
    # For LP with glpk  
    vec <- c( rep(1, nScenarios), rep(0, nAssets)) / nScenarios
    A <- rbind(
      c( rep(0, nScenarios), targetReturn = Mean),
      c( rep(0, nScenarios), budget = rep(1, nAssets)),
      cbind(-diag(nScenarios), Returns = lppData),
      cbind(+diag(nScenarios), Returns = lppData))
    dirA <- c( 
      rep("==", 2),
      rep("<=", nScenarios),
      rep(">=", nScenarios))      
    rhsA <- c(
      targetReturn = targetReturn, 
      budget = 1,
      lower = rep(0, nScenarios),
      upper = rep(0, nScenarios))
    
    
    # MAD with Box Constraints:
    madbox.linCons <- list(
      mat = rbind(
        c( rep(0, nScenarios), targetReturn = Mean),
        c( rep(0, nScenarios), budget = rep(1, nAssets)),
        cbind(-diag(nScenarios), Returns = lppData),
        cbind(+diag(nScenarios), Returns = lppData)),
      lower = c(
        targetReturn = targetReturn, 
        budget = 1, 
        rep(-Inf, nScenarios), 
        rep(   0, nScenarios)),
      upper = c(
        targetReturn = targetReturn, 
        budget = 1, 
        rep(  0, nScenarios), 
        rep(Inf, nScenarios)))
    
    
    
    # MAD with Box and Group Constraints:
    Bonds    = c(1, 0, 0, 1, 0, 0);  bonds    = 0.3  # >=  30%
    Foreign  = c(0, 0, 0, 1, 1, 1);  foreign  = 0.5  # <=  50%
    Equities = c(0, 0, 1, 0, 1, 1);  equities = 0.6  # <=  60%
    madgroup.linCons = list(
      mat = rbind(
        c( rep(0, nScenarios), targetReturn = Mean),
        c( rep(0, nScenarios), budget = rep(1, nAssets)),
        c( rep(0, nScenarios), bons = Bonds),
        c( rep(0, nScenarios), foreign = Foreign),
        c( rep(0, nScenarios), equities = Equities),
        cbind(negDiag = -diag(nScenarios), Returns = lppData),
        cbind(posDiag = +diag(nScenarios), Returns = lppData)),
      lower = c(
        targetReturn = targetReturn, 
        budget = 1, 
        bonds = 0.3,
        foreign = 0,
        equities = 0,
        rep(-Inf, nScenarios), 
        rep(   0, nScenarios)),
      upper = c(
        targetReturn = targetReturn, 
        budget = 1, 
        bonds = 1,
        foreign = 0.5,
        equities = 0.6,
        rep(   0, nScenarios), 
        rep(Inf, nScenarios)))  
    
    
    # QP Box Constrained Markowitz:
    Budget = rep(1, nAssets)
    investBudget = 1
    qpbox.linCons <- list(
      mat = rbind(Return = Mean, Budget = Budget),
      lower = c(Return=targetReturn, Budget=investBudget), 
      upper = c(Return=targetReturn, Budget=investBudget) )  
    
    
    # QP Box and Group Constrained Markowitz:
    Bonds    = c(1, 0, 0, 1, 0, 0);  bonds    = 0.3  # >=  30%
    Foreign  = c(0, 0, 0, 1, 1, 1);  foreign  = 0.5  # <=  50%
    Equities = c(0, 0, 1, 0, 1, 1);  equities = 0.6  # <=  60%    
    qpgroup.linCons <- list(
      mat = rbind(Return = Mean, Budget = Budget, 
                  Bonds = Bonds, Foreign = Foreign, Equtities = Equities),
      lower = c(Return=targetReturn, Budget=1, 
                Bonds=0.3, Foreign=0.0, Equtities=0.0), 
      upper = c(Return=targetReturn, Budget=1, 
                Bonds=1.0, Foreign=0.5, Equtities=0.6) )  
    
    
    # QP Box Constrained Markowitz:     
    qpbox.funCons <- list(
      fun = list(
        Return = function(x) { Mean %*%  x },
        Budget = function(x) { sum(x) } ),
      lower = c(Return=targetReturn, Budget=1), 
      upper = c(Return=targetReturn, Budget=1) )          
    
    
    
    qpgroup.funCons <- list(
      fun = list(
        Return = function(x) { Mean %*%  x },
        Budget = function(x) { sum(x) },
        Bonds = function(x) { Bonds %*% x },
        Foreign = function(x) { Foreign %*% x },
        Equtities = function(x) { Equities %*% x } ),
      lower = c(Return=targetReturn, Budget=1, Bonds=0.3, Foreign=0.0, Equtities=0.0), 
      upper = c(Return=targetReturn, Budget=1, Bonds=1.0, Foreign=0.5, Equtities=0.6) )        
    
    
    budget.linCons <- list(
      mat = rbind(Budget = Budget),
      lower = investBudget, 
      upper = investBudget)  
    
    budget.funCons <- list(
      fun = list(function(x) sum(x)),
      lower = investBudget, 
      upper = investBudget)  
    
    
    groupbudget.funCons <- list(
      fun = list(
        Budget = function(x) { sum(x) },
        Bonds = function(x) { Bonds %*% x },
        Foreign = function(x) { Foreign %*% x },
        Equtities = function(x) { Equities %*% x } ),
      lower = c(Budget=1, Bonds=0.3, Foreign=0.0, Equtities=0.0), 
      upper = c(Budget=1, Bonds=1.0, Foreign=0.5, Equtities=0.6) )              
    
  }


###############################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                     DESCRIPTION:
#  efficientPortfolio            Returns a frontier portfolio
#  maxratioPortfolio             Returns the max return/risk ratio portfolio
#  tangencyPortfolio             Returns the tangency portfolio
#  minriskPortfolio              Returns the minimum risk portfolio
#  minvariancePortfolio          Returns the minimum variance portfolio
#  maxreturnPortfolio            Returns the maximum return portfolio
################################################################################


efficientPortfolio <-
    function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Computes target risk and weights for an efficient portfolio

    # Arguments:
    #   data - a rectangular object of assets
    #   spec - an object of class 'fPFOLIOSPEC'
    #   constraints - a character vector or NULL

    # Example:
    #   data = as.timeSeries(data(LPP2005REC))[, 1:6]
    #   spec = portfolioSpec(); setTargetReturn(spec) <- mean(data)
    #   efficientPortfolio(data, spec)

    # FUNCTION:

    # Match Spec Versus Constraints:
    # .checkSpecVsConstraints(spec, constraints)

    # Optimize Portfolio:
    Solver <- match.fun(getSolver(spec))
    portfolio <- Solver(data, spec, constraints)

    # Set Parameters:
    # Do not use ...
    # setWeights(spec) = portfolio$weights
    # setTargetReturn(spec) = portfolio$targetReturn
    # setTargetRisk(spec) = portfolio$targetRisk
    # to provide overwriting use:
    spec@portfolio$weights <- portfolio$weights
    spec@portfolio$targetReturn <- portfolio$targetReturn
    spec@portfolio$targetRisk <- portfolio$targetRisk

    # Add Status:
    setStatus(spec) <- portfolio$status

    # Add Title:
    Title <- "Efficient Portfolio"

    # Compose Portfolio:
    portfolio <- feasiblePortfolio(data, spec, constraints)
    portfolio@call <- match.call()
    portfolio@title <- Title

    # Return Value:
    portfolio
}


# ------------------------------------------------------------------------------


maxratioPortfolio <-
    function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Computes Capital Market Line

    # Arguments:
    #   data - a rectangular object of assets
    #   spec - an object of class 'fPFOLIOSPEC'
    #   constraints - a character vector or NULL

    # Example:
    #   data <- as.timeSeries(data(LPP2005REC))[, 1:6]
    #   maxratioPortfolio(data)

    # FUNCTION:

    # Match Spec Versus Constraints:
    # .checkSpecVsConstraints(spec, constraints)

    # Transform Data:
    Data <- portfolioData(data, spec)

    # Compute Sharpe ratio to be minimized:
    ratioFun <- function(x, data, spec, constraints)
    {
        # x is the target return ...
        setTargetReturn(spec) <- x[1]
        Solver <- match.fun(getSolver(spec))
        ans <- Solver(data, spec, constraints)
        # 2012-02-21 DW: Return if Solver does not converge
          BIG <- 1e10
          if(ans$status != 0) return(-BIG)
        ratio = (x[1] - getRiskFreeRate(spec)) / ans$objective
        attr(ratio, "weights") <- ans$weights
        attr(ratio, "status") <- ans$status
        return(ratio)
    }

    # Start Solution - Equal Weights Portfolio:
    nAssets <- getNAssets(Data)
    setWeights(spec) <- rep(1/nAssets, times = nAssets)
    fp <- feasiblePortfolio(Data, spec, constraints)
    setTargetReturn(spec) <- getTargetReturn(fp)
    
    ## 2012-03-10 DW: 
    ## tol = 10*.Machine$double.eps - higher tolerance added
    portfolio <- optimize(f = ratioFun, interval = range(getMu(Data)),
        maximum = TRUE, data = Data, spec = spec, constraints = constraints,
        tol = 10*.Machine$double.eps)
        
    ## 2009-04-19 DW:
    ## It may happen, that the maximum ratio portfolio cannot be computed.
    ## One reason is that the portfolio does not exist since the constraints 
    ## are too restrictive. 
    ## Another reason is that the risk free rate is above the highest return
    ## point.
    ## In these cases we stop the computation here, and return an error message.
    
    STATUS = attr(portfolio$objective, "status")
    if (STATUS != 0) {
        # Error Message:
        cat("\nExecution stopped:")   
        cat("\n  The maximum ratio portfolio could not be computed.")
        cat("\nPossible Reason:")
        cat("\n  Your portfolio constraints may be too restrictive.")
        cat("\nStatus Information:")
        cat("\n  status=", STATUS, " from solver ", getSolver(spec), ".", sep = "")
        cat("\n")
        stop(call. = FALSE, show.error.messages = "\n  returned from Rmetrics")
    }
       
    # Continue: Succesfully computed the minimum risk portfolio ...     
    setWeights(spec) <- attr(portfolio$objective, "weights")
    setStatus(spec) <- attr(portfolio$objective, "status")

    # Compose Portfolio:
    portfolio <- feasiblePortfolio(data, spec, constraints)
    portfolio@call <- match.call()
    portfolio@title <- "Max Return/Risk Ratio Portfolio"

    # Return Value:
    portfolio
}


# ------------------------------------------------------------------------------


tangencyPortfolio <-
    function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Computes Markowitz tangency portfolio
    
    # Arguments:
    #   data - a rectangular object of assets
    #   spec - an object of class 'fPFOLIOSPEC'
    #   constraints - a character vector or NULL
    
    # FUNCTION:
    
    # Portfolio:
    portfolio <- maxratioPortfolio(data, spec, constraints)
    portfolio@title <- "Tangency Portfolio"

    # Return Value:
    portfolio
}


################################################################################


.minriskPortfolio <-
    function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Computes minimum risk portfolio

    # Arguments:
    #   data - a rectangular object of assets
    #   spec - an object of class 'fPFOLIOSPEC'
    #   constraints - a character vector or NULL

    # Example:
    #   minriskPortfolio(SWX[, 1:3])

    # FUNCTION:

    # Match Spec Versus Constraints:
    #   .checkSpecVsConstraints(spec, constraints)

    # Transform Data:
    Data <- portfolioData(data, spec)

    # Compute target risk to be minimized:
    targetRiskFun <- function(x, data, spec, constraints) {
        # x is the target return ...
        setTargetReturn(spec) = x[1]
        Solver <- match.fun(getSolver(spec))
        ans <- Solver(data, spec, constraints)
        targetRisk <- ans$objective
        attr(targetRisk, "weights") <- ans$weights
        attr(targetRisk, "status") <- ans$status
        return(targetRisk)
    }

    # Minimal Risk:
    portfolio <- optimize(targetRiskFun, interval = range(getMu(Data)),
        data = Data, spec = spec, constraints = constraints, 
        tol = .Machine$double.eps^0.5)
        
    ## 2009-04-19 DW:
    ## It may happen, that the minimum risk protfolio cannot be computed.
    ## One reason is that the portfolio does not exist since the constraints 
    ## are too restrictive. 
    ## In this case we stop the computation here, and return an error message.
    
    STATUS = attr(portfolio$objective, "status")
    if (STATUS != 0) {
        # Error Message:
        cat("\nExecution stopped:")   
        cat("\n  The minimum risk portfolio could not be computed.")
        cat("\nPossible Reason:")
        cat("\n  Your portfolio constraints may be too restrictive.")
        cat("\nStatus Information:")
        cat("\n  status=", STATUS, " from solver ", getSolver(spec), ".", sep = "")
        cat("\n")
        stop(call.= FALSE, show.error.messages = "\n  returned from Rmetrics")
    }
       
    # Continue: Succesfully computed the minimum risk portfolio ...  
    setWeights(spec) <- attr(portfolio$objective, "weights")
    setStatus(spec) <- attr(portfolio$objective, "status")

    # Compose Portfolio:
    portfolio <- feasiblePortfolio(data, spec, constraints)
    portfolio@call <- match.call()
    portfolio@title <- "Minimum Risk Portfolio"

    # Return Value:
    portfolio
}


minriskPortfolio <-
    function(data, spec = portfolioSpec(), constraints = "LongOnly")
{   
    # A function implemented by Diethelm Wuertz
    
    # Note:
    #    NEW VERSION DW
    
    # FUNCTION:
    
    setTargetReturn(spec) <- NULL
    
    portfolio <- efficientPortfolio(data, spec, constraints)
    portfolio@call <- match.call()
    portfolio@title <- "Minimum Risk Portfolio"
    
    # Return Value:
    portfolio
}    


# ------------------------------------------------------------------------------


minvariancePortfolio <-
    function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Computes global minimum variance portfolio
    
    # Arguments:
    #   data - a rectangular object of assets
    #   spec - an object of class 'fPFOLIOSPEC'
    #   constraints - a character vector or NULL
    
    # FUNCTION:
    
    # Portfolio:
    portfolio <- minriskPortfolio(data, spec, constraints)
    portfolio@title <- "Minimum Variance Portfolio"

    # Return Value:
    portfolio
}


################################################################################


maxreturnPortfolio <-
    function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Computes target risk and weights for an efficient portfolio

    # Arguments:
    #   data - a rectangular object of assets
    #   spec - an object of class 'fPFOLIOSPEC'
    #   constraints - a character vector or NULL

    # FUNCTION:

    # Match Spec Versus Constraints:
    # .checkSpecVsConstraints(spec, constraints)

    # Transform Data:
    data = portfolioData(data, spec)

    # Maximize Return:
    if(is.null(getTargetRisk(spec))) {
        stop("Missing target risk for maximum return optimization.")
    } else {
        # Optimize Portfolio:
        Solver = match.fun(getSolver(spec))
        portfolio = Solver(data, spec, constraints)
        setWeights(spec) = portfolio$weights
        setStatus(spec) = portfolio$status
        Title = "Return Maximized Efficient Portfolio"
    }

    # Compose Portfolio:
    portfolio <- feasiblePortfolio(data, spec, constraints)
    portfolio@call <- match.call()
    portfolio@title <- Title

    # Return Value:
    portfolio
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General 
# Public License along with this library; if not, write to the 
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
# MA  02111-1307  USA


###############################################################################
# GLOBAL SETTINGS:         DESCRIPTION: 
#  Data                     Defines global portfolio Data object
#  portfolioObjective       Defines global portfolio objective function
#  portfolioReturn          Defines global portfolio return function
#  portfolioRisk            Defines global portfolio risk function
###############################################################################


# Global Variables, do it properly ...


# -----------------------------------------------------------------------------


Data <- NA


# -----------------------------------------------------------------------------


portfolioObjective <- 
  function(weights) 
{ 
  NA 
} 


# -----------------------------------------------------------------------------


portfolioReturn <- 
  function(weights) 
{ 
  NA 
}


# -----------------------------------------------------------------------------


portfolioRisk <- 
  function(weights) 
{ 
  NA 
} 


###############################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:               DESCRIPTION:
#  solveRampl.MV           AMPL solver for a MV Long Only Portfolio
# FUNCTION:               DESCRIPTION:
#  solveRampl.CVAR         AMPL solver for a CVAR Long Only Portfolio
################################################################################


solveRampl.MV <-
    function (data, spec, constraints="LongOnly") 
{
    # A function written by Diethelm Wuertz
    
    # Description:
    #   A Mean-Variance (MV) Long Only portfolio solver
    
    # Note:
    #   No other constraints are allowed.
    
    # Example:
    #    data <- 100 * LPP2005.RET[, 1:6]
    #    spec <- portfolioSpec()
    #    setTargetReturn(spec) <- mean(data)
    #    setSolver(spec) <- "solveRampl.MV"
    #    solveRampl.MV(data, spec)
    
    # FUNCTION:
    
    # Check Consistency:
    stopifnot(getType(spec) == "MV")
    stopifnot(constraints == "LongOnly")
    
    # Force AMPL Settings:
    spec@ampl$ampl <- TRUE
    solver <- spec@ampl$solver
    project <- spec@ampl$project
    trace <- spec@ampl$trace
    
    # Get Portfolio Settings:
    Data <- portfolioData(data, spec)
    Sigma <- getSigma(Data)
    n <- nAssets <- getNAssets(Data)
    targetReturn <- getTargetReturn(spec)    
    
    # Create AMPL Model File:
    amplModelOpen(project)
    model <- c(
        "# Quadratic Programming",
        "# Long Only Markowitz",
        "param nAssets;",                        
        "param mu{1..nAssets};",                   
        "param Sigma{1..nAssets, 1..nAssets};",      
        "param targetReturn;",                                                    
        "var x{1..nAssets} >= 0;",            
        "minimize Risk: sum {i in 1..nAssets} sum{j in 1..nAssets} x[i]*Sigma[i,j]*x[j];",
        "subject to Return: sum{i in 1..nAssets} mu[i]*x[i] = targetReturn;",
        "subject to Budget: sum{i in 1..nAssets} x[i] = 1;",
        NULL )
    amplModelAdd(model, project)
    
    # Create AMPL Data File:
    amplDataOpen(project)
    amplDataAddValue(data="nAssets", value=n, project)
    amplDataAddVector(data="mu", vector=getMu(Data), project)
    amplDataAddMatrix(data="Sigma", matrix=getSigma(Data), project)
    amplDataAddValue(data="targetReturn", value=getTargetReturn(spec), project)
    
    # Create AMPL Run File:
    amplRunOpen(project)
    run <- c(
        paste("reset ;"),
        paste("option solver", solver, ";", sp = ""),
        paste("model ", project, ".mod ;", sep = ""),
        paste("data ", project, ".dat ;", sep = ""),
        paste("solve ;"),
        paste("display x > ", project, ".txt ;", sep = ""),
        paste("display solve_result_num > ", project, ".txt ;", sep = ""),
        paste("display solve_result > ", project, ".txt ;", sep = ""),
        paste("display solve_message > ", project, ".txt ;", sep = ""),
        paste("exit ;"),
        NULL )
    amplRunAdd(run, project)
   
    # Exec AMPL:
    command <- paste("ampl -t -vs", paste(project, "run", sep="."))
    solve <- system(command, intern=TRUE)                          
    
    # Read AMPL Output File: 
    file <- paste(project, "txt", sep = ".")
    out <- scan(file, what = character(0), sep="\n", quiet=TRUE)
  
    # Get Weights:
    Index <- (grep(";", out) - 1)[1]
    splits <- strsplit(paste(out[2:Index], collapse=" "), " ")[[1]]
    solution <- as.numeric(splits[splits != ""])[seq(2, 2*n, by=2)]
    Index <- as.numeric(splits[splits != ""])[seq(1, 2*n, by=2)]
    solution[Index] <- solution
    
    # Get Status:
    status <- strsplit(out[grep("solve_result", out)], split=" ")
    statusCode <- status[[1]][3]
    statusMessage <- status[[2]][3]
    
    # Version:
    version <- system(paste(solver, "-v"), intern=TRUE) 
    
    # Compose Result into a List:
    ans <- list(
        type = getType(spec), 
        solver = getSolver(spec), 
        optim = solve, 
        weights = solution, 
        targetReturn = targetReturn, 
        targetRisk = sqrt(solution %*% Sigma %*% solution)[[1, 1]], 
        objective = sqrt(solution %*% Sigma %*% solution)[[1, 1]], 
        status = statusCode, 
        message = statusMessage)
    class(ans) <- c("solver", "list")
      
    # Return Value: 
    ans
}


################################################################################


solveRampl.CVAR <-
    function(data, spec, constraints="LongOnly")
{    
    # A function written by Diethelm Wuertz
    
    # FUNCTION:
    
    # Portfolio Model
    #   max     CVaR
    #   s.t.    desired target Return
    #   s.t.    full Investement
    #   s.t.    long only positions

    # Check Consistency:
    stopifnot(getType(spec) == "CVAR")
    stopifnot(getSolver(spec) == "solveAMPLipoptCVAR")
    stopifnot(constraints == "LongOnly")
    
    # Force AMPL Settings:
    spec@ampl$ampl <- TRUE
    project <- spec@ampl$project
    trace <- spec@ampl$trace
    
    # Settings:
    Data <- portfolioData(data, spec)
    data <- getSeries(Data)
    n <- nAssets <- getNAssets(Data)
    nScenarios <- nrow(getSeries(Data))
    Mean <- getMean(Data)
    targetReturn <- getTargetReturn(spec)
    alpha <- getAlpha(spec)
    Type <- getType(spec)
    project <- "ampl"
    solver <- "ipopt"

    # Model File:
    amplModelOpen(project)
    model <- c(
        "param alpha ;",
        "param nAssets ;",
        "param nScenarios ;",
        "param Mean{1..nAssets} ;",
        "param targetReturn ;",
        "param Data{1..nScenarios,1..nAssets} ;",
        "var weights{1..nAssets} ;",
        "var VaR ;",
        "var z{1..nScenarios};",
        "maximize Risk: VaR - ( sum{i in 1..nScenarios} z[i] ) / ( alpha  * nScenarios ) ;",
        "subject to Return: sum{i in 1..nAssets} weights[i] * Mean[i] = targetReturn ;" ,
        "subject to Weights{i in 1..nAssets}:  weights[i] >= 0 ;", 
        "subject to Budget: sum{i in 1..nAssets} weights[i] = 1 ;", 
        "subject to Scenarios{k in 1..nScenarios}: -VaR + z[k] + sum{i in 1..nAssets} Data[k,i]*weights[i] >= 0 ;" ,
        "subject to Z{i in 1..nScenarios}:  z[i] >= 0 ;", 
        NULL )
    amplModelAdd(model, project)

    # Data File:
    amplDataOpen(project)
    amplDataAddValue(data="alpha", value=alpha, project)
    amplDataAddValue(data="nAssets", value=nAssets, project)
    amplDataAddValue(data="nScenarios", value=nScenarios, project)
    amplDataAddValue(data="targetReturn", value=targetReturn, project)
    amplDataAddVector(data="Mean", vector=Mean, project)
    amplDataAddMatrix(data="Data", matrix=data, project)

    # Run File:
    amplRunOpen(project)
    run <- c(
        paste("option solver cplex ;"),
        paste("model ", project, ".mod ;", sep = ""),
        paste("data ", project, ".dat ;", sep = ""),
        "solve ;",
        paste("display weights > ", project, ".txt ;", sep = ""),
        paste("display VaR > ", project, ".txt ;", sep = ""),
        paste("exit ;"),
        NULL)
    amplRunAdd(run, project)

    # Exec AMPL:
    command <- paste("ampl -t -vs", paste(project, "run", sep="."))
    solve <- system(command, intern=TRUE)                          
    
    # Read AMPL Output File: 
    file <- paste(project, "txt", sep = ".")
    out <- scan(file, what = character(0), sep="\n", quiet=TRUE)
  
    # Get Weights:
    Index <- (grep(";", out) - 1)[1]
    splits <- strsplit(paste(out[2:Index], collapse=" "), " ")[[1]]
    solution <- as.numeric(splits[splits != ""])[seq(2, 2*n, by=2)]
    Index <- as.numeric(splits[splits != ""])[seq(1, 2*n, by=2)]
    solution[Index] <- solution
    
    # Get Status:
    status <- strsplit(out[grep("solve_result", out)], split=" ")
    statusCode <- status[[1]][3]
    statusMessage <- status[[2]][3]
    
    # Version:
    version <- system(paste(solver, "-v"), intern=TRUE) 

    # Result:
    ans <- list(
        type = "CVAR", 
        solver = "solveRampl.CVAR", 
        optim = optim, 
        weights = weights,
        targetReturn = targetReturn, 
        targetRisk = -optim$optimum,
        objective = -optim$optimum, 
        status = optim$status[[1]],
        message = "")
    class(ans) <- c("solver", "list")
      
    # Return Value: 
    ans 
}


# #############################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


###############################################################################
# FUNCTION:                    DESCRIPTION:
#  rglpkLP                      Rmetrics Interface for Rglpk LP solver 
#  glpkLP                       Convenience wrapper for Rglpk LP solver
#  glpkLPControl                Rglpk LP control parameter list
###############################################################################


rglpkLP <- 
    function(objective, lower=0, upper=1, linCons, control=list())
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Convenience wrapper for solver Rglpk_solve_LP()
    
    # Argments:
    #    objective - numeric vector.
    #    lwer, upper - box constraints
    #    linCons - linear constraints: mat, lower and upper
    #    control - control list
    
    # FUNCTION:
    
    # Update Control List:
    ctrl <- glpkLPControl()
    if (length(control) > 0)
        for (name in names(control)) ctrl[name] = control[name]
    control <- ctrl
        
    # Box Constraints:
    N <- length(objective)
    if(length(lower) == 1) {
        par.lower = rep(lower, N)
    } else {
        par.lower = lower
    }
    if(length(upper) == 1) {
        par.upper = rep(upper, N)
    } else {
        par.upper = upper
    }
    bounds <- list(
        lower = list(ind = 1:N, val = par.lower),
        upper = list(ind = 1:N, val = par.upper))
        
    # Linear Constraints:
    mat <- linCons[[1]]
    M <- nrow(mat)
    lower <- as.vector(linCons[[2]])
    upper <- as.vector(linCons[[3]])
    
    if(length(lower) == 1) {
        lower <- rep(lower, M)
    } else {
        lower <- lower
    }
    if(length(upper) == 1) {
        upper <- rep(upper, M)
    } else {
        upper <- upper
    }
    
    eqIndex <- which(lower == upper)
    ineqIndex <- which(lower != upper)
    
    eqA <- mat[eqIndex, ]
    ineqA <- mat[ineqIndex, ]
    mat <- rbind(eqA, ineqA, ineqA)
    dir <- c(rep("==", length(eqIndex)), rep("<=", length(ineqIndex)), rep(">=", length(ineqIndex)))
    rhs <- c(upper[eqIndex], upper[ineqIndex], lower[ineqIndex])
    mat <- mat[is.finite(rhs), ]
    dir <- dir[is.finite(rhs)]
    rhs <- rhs[is.finite(rhs)]

    # Optimize Portfolio:
    optim <- glpkLP(
        obj = objective, 
        mat = mat,
        dir = dir,
        rhs = rhs,
        types = NULL,
        max = FALSE,
        bounds = bounds,
        verbose = control$trace) 
    
    # Version:
    version <- paste(packageDescription("Rglpk")[1:3], collapse=" ")
         
    # Return Value:
    value <- list(
        opt = optim,
        solution = optim$solution, 
        objective = optim$optimum, 
        status = optim$status,
        message = "none",
        solver = paste("R", control$solver),
        version = version)
    class(value) <- c("solver", "list")
    value
}


# -----------------------------------------------------------------------------


glpkLP <- Rglpk::Rglpk_solve_LP


# -----------------------------------------------------------------------------


glpkLPControl <- 
    function(solver = "glpk", project="r", trace=FALSE) 
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns control parameter list
    
    # FUNCTION:
    
    # Return Value:
    list(solver=solver, trace=trace)
}


###############################################################################

    

################################################################################
# FUNCTION:               DESCRIPTION:
#  portfolioConstraints    Returns an object of class fPFOLIOCON
# FUNCTION:               DESCRIPTION:
#  minWConstraints         Returns vector with min box constraints
#  maxWConstraints         Returns vector with max box constraints
#  eqsumWConstraints       Returns list with group equal vec/matrix constraints
#  minsumWConstraints      Returns list with group min vec/matrix constraints
#  maxsumWConstraints      Returns list with group max vec/matrix constraints
#  minBConstraints         Returns vector with min cov risk budget constraints
#  maxBConstraints         Returns vector with max cov risk budget constraints
#  minFConstraints         Returns vector with min nonlin functions constraints
#  maxFConstraints         Returns vector with max nonlin functions constraints
#  minBuyinConstraints     Returns lower bound of buyin constraints
#  maxBuyinConstraints     Returns upper bound of buyin constraints
#  nCardConstraints        Returns number of Cardinalities
#  minCardConstraints      Returns lower bound of Cardinalities
#  maxCardConstraints      Returns upper bound of Cardinalities
################################################################################


portfolioConstraints <-
function(data, spec=portfolioSpec(), constraints="LongOnly", ...)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns an object of class fPFOLIOCON

    # Arguments:
    #   data - a timeSeries or a fPFOLIODATA object
    #   spec - a fPFOLIOSPEC object
    #   constraints - a constraints string
    #       validStrings = c(
    #           "LongOnly", "Short",      # LongOnly and Short Notification
    #           "minW", "maxW",           # Box Constraints
    #           "minsumW", "maxsumW",     # left/right Sided Group Constraints
    #           "minB", "maxB",           # Covariance Risk Budgets
    #           "listF", "minF", "maxF",  # Nonlinear Functions Constraints
    #     NEW:  "minBuyin"," maxBuyin",
    #     NEW:  "nCard", "minCard", "maxCard") 
    
    # Details:
    #   This function takes the constraints strings and converts them to
    #   constraints vectors and matrices of the following form:
    #   1. boxConstraints           W_min <= W <= W_max
    #   2. groupEqConstraints       A_eq W = c_eq
    #   3. groupMatConstraints      a_vec <= A_mat W <= b_vec
    #   4. riskBudgetConstraints    a <= RiskBudget <= b
    #      cardinalityConstraints   eps*z <= W <- delta*z,  z[0,1],  Sum(z)=K
    #   These values are returned as list in four slots.

    # Example:
    #   data = .lppData; spec=.mvSpec
    #   portfolioConstraints(data, spec, "LongOnly")
    #   constraints=c("minW[1:3]=0.1", "maxW[4:6]=0.9", "minsumW[c(2,5)]=0.2", "maxsumW[c(1,4)]=0.9")
    #   portfolioConstraints(data, spec, constraints)

    # FUNCTION:

    # Already done ...
    if (inherits(constraints,"fPFOLIOCON")) return(constraints)

    # Missing target Return ...
    if (is.null(getTargetReturn(spec))) setTargetReturn(spec) <- NA

    # Handle NULL - A NULL  :
    if (is.null(constraints)) constraints="LongOnly"

    # Check Vector of Valid Strings - these are strings ...
    validStrings = c(
        "LongOnly", "Short",      # LongOnly and Short Notification
        "minW", "maxW",           # Box Constraints
        "minsumW", "maxsumW",     # left and right Sided Group Constraints
        "minB", "maxB",           # Covariance Risk Budgets
        "listF", "minF", "maxF",  # Nonlinear Functions Constraints
        "nCard", "minCard", "maxCard")
    if (any(constraints == "Short")) setSolver(spec) = "solveRshortExact"
    # usedStrings <- unique(sort(sub("\\[.*", "", constraints)))
    # checkStrings <- usedStrings %in% validStrings
    # check <- (sum(!checkStrings) == 0)
    # if (check) check <- "valid" else stop("Invalid Constraints String(s)")
    stringConstraints <- constraints
    # attr(stringConstraints, "control") = check

    # Data:
    Data <- portfolioData(data, spec)
    
    # Constraints:
    minW <- minWConstraints(Data, spec, constraints)
    maxW <- maxWConstraints(Data, spec, constraints)
    eqsumW <- eqsumWConstraints(Data, spec, constraints)
    minsumW <- minsumWConstraints(Data, spec, constraints)
    maxsumW <- maxsumWConstraints(Data, spec, constraints)
    minB <- minBConstraints(Data, spec, constraints)
    maxB <- maxBConstraints(Data, spec, constraints)
    listF <- listFConstraints(Data, spec, constraints)
    minF <- minFConstraints(Data, spec, constraints)
    maxF <- maxFConstraints(Data, spec, constraints)
    minBuyin <- minCardConstraints(Data, spec, constraints)
    maxBuyin <- maxCardConstraints(Data, spec, constraints)
    nCard <- nCardConstraints(Data, spec, constraints)
    minCard <- minCardConstraints(Data, spec, constraints)
    maxCard <- maxCardConstraints(Data, spec, constraints)

    if(is.null(minW)) minW = numeric()
    if(is.null(maxW)) maxW = numeric()
    if(is.null(eqsumW)) eqsumW = matrix(NA)
    if(is.null(minsumW)) minsumW = matrix(NA)
    if(is.null(maxsumW)) maxsumW = matrix(NA)
    if(is.null(minB)) minB = numeric()
    if(is.null(maxB)) maxB = numeric()
    if(is.null(maxsumW)) maxsumW = matrix(NA)
    if(is.null(minF)) minF = numeric()
    if(is.null(maxF)) maxF = numeric()
    if(is.null(minBuyin)) minBuyin = numeric()
    if(is.null(maxBuyin)) maxBuyin = numeric()
    if(is.null(nCard)) nCard = integer()
    if(is.null(minCard)) minCard = numeric()
    if(is.null(maxCard)) maxCard = numeric()
    
    # Return Value:
    new("fPFOLIOCON",
        stringConstraints=stringConstraints,
        minWConstraints=minW,
        maxWConstraints=maxW,
        eqsumWConstraints=eqsumW,
        minsumWConstraints=minsumW,
        maxsumWConstraints=maxsumW,
        minBConstraints=minB,
        maxBConstraints=maxB,
        listFConstraints=listF,
        minFConstraints=minF,
        maxFConstraints=maxF,
        minBuyinConstraints=minBuyin,
        maxBuyinConstraints=maxBuyin,
        nCardConstraints=nCard,
        minCardConstraints=minCard,
        maxCardConstraints=maxCard 
        )
}


################################################################################


minWConstraints <-
function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns a vector with min box constraints

    # Details:
    #   Takes care of "minW" strings, i.e. lower blounds
    #   W >= c

    # Arguments:
    #   data - a timeSeries or a fPFOLIODATA object
    #   spec - a fPFOLIOSPEC object
    #   constraints - a constraints string

    # Example:
    #   data <- as.timeSeries(data(LPP2005REC))[, 1:6]
    #   spec <- portfolioSpec()
    #   constraints <- c("minW[3:4]=0.1", "maxW[5:6]=0.8")
    #   minWConstraints(data, spec, constraints)

    # FUNCTION:

    # Settings:
    Data <- portfolioData(data, spec)
    if (inherits(data,"fPFOLIODATA")) data <- getSeries(Data)
    nAssets <- getNAssets(Data)
    assetsNames <- getUnits(Data)

    # Consider LongOnly:
    if("LongOnly" %in% constraints) {
        minW <- rep(0, nAssets)
        names(minW) <- assetsNames
        return(minW)
    }

    # Consider Unlimited Short:
    if("Short" %in% constraints) {
        minW <- rep(-Inf, nAssets)
        names(minW) <- assetsNames
        return(minW)
    }

    # Extract and Compose Vectors a_vec and b_vec:
    minW = rep(0, nAssets)
    names(minW) = assetsNames
    if (!is.null(constraints)) {
        nC <- length(constraints)
        what <- substr(constraints, 1, 4)
        for (i in 1:nC) {
            if (what[i] == "minW") eval(parse(text = constraints[i]))
        }
    }
    names(minW) <- assetsNames
    return(minW)

    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------


maxWConstraints <-
function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns a vector with max box constraints

    # Details:
    #   Takes care of "maxW" strings, i.e. upper bounds
    #   W >= c

    # Arguments:
    #   data - a timeSeries or a fPFOLIODATA object
    #   spec - a fPFOLIOSPEC object
    #   constraints - a constraints string

    # Example:
    #   data = as.timeSeries(data(LPP2005REC))[, 1:6]
    #   spec=portfolioSpec()
    #   constraints=c("minW[3:4]=0.1", "maxW[5:6]=0.8")
    #   maxWConstraints(data, spec, constraints)

    # FUNCTION:

    # Settings:
    Data <- portfolioData(data, spec)
    if (inherits(data,"fPFOLIODATA")) data <- getSeries(Data)
    nAssets <- getNAssets(Data)
    assetsNames <- getUnits(Data)

    # Consider LongOnly:
    if("LongOnly" %in% constraints) {
        maxW <- rep(1, nAssets)
        names(maxW) <- assetsNames
        return(maxW)
    }

    # Consider Unlimited Short:
    if("Short" %in% constraints) {
        maxW <- rep(Inf, nAssets)
        names(maxW) = assetsNames
        return(maxW)
    }

    # Extract and Compose Vectors a_vec and b_vec:
    maxW = rep(1, nAssets)
    names(maxW) <- assetsNames
    if (!is.null(constraints)) {
        nC <- length(constraints)
        what <- substr(constraints, 1, 4)
        for (i in 1:nC) {
            if (what[i] == "maxW") eval(parse(text = constraints[i]))
        }
    }
    names(maxW) = assetsNames
    return(maxW)

    # Return Value:
    invisible()
}


################################################################################


eqsumWConstraints <-
function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns a list with group equal matrix and vectors constraints

    # Details:
    #   Takes care of "eqsumW" strings
    #   A_eq W = c_eq

    # Arguments:
    #   data - a timeSeries or a fPFOLIODATA object
    #   spec - a fPFOLIOSPEC object
    #   constraints - a constraints string

    # Example:
    #   data = as.timeSeries(data(LPP2005REC))[, 1:6]
    #   spec=portfolioSpec(); setTargetReturn(spec) = mean(data)
    #   constraints="eqsumW[1:6]=1"
    #   eqsumWConstraints(data, spec, constraints)
    #   eqsumWConstraints(data, spec, constraints="LongOnly")
    #   eqsumWConstraints(data, spec, constraints=c("LongOnly","Partial"))

    # FUNCTION:

    # Get Statistics:
    Data <- portfolioData(data, spec)
    if (inherits(data,"fPFOLIODATA")) data <- getSeries(Data)
    targetReturn <- getTargetReturn(spec)[1]
    if (is.null(targetReturn)) {
        targetReturn = NA
        # stop("Target Return is Missing")
    }

    # Get Data:
    mu <- getMu(Data)
    nAssets <- getNAssets(Data)
    assetsNames <- getUnits(Data)

    # Target Return:
    Aeq <- matrix(mu, byrow = TRUE, ncol = nAssets)

    # Full or partial Investment?
    if ("partial" %in% tolower(constraints))
        fullInvest = FALSE else fullInvest = TRUE

    # Full Investment:
    #   - negative to handle better partial Investment in Rquadprog:
    if (fullInvest) Aeq <- rbind(Aeq, -rep(1, nAssets))

    # Dimension Names:
    colnames(Aeq) <- assetsNames
    if (fullInvest) {
        rownames(Aeq) <- c("Return", "Budget")
    } else {
        rownames(Aeq) <- "Return"
    }
    
    # RHS Vector:
    if (fullInvest) {
        ceq <- c(Return = targetReturn, Budget = -1)
    } else {
        ceq <- c(Return = targetReturn)
    }
    
    # Extract and Compose Matrix and Vector:
    what6 = substr(constraints, 1, 6)
    if (!is.null(constraints)) {
        nC = length(constraints)
        for (i in 1:nC) {
            if (what6[i] == "eqsumW")  {
                eqsumW = rep(0, times = nAssets)
                names(eqsumW) <- assetsNames
                eval(parse(text = constraints[i]))
                Aeq = rbind(Aeq, eqsumW = sign(eqsumW))
                a = strsplit(constraints[i], "=")[[1]][2]
                ceq = c(ceq, eqsumW = as.numeric(a))
            }
        }
    }
    eqsumW <- cbind(ceq, Aeq)

    # If target Return is missing:
    eqsumW <- na.omit(eqsumW)
    
    # Return Value:
    eqsumW
}


# ------------------------------------------------------------------------------


minsumWConstraints <-
function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns a list with group matrix and vectors constraints

    # Arguments:
    #   data - a timeSeries or a fPFOLIODATA object
    #   spec - a fPFOLIOSPEC object
    #   constraints - a constraints string$

    # Details:
    #   Takes care of "minsumW" strings
    #   a_vec <= A_mat W

    # Example:
    #   data = as.timeSeries(data(LPP2005REC))[, 1:6]
    #   spec=portfolioSpec(); setTargetReturn(spec) = mean(data)
    #   constraints=c("minsumW[2:3]=0.2", "minsumW[c(1,4:6)]=0.2")
    #   minsumWConstraints(data, spec, constraints)
    #   minsumWConstraints(data, spec)

    # FUNCTION:

    # Get Statistics:
    data <- portfolioData(data, spec)

    # Get Specifications:
    mu <- getMu(data)
    nAssets <- getNAssets(data)
    assetsNames <- getUnits(data)

    # Extrac and Compose Matrix and Vectors:
    what7 <- substr(constraints, 1, 7)

    if (!is.null(constraints)) {
        nC <- length(constraints)

        count <- 0
        Amat <- NULL
        avec <- NULL

        # Partial Investment:
        if ("partial" %in% tolower(constraints)) {
            Amat <- rbind(Amat, rep(1, times = nAssets))
            avec <- c(avec, 0)
        }

        for (i in 1:nC) {
            if (what7[i] == "minsumW")  {
                count = count + 1
                minsumW = rep(0, times = nAssets)
                names(minsumW) <- assetsNames
                eval(parse(text = constraints[i]))
                Amat = rbind(Amat, minsumW = sign(minsumW))
                a = strsplit(constraints[i], "=")[[1]][2]
                avec = c(avec, as.numeric(a))
            }
        }
        if (!is.null(Amat)){
            colnames(Amat) = assetsNames
            names(avec) = rep("lower", count)
        }

    }

    # Return Value:
    cbind(avec = avec, Amat = Amat)
}


# ------------------------------------------------------------------------------


maxsumWConstraints <-
function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns a list with group matrix and vectors constraints

    # Arguments:
    #   data - a timeSeries or a fPFOLIODATA object
    #   spec - a fPFOLIOSPEC object
    #   constraints - a constraints string$

    # Details:
    #   Takes care of "minsumW" and "maxsumW" strings
    #   a_vec <= A_mat W <= b_vec

    # Example:
    #   data = as.timeSeries(data(LPP2005REC))[, 1:6]
    #   spec=portfolioSpec(); setTargetReturn(spec) = mean(data)
    #   constraints=c("maxsumW[2:3]=0.7", "maxsumW[c(1,4:6)]=0.8")
    #   maxsumWConstraints(data, spec, constraints)
    #   maxsumWConstraints(data, spec)

    # FUNCTION:

    # Get Statistics:
    data <- portfolioData(data, spec)

    # Get Specifications:
    mu <- getMu(data)
    nAssets <- getNAssets(data)
    assetsNames <- getUnits(data)

    # Extract and Compose Matrix and Vectors:
    what7 = substr(constraints, 1, 7)

    if (!is.null(constraints)) {
        nC <- length(constraints)

        count <- 0
        Amat <- NULL
        avec <- NULL

        # Partial Investment:
        if ("partial" %in% tolower(constraints)) {
            Amat <- rbind(Amat, rep(1, times = nAssets))
            avec <- c(avec, 1)
        }

        for (i in 1:nC) {
            if (what7[i] == "maxsumW")  {
                count = count + 1
                maxsumW = rep(0, times = nAssets)
                names(maxsumW) <- assetsNames
                eval(parse(text = constraints[i]))
                Amat = rbind(Amat, maxsumW = sign(maxsumW))
                a = strsplit(constraints[i], "=")[[1]][2]
                avec = c(avec, as.numeric(a))
            }
        }
        if (!is.null(Amat)) {
            colnames(Amat) = assetsNames
            names(avec) = rep("upper", count)
        }
    }

    # Return Value:
    cbind(avec = avec, Amat = Amat)
}


################################################################################


minBConstraints <-
function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Returns a list with min risk budget constraints vectors

    # Arguments:
    #   constraints - a constraints string

    # Example:
    #   data = as.timeSeries(data(LPP2005REC))[, 1:6]
    #   spec=portfolioSpec()
    #   constraints=c("minB[3:4]=0.1","maxB[1:3]=0.3","maxB[c(4,6)]=0.4")
    #   minBConstraints(data, spec, constraints)
    #   minBConstraints(data, spec)

    # FUNCTION:

    # Create Data Object:
    Data <- portfolioData(data, spec)
    if (inherits(data,"fPFOLIODATA")) data <- getSeries(Data)

    # Get Specifications:
    nAssets <- getNAssets(Data)
    assetsNames <- getUnits(Data)

    # Extract and Compose Risk Budgets:
    minB = rep(-Inf, nAssets)
    names(minB) <- assetsNames
    if (!is.null(constraints)) {
        nC = length(constraints)
        what = substr(constraints, 1, 4)
        for (i in 1:nC) {
            if (what[i] == "minB") eval(parse(text = constraints[i]))
        }
    }

    # Return Value:
    minB
}


# ------------------------------------------------------------------------------


maxBConstraints <-
function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns a list with max risk budget constraints vectors

    # Arguments:
    #   constraints - a constraints string

    # Example:
    #   data = as.timeSeries(data(LPP2005REC))[,1:6]
    #   spec=portfolioSpec()
    #   constraints=c("minB[3:4]=0.1","maxB[1:3]=0.3","maxB[c(4,6)]=0.4")
    #   maxBConstraints(data, spec, constraints)
    #   maxBConstraints(data, spec)

    # FUNCTION:

    # Create Data Object:
    Data <- portfolioData(data, spec)
    if (inherits(data,"fPFOLIODATA")) data <- getSeries(Data)

    # Get Specifications:
    N <- nAssets <- getNAssets(Data)
    assetsNames <- getUnits(Data)

    # Extract and Compose Risk Budgets:
    maxB <- rep(1, N)
    names(maxB) <- assetsNames
    if (!is.null(constraints)) {
        nC = length(constraints)
        what = substr(constraints, 1, 4)
        for (i in 1:nC) {
            if (what[i] == "maxB") eval(parse(text = constraints[i]))
        }
    }

    # Return Value:
    maxB
}


################################################################################


listFConstraints <-
function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Nonlinear Constraints

    # Example: 
    #    maxdd <- function(x) max(drawdowns(x))
    #    listFConstraints(data <- NULL, constraints=c("minF=-0.04", "listF(maxdd)"))

    # FUNCTION:

    # Parse:
    nlin <- list()
    matched <- pmatch("listF" , constraints)
    if(!is.na(matched)) {
        constraints = paste("nlin = ", constraints[matched])
        constraints = sub("listF", "list", constraints)
        eval(parse(text = constraints))
    }
    
    # Return Value:
    return(nlin)
}


# ------------------------------------------------------------------------------


minFConstraints <-
function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Nonlinear Constraints

    # Example: 
    #    minFConstraints("minF=-0.04")

    # FUNCTION:

    # Parse:
    minF <- NULL
    matched <- pmatch("minF" , constraints)
    if(!is.na(matched)) eval(parse(text = constraints[matched]))
    
    # Return Value:
    return(minF)
}


# ------------------------------------------------------------------------------


maxFConstraints <-
function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Nonlinear Constraints

    # Example: 
    #    maxFConstraints(c("LongOnly", "maxF=0"))

    # FUNCTION:

    # Parse:
    maxF <- NULL
    matched <- pmatch("maxF" , constraints)
    if(!is.na(matched)) eval(parse(text = constraints[matched]))
    
    # Return Value:
    return(maxF)
}


################################################################################


minBuyinConstraints <-
function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Lower Buyin Constraints

    # Example: minBuyinConstraints(c("LongOnly", "minBuyin[1:3]=0.1")

    # FUNCTION:

    # Parse:
    Data <- portfolioData(data)
    nAssets <- getNAssets(Data)
    minBuyin <- rep(0, nAssets)
    matched <- pmatch("minBuyin", constraints)
    if(!is.na(matched)) eval(parse(text = constraints[matched]))
    names(minBuyin) <- getUnits(Data)
    
    # Return Value:
    return(minBuyin)
}


# ------------------------------------------------------------------------------


maxBuyinConstraints <-
function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Upper Buyininality Constraints

    # Example: maxBuyinConstraints(c("LongOnly", "maxBuyin[5]=0.9")

    # FUNCTION:

    # Parse:
    Data <- portfolioData(data)
    nAssets <- getNAssets(Data)
    maxBuyin <- rep(1, nAssets)
    matched <- pmatch("maxBuyin", constraints)
    if(!is.na(matched)) eval(parse(text = constraints[matched]))
    names(maxBuyin) <- getUnits(Data)
    
    # Return Value:
    return(maxBuyin)
}


################################################################################


nCardConstraints <-
function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Cardinality Constraints

    # Example: ncardConstraints(c("LongOnly", "ncard=4")

    # FUNCTION:

    # Parse:
    Data <- portfolioData(data)
    nAssets <- getNAssets(Data)
    nCard <- nAssets
    matched <- pmatch("nCard", constraints)
    if(!is.na(matched)) eval(parse(text = constraints[matched]))
    
    # Return Value:
    return(nCard)
}


# ------------------------------------------------------------------------------


minCardConstraints <-
function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Lower Cardinality Constraints

    # Example: minCardConstraints(c("LongOnly", "minCard[1:3]=0.1")

    # FUNCTION:

    # Parse:
    Data <- portfolioData(data)
    nAssets <- getNAssets(Data)
    minCard <- rep(0, nAssets)
    matched <- pmatch("minCard", constraints)
    if(!is.na(matched)) eval(parse(text = constraints[matched]))
    names(minCard) <- getUnits(Data)
    
    # Return Value:
    return(minCard)
}


# ------------------------------------------------------------------------------


maxCardConstraints <-
function(data, spec=portfolioSpec(), constraints="LongOnly")
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Upper Cardinality Constraints

    # Example: maxCardConstraints(c("LongOnly", "maxCard[5]=0.9")

    # FUNCTION:

    # Parse:
    Data <- portfolioData(data)
    nAssets <- getNAssets(Data)
    maxCard <- rep(1, nAssets)
    matched <- pmatch("maxCard", constraints)
    if(!is.na(matched)) eval(parse(text = constraints[matched]))
    names(maxCard) <- getUnits(Data)
    
    # Return Value:
    return(maxCard)
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General 
# Public License along with this library; if not, write to the 
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
# MA  02111-1307  USA


################################################################################
# FUNCTION:                  DESCRIPTION:
#  .tailDependenceCoeffs      Returns Lower and Upper Tail Dependence Coeffs 
# FUNCTION:                  MIXED GUMBEL-SURVIVALGUMBEL-NORMAL COPULA:
#  .rgsgnormCopula            Generates G-SG-NORM copula random variates
#  .dgsgnormCopula            Computes G-SG-NORM copula density
# FUNCTION:                  MIXED G-SG-NORM COPULA FIT:
#  .gsgnormCopulaFit          Estimates the parameters of the G-SG-NORM copula
# FUNCTION:                  NON-PARAMETRIC TAIL DEPENDECY ESTIMATOR:
#  .cfgTDE                    Estimates non-parametrically tail dependence
# FUNCTION:                  COPULA FIT WITH NORM, NIG OR  GHT MARGINALS:
#  .empiricalDependencyFit    Estimates tail dependence with empirical marginals
#  .normDependencyFit         Estimates tail dependence with normal marginals
#  .gldDependencyFit          Estimates tail dependence with GLD marginals
#  .ghtDependencyFit          Estimates tail dependence with GHT marginals     
################################################################################

 
.tailDependenceCoeffs <-
    function(x, method = c("nig", "norm", "ght"), 
    trace = FALSE, doplot = TRUE)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns a list with lower and upper bivariate tail depenence matrixes
    
    # Example:
    #   x <- 100 * LPP2005.RET[, 1:6]; .tailDependenceCoeffs(x)
    
    # Notes:
    #   Tested only for NIG marginal distributions.
    
    # FUNCTION:
    
    # Check Settings:
    method <- match.arg(method)
    
    # Compute Coeffs with desired Marginals:
    fun <- paste(".", method, "DependencyFit", sep = "")
    funFit <- match.fun(fun)
    coeffs <- funFit(x, doplot = doplot, trace = trace) 
    
    # Return Value:
    coeffs
}


################################################################################


.rgsgnormCopula <- 
    function(n = 1000, alpha = c(2, 2), rho = 0, weights = c(1/3, 1/3))
{   
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Computes RVs from a mixed Gumbel-SurvivalGumbel-Normal Copula
    
    # Arguments:
    #   n - an integer value, the number of random variates to be
    #       generated.
    #   alpha - a numeric vector with two entries. The first denotes 
    #       the parameter value of alpha for the Gumbel copula, and
    #       the second for the Survival Gumbel Copula.
    #   rho - a numeric value denoting the correlation parameter for
    #       the normal copula.
    #   weights - a numeric vector with two entries. The first denotes 
    #       the weight of the Gumbel copula, and the second the weight
    #       of the Survival Gumbel Copula. The weight for the normal
    #       copula is evaluated by 1 - sum(weights).
    
    # Example:
    #   .rgsgnormCopula(20)
    
    # FUNCTION:
    
    # Checking:
    stopifnot(any(weights >= 0))
    stopifnot(sum(weights) <= 1)
       
    # Upper Gumbel = 1 , Lower Gumbel = 2, t = 3:
    weights <- c(weights, 1-sum(weights))
    N <- round(n*weights[1:2])
    N <- c(N, n-sum(N))
       
    # Random Variates:
    r <- rbind(
        if (N[1] > 0) fCopulae::rgumbelCopula(N[1], alpha[1]),
        if (N[2] > 0) 1-fCopulae::rgumbelCopula(N[2], alpha[2]),
        if (N[3] > 0) fCopulae::rellipticalCopula(N[3], rho, type = "norm") )
    index <- sample(1:n)
    ans <- r[index, ]
    
    N <- c(n, N)
    names(N) <- c("n", "n1", "n2", "n3")
    attr(ans, "control") < -N
    
    # Return Value:
    ans
} 


# ------------------------------------------------------------------------------


.dgsgnormCopula <- 
    function(u = 0.5, v = u, alpha = c(2, 2), rho = 0, weights = c(1/3, 1/3),
    output = c("vector", "list"))
{   
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Computes mixed Gumbel-SurvivalGumbel-Normal Copula density
    
    # Example:
    #   .perspPlot(.dgsgnormCopula(u=grid2d()$x, v=grid2d()$y, output = "list"))
   
    # FUNCTION:
    
    # Settings:
    if (is.list(u)) {
        v = u[[2]]
        u = u[[1]]
    }
    if (is.matrix(u)) {
        v = u[, 1]
        u = u[, 2]
    }
    
    # Match Arguments:
    output = match.arg(output)
    
    # Mixed Copula:
    weights <- c(weights, 1-sum(weights))
    dCopula1 <- fCopulae::dgumbelCopula(u, v, alpha[1])
    dCopula2 <- fCopulae::dgumbelCopula(1-u, 1-v, alpha[2])
    dCopula3 <- fCopulae::dellipticalCopula(u, v, rho, type = "norm")
    c.uv <- weights[1]*dCopula1 + weights[2]*dCopula2 + weights[3]*dCopula3
    
    attr(c.uv, "control") <- c(alpha = alpha, rho = rho, weights = weights)
    if (output == "list") {
        N = sqrt(length(u))
        x = u[1:N]
        y = matrix(v, ncol = N)[1, ]
        c.uv = list(x = x, y = y, z = matrix(c.uv, ncol = N))
    }
    
    # Return Value:
    c.uv
}


# ------------------------------------------------------------------------------


.gsgnormCopulaFit <- 
    function(u, v =  NULL, trace = FALSE)
{   
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Fits parameters for a mixed GSG copula
    
    # FUNCTION:
    
    # Settings:
    U <- u
    V <- v
    if (is.list(u)) {
        U = u[[1]]
        V = u[[2]]
    }
    if (is.matrix(u)) {
        U = u[, 1]
        V = u[, 2]
    }
   
    # From weights to gamma ...
    #   G-SG-NORM: W1*Gumbel + W2*SurvivalGumbul + W3*NORM
    #   with W3 = 1-W2-W3
    # Transformation: Use Weights W1 and W3
    #   gamma = c(gamma1, gamma2)
    #       0 < gamma1 = W1/(1-W3) < 1
    #       0 < gamma2 = 1-W3 < 1
    #       W3 = 1-gamma2
    #       W1 = gamma1*gamma2
    #       W2 = gamma2*(1-gamma1)
    # Note, this transformation has the advantage, that gamma1
    #   and gamma2 are independent from each other.
     
    # Estimate Copula:
    start = c(alpha1 = 1.5, alpha2 = 1.5, rho = 0, gamma1 = 1/2, gamma2 = 1/2)
    fun = function(x, U, V, trace) 
    {
        alpha = x[1:2]
        rho = x[3]
        gamma = x[4:5]
        weights = c(
            weights1 = gamma[[1]]*gamma[[2]], 
            weights2 = gamma[[2]]*(1-gamma[[1]]))
        
        density = .dgsgnormCopula(u = U, v = V, alpha, rho, weights = weights)
            
        density = density[!is.na(density)]
        f = -mean( log(density) )
        if (trace) {
            params = round(c(x[1:3], 
                weights[1], weights[2], 1-weights[1]-weights[2]), 4)
            names(params) = c("alpha1", "alpha2", "rho", 
                "gumbel", "survival", "norm")
            cat("\n Objective Function Value:  ", -f)
            cat("\n Parameter Estimates:       ", params, "\n") 
                
        }
        f
    }

    # Fit:
    fit = nlminb(
        start = start, objective = fun, 
        lower = c(  1,   1, -0.999, 0, 0), 
        upper = c(Inf, Inf,  0.999, 1, 1), U = U, V = V, trace = trace)
     
    # Fitted Parameters:   
    param = fit$par
    
    # Named Parameters:
    alpha1 = param[1]
    alpha2 = param[2]
    rho = param[3]
    gamma1 = param[4]
    gamma2 = param[5]
    
    # Weights:
    weights3 =  1-gamma2
    weights1 = gamma1*gamma2
    weights2 = gamma2*(1-gamma1)
    
    # Tail Coefficients
    upperLambda = (weights1*(2 - 2^(1/alpha1)))[[1]]
    lowerLambda = (weights2*(2 - 2^(1/alpha2)))[[1]]
    params = c(param[1:3], param[5]*param[4], param[5]*(1-param[4]), 1-param[5])
    names(params) = c("alpha1", "alpha2", "rho", "gumbel", "survival", "norm")
    Lambda = c(lower = lowerLambda, upper = upperLambda)
      
    # Return Value:
    list(param = params, lambda = Lambda, fitted = fit$par)
}


################################################################################


.cfgTDE <- 
    function(x, y)
{   
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Estimates non-parametrically tail dependency coefficient
    
    # FUNCTION:
    
    # Upper Tail:
    lambda = NULL
    n = length(x)
    for(i in 1:n){
        lambda = c(lambda,
            log(sqrt(log(1/x[i])*log(1/y[i]))/log(1/max(x[i],y[i])^2)))
    }
    upper <- 2 - 2*exp(sum(lambda/n))
    
    # Lower Tail:
    x = 1-x
    y = 1-y
    lambda = NULL
    n = length(x)
    for(i in 1:n){
        lambda = c(lambda,
            log(sqrt(log(1/x[i])*log(1/y[i]))/log(1/max(x[i],y[i])^2)))
    }
    lower <- 2 - 2*exp(sum(lambda/n))
    
    # Return Value:
    c(lower = lower, upper = upper)
}


################################################################################


.empiricalDependencyFit <- 
    function(x, doplot = TRUE, trace = FALSE)
{   
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Estimates tail dependency coefficients with Normal marginals
    
    # Arguments:
    #   x - a multivariate 'timeSeries' object
    
    # FUNCTION:
    
    # Settings: 
    N = ncol(x)
    lowerLambda = upperLambda = 0*diag(N)
    assetsNames = colnames(x)
    P = NULL
    
    for (i in 1:(N-1)) {
        # First asset:
        r1 = as.vector(x[, i])
        fit1 = nFit(r1)
        estim1 = fit1@fit$estimate
        p1 = pnorm(r1, estim1[1], estim1[2]) 
        Main1 = assetsNames[i]
        P = cbind(P, p1)
        for (j in (i+1):N) 
        {  
            # Second asset:
            r2 = as.vector(x[, j])
            fit2 = nFit(r2) 
            estim2 = fit2@fit$estimate      
            p2 = pnorm(r2, estim2[1], estim2[2]) 
            Main2 = assetsNames[j]
            # Optional Plot:
            if (doplot) 
            {
                # Plot Distribution:
                MainR = paste("Distribution:", Main1, "-", Main2)
                plot(r1, r2, pch = 19, main = MainR)
                grid()
                
                # Plot Copula:
                MainP = paste("Copula:", Main1, "-", Main2)
                plot(p1, p2, pch = 19, main = MainP)
                grid()
            }
            
            # Fit GSG copula parameters:
            fit = .gsgnormCopulaFit(u = p1, v = p2, trace = trace)
            if (trace)
                cat(assetsNames[c(i,j)], round(fit$lambda, 3), "\n")  
            
                # Compose lambda Matrix:
            lowerLambda[i, j] = lowerLambda[j, i] = fit$lambda[1]
            upperLambda[i, j] = upperLambda[j, i] = fit$lambda[2]
        }
    }
    
    # Result:
    colnames(lowerLambda) = rownames(lowerLambda) = assetsNames
    colnames(upperLambda) = rownames(upperLambda) = assetsNames
    ans = list(lower = lowerLambda, upper = upperLambda)
     
    # Return Value:
    ans 
}


# ------------------------------------------------------------------------------


.normDependencyFit <- 
    function(x, doplot = TRUE, trace = FALSE)
{   
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Estimates tail dependency coefficients with Normal marginals
    
    # Arguments:
    #   x - a multivariate 'timeSeries' object
    
    # FUNCTION:
    
    # Settings: 
    N = ncol(x)
    lowerLambda = upperLambda = 0*diag(N)
    assetsNames = colnames(x)
    P = NULL
    
    for (i in 1:(N-1)) {
        # First asset:
        r1 = as.vector(x[, i])
        fit1 = nFit(r1)
        estim1 = fit1@fit$estimate
        p1 = pnorm(r1, estim1[1], estim1[2]) 
        Main1 = assetsNames[i]
        P = cbind(P, p1)
        for (j in (i+1):N) 
        {  
            # Second asset:
            r2 = as.vector(x[, j])
            fit2 = nFit(r2) 
            estim2 = fit2@fit$estimate      
            p2 = pnorm(r2, estim2[1], estim2[2]) 
            Main2 = assetsNames[j]
            # Optional Plot:
            if (doplot) 
            {
                # Plot Distribution:
                MainR = paste("Distribution:", Main1, "-", Main2)
                plot(r1, r2, pch = 19, main = MainR)
                grid()
                
                # Plot Copula:
                MainP = paste("Copula:", Main1, "-", Main2)
                plot(p1, p2, pch = 19, main = MainP)
                grid()
            }
            
            # Fit GSG copula parameters:
            fit = .gsgnormCopulaFit(u = p1, v = p2, trace = trace)
            if (trace)
                cat(assetsNames[c(i,j)], round(fit$lambda, 3), "\n")  
            # Compose lambda Matrix:
            lowerLambda[i, j] = lowerLambda[j, i] = fit$lambda[1]
            upperLambda[i, j] = upperLambda[j, i] = fit$lambda[2]
        }
    }
    
    # Result:
    colnames(lowerLambda) = rownames(lowerLambda) = assetsNames
    colnames(upperLambda) = rownames(upperLambda) = assetsNames
    ans = list(lower = lowerLambda, upper = upperLambda)
     
    # Return Value:
    ans 
}


# ------------------------------------------------------------------------------


.gldDependencyFit <- 
    function(x, doplot = TRUE, trace = FALSE)
{   
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Estimates tail dependency coefficients with NIG marginals
    
    # Arguments:
    #   x - a multivariate 'timeSeries' object
    
    # FUNCTION:
    
    # Settings:
    N <- ncol(x)
    lowerLambda <- upperLambda <- 0*diag(N)
    assetsNames <- colnames(x)
    P <- NULL
    
    for (i in 1:(N-1)) {
        # First asset:
        r1 <- as.vector(x[, i])
        fit1 <- gldFit(r1, doplot = FALSE, trace = trace)
        estim1 <- fit1@fit$estimate
        p1 <- pgld(r1, estim1[1], estim1[2], estim1[3], estim1[4]) 
        Main1 <- assetsNames[i]
        P <- cbind(P, p1)
        for (j in (i+1):N) {  
            # Second asset:
            r2 <- as.vector(x[, j])
            fit2 <- gldFit(r2, doplot = FALSE, trace = trace) 
            estim2 = fit2@fit$estimate      
            p2 <- pgld(r2, estim2[1], estim2[2], estim2[3], estim2[4]) 
            Main2 <- assetsNames[j]
            # Optional Plot:
            if (doplot) {
                ## MainR = paste("Distribution:", Main1, "-", Main2)
                ## plot(r1, r2, pch = 19, main = MainR)
                ## grid()
                MainP <- paste("Copula:", Main1, "-", Main2)
                plot(p1, p2, pch = 19, main = MainP, xlab = "", ylab = "", cex=0.5)
                grid()
            }
            # Fit GSG copula parameters:
            fit <- .gsgnormCopulaFit(u = p1, v = p2, trace = trace)
            if (trace) cat(assetsNames[c(i,j)], round(fit$lambda, 3), "\n")  
            # Compose lambda Matrix:
            lowerLambda[i, j] <- lowerLambda[j, i] <- fit$lambda[1]
            upperLambda[i, j] <- upperLambda[j, i] <- fit$lambda[2]
        }
    }
    
    # Result:
    colnames(lowerLambda) <- rownames(lowerLambda) <- assetsNames
    colnames(upperLambda) <- rownames(upperLambda) <- assetsNames
    ans = list(lower = lowerLambda, upper = upperLambda)
     
    # Return Value:
    ans 
}


# ------------------------------------------------------------------------------


.ghtDependencyFit <- 
    function(x, doplot = TRUE, trace = FALSE)
{   
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Estimates tail dependency coefficients with GH Student-t marginals
    
    # Arguments:
    #   x - a multivariate 'timeSeries' object
    
    # FUNCTION:
    
    # Settings: 
    N = ncol(x)
    lowerLambda = upperLambda = 0*diag(N)
    assetsNames = colnames(x)
    P = NULL
    
    for (i in 1:(N-1)) {
        # First asset:
        r1 = as.vector(x[, i])
        fit1 = ghtFit(r1, doplot = FALSE, trace = trace)
        estim1 = fit1@fit$estimate
        p1 = pght(r1, estim1[1], estim1[2], estim1[3], estim1[4]) 
        Main1 = assetsNames[i]
        P = cbind(P, p1)
        for (j in (i+1):N) {  
            # Second asset:
            r2 = as.vector(x[, j])
            fit2 = ghtFit(r2, doplot = FALSE, trace = trace) 
            estim2 = fit2@fit$estimate      
            p2 = pght(r2, estim2[1], estim2[2], estim2[3], estim2[4]) 
            Main2 = assetsNames[j]
            # Optional Plot:
            if (doplot) {
                MainR = paste("Distribution:", Main1, "-", Main2)
                plot(r1, r2, pch = 19, main = MainR)
                grid()
                MainP = paste("Copula:", Main1, "-", Main2)
                plot(p1, p2, pch = 19, main = MainP)
                grid()
            }
            # Fit GSG copula parameters:
            fit = .gsgnormCopulaFit(u = p1, v = p2, trace = trace)
            if (trace)
                cat(assetsNames[c(i,j)], round(fit$lambda, 3), "\n")  
            # Compose lambda Matrix:
            lowerLambda[i, j] = lowerLambda[j, i] = fit$lambda[1]
            upperLambda[i, j] = upperLambda[j, i] = fit$lambda[2]
        }
    }
    
    # Result:
    colnames(lowerLambda) = rownames(lowerLambda) = assetsNames
    colnames(upperLambda) = rownames(upperLambda) = assetsNames
    ans = list(lower = lowerLambda, upper = upperLambda)
     
    # Return Value:
    ans 
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################
# FUNCTION:             DESCRIPTION:
#  amplModelOpen         Opens a writes to an AMPL model file
#  amplModelAdd          Adds model specs to an existing AMPL model file
#  amplModelShow         Shows the content of an AMPL .mod file
# FUNCTION:             DESCRIPTION:
#  amplDataOpen          Opens and writes the header to an AMPL data file
#   amplDataAddValue      Adds a numeric value to an AMPL data file
#   amplDataAddVector     Adds a numeric vector to an AMPL data file
#   amplDataAddMatrix     Adds a numeric matrix to an AMPL data file
#   amplDataSemicolon     Adds a semicolon on the end of a data input line
#  amplDataShow           Shows the content of an AMPL data file
# FUNCTION:             DESCRIPTION:
#  amplRunOpen           Opens a run file
#  amplRunAdd            Adds run specs to an existing AMPL run file
#  amplRunShow           Shows the content of an AMPL run file   
# FUNCTION:             DESCRIPTION:
#  amplOutShow           Shows the content of an AMPL output txt file     
################################################################################


amplModelOpen <-
    function(project)
{
    # A function written by Diethelm Wuertz
    
    # Description:
    #   Writes an AMPL model to a .mod file
    
    # Arguments:
    #   project - a project name, gives the root name of the model file
    
    # FUNCTION:
    
    # Compose File Name:
    file <- paste(project, "mod", sep = ".")
    
    # Write Model to File:
    write("", file=file, ncolumns=1, append=FALSE)   

    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------


amplModelAdd <-
    function(model, project)
{
    # A function written by Diethelm Wuertz
    
    # Description:
    #   Writes an AMPL model to a .mod file
    
    # Arguments:
    #   model - a character vector with the lines making the model file
    #   project - a project name, gives the root name of the model file
    
    # FUNCTION:
    
    # Compose File Name:
    file <- paste(project, "mod", sep = ".")
    
    # Write Model to File:
    write(model, file=file, ncolumns=1, append=TRUE)   

    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------


amplModelShow <-
    function(project)
{
    # A function written by Diethelm Wuertz
    
    # Description:
    #   Prints an AMPL .mod file
    
    # Arguments:
    #   project - a project name, gives the root name of the model file
    
    # FUNCTION:
    
    # Compose File Name:
    file <- paste(project, "mod", sep = ".")
    
    # Trace File:
    cat(readLines(file), sep = "\n")

    # Return Value:
    invisible()
}


################################################################################


amplDataOpen <-
    function(project)
{
    # A function written by Diethelm Wuertz
    
    # Description:
    #   Writes the header to an AMPL data file
    
    # Arguments:
    #   project - a project name, gives the root name of the model file
    
    # FUNCTION:
    
    # Compose File Name:
    file <- paste(project, "dat", sep = ".")
    
    # Write to the file:
    write("", file=file, ncolumns=1, append=FALSE)
    
    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------


amplDataAdd <- 
    function(name, data, type, project)
{
    # A function written by Diethelm Wuertz
    
    # Description:
    #   Check data type and adds to AMPL data file
    
    # Arguments:
    #   name - AMPL data name
    #   data - the data object, a numeric value, vector or matrix
    #   type - eiher "value", "vector" or "matrix"
    #   project - a project name, gives the root name of the model file
    
    # FUNCTION:

    # Add Data:
    if (type == "value") {
        amplDataAddValue(data=name, value=data, project=project) }
    if (type == "vector") {
        amplDataAddVector(data=name, vector=data, project=project) }
    if (type == "matrix") {
        amplDataAddMatrix(data=name, matrix=data, project=project) }
        
    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------


amplDataShow <-
    function(project)
{
    # A function written by Diethelm Wuertz
    
    # Description:
    #   Prints an AMPL data file
    
    # Arguments:
    #   project - a project name, gives the root name of the model file
    
    # FUNCTION:
    
    # Compose File Name:
    file <- paste(project, "dat", sep = ".")
    
    # Trace File:
    cat(readLines(file), sep = "\n")

    # Return Value:
    invisible()
}


# -----------------------------------------------------------------------------


amplDataSemicolon <-
    function(project)
{
    # A function written by Diethelm Wuertz
    
    # Description:
    #   Adds a semicolon on the end of a data input line
    
    # Arguments:
    #   project - a project name, gives the root name of the model file
    
    # FUNCTION:
    
    # Compose File Name:
    file <- paste(project, "dat", sep = ".")
    
    # Add a Semicolon and an Empty Line:
    write(";", file=file, ncolumns=1, append=TRUE)
    write(" ", file=file, ncolumns=1, append=TRUE)
    
    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------


amplDataAddValue <-
    function(data, value, project)
{
    # A function written by Diethelm Wuertz
    
    # Description:
    #   Adds a numeric value to an AMPL data file
    
    # Arguments:
    #   data - a character string, the name of the value
    #   value - a numeric value, the value of the numeric input variable
    #   project - a project name, gives the root name of the model file
    
    # FUNCTION:
    
    name <- data
    
    # Compose File Name:
    file <- paste(project, "dat", sep = ".")
    
    # Write Name:
    x <- paste("param", name, ":=", collapse=" ")
    write(x, file=file, ncolumns=1, append=TRUE)
    
    # Write Numeric Value:
    write(value, file=file, ncolumns=1, append=TRUE)
    amplDataSemicolon(project) 
    
    # Return Value:
    invisible()
}    


# ------------------------------------------------------------------------------


amplDataAddVector <-
    function(data, vector, project)
{
    # A function written by Diethelm Wuertz
    
    # Description:
    #   Adds a numeric vector to an AMPL data file
    
    # Arguments:
    #   data - a character string, the name of the vector
    #   value - a numeric vector, the values of the numeric input vector
    #   project -  a project name, gives the root name of the model file
    
    # FUNCTION:
    
    name <- data
    
    # Compose File Name:
    file = paste(project, "dat", sep = ".")
    
    # Write Name:
    x <- paste("param", name, ":=", collapse=" ") 
    write(x, file=file, ncolumns=1, append=TRUE)
    
    # Write Vector:
    vector <- as.vector(vector)
    N <- length(vector)
    write(t(cbind(1:N, vector)), file=file, ncolumns=2, append=TRUE)
    amplDataSemicolon(project) 
    
    # Return Value:
    invisible()
}   


# ------------------------------------------------------------------------------


amplDataAddMatrix <-
    function(data, matrix, project)
{
    # A function written by Diethelm Wuertz
    
    # Description:
    #   Adds a numeric matrix to an AMPL data file
    
    # Arguments:
    #   data - a character string, the name of the matrix
    #   value - a numeric matrix, the values of the numeric input matrix
    #   project -  a project name, gives the root name of the model file
    
    # FUNCTION:
    
    name <- data
    
    # Compose File Name:
    file <- paste(project, "dat", sep = ".")
    
    # Write Name:
    x <- paste("param", name, ":", collapse=" ")
    write(x, file=file, ncolumns=1, append=TRUE)
    
    # Write Matrix:
    N <- ncol(matrix)
    x <- paste(paste(1:N, collapse = " "), ":=")
    write(x, file = file, ncolumns = 1, append = TRUE)
    X <- cbind(1:nrow(matrix), matrix)
    colnames(X) = rownames(X) = NULL
    write(t(X), file = file, ncolumns = 1, append = TRUE, sep = " ")
    amplDataSemicolon(project) 
    
    # Return Value:
    invisible()
}


################################################################################


amplRunOpen <-
    function(project)
{
    # A function written by Diethelm Wuertz
    
    # Description:
    #   Writes an AMPL run to a .run file   
    
    # Arguments:
    #   project - a project name, gives the root name of the model file
    
    # FUNCTION:
    
    # Compose File Name:
    file <- paste(project, "run", sep = ".")
    
    # Open:
    write("", file=file, ncolumns=1, append=FALSE) 
    
    # Return Value:
    invisible()
}


# -----------------------------------------------------------------------------


amplRunAdd <-
    function(run, project)
{
    # A function written by Diethelm Wuertz
    
    # Description:
    #   Writes an AMPL run to a .run file   
    
    # Arguments:
    #   run - a character vector with the lines making the model file
    #   project -  a project name, gives the root name of the model file
    
    # FUNCTION:
    
    # Compose File Name:
    file <- paste(project, "run", sep = ".")
    
    # Write Run Commands to File:
    write(run, file=file, ncolumns=1, append=FALSE) 
    
    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------


amplRunShow <-
    function(project)
{
    # A function written by Diethelm Wuertz
    
    # Description:
    #   Prints an AMPL .run file
    
    # Arguments:
    #   project -  a project name, gives the root name of the model file
    
    # FUNCTION:
    
    # Compose File Name:
    file = paste(project, "run", sep = ".")
    
    # Trace File:
    cat(readLines(file), sep = "\n")

    # Return Value:
    invisible()
}


################################################################################


amplOutShow <-
    function(project)
{
    # A function written by Diethelm Wuertz
    
    # Description:
    #   Shows the content of an AMPL output txt file     
    
    # Arguments:
    #   project - a project name, gives the root name of the model file
    
    # FUNCTION:
    
    # Compose File Name:
    file <- paste(project, "txt", sep = ".")
    
    # Trace File:
    cat(readLines(file), sep = "\n")

    # Return Value:
    invisible()
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                DESCRIPTION:
#  feasiblePortfolio        Returns a feasible portfolio
################################################################################


feasiblePortfolio <-
    function(data, spec = portfolioSpec(), constraints = "LongOnly")
{
    # A function implemented Diethelm Wuertz

    # Description:
    #   Computes Risk and Return for a feasible portfolio

    # Arguments:
    #   data - a rectangular timeSeries object of assets
    #   spec - an object of class 'fPFOLIOSPEC'
    #   constraints - a character vector or NULL

    # FUNCTION:

    # Data and Assets Names:
    Data <- portfolioData(data, spec)
    if(inherits(data,"fPFOLIODATA")) data <- getSeries(Data) 
    assetsNames <- getUnits(Data)
    
    # Specification:
    Spec <- spec
    
    # Constraints:
    Constraints <- portfolioConstraints(Data, spec, constraints)

    # Get Weights:
    if(is.null(getWeights(spec))) {
        stop("Missing weights")
    }
    weights <- as.vector(getWeights(spec))
    names(weights) <- assetsNames

    if (inherits(getSeries(Data),"timeSeries")) {
    
        # Compute Returns:
        targetReturn <- c(
            mean = (Data@statistics$mean %*% weights)[[1]],
            mu = (Data@statistics$mu %*% weights)[[1]])
        setTargetReturn(spec) <- targetReturn
    
        # Compute Covariance Risk:
        Cov <- Data@statistics$Cov
        cov <- sqrt((weights %*% Cov %*% weights)[[1]])
    
        # Check Solver:
        # if (any(constraints@stringConstraints == "Short")) {
        #     setSolver(spec) = "solveRshortExact"
        #     warning("Short Constraints Specified: Solver forced to solveRshortExact")
        # }
        
        # Compute Alternative/Robust Covariance Risk:
        if (getType(spec) == "SPS") {
            myCheck <- TRUE
            funSigma <- match.fun(getObjective(spec)[1])
            rcov <- funSigma(as.vector(weights))
        } else {
            Sigma <- Data@statistics$Sigma
            rcov <- sqrt((weights %*% Sigma %*% weights)[[1]])
        }

        # Compute VaR:
        alpha <- getAlpha(spec)
        returns <- getDataPart(getSeries(Data)) %*% weights
        VaR <- quantile(returns, alpha, type = 1)

        # Compute CVaR:
        CVaR <- VaR - 0.5*mean(((VaR-returns) + abs(VaR-returns))) / alpha

        # Compose Risks:
        targetRisk <- c(cov, rcov, -CVaR, -VaR)
        names(targetRisk) <- c("Cov", "Sigma", "CVaR", "VaR")
        alpha <- getAlpha(Spec)
        
    } else if (inherits(getSeries(Data),"logical")) {
       
        # Compute Returns:
        targetReturn <- c(
            mean = (Data@statistics$mean %*% weights)[[1]],
            mu = NA)
        setTargetReturn(spec) <- targetReturn
        
        # Compute Covariance Risk:
        Cov <- Data@statistics$Cov
        cov <- sqrt((weights %*% Cov %*% weights)[[1]])
        
        # Compose Risks:
        targetRisk <- c(cov, NA, NA, NA)
        names(targetRisk) <- c("Cov", "Sigma", "CVaR", "VaR")
        alpha <- NA
    
    }
        
    # Compute Risk Budgets:
    covRiskBudgets <- (weights * Cov %*% weights)[, 1] / cov^2
    names(covRiskBudgets) <- assetsNames
    
      
    # Compose Portfolio:
    Portfolio <- new("fPFOLIOVAL",
        portfolio = list(
            weights = weights,
            covRiskBudgets = covRiskBudgets,
            targetReturn = targetReturn,
            targetRisk = targetRisk,
            targetAlpha = alpha,
            status = getStatus(spec)))
    
    # Return Value:
    new("fPORTFOLIO",
        call = match.call(),
        data = Data,
        spec = Spec,
        constraints = Constraints,
        portfolio = Portfolio,
        title = "Feasible Portfolio",
        description = description() )
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                    DESCRIPTION:
#  solveRglpk.MAD               Portfolio interface to solver Rglpk
#  .madRglpkArguments           Returns MAD arguments for solver
# FUNCTION:                    DESCRIPTION:
#  .rglpk.MAD                   Wrapper to solver function
################################################################################


solveRglpk.MAD <-
    function(data, spec, constraints)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Portfolio interface to solver Rglpk

    # FUNCTION:

    # Settings:
    Data <- portfolioData(data, spec)
    data <- getSeries(Data)
    nAssets <- getNAssets(Data)
    type <- getType(spec)

    # Compile Arguments for Solver:
    args <- .madRglpkArguments(Data, spec, constraints)

    # Solve Multiassets Portfolio:
    ans <- .rglpk.MAD(
        obj = args$obj,
        mat = args$mat,
        dir = args$dir,
        rhs = args$rhs,
        types = args$types,
        max = args$max,
        bounds = args$bounds,
        verbose = args$verbose,
        nScenarios = args$nScenarios,
        nAssets = args$nAssets,
        targetReturn = args$targetReturn,
        Alpha = args$Alpha,
        Type = args$Type)
    ans$solver <- "solveRglpk.MAD"
    
    # Return Value:
    class(ans) = c("solver", "list")
    ans
}


################################################################################


.madRglpkArguments <-
    function(data, spec, constraints)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns glpk conform MAD arguments for the solver

    # Details:
    #       max/min:                   obj %*% x
    #       subject to:
    #                           mat %*% x  ?=  rhs
    #                                  dir  =  "?="
    #                              upper/lower bounds
    #
    #   Rglpk_solve_LP(obj, mat, dir, rhs, types = NULL, max = FALSE,
    #       bounds = NULL, verbose = FALSE)

    # FUNCTION:

    # Settings:
    Data <- portfolioData(data, spec)
    data <- getSeries(Data)
    nAssets <- ncol(data)
    nScenarios <- nrow(data)
    series <- getDataPart(data)
    series <- series - matrix(rep(colMeans(series), times=nScenarios), byrow=TRUE, ncol=nAssets)
    targetReturn <- getTargetReturn(spec)
    Type <- getType(spec)

    # Objective Function to be Maximized:
    objNames <- c(paste("e", 1:nScenarios, sep = ""), colnames(data))
    obj <- c(rep(1/nScenarios, nScenarios), rep(0, nAssets))
    names(obj) <- objNames

    # 1:
    # The negative MAD Equation Constraints:
    #   (-diag + [Returns-mu]) %*% (es, W) <= 0
    Aneg <- cbind(-diag(nScenarios), series)
    aneg <- rep(0, nrow(Aneg))
    dneg <- rep("<=", nrow(Aneg))

    # 2:
    # The positive MAD Equation Constraints:
    #   (+diag + [Returns-mu]) %*% (es, W) >= 0
    Apos <- cbind( diag(nScenarios), series)
    apos <- rep(0, nrow(Apos))
    dpos <- rep(">=", nrow(Apos))

    # 3 and 4:
    # The A_equal Equation Constraints: A_eq %*% x == a_eq
    eqsumW <- eqsumWConstraints(Data, spec, constraints)
    Aeq <- cbind(
        matrix(0, ncol=nScenarios, nrow=nrow(eqsumW)), 
        matrix(eqsumW[, -1], ncol=nAssets) )
    aeq <- eqsumW[, 1]
    deq <- rep("==", nrow(eqsumW))

    # 5:
    # The e_s > = 0 Equation Constraints:
    Aes <- cbind(diag(nScenarios), matrix(0, nrow=nScenarios, ncol=nAssets))
    aes <- rep(0, nrow(Aes))
    des <- rep(">=", nrow(Aes))

    # 6:
    # Group Constraints: A W >= a
    minsumW <- minsumWConstraints(Data, spec, constraints)
    if (is.null(minsumW)){
        Aminsum <- aminsum <- dminsum <- NULL
    } else {
        Aminsum <- cbind(
            matrix(0, nrow=nrow(minsumW), ncol=nScenarios),
            minsumW[, -1, drop=FALSE] )
        aminsum <- minsumW[, 1]
        dminsum <- rep(">=", nrow(minsumW))
    }

    # 7:
    # Group Constraints: A W <= b
    maxsumW <- maxsumWConstraints(Data, spec, constraints)
    if (is.null(maxsumW)){
        Amaxsum <- amaxsum <- dmaxsum <- NULL
    } else {
        Amaxsum <- cbind(
            matrix(0, nrow=nrow(maxsumW), ncol=nScenarios),
            maxsumW[, -1, drop=FALSE] )
        amaxsum <- maxsumW[, 1]
        dmaxsum <- rep("<=", nrow(maxsumW))
    }

    # Putting all Together:
    mat <- rbind(Aeq, Apos, Aneg, Aes, Aminsum, Amaxsum)
    rhs <-     c(aeq, apos, aneg, aes, aminsum, amaxsum)
    dir <-     c(deq, dpos, dneg, des, dminsum, dmaxsum)

    # Box Constraints: Upper and Lower Bounds as listn required ...
    minW <- minWConstraints(Data, spec, constraints)
    maxW <- maxWConstraints(Data, spec, constraints)
    nInd <- 1:(nScenarios+nAssets)
    bounds <- list(
        lower = list(ind = nInd, val = c(rep(  0, nScenarios), minW)),
        upper = list(ind = nInd, val = c(rep(Inf, nScenarios), maxW)) )

    # What variable Types, All Continuous:
    types <- NULL

    # Should I minimize or maximize ?
    max <- FALSE

    # Return Value:
    list(
        obj = obj, mat = mat, dir = dir, rhs = rhs,
        types = types, max = max, bounds = bounds, verbose = FALSE,
        nScenarios = nScenarios, nAssets = nAssets,
        targetReturn = targetReturn, Alpha = NA, Type = Type)
}


################################################################################


.rglpk.MAD <-
    function(obj, mat, dir, rhs, types, max, bounds, verbose,
    nScenarios, nAssets, targetReturn, Alpha, Type)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Rglpk MAD Solver

    # FUNCTION:

    # Solve - use Rglpk_solve_LP:
    optim <- Rglpk::Rglpk_solve_LP(
        obj = obj,
        mat = mat,
        dir = dir,
        rhs = rhs,
        types = types,
        max = max,
        bounds = bounds,
        verbose = verbose)

    # Extract Weights:
    weights <- .checkWeights(rev(rev(optim$solution)[1:nAssets]))
    attr(weights, "invest") = sum(weights)

    # Result:
    ans <- list(
        type = Type,
        solver = "Rglpk.MAD",
        optim = optim,
        weights = weights,
        solution = weights,
        targetReturn = targetReturn,
        targetRisk = -optim$optimum,
        objective = -optim$optimum,
        status = optim$status[[1]],
        message = "NA")

    # Return Value:
    ans
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  US


################################################################################
# FUNCTION:              DESCRIPTION:
#  backtestStats          Wrapper function for calculating rolling statistics
# FUNCTION:              DESCRIPTION:
#  rollingSigma           Rolling portfolio Sigma risk
#  rollingVaR             Rolling Value at Risk
#  rollingCVaR            Rolling Conditional Value at Risk
#  rollingDar             Rolling Drawdowns at Risk
#  rollingCDaR            Rolling Conditional Drawdowns at Risk
################################################################################


backtestStats <- 
  function(object, FUN = "rollingSigma", ...)
  {
    # A function implemented by William Chen
    
    # Description:
    #   Wrapper function for calculating rolling statistics
    
    # Arguments:
    #   object - a list as returned by the function portfolioBacktesting()
    #   FUN - a character string, the name of the statistics function
    
    # Example:
    #   data = returns(align(SPISECTOR))
    #   formula <- SPI ~ BASI+INDU+CONG+HLTH+CONS+TELE+UTIL+FINA+TECH
    #   backtests <- portfolioBacktesting(formula, data, trace = FALSE) 
    #   portfolios <- portfolioSmoothing(backtests, portfolioBacktest())
    
    # FUNCTION:
    
    # Perform Statistics:
    statsFun <- match.fun(FUN)
    ans <- statsFun(object, ...)
    
    # Return Value
    ans
  }

# ------------------------------------------------------------------------------


rollingSigma <-
  function(object)
  {
    # A function implemented by William Chen and Diethelm Wuertz
    
    # Description:
    #   Returns rolling sigmas from an object of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - a list as returned by the function portfolioBacktesting()
    
    # Example:
    #   rollingSigma(object)
    
    # FUNCTION:
    
    # quick fix ... there is some confusion with getTargetRisk of
    # @portfolio and @spec
    portfolios <- object$strategyList
    prtval <- lapply(portfolios, slot, "portfolio")
    ans <- sapply(prtval, function(x) getTargetRisk(x)["Sigma"])
    dates <- sapply(portfolios, function(x) rev(rownames(getSeries(x)))[1])
    
    # Return Value:
    timeSeries(ans, charvec = dates, units = "Sigma")
  }


# ------------------------------------------------------------------------------


rollingVaR <-
  function(object)
  {
    # A function implemented by William Chen
    
    # Description:
    #   Returns rolling VaR from an object of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - a list as returned by the function portfolioBacktesting()
    
    # Example:
    #   rollingVaR(object)
    
    # FUNCTION:
    
    # calculate VaR for one portfolio:
    .var <- function(x) {
      alpha <- getAlpha(x)
      R <- as.numeric(getSeries(x) %*% getWeights(x))
      quantile(R, probs = alpha)
    }
    
    # Get Portfolios:
    portfolios <- object$strategyList
    
    # Calculates VaR for all portfolios:
    ans <- sapply(portfolios, FUN = .var)
    
    # Extracts the dates:
    dates <- sapply(portfolios, function(x) rev(rownames(getSeries(x)))[1])
    
    # Return Value:
    alpha <- getAlpha(portfolios[[1]])
    timeSeries(ans, charvec = dates, units = paste("VaR", alpha, sep = "."))
  }


# ------------------------------------------------------------------------------


rollingCVaR <- 
  function(object)
  {
    # A function implemented by William Chen
    
    # Description:
    #   Returns rolling DVaR from an object of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - a list as returned by the function portfolioBacktesting()
    
    # Example:
    #   rollingCVaR(object)
    
    # FUNCTION:
    
    # Calculate CVaR for one portfolio:
    .cvar <- function(x) {
      alpha <- getAlpha(x)
      R <- as.numeric(getSeries(x) %*% getWeights(x))
      z <- quantile(R, probs = alpha)
      mean(R[R <= z], na.rm = TRUE)
    }
    
    # Get Portfolios:
    portfolios <- object$strategyList
    
    # Calculate CVaR for all portfolios:
    ans <- sapply(portfolios, FUN = .cvar)
    
    # Extract the Dates:
    dates <- sapply(portfolios, function(x) rev(rownames(getSeries(x)))[1])
    
    # Return:
    alpha <- getAlpha(portfolios[[1]])
    timeSeries(ans, charvec = dates, units = paste("CVaR", alpha, sep = "."))
  }


# ------------------------------------------------------------------------------


rollingDaR <-
  function(object)
  {
    # A function implemented by William Chen
    
    # Description:
    #   Returns rolling DaR from an object of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - a list as returned by the function portfolioBacktesting()
    
    # Example:
    #   rollingDaR(object)
    
    # FUNCTION:
    
    # calculate DaR for one portfolio:
    .dar <- function(x) {
      alpha <- getAlpha(x)
      R <- as.numeric(getSeries(x) %*% getWeights(x))
      dd <- 100 * drawdowns(as.timeSeries(R)/100)
      quantile(dd, probs = alpha)
    }
    
    # Get Portfolios:
    portfolios <- object$strategyList
    
    # Calculates DaR for all portfolios:
    ans <- sapply(portfolios, FUN = .dar)
    
    # Extracts the dates:
    dates <- sapply(portfolios, function(x) rev(rownames(getSeries(x)))[1])
    
    # Return:
    alpha <- getAlpha(portfolios[[1]])
    timeSeries(ans, charvec = dates, units = paste("DaR", alpha, sep = "."))
  }


# ------------------------------------------------------------------------------


rollingCDaR <- 
  function(object)
  {
    # A function implemented by William Chen
    
    # Description:
    #   Returns rolling CDaR from an object of class fPFOLIOBACKTEST
    
    # Arguments:
    #   object - a list as returned by the function portfolioBacktesting()
    
    # Example:
    #   rollingCDaR(object)
    
    # FUNCTION:
    
    # Calculate CDaR for one portfolio:
    .cdar <- function(x){
      alpha <- getAlpha(x)
      R <- as.numeric(getSeries(x) %*% getWeights(x))
      dd <- 100 * drawdowns(as.timeSeries(R)/100)
      z <- quantile(as.numeric(dd), probs = alpha)
      mean(dd[dd <= z])
    }
    
    # Get Portfolios:
    portfolios <- object$strategyList
    
    # Calculate CVaR for all portfolios:
    ans <- sapply(portfolios, FUN = .cdar)
    
    # Extract the dates:
    dates <- sapply(portfolios, function(x) rev(rownames(getSeries(x)))[1])
    
    # Return:
    alpha <- getAlpha(portfolios[[1]])
    timeSeries(ans, charvec = dates, units = paste("CDaR", alpha, sep = "."))
  }


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################
# FUNCTION:                    DESCRIPTION:
#  portfolioBacktest            Returns an object of class fPFOLIOBACKTEST
################################################################################


portfolioBacktest <-
function(
    windows = list(
        windows = "equidistWindows",
        params = list(
            horizon = "12m")),
    strategy = list(
        strategy = "tangencyStrategy",
        params = list()),
    smoother = list(
        smoother = "emaSmoother",
        params = list(
            doubleSmoothing = TRUE,
            lambda = "3m",
            skip = 0,
            initialWeights = NULL)),
    messages = list() )
{
    # A function implemented by William Chen and Diethelm Wuertz
        
    # Description:
    #   Specifies a portfolio to be optimized from scratch

    # Example:
    #   portfolioBacktest()
   
    # Arguments:
    #   windows - rolling windows slot:
    #       windows - the name of the rollings windows function
    #       params - parameter list for windows settings:
    #           horizon - length of the rolling windows
    #   strategy - portfolio strategy slot:
    #       strategy - the name of the portfolio strategy function
    #       params - parameter list for strategy settings:
    #   smoother - smoother approach slot:
    #       smoother - the name of the portfolio weights smoother function
    #       params - parameter list for smoother settings:
    #           doubleSmoothing - a flag sould we double smooth the weights?
    #           lambda - length of the ema smoothing parameter
    #           skip - hoqw many periods should be skipped for smoothing ?
    #           initialWeights - vector of initial weights
   
    # FUNCTION:

    # Return Value:
    new("fPFOLIOBACKTEST",
        windows = windows,
        strategy = strategy,
        smoother = smoother,
        messages = messages)
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


############################################################################### 
# FUNCTION:              DESCRIPTION: 
#  stabilityAnalytics      Retroactive stability analytics 
# FUNCTION:              DESCRIPTION:        
#  turnsAnalytics          Retroactive turning point analytics 
#  drawdownsAnalytics      Retroactive maximum drawdown analytics
#  garchAnalytics          Retroactive Garch volatility analytics
#  riskmetricsAnalytics    Retroactive Riskmetrics analytics
#  bcpAnalytics            Retroactive Bayesian changepoints analytics
#  pcoutAnalytics          Retroactive Principal component outlier analytics
# FUNCTION:              DESCRIPTION:
#  addRainbow              Adds rainbow colored stability indicators
# FUNCTION:              DESCRIPTION: 
#  waveletSpectrum         Retroactive Morlet wavelet analytics
# FUNCTION:              DESCRIPTION:
#  parAnalytics            Graph frame settings for a desired analytics
############################################################################### 


stabilityAnalytics <- 
  function(index, method=c("turns", "drawdowns", "garch", 
                           "riskmetrics", "bcp", "pcout"), ...)
  {
    # A function implemented by Diethelm Wuertz and Tobias Setz
    
    # Description:
    # Retroactive stability analytics 
    
    # FUNCTION:
    
    # Run Analytics:
    method <- match.arg(method)
    FUN <- paste(method, "Analytics", sep = "")
    fun <- match.fun(FUN)
    
    # Return Value:
    fun(index, ...)
  }


###############################################################################


turnsAnalytics <- 
  function(index, spar=0.5, main=NULL, 
           trace=TRUE, doplot=TRUE, at=pretty(index), format="%m/%y")
  {
    # A function implemented by Diethelm Wuertz and Tobias Setz
    
    # Description:
    #   Retroactive Turning Points Analytics
    
    # Arguments:
    #   index - an index or price S4 'timeSeries' object
    #   spar - a numeric between 0 and, the degree of smoothness
    #   main - main plot title
    #   trace - a logical, should the results be traced?
    #   plot -  a logical, should the results be plotted
    #   at - generate pretty axis positions
    #   format -  a string describing the label format
    
    # Note:
    #   The lowess and supsmu smoothers are by far not as good as the
    #   spline smoother.
    
    # FUNCTION:
    
    # Settings:
    stopifnot(isUnivariate(index))
    if (is.null(main)) main <- "Retroactive Turnpoints Analytics"
    
    # Smooth Return Series:
    rets <- returns(index)
    indexSmu <- smoothSpline(log(index), spar=spar)
    
    # Turnpoints:
    warn <- getOption("warn")
    options(warn=-1)
    indexTurns <- turns(indexSmu[, 2])
    options(warn=warn)
    indexTurns <- indexTurns[indexTurns[, 2] !=0, ]
    
    # Add Verical Lines:
    turns.tps <- format(time(indexTurns))
    n.tps <- length(turns.tps)
    
    # Trace Results:
    if (trace){
      cat("Series:\n", colnames(index), "\n")
      cat("Turning Points:", n.tps, "\n")
      print(turns.tps) 
    }
    
    # Positions:
    positions <- sign(returns(exp(indexSmu[, 2]), trim=FALSE))
    positions[1, 1] <- 0
    ablines <- time(positions)[as.vector(positions) < 0]
    
    # Plot:
    if (doplot) {
      
      # Turnpoints:
      range <- range(log(index))
      ylim <- c(range[1], range[2] + diff(range(log(index)))/4)
      plot(indexSmu[, 1], ylim=ylim, main="", ylab="", xlab="", las=2, 
           at=at, format=format, col="black")
      title(main=main, ylab=paste("log Index", colnames(index)), xlab="")
      abline(v=ablines, lty=3, lwd=2, col="steelblue")
      lines(indexSmu[, 1], col="black") 
      lines(indexSmu[, 2], col="red") 
      if (turns.tps[2] > 0) points(indexTurns[, 1], pch=19, col="red")
      abline(v=as.POSIXct(at), col="darkgrey", lty=3)
      box(col="white")
      box(bty="l")
      
      # Add Returns:
      center <- range[2] + diff(range(log(index)))/4/2
      scale <- diff(range(log(index)))/4
      returnsScaled <- (rets-mean(rets))/max(abs(rets)) * scale/2 + center
      lines(returnsScaled, col="orange")
      abline(h=mean(returnsScaled), col="darkgrey", lty=3)
      box(col="white")
      box(bty="l")
      
      # Add Rmwtrics - Do not Remove!
      mtext("Rmetrics", adj=0, side=4, cex=0.7, col="darkgrey")
      
    }
    
    # Check Turnpoints:
    if ( turns.tps[1] == "1" & turns.tps[2] == "0")
      n.tps <- 0
    
    # Return Value:
    invisible(list(
      data=indexSmu, 
      turns=turns.tps, 
      positions=positions, 
      ablines=ablines, 
      n=n.tps, 
      smooth=spar))
  }


###############################################################################    


drawdownsAnalytics <- 
  function(index, spar=0.5, main=NULL, 
           trace=TRUE, doplot=TRUE, at=pretty(index), format="%m/%y")
  {
    # A function implemented by Diethelm Wuertz and Tobias Setz
    
    # Description:
    #   Retroactive Maximum Drawdown Analytics
    
    # Arguments:
    #   index - an index or price S4 'timeSeries' object
    #   spar - a numeric between 0 and, the degree of smoothness
    #   main - main plot title
    #   trace - a logical, should the results be traced?
    #   plot -  a logical, should the results be plotted
    #   at - generate pretty axis positions
    #   format -  a string describing the label format
    
    # FUNCTION:
    
    # Settings:
    stopifnot(isUnivariate(index))
    if (is.null(main)) main <- "Retroactive Drawdowns Analytics"
    
    # Series:
    rets<- returns(index)
    
    # Turning Points:
    tps <- turnsAnalytics(index=index, spar=spar, trace=trace, doplot=FALSE)
    ablines <- tps$ablines
    
    # Drawdowns:
    maxdd <- drawdowns(rets)
    
    # Plot:
    if(doplot) {
      
      # Plot:
      plot(maxdd, main=main, xlab="", ylab=paste("Drawdwons", colnames(index)),
           las=2, at=at, format=format)
      abline(v=ablines, lty=3, lwd=2, col="steelblue")
      lines(maxdd)
      box(col="white")
      box(bty="l")
      
      # Add Rmwtrics - Do not Remove!
      mtext("Rmetrics", adj=0, side=4, cex=0.7, col="darkgrey")
      
    }
    
    # Return Value:
    invisible(list(
      index = index, 
      series = maxdd))
  }


###############################################################################


garchAnalytics <-
  function (index, spar = 0.5, main=NULL, 
            trace=TRUE, doplot=TRUE, at=pretty(index), format="%m/%y")
  {
    # A function implemented by Diethelm Wuertz and Tobias Setz
    
    # Arguments:
    #   index - an index or price S4 'timeSeries' object
    #   spar - a numeric between 0 and, the degree of smoothness
    #   main - main plot title
    #   trace - a logical, should the results be traced?
    #   plot -  a logical, should the results be plotted
    #   at - generate pretty axis positions
    #   format -  a string describing the label format
    
    # Description:
    #   Retroactive Garch11 Volatility Analytics
    
    # FUNCTION:
    
    # Load Library:
    # require(fGarch)
    
    # Settings:
    stopifnot(isUnivariate(index))
    if (is.null(main)) main <- "Retroactive Garch Analytics"
    
    # Fit Garch11 Model:
    fit <- fGarch::garchFit(data = 100 * returns(index), trace=trace)
    xseries <- as.timeSeries(fit@data)/100
    Index <- cumulated(xseries)
    colnames(Index) <- colnames(index)
    
    # Turning Points:
    tps <- turnsAnalytics(index=index, spar=spar, trace=trace, doplot=FALSE)
    ablines <- tps$ablines
    
    # Plot Return Series and Standard Deviations:
    xcsd <- timeSeries(data = fit@sigma.t/100, charvec = time(xseries))
    if (doplot) {
      
      # Plot:
      sdPlus <- mean(xseries) + 2 * xcsd
      sdMinus <- mean(xseries) - 2 * xcsd
      range <- range(xseries, sdPlus, sdMinus)
      plot(xseries, 
           main=main, xlab="", ylab=paste("Volatility", colnames(index)), 
           at=at, format=format, 
           # type="l", 
           col="steelblue", ylim=range)
      abline(v = ablines, lty = 3, lwd = 2, col = "grey")
      lines(xseries, col = "steelblue")
      lines(sdPlus, col = "red", lwd = 2)
      lines(sdMinus, col = "red", lwd = 2)
      abline(h = 0, col = "grey", lty = 3)
      box(col = "white")
      box(bty = "l")
      
      # Margin Text:
      # mtext("Volatility Band: 2 sd", adj = 0, side = 4, cex = 0.7, 
      #     col = "darkgrey")
      
      # Add Rmwtrics - Do not Remove!
      mtext("Rmetrics", adj=0, side=4, cex=0.7, col="darkgrey")
      
    }
    
    # Return Value:
    invisible(list(
      index = index, 
      residuals = xseries,
      volatility = xcsd, 
      fit = fit))
  }


###############################################################################


riskmetricsAnalytics <- 
  function(index, spar=0.5, lambda=0.9, main=NULL, 
           trace=TRUE, doplot=TRUE, at=pretty(index), format="%m/%y")
  {    
    # A function implemented by Diethelm Wuertz and Tobias Setz
    
    # Description:
    #   Retroactive riskmetrics analytics
    
    # Arguments:
    #   index - an index or price S4 'timeSeries' object
    #   spar - a numeric between 0 and, the degree of smoothness
    #   main - main plot title
    #   trace - a logical, should the results be traced?
    #   plot -  a logical, should the results be plotted
    #   at - generate pretty axis positions
    #   format -  a string describing the label format
    
    # FUNCTION:
    
    # Settings:
    stopifnot(isUnivariate(index))
    if (is.null(main)) main <- "Retroactive RiskMetrics Analytics"
    
    # Series:
    rets <- returns(index)
    
    # Turning Points:
    tps <- turnsAnalytics(index=index, spar=spar, trace=trace, doplot=FALSE)
    ablines <- tps$ablines
    
    # Indicator:
    sigma <- .emaIndicator(abs(rets), lambda)
    sd <- sqrt(.emaIndicator(rets^2, lambda))
    
    # Plot:
    if(doplot) {
      
      # Plot:
      sdPlus <- mean(rets) + 2 * sd
      sdMinus <- mean(rets) - 2 * sd
      range <- range(rets, sdPlus, sdMinus)
      plot(rets, 
           main="", xlab="", ylab=paste("Volatility", colnames(index)),
           at=at, format=format,
           ylim=range) 
      abline(v=ablines, lty=3, lwd=2, col="grey")
      lines(rets, col="steelblue")
      lines(mean(rets) + 2*sd, col="orange", lwd=2)
      lines(mean(rets) - 2*sd, col="orange", lwd=2)
      lines(mean(rets) + 2*sigma, col="red", lwd=2)
      lines(mean(rets) - 2*sigma, col="red", lwd=2)
      abline(h=0, col="grey", lty=3)
      box(col="white")
      box(bty="l")
      
      # Margin Text:
      # mtext(paste("sd/var Volatility Bands: 2 sd  |  lambda", lambda),
      #   adj=0, side=4, cex=0.7, col="darkgrey")  
      
      # Add Rmwtrics - Do not Remove!
      mtext("Rmetrics", adj=0, side=4, cex=0.7, col="darkgrey")
      
    }
    
    # Return Value:
    invisible(list(
      index = index, 
      analytics = sigma))
  }


###############################################################################


bcpAnalytics <-
  function (index, spar=0.5, FUN=returns, method=c("prob", "mean", "var"),
            main=NULL, trace=TRUE, doplot=TRUE, at=pretty(index), format="%m/%y")
  {
    # A function implemented by Diethelm Wuertz and Tobias Setz
    
    # Description:
    #   Retroactive Bayesian Change Points Analytics
    
    # Arguments:
    #   index - an index or price S4 'timeSeries' object
    #   spar - a numeric between 0 and, the degree of smoothness
    #   main - main plot title
    #   trace - a logical, should the results be traced?
    #   plot -  a logical, should the results be plotted
    #   at - generate pretty axis positions
    #   format -  a string describing the label format
    
    # FUNCTION:
    
    # Load Library:
    # require(bcp)
    
    # Settings:
    stopifnot(isUnivariate(index))
    if (is.null(main)) main <- "Retroactive Change Points Analytics"
    
    # Turning Points:
    tps <- turnsAnalytics(index=index, spar=spar, trace=trace, doplot=FALSE)
    positions <- tps$positions
    ablines <- tps$ablines
    
    # BCP Analytics:
    fun <- match.fun(FUN)
    series <- fun(index)
    analytics <- bcp::bcp(series)
    
    # Compose Series:
    method <- match.arg(method)
    select <- c(mean="posterior.mean", var="posterior.var", prob="posterior.prob")
    series(series) <- analytics[[select[method]]]
    
    # Compose y-axis Label:
    ylab <- c(mean="Mean", var="Variance", prob="Probability")
    ylab <- paste("Posterior", ylab[method], colnames(index))
    
    # Select:
    if (method == "prob") {
      ylim <- c(0, 1) 
      prob <- timeSeries(data=analytics$posterior.prob, charvec=time(series))
      prob <- na.omit(prob)
    } else {
      ylim <- range(na.omit(series))
      prob <- NA
    }
    
    # Plot:
    if (doplot) {
      
      # Plot:
      plot(series, type="h", ylim=ylim, las=2, col="grey",
           main=main, xlab="", ylab=ylab,
           at=at, format=format)
      abline(v=ablines, lty=3, lwd=2, col="steelblue")
      points(series, pch=19, cex=0.5)       
      
      box(col = "white")
      box(bty = "l")
      
      # MarginText:
      # mtext(paste("Smooth:", spar), adj = 0, side = 4, cex = 0.7, col = "darkgrey")
      
      # Add Rmwtrics - Do not Remove!
      mtext("Rmetrics", adj=0, side=4, cex=0.7, col="darkgrey")
      
    }
    
    # Return Value:
    invisible(list(
      index = index, 
      analytics = analytics,
      prob = prob))
  }


###############################################################################


pcoutAnalytics <-
  function (index, spar=0.5, main=NULL, 
            trace=TRUE, doplot=TRUE, at=pretty(index), format="%m/%y",
            strong=TRUE, k=2, cs=0.25, outbound=0.25) 
  {
    # A function implemented by Diethelm Wuertz and Tobias Setz
    
    # Description:
    #   Retroactive PCA outlier analytics
    
    # Arguments:
    #   index - an index or price S4 'timeSeries' object
    #   spar - a numeric between 0 and, the degree of smoothness
    #   main - main plot title
    #   trace - a logical, should the results be traced?
    #   plot -  a logical, should the results be plotted
    #   at - generate pretty axis positions
    #   format -  a string describing the label format
    
    # FUNCTION:
    
    # Load Library:
    # require(mvoutlier)
    
    # Settings:
    stopifnot(isUnivariate(index))
    if (is.null(main)) main <- "Retroactive Outlier Analytics"
    
    # Turning Points:
    tps <- turnsAnalytics(index=index, spar=spar, trace=trace, doplot=FALSE)   
    positions <- tps$positions
    ablines <- tps$ablines
    
    # Series Settings:
    X <- log(index)
    Y <- returns(index)    
    Z <- cbind(X[-1, ], Y)
    u <- Y
    v <- lag(u, k=-k:k, trim=TRUE)
    U <- u[time(v), ]
    
    # Principal Component Outlier Analytics: 
    ans <- mvoutlier::pcout(v, makeplot=FALSE, explvar=0.99, crit.M1=1/3, 
                            crit.c1=2.5, crit.M2=1/4, crit.c2=0.99, cs=cs, 
                            outbound=outbound)
    
    # Plot: 
    colnames(X) <- paste(colnames(X), "X", sep=":")
    colnames(Y) <- paste(colnames(Y), "Y", sep=":")
    Z <- cbind(X[time(v), ], Y[time(v), ])
    Z@recordIDs <- data.frame(wfinal01=ans$wfinal01, wfinal=ans$wfinal, 
                              wloc=ans$wloc, wscat=ans$wscat, x.dist1=ans$x.dist1, 
                              x.dist2=ans$x.dist2)
    rownames(Z@recordIDs) <- rownames(Z)
    wfinal01 <- ans$wfinal01
    
    if (strong) {
      V <- as.timeSeries(Z@recordIDs)[, "wfinal01"]
      W <- lag(V, k=k, trim=TRUE)
      S <- timeSeries(data=rowSums(W))
      Sfinal <- S == 0
      z <- timeSeries(charvec=time(Z), data=ans$wfinal01)
      z[time(S), ] <- as.integer(!Sfinal)
      wfinal01 <- as.numeric(z)
      Z@recordIDs[, "wfinal01"] <- wfinal01
    }
    
    ans$wfinal <- ans$wfinal01 <- ans$wloc <- ans$wscat <- NULL
    ans$x.dist1 <- ans$x.dist2 <- NULL
    
    U <- Z[, 1]
    wfinal01 <- Z@recordIDs$wfinal01
    datapoints <- length(U)
    
    U <- Z[, 2]
    outliers <- length(U) - sum(wfinal01)
    percent <- round(100 * outliers/length(U), 2)   
    weights <- as.timeSeries(Z@recordIDs)[, "wfinal"]
    
    # Analytics:
    Indicator <- 1 - weights
    invWeights <- as.vector(Indicator)
    extreme = sum(invWeights[invWeights > 0.75]) / length(invWeights)
    analytics <- c(
      mean = mean(1-weights), 
      sd = sd(1-weights), 
      extreme75 = extreme, 
      indicator = Indicator)
    
    ylab <- paste("PC Outlier Prob", colnames(index))
    
    # Plot:
    if(doplot) {
      
      # Plot:
      plot(Indicator, type="h", ylim=c(0, 1), las=2, col="grey", 
           main=main, xlab="", ylab=ylab, 
           at=at, format=format)
      abline(v=ablines, lty=3, lwd=2, col="steelblue")
      points(Indicator, pch=19, cex=0.5)         
      
      # Add Grid and Box:
      grid(NA, ny=NULL) 
      box(col="white")
      box(bty="l")
      
      # Margin Text:
      # mtext(paste("Smooth:", spar), adj=0, side=4, cex=0.7, col="darkgrey")
      
      # Add Rmetrics - Do not Remove!
      mtext("Rmetrics", adj=0, side=4, cex=0.7, col="darkgrey")
      
    }
    
    # Return Value:
    ans <- invisible(list(
      index = index, 
      pcout = ans, 
      series = Z, 
      analytics = analytics,
      prob = Indicator))
    class(ans) <- c("analytics", "list")
    ans
  }


# -----------------------------------------------------------------------------


addRainbow <- 
  function(analytics, palette=rainbow, a=0.3, b=0.8, K=100)
  {
    # A function implemented by Diethelm Wuertz and Tobias Setz
    
    # Example:
    #    analytics <- pcoutAnalytics(index); addRainbow(analytics)
    #    analytics <- bcpAnalytics(index); addRainbow(analytics)
    
    # FUNCTION:
    
    # Get Probability Indicator:
    Indicator <- analytics$prob
    
    # Check:
    stopifnot(isUnivariate(Indicator))
    stopifnot(min(Indicator) >= 0)
    stopifnot(max(Indicator) <= 1)
    
    # Add Spline Smoothed Indicators:
    k.spar <- seq(a, b, length=K)
    for (k in 1:K) {
      curve <- timeSeries::smoothSpline(Indicator, spar=k.spar[k])[, 2]
      if (k == 1) curve1 <- curve
      if (k == K) diff <- curve - curve1
      lines(curve, lwd=2, col=palette(K)[k]) }
    
    # Add Difference Indicator Line:
    lines(diff + 0.5, lwd=2)
    abline(h=0.5, lwd=2, col="orange")
    
    # Return Value:
    invisible()
  }


###############################################################################


waveletSpectrum <-
  function(index, spar=0.5, main=NULL,
           trace=TRUE, doplot=TRUE, at=pretty(index), format="%m/%y") 
  {
    # A function implemented by Diethelm Wuertz and Tobias Setz
    
    # Description:
    #    Morlet Wavelet Analytics
    
    # Arguments:
    #   index - an index or price S4 'timeSeries' object
    #   spar - a numeric between 0 and, the degree of smoothness
    #   main - main plot title
    #   trace - a logical, should the results be traced?
    #   plot -  a logical, should the results be plotted
    #   at - generate pretty axis positions
    #   format -  a string describing the label format
    
    # FUNCTION:
    
    # Load Library:
    # require(dplR)
    
    # Settings:
    stopifnot(isUnivariate(index))
    if (is.null(main)) main <- "Morlet Wavelet Spectrum"
    
    # Index Returns:
    returns <- returns(index)
    Time <- 2:length(index)
    Returns <- as.vector(returns)
    
    ans <- dplR::morlet(y1=returns, x1=Time)
    # Returns a list containing:
    # y          Numeric. The original time series.
    # x          Numeric. The time values.
    # wave       Complex. The wavelet transform.
    # coi        Numeric. The cone of influence.
    # period     Numeric. The period.
    # Scale      Numeric. The scale.
    # Signif     Numeric. The significant values.
    # Power      Numeric. The squared power.
    
    # Turning Points:
    tps <- turnsAnalytics(index=index, spar=spar, trace=trace, doplot=FALSE)
    
    # Wavelet Spectrum:     
    p <- as.vector(ans$Power)
    vec <- c(mean=mean(p), sd=sd(p), skew=skewness(p), kurt=kurtosis(p))
    mat <- c(
      O = base::norm(ans$Power, "O"), # "One Norm" 
      I = base::norm(ans$Power, "I"), # "Inf Norm" 
      F = base::norm(ans$Power, "F"), # "Frobenius" 
      M = base::norm(ans$Power, "M")) # "Max Modulus" 
    if (trace) {
      cat("Stats Measures:\n")
      print(vec)
      print(mat)
    }
    analytics <- c(vec, mat)
    
    # Wavelet Parameters:  
    wave.list <- ans  
    wavelet.levels <- quantile(wave.list$Power, probs=seq(from=0, to=1, by=0.1))
    add.coi <- TRUE 
    add.sig <- TRUE
    crn.lab <- "RWI" 
    key.cols <- rev(heat.colors(length(wavelet.levels)))
    key.lab <- expression(paste("Power"^2))
    nyrs=NULL
    crn.col <- "black"
    crn.lwd <- 1
    crn.ylim <- range(wave.list$y) * 1.1
    
    # Settings:
    y <- wave.list$y
    x <- wave.list$x
    wave <- wave.list$wave
    period <- wave.list$period
    Signif <- wave.list$Signif
    coi <- wave.list$coi
    coi[coi == 0] <- 1e-12
    Power <- wave.list$Power
    siglvl <- wave.list$siglvl
    Signif <- t(matrix(Signif, dim(wave)[2], dim(wave)[1]))
    Signif <- Power/Signif
    period2 <- log(period)/log(2)
    ytick <- unique(trunc(period2))
    ytickv <- 2^(ytick)
    coi2 <- log(coi)/log(2)
    coi2[coi2 < 0] <- 0
    coi2.yy <- c(coi2, rep(max(period2, na.rm=TRUE), length(coi2)))
    coi2.yy[is.na(coi2.yy)] <- coi[2]
    yr.vec.xx <- c(x, rev(x))
    # DW
    # par.orig <- par(c("mar", "las", "mfrow"))
    # on.exit(par(par.orig))
    nlevels <- length(wavelet.levels)
    key.labs <- formatC(wavelet.levels, digits=4, format="f")
    asp <- NA
    las <- 1   
    xlim <- range(x, finite=TRUE)
    ylim <- range(period2, finite=TRUE)
    ylim[2] <- ylim[2] * 1.1
    
    # Image Plot:
    # DW
    # ... now you can create more than one plot on one page
    # plot.new()
    # plot.window(xlim, ylim, xaxs="r", yaxs="r")
    # Use instead:
    plot(xlim, ylim, xaxs = "r", yaxs = "r", 
      col = "white", axes=FALSE, frame=FALSE, ann=FALSE)
    
    # DW
    # .Internal(filledcontour()) no longer works on 3.0.
    # .Internal(filledcontour(
    #   as.double(x), as.double(period2), Power, 
    #   as.double(wavelet.levels), col=key.cols))
    # Use instead:
    graphics::.filled.contour(
      x = as.double(x), 
      y = as.double(period2), 
      z = Power,
      levels = as.double(wavelet.levels), 
      col = key.cols)
    
    title(main=main, xlab="", ylab=paste("Period", colnames(index)))
    box(col="white")
    box(bty="l")
    
    # Add Contours: 
    contour(x, period2, Signif, levels=1, labels=siglvl, 
      drawlabels=FALSE, axes=FALSE, frame.plot=FALSE, 
      add=TRUE, lwd=2, col="black")
    
    # Add Coin of Influence:
    polygon(yr.vec.xx, coi2.yy, density=c(10, 20), 
      angle=c(-45, 45), col="black")
    
    # Add Axis Labels:
    xtick <- NULL
    for (i in 1:length(time(index)))
      xtick <- c(xtick, which.min(abs(time(index) - at[i])))
    axis(1, at=xtick, labels=format(at, format=format))
    axis(2, at=ytick, labels=ytickv, las=2)   
    
    # Add Scale Legend:
    nCol<- length(key.cols)
    positions <- seq(min(x), max(x), length=nCol+1)
    colLevels <- paste(signif(wavelet.levels,2))
    for (i in 1:nCol) {
      lines(x=c(positions[i], positions[i+1]), 
            y=c(1.03,1.03)*max(period2), lwd=3, col=key.cols[i])
      text(x=(positions[i]+positions[i+1])/2, 1.07*max(period2), 
           colLevels[i], cex=0.6) }
    points(positions, rep(1.03*max(period2), length=nCol+1), 
           pch=19, cex=0.7)
    
    # Add Rmetrics - Do not Remove!
    mtext("Rmetrics", adj=0, side=4, cex=0.7, col="darkgrey")
    
    # Return Value:
    invisible(list(
      index = index, 
      spar = spar, 
      wavelet = ans, 
      analytics = list(Time=x, Period=period2, Power=Power)))
  }     


###############################################################################


parAnalytics <- 
  function()
  {
    # A function implemented by Diethelm Wuertz and Tobias Setz
    
    # Description:
    #    Sets the graph frame for an analytics chart
    
    # FUNCTION:
    
    # Graph Frame:
    par(mfrow = c(2, 1))
    par(mar = c(2, 4, 2, 2) + 0.1)
    par(omi = 0.2*c(1, 0.7, 1, 0.7))
    
    # Return Value:
    invisible()
  }


############################################################################### 

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the 
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General 
# Public License along with this library; if not, write to the 
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
# MA 02111-1307 USA


################################################################################
# FUNCTION:                     DESCRIPTION:             
#  summary.fPORTFOLIO            S3 Summary method for 'fPORTFOLIO' objects
################################################################################


summary.fPORTFOLIO <- 
    function(object, ...)
{   
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Plot method for an object of class 'fPORTFOLIO'
    
    # Note:
    #   This method can also be used for plotting graphs fitted by 
    #   the function 'garch' from the contributed R package 'tseries'.
    
    # FUNCTION:

    # Summary:
    print(object)
    funCalled = as.character(object@call[1])
    if (funCalled == "portfolioFrontier") {      
        weightsPlot(object)
        weightedReturnsPlot(object)
        covRiskBudgetsPlot(object)
        # Plot Frontier:
        plot(object, which = 1)
    } else {
        weightsPie(object)
        weightedReturnsPie(object)
        covRiskBudgetsPie(object)
    }
          
    # Return Value:
    invisible(object)
} 


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################
# FUNCTION:                DESCRIPTION:
#   .solveNLP.demo          Mean-variance portfolio demo solved by NLP
################################################################################


.solveNLP.demo <- 
  function()
  {
    # Solve Mean-Variance Portfolio:
    
    # Load Dataset
    dataSet <- data("LPP2005REC", package="timeSeries", envir=environment())
    LPP2005REC <- get(dataSet, envir=environment())
    
    # Load Swiss Pension Fund Data:
    nAssets <- 6
    data <- 100 * LPP2005REC[, 1:nAssets]     
    mu <- colMeans(data)
    Sigma <- cov(data)
    
    # Arguments: 
    start <- rep(1, nAssets)/nAssets
    objective <- function(x) { 0.5 * (x %*% Sigma %*% x)[[1]] }
    lower <- rep(0, nAssets)
    upper <- rep(1, nAssets)
    mat <- rbind(
      budget = rep(1, times=nAssets), 
      returns = colMeans(data))
    matLower <- c(
      budget = 1, 
      return = mean(data))
    matUpper <- matLower
    linCons <- list(mat, matLower, matUpper)
    control <- list()
    
    # ### REMOVED ### donlp2 Solver:
    # require(Rdonlp2)
    ## ans <- rdonlp2NLP(start, objective, lower, upper, linCons)
    ## ans
    
    # solnp Solver:
    # require(Rsolnp)
    ans <- rsolnpNLP(start, objective, lower, upper, linCons)
    ans
    
    # nlminb2 Solver:
    ans <- rnlminb2NLP(start, objective, lower, upper, linCons)
    ans
  }


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                    DESCRIPTION:
#  solveRipop                   Portfolio interface to solver Ripop
#  .ripopArguments              Returns arguments for solver
################################################################################


solveRipop <-
    function(data, spec, constraints)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Portfolio interface to solver Ripop
    
    # Example:
    #    data <- 100 * LPP2005.RET[, 1:6]
    #    spec <- portfolioSpec()
    #    setTargetReturn(spec) <- mean(data)
    #    setSolver(spec) <- "solveRipop"
    #    solveRipop(data, spec)  

    # FUNCTION:

    # Transform Data:
    Data <- portfolioData(data, spec)
    data <- getSeries(Data)
    nAssets <- getNAssets(Data)

    # Compile Arguments for Solver:
    args <- .ripopArguments(data, spec, constraints)

    # Optimize Portfolio:
    ans <- try(ipopQP(
        objective = args$objective, 
        par.lower = args$par.lower, 
        par.upper = args$par.upper, 
        eqA = args$eqA, 
        eqA.bound = args$eqA.bound, 
        ineqA = args$ineqA, 
        ineqA.lower = args$ineqA.lower, 
        ineqA.upper = args$ineqA.upper, 
        control = list()), 
        silent = TRUE)
        
    if (inherits(ans, "try-error")) {
        # When Optimization Failed:
        ans <- list(
            opt = list(dvec=NA, Dmat=NULL),
            objective = 1e99,
            status = 1,
            message = "error",
            weights = rep(0, times=nAssets))
        run <- "failed"
    } else {
        # Set Tiny Weights to Zero:
        ans$weights <- .checkWeights(ans$solution)
        ans$solution <- NULL
        run <- "passed"
    }
    ans$solver <- "solveRipop"
    ans$solution <- ans$weights
    attr(ans$weights, "invest") <- sum(ans$weights)
    attr(ans$opt, "args") <- args

    # Class:
    class(ans) <- c("solver", "list")

    # Return Value:
    ans
}
       

# -----------------------------------------------------------------------------


.ripopArguments <-
    function(data, spec, constraints)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns ipopQP conform arguments for the solver

    # FUNCTION:

    # Data and Constraints as S4 Objects:
    Data <- portfolioData(data)
    data <- getSeries(Data)
    nAssets <- getNAssets(Data)
    Sigma <- getSigma(Data)

    # Box Constraints:
    par.lower <- minWConstraints(data, spec, constraints)
    par.upper <- maxWConstraints(data, spec, constraints)
        
    # Set up Equality Constraints:
    eqsumW <- eqsumWConstraints(data, spec, constraints)
    eqA <- eqsumW[, -1]
    eqA.bound <- eqsumW[, 1]
    
    # Set up Inequality Constraints:
    minsumW <- minsumWConstraints(data, spec, constraints)
    maxsumW <- maxsumWConstraints(data, spec, constraints)
    ineqA <- NULL
    if(!is.null(minsumW)) ineqA = rbind(ineqA, minsumW[, -1])
    if(!is.null(maxsumW)) ineqA = rbind(ineqA, maxsumW[, -1])
    ineqA.lower <- ineqA.upper <- NULL
    if(!is.null(minsumW)) {
        ineqA.lower = c(ineqA.lower, +minsumW[, 1])
        ineqA.upper = c(ineqA.upper, rep(sum(par.upper), times=length(minsumW[, 1]))) }
    if(!is.null(maxsumW)) {
        ineqA.lower = c(ineqA.lower, rep(sum(par.lower), times=length(maxsumW[, 1])))
        ineqA.upper = c(ineqA.upper, maxsumW[, 1]) }

    # Return Value:
    list(
        objective = list(dvec = rep(0, nAssets), Dmat = Sigma), 
        par.lower = par.lower,
        par.upper = par.upper,
        eqA = eqA,
        eqA.bound = eqA.bound ,
        ineqA = ineqA,
        ineqA.lower = ineqA.lower,
        ineqA.upper = ineqA.upper
        )
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                   DESCRIPTION:
#  covRisk                     Computes covariance risk as standard deviation
#  varRisk                     Computes Value at Risk
#  cvarRisk                    Computes Conditional Value at Risk
# FUNCTION:                   DESCRIPTION - DEPRECATED:
#  .covRisk                    Computes Covariance Risk
#  .varRisk                    Computes Value at Risk
#  .cvarRisk                   Computes Conditional Value at Risk
# FUNCTION:                   DESCRIPTION:
#  .cfgFit                     Fits bivariate tail dependency parameter lambda
#  .lambdaTailRisk             Fits tail lambda for multivariate data
################################################################################


covRisk <-
    function(data, weights)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Computes Covariance Risk for assets given weights

    # Arguments:
    #   data - any univariate or multivariate object which can
    #       be transformed into a matrix
    #   weights - a numeric vector, the weights vector

    # Example:
    #   data = LPP2005.RET[, 1:6]; weights = rep(1/6, times = 6)
    #   covRisk(data, weights)

    # FUNCTION:

    # Data:
    if (inherits(data, "timeSeries"))
        data <- getDataPart(data)
    Data <- as.matrix(data)
    nAssets = dim(Data)[2]

    # Covariance Matrix:
    Sigma = cov(Data)

    # Risk:
    weights = as.vector(weights)
    Std = sqrt( as.numeric( weights %*% Sigma %*% weights ) )
    names(Std) = "Cov"

    # Return Value:
    Std

}


# ------------------------------------------------------------------------------


varRisk <-
    function(data, weights, alpha = 0.05)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Computes VaR for assets given weights and alpha

    # Arguments:
    #   data - any univariate or multivariate object which can
    #       be transformed into a matrix
    #   weights - a numeric vector, the weights vector
    #   alpha - a numeric value, the quantile

    # Example:
    #   data = LPP2005.RET[, 1:6]; weights = rep(1/6, times = 6)
    #   varRisk(data, weights)

    # FUNCTION:
    if (inherits(data, "timeSeries"))
        data <- getDataPart(data)

    # VaR:
    weights = as.vector(weights)
    X = as.matrix(data) %*% weights
    VaR = quantile(X, alpha, type = 1)
    names(VaR) <- paste("VaR.", alpha*100, "%", sep = "")

    # Return Value:
    VaR
}


# ------------------------------------------------------------------------------


cvarRisk <-
    function(data, weights, alpha = 0.05)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Computes CVaR for assets given weights and alpha

    # Arguments:
    #   data - any univariate or multivariate object which can
    #       be transformed into a matrix
    #   weights - a numeric vector, the weights vector
    #   alpha - a numeric value, the quantile

    # Example:
    #   data = LPP2005.RET[, 1:6]; weights = rep(1/6, times = 6)
    #   cvarRisk(data, weights)

    # FUNCTION:
    if (inherits(data, "timeSeries"))
        data <- getDataPart(data)

    # CVaR:
    weights = as.vector(weights)
    X = as.matrix(data) %*% weights
    VaR = quantile(X, alpha, type = 1)
    CVaR = c(CVaR = VaR - 0.5 * mean(((VaR-X) + abs(VaR-X))) / alpha)
    names(CVaR) <- paste("CVaR.", alpha*100, "%", sep = "")

    # Return Value:
    CVaR
}


################################################################################
# OLD FUNCTIONS:
#   check where they are still used


.covRisk <-
    function(data, weights)
{
    # A function implemented by Rmetrics

    # Description:
    #   Computes Covariance Risk for assets given weights and alpha

    # FUNCTION:
    if (inherits(data, "timeSeries"))
        data <- getDataPart(data)

    # Data:
    Data = as.matrix(data)
    nAssets = dim(Data)[2]

    # Mean Vector and Covariance:
    mu = colMeans(Data)
    Sigma = cov(Data)

    # Return and Risk:
    return = as.numeric( weights %*% mu )
    risk = sqrt( as.numeric( weights %*% Sigma %*% weights ) )

    # Return Value:
    list(risk = risk, return = return)
}


# ------------------------------------------------------------------------------


.varRisk <-
    function(x, weights, alpha = 0.05)
{
    # A function implemented by Rmetrics

    # Description:
    #   Computes VaR for assets given weights and alpha

    # Arguments:
    #   x - any univariate or multivariate object which can
    #       be transformed into a matrix
    #   weights - a numeric vector, the weights vector
    #   alpha - a numeric value, the quantile

    # FUNCTION:
    if (inherits(x, "timeSeries"))
        x <- getDataPart(x)

    # VaR:
    X = as.matrix(x) %*% weights
    VaR = quantile(X, alpha, type = 1)
    names(VaR) <- paste("VaR.", alpha*100, "%", sep = "")

    # Return Value:
    VaR
}


# ------------------------------------------------------------------------------


.cvarRisk <-
    function(x, weights, alpha = 0.05)
{
    # A function implemented by Rmetrics

    # Description:
    #   Computes CVaR for assets given weights and alpha

    # Arguments:
    #   x - any univariate or multivariate object which can
    #       be transformed into a matrix
    #   weights - a numeric vector, the weights vector
    #   alpha - a numeric value, the quantile

    # FUNCTION:
    if (inherits(x, "timeSeries"))
        x <- getDataPart(x)

    # CVaR:
    X = as.matrix(x) %*% weights
    VaR = quantile(X, alpha, type = 1)
    CVaR = c(CVaR = VaR - 0.5 * mean(((VaR-X) + abs(VaR-X))) / alpha)
    names(CVaR) <- paste("CVaR.", alpha*100, "%", sep = "")

    # Return Value:
    CVaR
}


################################################################################


.cfgFit <-
    function(x, y, tail = c("upper", "lower"))
{
    # Description:
    #   Fits bivariate tail dependency parameter lambda

    # Arguments:
    #   data - multivariate time series object of class S4 or a numeric
    #       matrix
    #   tail - which tail should be considered?

    # FUNCTION:

    # Match Arguments:
    tail = match.arg(tail)

    # If Lower Tail:
    if(tail == "lower") {
        x = 1-x
        y = 1-y
    }

    # Fit lambda:
    lambda = NULL
    n = length(x)
    for(i in 1:n){
        lambda = c(lambda,
            log(sqrt(log(1/x[i])*log(1/y[i]))/log(1/max(x[i],y[i])^2)))
    }
    ans = (2-2*exp(sum(lambda/n)))
    attr(ans, "control") <- c(tail = tail)

    # Return Value:
    ans
}


# ------------------------------------------------------------------------------


.lambdaTailRisk <-
    function(data, tail = c("upper", "lower"), margins = "norm", ...)
{
    # Description:
    #   Fits tail dependency parameter lambda for multivariate data

    # Arguments:
    #   data - multivariate time series object of class S4 or a numeric
    #       matrix
    #   tail - which tail should be considered?

    # Example:
    #   r = rarchmCopula(60, alpha = 2, type = "4")
    #   .cfgFit(r[, 1], r[, 2])
    #   x = cbind(qnorm(r[, 1]), qnorm(r[, 2]))
    #   .lambdaTailRisk(x)

    # FUNCTION:

    # Check Data:
    if(is.timeSeries(data)) data = series(data)
    n = ncol(data)

    # Normal Margins - Create Bivariate Copulae:
    x = data
    for (i in 1:n) {
        y = as.vector(data[, i])
        x[, i] = pnorm(y, mean(y), sd(y))
    }

    # Match Arguments:
    tail = match.arg(tail)

    # Compute Tail Risks:
    riskMatrix = diag(n)
    # Compute lambda:
    for ( i in 1:(n-1) ) {
        for ( j in (i+1):n ) {
            riskMatrix[i, j] = riskMatrix[j, i] =
                .cfgFit(x[, i], x[, j], tail = tail)
        }
    }
    attr(riskMatrix, "control") <- c(tail = tail)

    # Return Value:
    riskMatrix
}


################################################################################

################################################################################
# FUNCTION:                     DESCRIPTION:
#  print.solver                  S3 Print method for solver objects
################################################################################


print.solver <- 
    function(x, ...)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   S3 Print method for solver objects

    # Arguments:
    #   x - a list as returned by solver functions

    # FUNCTION:

    cat("\nMathematical Programming:\n ")
    
    cat(x$version, sep="\n")
    cat("\n")
    
    cat(" Objective:       ", x$objective, "\n")
    cat(" Solution:        ", head(x$solution),  "...\n")
    cat(" Status Code:     ", x$status,    "\n")
    cat(" Message:         ", x$message,   "\n")
    cat(" Solver:          ", x$solver,    "\n")
    
    #cat(" Version:         ", x$version,   "\n")
    
    # Return Value:
    invisible(x)
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                    DESCRIPTION:
#  frontierPlot                 Plots efficient frontier
#   minvariancePoints             Adds minimum variance point
#   cmlPoints                     Adds market portfolio
#   cmlLines                      Adds capital market Line
#   tangencyPoints                Adds tangency portfolio point
#   tangencyLines                 Adds tangency line
#   equalWeightsPoints            Adds point of equal weights portfolio
#   singleAssetPoints             Adds points of single asset portfolios
#   twoAssetsLines                Adds EF for all combinations of two assets
#   sharpeRatioLines              Adds Sharpe ratio line
#   monteCarloPoints              Adds randomly produced feasible portfolios
# FUNCTION:                    DESCRIPTION:
#  frontierPlotControl          Sets frontier plot control parameters
# FUNCTION:                    DESCRIPTION:
#  tailoredFrontierPlot         Tailored frontier plot with addons
################################################################################


frontierPlot <-
function(object, frontier = c("both", "lower", "upper"),
    col = c("black", "grey"), add = FALSE, labels = TRUE,
    return = c("mean", "mu"), risk = c("Cov", "Sigma", "CVaR", "VaR"),
    auto = TRUE, title = TRUE, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Plots the efficient frontier
    
    # Arguments:
    
    # FUNCTION:

    # Check Settings:
    stopifnot(length(col) == 2)

    # Settings:
    frontier <- match.arg(frontier)
    fullFrontier = frontierPoints(object, frontier = "both",
        return = return, risk = risk, auto = auto)
    upperFrontier <- frontierPoints(object, frontier = "upper",
        return = return, risk = risk, auto = auto)
    lowerFrontier <- frontierPoints(object, frontier = "lower",
        return = return, risk = risk, auto = auto)

    # Check for 'xlim' Argument:
    Arg <- match.call(expand.dots = TRUE)
    m <- match(c("xlim", "ylim"), names(Arg), Arg)
    xArg <- as.character(Arg[c(1, m)])[2]
    yArg <- as.character(Arg[c(1, m)])[3]

    # Plot:
    if(xArg == "NULL" & yArg == "NULL") {
        yLim <- range(fullFrontier[, 2])
        xRange <- range(fullFrontier[, 1])
        xDiff <- diff(xRange)
        xLim <- c(xRange[1] - 2.5*xDiff/10, xRange[2] + xDiff/10)

        # Plot:
        if(!add){
            if(frontier == "upper" | frontier == "both") {
                plot(upperFrontier, col = col[1], xlim = xLim, ylim = yLim,
                    ann = FALSE, ...)
            } else {
                if( frontier == "both") {
                    points(fullFrontier, col = col[2],
                        xlim = xLim, ylim = yLim, ...)
                }
                if(frontier == "lower" ) {
                    plot(lowerFrontier, col = col[2],
                        xlim = xLim, ylim = yLim, ann = FALSE, ...)
                }
            }
        }
        if(frontier == "upper" | frontier == "both") {
            points(upperFrontier, col = col[1],  ...)
        }
        if(frontier == "lower" | frontier == "both") {
            points(lowerFrontier, col = col[2], ...)
        }
    } else if (xArg != "NULL" & yArg == "NULL") {
        # In this case only xlim is specified in the argument list
        yLim = range(fullFrontier[, 2])
        # Plot:
        if(!add){
            if(frontier == "upper" | frontier == "both") {
                plot(upperFrontier, col = col[1], ylim = yLim,
                    ann = FALSE, ...)
            } else {
                if( frontier == "both") {
                    points(fullFrontier, col = col[2], ylim = yLim, ...)
                }
                if(frontier == "lower" ) {
                    plot(fullFrontier, col = col[2], ylim = yLim,
                        ann = FALSE, ...)
                }
            }
        }
        if(frontier == "upper" | frontier == "both") {
            points(upperFrontier, col = col[1], ...)
        }
        if(frontier == "lower" | frontier == "both") {
            points(lowerFrontier, col = col[2], ...)
        }
    } else if(xArg == "NULL" & yArg != "NULL") {
        # In this only ylim is specified in the argument list
        xRange = range(fullFrontier[, 1])
        xDiff = diff(xRange)
        xLim = c(xRange[1] - 2.5*xDiff/10, xRange[2] + xDiff/10)
        # Plot:
        if(!add){
            if(frontier == "upper" | frontier == "both") {
                plot(upperFrontier, col = col[1], xlim = xLim,
                    ann = FALSE, ...)
            } else {
                if( frontier == "both") {
                    points(fullFrontier, col = col[2], xlim = xLim, ...)
                }
                if(frontier == "lower" ) {
                    plot(lowerFrontier, col = col[2], xlim = xLim,
                        ann = FALSE,...)
                }
            }
        }
        if(frontier == "upper" | frontier == "both") {
            points(upperFrontier, col = col[1], ...)
        }
        if(frontier == "lower" | frontier == "both") {
            points(lowerFrontier, col = col[2], ...)
        }
    } else if (xArg != "NULL" & yArg != "NULL"){
        #  If both xlim and ylim are not defined in argument list ...
        if(!add){
            if(frontier == "upper" | frontier == "both") {
                plot(fullFrontier, type = "n", ann = FALSE, ...)
                points(upperFrontier, col = col[1], ...)
            }
            if(frontier == "both") {
                points(lowerFrontier, col = col[2], ...)
            }
            if(frontier == "lower") {
                plot(lowerFrontier, col = col[2], ann = FALSE, ...)
            }
        } else{
            if(frontier == "upper" | frontier == "both") {
                points(upperFrontier, col = col[1], ...)
            }
            if(frontier == "lower" | frontier == "both") {
                points(lowerFrontier, col = col[2], ...)
            }
        }
    }

    # Add Title:
    if (title) {
        labs = attr(fullFrontier, "control")
        title(
            main = "Efficient Frontier",
            xlab = paste("Target Risk[", labs[1], "]", sep = ""),
            ylab = paste("Target Return[", labs[2], "]", sep = ""))
    }
    
    # Add Rmetrics - Do not Remove!
    mtext("Rmetrics", adj=0, side=4, cex=0.7, col="darkgrey")

    # Return Value:
    invisible(fullFrontier)
}


# ------------------------------------------------------------------------------


minvariancePoints <-
function(object,
    return = c("mean", "mu"), risk = c("Cov", "Sigma", "CVaR", "VaR"),
    auto = TRUE, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Adds the minimum risk point to a MV and CVaR portfolio plot

    # Arguments:
    
    # FUNCTION:

    # Match Arguments:
    return <- match.arg(return)
    risk <- match.arg(risk)

    # Get Portfolio Slots:
    data <- getSeries(object)
    spec <- getSpec(object)
    constraints <- getConstraints(object)

    # Add Minimum Variance Point:
    mvPortfolio <- minvariancePortfolio(data, spec, constraints)
    assets <- frontierPoints(mvPortfolio, return = return, risk = risk,
        auto = auto)
    points(assets, ...)

    # Return Value:
    invisible(assets)
}


# ------------------------------------------------------------------------------


cmlPoints <-
function(object,
    return = c("mean", "mu"), risk = c("Cov", "Sigma", "CVaR", "VaR"),
    auto = TRUE, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Adds the capital market line to a portfolio plot

    # Arguments:
    
    # FUNCTION:

    # Match Arguments:
    return <- match.arg(return)
    risk <- match.arg(risk)

    # Get Portfolio Statistics:
    data <- getSeries(object)
    spec <- getSpec(object)
    constraints <- getConstraints(object)

    # Add Capital Market Line Tangency Point:
    cmlPortfolio <- tangencyPortfolio(data, spec, constraints)
    assets <- frontierPoints(cmlPortfolio, return = return, risk = risk,
        auto = auto)
    points(assets, ...)

    # Return Value:
    invisible(assets)
}


# ------------------------------------------------------------------------------


cmlLines <-
function(object,
    return = c("mean", "mu"), risk = c("Cov", "Sigma", "CVaR", "VaR"),
    auto = TRUE, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Adds the capital market line to a portfolio plot

    # Arguments:
    
    # FUNCTION:

    # Match Arguments:
    return <- match.arg(return)
    risk <- match.arg(risk)

    # Get Portfolio Statistics:
    data <- getSeries(object)
    spec <- getSpec(object)
    constraints <- getConstraints(object)

    # Add Capital Market Line:
    cmlPortfolio <- tangencyPortfolio(data, spec, constraints)
    riskFreeRate <- getRiskFreeRate(spec)
    slope <- ((getTargetReturn(cmlPortfolio)[, "mean"] - riskFreeRate) /
        getTargetRisk(cmlPortfolio@portfolio)[, "Cov"])
    if(slope > 0) { 
        abline(riskFreeRate, slope, ...)
    } else {
        warning("CML Line does not exist")
    }
    
    # Return Value:
    invisible(slope)
}


# ------------------------------------------------------------------------------


tangencyPoints <-
function(object,
    return = c("mean", "mu"), risk = c("Cov", "Sigma", "CVaR", "VaR"),
    auto = TRUE, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Adds tangency point and line to a MV and CVaR portfolio plot

    # Arguments:
    
    # FUNCTION:

    # Match Arguments:
    return <- match.arg(return)
    risk <- match.arg(risk)

    # Get Portfolio Slots:
    data <- getSeries(object)
    spec <- getSpec(object)
    constraints <- getConstraints(object)

    # Compute Tangency Portfolio:
    tgPortfolio <- tangencyPortfolio(data, spec, constraints)

    # Add Tangency Point:
    assets <- frontierPoints(tgPortfolio, return = return, risk = risk,
        auto = auto)
    points(assets, ...)

    # Return Value:
    invisible(assets)
}


# ------------------------------------------------------------------------------


tangencyLines <-
function(object,
    return = c("mean", "mu"), risk = c("Cov", "Sigma", "CVaR", "VaR"),
    auto = TRUE, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Adds tangency point and line to a MV and CVaR portfolio plot

    # Arguments:
    
    # FUNCTION:

    # Match Arguments:
    return = match.arg(return)
    risk = match.arg(risk)

    # Get Portfolio Slots:
    data <- getSeries(object)
    spec <- getSpec(object)
    constraints <- getConstraints(object)
    riskFreeRate <- getRiskFreeRate(object)

    # Compute Tangency Portfolio:
    tgPortfolio = tangencyPortfolio(data, spec, constraints)

    # Add Tangency Line:
    assets <- frontierPoints(tgPortfolio, return = return, risk = risk,
        auto = auto)
    slope <-( assets[2] - riskFreeRate ) / assets[1]
    if (slope > 0) {
        abline(riskFreeRate, slope, ...)
    } else {
        warning("Tangency point does not exist")
    }

    # Return Value:
    invisible(list(slope = slope, assets = assets))
}


# ------------------------------------------------------------------------------


equalWeightsPoints =
function(object,
    return = c("mean", "mu"), risk = c("Cov", "Sigma", "CVaR", "VaR"),
    auto = TRUE, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Adds equal weights portfolio to a portfolio plot

    # Arguments:
    
    # FUNCTION:

    # Match Arguments:
    return <- match.arg(return)
    risk <- match.arg(risk)

    # Get Portfolio Statistics:
    data <- getSeries(object)
    spec <- getSpec(object)
    constraints <- getConstraints(object)
    numberOfAssets <- getNAssets(object)

    # Set Equal Weights:
    setWeights(spec) <- rep(1/numberOfAssets, times = numberOfAssets)

    # Add Equal Weights Portfolio:
    ewPortfolio <- feasiblePortfolio(data, spec, constraints)
    assets <- frontierPoints(ewPortfolio, return = return, risk = risk,
        auto = auto)
    points(assets, ...)

    # Return Value:
    invisible(assets)
}


# ------------------------------------------------------------------------------


singleAssetPoints <-
function(object,
    return = c("mean", "mu"), risk = c("Cov", "Sigma", "CVaR", "VaR"),
    auto = TRUE, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Adds all single assets returns and risks to a portfolio plot

    # Arguments:
    
    # FUNCTION:

    # Add Single Assets:
    Statistics <- getStatistics(object)
    Type <- getType(object)

    # Match Arguments:
    return <- match.arg(return)
    risk <- match.arg(risk)

    # Get auto Risk:
    if (auto) {
        return = "mu"
        Type = getType(object)
        Estimator = getEstimator(object)
        if (Type == "MV") risk = "Cov"
        if (Type == "MV" & Estimator != "covEstimator") risk = "Sigma"
        if (Type == "QLPM") risk = "Sigma"
        if (Type == "CVaR") risk = "CVaR"
    }

    # Extract Return:
    if (return == "mean") {
        Return = Statistics$mean
    } else if (return == "mu") {
        Return = Statistics$mu
    }

    # Extract Risk:
    if (risk == "Cov") {
        Risk = sqrt(diag(Statistics$Cov))
    } else if (risk == "Sigma") {
        Risk = sqrt(diag(Statistics$Sigma))
    } else if (risk == "CVaR") {
        nAssets = getNAssets(object)
        Data = getSeries(object)
        alpha = getAlpha(object)
        Risk = NULL
        for (i in 1:nAssets) Risk = c(Risk, -.cvarRisk(Data[ ,i], 1, alpha))
    } else if (risk == "VaR") {
        nAssets = getNAssets(object)
        Data = getSeries(object)
        alpha = getAlpha(object)
        Risk = NULL
        for (i in 1:nAssets) Risk = c(Risk, -.varRisk(Data[ ,i], 1, alpha))
    }
    Risk = as.vector(Risk)

    # Add Points:
    assets = cbind(targetRisk = Risk, targetReturn = Return)
    attr(assets, "control") <-
        c(targetRisk = risk, targetReturn = return, auto = as.character(auto))
    points(assets, ...)

    # Return Value:
    invisible(assets)
}


# ------------------------------------------------------------------------------


twoAssetsLines <-
function(object,
    return = c("mean", "mu"), risk = c("Cov", "Sigma", "CVaR", "VaR"),
    auto = TRUE, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Adds efficient long-only frontier of all portfolio pairs

    # Arguments:
    
    # Note:
    #   Only supported for "Short" and "LongOnly" Constraints!

    # FUNCTION:

    # Supported ?
    check <- rev(attr(object@constraints, "model"))[1]

    # Match Arguments:
    return <- match.arg(return)
    risk <- match.arg(risk)

    # Get Portfolio Statistics:
    data <- getSeries(object)
    Data <- getData(object)
    mu <- getMu(Data)
    Sigma <- getSigma(Data)
    spec <- getSpec(object)
    constraints <- getConstraints(object)

    # Add Frontiers for all Two-Assets Portfolios:
    N <- getNAssets(object)
    setWeights(spec) = NULL
    for (i in 1:(N-1) ) {
        for (j in (i+1):N ) {
            index = c(i, j)
            data2 = data[, index]
            Data2 = portfolioData(data2, spec)
            Data2@statistics$mu <- mu[index]
            Data2@statistics$Sigma <- Sigma[index, index]
            ans = portfolioFrontier(data = Data2, spec = spec)
            lines(frontierPoints(ans,
                return = return, risk = risk, auto = auto), ...)
        }
    }

    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------


sharpeRatioLines <-
function(object,
    return = c("mean", "mu"), risk = c("Cov", "Sigma", "CVaR", "VaR"),
    auto = TRUE, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Adds Sharpe Ratio

    # Arguments:
    
    # FUNCTION:

    # Match Arguments:
    return = match.arg(return)
    risk = match.arg(risk)

    # Get Portfolio Slots:
    data <- getSeries(object)
    spec <- getSpec(object)
    constraints <- getConstraints(object)
    riskFreeRate <- getRiskFreeRate(object)
    Type <- getType(object)

    # Efficient Frontier:
    frontPoints <- frontierPoints(object, frontier = "upper",
        return = return, risk = risk, auto = auto)
    x <- frontPoints[, 1]
    y <- frontPoints[, 2] - riskFreeRate

    # Tangency Portfolio:
    tangencyPortfolio <- tangencyPortfolio(data, spec, constraints)
    # x.tg = getTargetReturn(tangencyPortfolio@portfolio)["mean"]
    x.tg = frontierPoints(tangencyPortfolio,
        return = return, risk = risk, auto = auto)[, 2]

    # Normalization to fit in EF Plot:
    norm <- x.tg / max(y/x)
    index <- 2:length(x)
    #index = index[diff(x) > 0]
    x <- x[index]
    y <- y[index]
    y.norm <- (y/x*norm)
    assets <- cbind(x, y.norm)
    lines(x, y.norm, ...)
    
    # Search for Maximum:
    index <- which.max(y.norm)
    points(x[index], y.norm[index], col = "cyan", cex = 1.5)

    # Add Tailored Labels -  2 may be a good Number ...
    x.tg <- x.tg[index]
    norm2 <- x.tg / max(y)
    Range <- range(y/x * norm)

    # Take a reasonable number of significant digits to plot, e.g. 2 ...
    nPrecision <- 3 
    Labels <- signif(Range, nPrecision)
    axis(4, at = Range, labels = c(" ", " "), cex.axis = 0.75)
    axis(4, at = mean(Range), labels = paste(Labels[1], "   ", Labels[2]),
        cex.axis = 0.75)

    # Add Axis Labels and Title:
    mtext("Sharpe Ratio", side = 4, line = 2, cex = 0.75)
    
    # Return Value:
    invisible(assets)
}


# ------------------------------------------------------------------------------


monteCarloPoints <-
function(object, mcSteps = 5000,
    return = c("mean", "mu"), risk = c("Cov", "Sigma", "CVaR", "VaR"),
    auto = TRUE, ...)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Adds randomly feasible portfolios to a plot

    # Arguments:
    
    # FUNCTION:

    # Match Arguments:
    return = match.arg(return)
    risk = match.arg(risk)

    # Get Portfolio Statistics:
    Statistics = getStatistics(object)
    Type = getType(object)
    mu = Statistics$mu
    Sigma = Statistics$Sigma
    N = length(mu)

    # Get Specification:
    if (Type == "MV") {
        # Get Constraints Model:
        Model = rev(attr(object@constraints, "model"))[1]
        Model = "LongOnly"
        if (Model == "Short" | any(getConstraints(object) == "Short")) {
            # Monte Carlo Loop - Short:
            for (k in 1:mcSteps) {
                s = sign(rnorm(N, mean = rnorm(1)))
                weights = s * abs(rcauchy(N))
                weights = weights / sum(weights)
                Return = as.numeric(mu %*% weights)
                Risk = sqrt( as.numeric( t(weights) %*% Sigma %*% (weights) ) )
                points(Risk, Return, ...)
            }
        } else if (Model == "LongOnly" | any(getConstraints(object) == "LongOnly")) {
            # Monte Carlo Loop - Long Only:
            for (k in 1:mcSteps) {
                weights = abs(rcauchy(N))
                weights = weights / sum(weights)
                Return = as.numeric(mu %*% weights)
                Risk = sqrt( as.numeric( t(weights) %*% Sigma %*% (weights) ) )
                points(Risk, Return, ...)
            }
        } else {
            cat("\n\tOnly for Short and LongOnly Portfolios\n")
        }
    } else if (Type == "CVaR") {
        # Monte Carlo Loop - Long Only:
        x = getSeries(object)
        alpha = getAlpha(object)
        for (k in 1:mcSteps) {
            weights = abs(rcauchy(N))
            weights = weights / sum(weights)
            Return = as.numeric(mu %*% weights)
            Risk = .cvarRisk(x, weights, alpha)
            points(-Risk, Return, ...)
        }
    }

    # Return Value:
    invisible()
}


################################################################################


frontierPlotControl <-
function(

    # Colors:
    sharpeRatio.col   = "blue",
    minvariance.col   = "red",
    tangency.col      = "steelblue",
    cml.col           = "green",
    equalWeights.col  = "blue",
    singleAsset.col   = "topo.colors",
    twoAssets.col     = "grey",
    monteCarlo.col    = "black",

    # Point Sizes:
    minvariance.cex   = 1.25,
    tangency.cex      = 1.25,
    cml.cex           = 1.25,
    equalWeights.cex  = 1.25,
    singleAsset.cex   = 1.25,
    twoAssets.cex     = 0.01,
    monteCarlo.cex    = 0.01,
    sharpeRatio.cex   = 0.1,

    # Limits:
    xlim              = NULL,
    ylim              = NULL,

    # MC Steps:
    mcSteps           = 5000,

    # Pie Settings:
    pieR              = NULL,
    piePos            = NULL,
    pieOffset         = NULL
    )
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Sets frontier plot control parameters

    # Arguments:
    
    # FUNCTION:

    # Return Value:
    list(

        # Colors:
        sharpeRatio.col  = sharpeRatio.col,
        minvariance.col  = minvariance.col,
        tangency.col     = tangency.col,
        cml.col          = cml.col,
        equalWeights.col = equalWeights.col,
        singleAsset.col  = singleAsset.col,
        twoAssets.col    = twoAssets.col,
        monteCarlo.col   = monteCarlo.col,

        # Point Sizes:
        minvariance.cex  = minvariance.cex,
        tangency.cex     = tangency.cex,
        cml.cex          = cml.cex,
        equalWeights.cex = equalWeights.cex,
        singleAsset.cex  = singleAsset.cex ,
        twoAssets.cex    = twoAssets.cex,
        monteCarlo.cex   = monteCarlo.cex,
        sharpeRatio.cex  = sharpeRatio.cex,

        # Limits:
        xlim             = xlim,
        ylim             = ylim,

        # MC Steps:
        mcSteps          = 5000,

        # Pie Settings:
        pieR             = pieR,
        piePos           = piePos,
        pieOffset        = pieOffset

        )

}


################################################################################


tailoredFrontierPlot <-
function(object,
    return = c("mean", "mu"), risk = c("Cov", "Sigma", "CVaR", "VaR"),
    mText = NULL, col = NULL, xlim = NULL, ylim = NULL, 
    twoAssets = FALSE, sharpeRatio = TRUE, 
    title = TRUE, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Creates an easy to use tailored frontier plot
    
    # Arguments:
    #   object - a portfolio object
    #   mtext - not used
    
    # FUNCTION:
    
    # 1. Plot the Frontier, add margin text, grid and ablines:
    offset <- 0.10
    risk <- match.arg(risk)
    return <- match.arg(return)
    
    # x - Range:
    if (is.null(xlim)) {
        if (risk == "Cov") {
            xmax <- max(sqrt(diag(getCov(object))))
        }
        if (risk == "Sigma") {
            xmax <- max(sqrt(diag(getSigma(object))))
        }
        if (risk == "CVaR") {
            alpha <- getAlpha(object)
            quantiles <- colQuantiles(getSeries(object), prob = alpha)
            n.max <- which.max(-quantiles)
            r <- getSeries(object)[, n.max]
            r <- r[r < quantiles[n.max]]
            xmax <- -mean(r)
        }
        if (risk == "VaR") {
            xmax <- max(-colQuantiles(getSeries(object), prob = alpha))
        }
        xlim <- c(0, xmax)
        Xlim <- c(xlim[1]-diff(xlim)*offset, xlim[2]+diff(xlim)*offset)
    } else {
        Xlim <- xlim
    }
    
    # y - Range:
    if (is.null(ylim)) {
        if (return == "mean") {
            ylim <- range(getMean(object))
        } else {
            ylim <- range(getMu(object))
        }
        Ylim <- c(ylim[1]-diff(ylim)*offset, ylim[2]+diff(ylim)*offset)
    } else {
        Ylim = ylim
    }
    
    # Frontier Plot:
    frontierPlot(object, labels = FALSE,
        return = return, risk = risk, auto = FALSE, 
        xlim = Xlim, ylim = Ylim, title = title, pch = 19, ...)
    
    # Add Grid:
    grid()
    
    # Add Center-Hair Cut:
    abline(h = 0, col = "grey")
    abline(v = 0, col = "grey")

    # 2. Add minimum risk (variance) Portfolio Point:
    data <- getData(object)
    spec <- getSpec(object)
    constraints <- getConstraints(object)
    mvPortfolio <- minvariancePortfolio(data, spec, constraints)
    minvariancePoints(object, return = return, risk = risk, auto = FALSE,
        pch = 19, col = "red")

    # 3. Add Tangency Portfolio Point and Tangency Line:
    tangencyPoints(object, return = return, risk = risk, auto = FALSE,
        pch = 19, col = "blue")
    tangencyLines(object, return = return, risk = risk, auto = FALSE,
        col = "blue")

    # 4. Add Equal Weights Portfolio:
    xy <- equalWeightsPoints(object, return = return, risk = risk, 
        auto = FALSE, pch = 15, col = "grey")
    text(xy[, 1]+diff(xlim)/20, xy[, 2]+diff(ylim)/20, "EWP",
        font = 2, cex = 0.7)

    # 5. Add all Assets Points:
    if (is.null(col)) col = rainbow(6)
    xy <- singleAssetPoints(object, return = return, risk = risk, 
        auto = FALSE, cex = 1.5,
        col = col, lwd = 2)
    text(xy[, 1]+diff(xlim)/20, xy[, 2]+diff(ylim)/20,
        rownames(xy), font = 2, cex = 0.7)

    # 6. Add optionally all Two Assets  Lines
    if (twoAssets) {
        twoAssetsLines(object, return = return, risk = risk, auto = FALSE,
            lty = 3, col = "grey")
    }

    # 6. Add Sharpe Ratio Line:
    if(sharpeRatio) {
        sharpeRatioLines(object, return = return, risk = risk, auto = FALSE,
            col = "orange", lwd = 2)
    }
    
    # Add Rmetrics - Do not Remove!
    mtext("Rmetrics", adj=0, side=4, cex=0.7, col="darkgrey")

    # Return Value:
    invisible(list(object=object, xlim=Xlim, ylim=Ylim))
}


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


###############################################################################
# FUNCTION:                    DESCRIPTION:
#  ripopQP                      Rmetrics Interface for LOQO QP solver 
#  ipopQP                       Convenience wrapper for LOQO QP solver
#  ipopQPControl                LOQO QP control parameter list   
#  ripop                        Synonyme name for kernlab::ipop function           
###############################################################################

    
ripopQP <- 
    function(objective, lower=0, upper=1, linCons, control=list())
{   
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Implements Vandenberg's LOQO Algorithm
    
    # Arguments:
    #   objective - list(dvec=NULL, Dmat=NULL)
    #   lower - lower box constraints
    #   upper - upper box constraints
    #   linCons - linear constraints, list with entries:
    #       mat, lower, upper.
    #   control - control list
    
    # FUNCTION:
    
    # Control List:
    ctrl <- ipopQPControl()
    if (length(control) > 0)
        for (name in names(control)) ctrl[name] <- control[name]
    control <- ctrl
    BIG <- control$inf
    
    # General Settings:
    dvec <- objective$dvec
    Dmat <- objective$Dmat
    Names <- colnames(rbind(dvec, Dmat))
    N <- ncol(rbind(dvec, Dmat))
    
    # Box Constraints:
    if(length(lower) == 1) {
        par.lower <- rep(lower, N)
    } else {
        par.lower <- lower
    }
    if(length(upper) == 1) {
        par.upper <- rep(upper, N)
    } else {
        par.upper <- upper
    }
    par.lower[is.infinite(par.lower)] <- BIG*sign(par.lower[is.infinite(par.lower)])
    par.upper[is.infinite(par.upper)] <- BIG*sign(par.upper[is.infinite(par.upper)])
      
    # Linear Constraints:  
    if(missing(linCons)) {
        eqA <- ineqA <- NULL
        eqA.bound < ineqA.lower <- ineqA.upper <- NULL
    } else {
        mat <- linCons[[1]]
        M <- nrow(mat)
        lower <- linCons[[2]]
        upper <- linCons[[3]]
        if(length(lower) == 1) lower <- rep(lower, M)
        if(length(upper) == 1) upper <- rep(upper, M)
        lower[is.infinite(lower)] <- BIG*sign(lower[is.infinite(lower)])
        upper[is.infinite(upper)] <- BIG*sign(upper[is.infinite(upper)])
        
        eqIndex <- which(lower == upper)
        ineqIndex <- which(lower != upper)
        
        if (length(eqIndex) == 0) {
            eqA <- NULL
            eqA.bound <- NULL
        } else {
            eqA <- mat[eqIndex, ]
            eqA.bound <- lower[eqIndex]
        }
        if (length(ineqIndex) == 0) {
            ineqA <- NULL
            ineqA.lower <- NULL
            ineqA.upper <- NULL
        } else {
            ineqA <- mat[ineqIndex, ]
            ineqA.lower <- lower[ineqIndex]
            ineqA.upper <- upper[ineqIndex]
        }
    }   
    
    # Optimize Portfolio:
    optim <- ipopQP(
        objective, 
        par.lower, par.upper, 
        eqA, eqA.bound,  
        ineqA, ineqA.lower, ineqA.upper,
        control)
    
    # Return Value:
    value <- list(
        opt = optim, 
        solution = optim$solution, 
        objective = optim$objective,
        status = optim$status,
        message = optim$message,
        solver = "ipop",
        version = optim$version)
    class(value) = c("solver", "list")
    value
}


###############################################################################


ipopQP <- 
    function(
        objective=list(dvec=NULL, Dmat = NULL), 
        par.lower=NULL, par.upper=NULL, 
        eqA=NULL, eqA.bound=NULL,
        ineqA=NULL, ineqA.lower=NULL, ineqA.upper=NULL,
        control=list())
{   
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Implements Vandenberg's LOQO Algorithm
    
    # FUNCTION:
    
    # Control List:
    ctrl <- ipopQPControl()
    if (length(control) > 0)
        for (name in names(control)) ctrl[name] <- control[name]
    control <- ctrl
    
    # General Settings:
    dvec <- objective$dvec
    Dmat <- objective$Dmat 
    Names <- colnames(rbind(dvec, Dmat))
    N <- ncol(rbind(dvec, Dmat))
        
    # Solve:
    optim <- ripop(
        c = objective$dvec, 
        H = objective$Dmat, 
        A = rbind(eqA, ineqA), 
        b = c(eqA.bound, ineqA.lower), 
        l = par.lower, 
        u = par.upper, 
        r = c(eqA.bound, ineqA.upper) - c(eqA.bound, ineqA.lower), 
        sigf = control$sigf, 
        maxiter = control$maxiter, 
        margin = control$margin, 
        bound = control$bound, 
        verb = control$verb)
    
    # Add:  
    if (optim@how == "converged") Status <- 0 else Status <- 1
    par <- optim@primal
    names(par) <- Names
    Message <- optim@how 
    
    # Version:
    package <- packageDescription(pkg="kernlab")
    version <- paste(package$Package, package$Version, package$Date)
        
    # Return Value:
    ans <- list(
        opt = optim, 
        solution = par, 
        objective = c( dvec %*% par + 0.5 * par %*% Dmat %*% par )[[1]],
        status = Status,
        message = Message,
        solver = "ipop",
        version = version)
    class(ans) = c("solver", "list")
    ans
}


###############################################################################


ripop <- kernlab::ipop


###############################################################################


ipopQPControl <- 
    function(
        sigf=12, maxiter=400, margin=0.05, bound=10, verb=0, 
        inf=1e12, solver="ipop", trace=FALSE)
{
    # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Returns control parameter list
    
    # FUNCTION:
    
    # Control Parameter:
    control <- list(
        sigf = 12, 
        maxiter = 400, 
        margin = 0.05, 
        bound = 10, 
        verb = 0,
        inf = inf,
        solver = solver,
        trace = trace)
        
    # Return Value:
    control
}   
    

###############################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################
# FUNCTION:                    DESCRIPTION:
#  netPerformance               Returns performance from a portfolio backtest
# FUNCTION:                    DESCRIPTION:
#  .netPerformanceYTD           Returns year-to-date performance 
#  .netPerformanceCalendar      Returns calendar performance
#  .netPerformancePlot          Creates a net performance plot
################################################################################
  

# DW - TO DO:
#   Add argument doplot to function netPerformance
#   Make plot() a generic function
#   Problems in plot function


# ------------------------------------------------------------------------------ 
 

netPerformance <-  
    function(object, format = "%Y-%m-%d")
{
    # A function implemented by William Chen
    
    # Description:
    #   Returns performance from a portfolio backtest
    
    # Arguments:
    #   object - an object as returned by the function portfolioSmoothing.
    #   format -
    
    # FUNCTION:
        
    # Settings:
    cumP <- object$portfolioReturns
    cumB <- object$benchmarkReturns
    P <- as.numeric(cumP)
    B <- as.numeric(cumB)
    monthlyP <- object$P
    monthlyB <- object$B
    char.dates <- rownames(cumP)
    dates <- strptime(char.dates, format =  format)      
    # nye <- new years eve
    nye <- as.character(dates[dates$mon == 11] )
    years <- substr(nye, 1,4)        
    nYears <- length(years)
    
    # Net Performance Plot:
    # DW: Needs repair, all following plots fails once called
    # .netPerformancePlot(dates, char.dates, years, nye, P)
    
    # NET PERFORMANCE TO YTD:
    netYTD = rbind(.netPerformanceYTD(char.dates, monthlyP, P, nYears),
        .netPerformanceYTD(char.dates, monthlyB, B, nYears))
    rownames(netYTD) = c("Portfolio", "Benchmark")                     
    
    # NET PERFORMANCE CALENDAR YEAR:
    netCalendar <- rbind(.netPerformanceCalendar(nye, char.dates, P),
        .netPerformanceCalendar(nye, char.dates, B))
    rownames(netCalendar) = c("Portfolio", "Benchmark")
    
    # Print Summary:
    cat("\nNet Performance % to", 
        paste(rev(char.dates)[1], ":",sep = ""), "\n")
    print(round(netYTD,2))
    cat("\n\nNet Performance % Calendar Year:\n")
    print(round(netCalendar,2))
    cat("\n")
    
    # Return Value:
    ans = list(YTD = netYTD, Calendar = netCalendar)
    invisible(ans)               
}
   

# ------------------------------------------------------------------------------
  
        
.netPerformanceYTD = 
    function(char.dates, monthlyP, P, nYears, ...)
{   
    # A function implemented by William Chen
    
    # Description:
    #   Returns year-to-date performance from a portfolio backtest
    
    # Arguments:
    
    # FUNCTION:
        
    # NET PERFORMANCE TO YTD:       
    #   summaries for last 1, 3, 6 months, 
    #   1 year, 3 years, 5 years, 3 years annualised, 
    #   5 years annualised (if possible)
      
    monthly <- c(rev(monthlyP)[1], sum(rev(monthlyP)[1:3]), 
        sum(rev(monthlyP)[1:6]))
    
    if (nYears >= 5){
        IDX <- 1 + c(0,1,2,5)*12
        yearly <- numeric(length(IDX)-1)
        for (i in 1:(length(IDX)-1)){
            yearly[i] <- rev(P)[IDX[1]] - rev(P)[IDX[i+1]]   
        }
        annualised <- c((1+yearly[2]/100)^(1/3) - 1, 
            (1+yearly[3]/100)^(1/5) - 1) * 100
        combine <- c(monthly, yearly, annualised)
        names(combine) = c("1 mth", paste(c(3,6), "mths"), 
            "1 yr", paste(c(3,5), "yrs"), 
            paste(c(3,5), "yrs p.a."))
    } else {
        IDX <- 1 + seq(0, nYears) * 12
        yearly <- numeric(length(IDX)-1)
        for (i in 1:(length(IDX)-1)){
            yearly[i] <- rev(P)[IDX[1]] - rev(P)[IDX[i+1]]   
        }
        Names = paste(seq(1:nYears), "yrs")
        # calculate some annualised rates
        if (nYears > 1){
            ys <- seq(nYears)
            annualised <- NULL
            for (i in 2:nYears){
                annualised = c(annualised, (1+yearly[i]/100)^(1/i)-1)                    
            }
            annualised <- annualised * 100
            yearly <- c(yearly, annualised)
            Names <- c(Names, paste(2:nYears, "yrs p.a."))
        }
         
        combine <- c(monthly, yearly)
        names(combine) <- c("1 mth", paste(c(3,6), "mths"), Names)
    }
            
    # Return Value:                            
    combine
}


# ------------------------------------------------------------------------------        
            
 
.netPerformanceCalendar <- 
    function(nye, char.dates, P, ...)
{
    # A function implemented by William Chen
    
    # Description:
    #   Returns calendar performance
    
    # Arguments:
    
    # FUNCTION:
        
    # Net Performance Calendar Year: 
    nye1 = c(nye, char.dates[length(char.dates)])

    # First entry is the cumulated return at the end of first year
    # assume we start with 0 returns
    annuals <- P[char.dates == nye[1]]
    for (i in 1:(length(nye1)-1)){
        annuals <- c(annuals, 
            P[char.dates == nye1[i+1]] - P[char.dates == nye1[i]])
    }
        
    Annual <- c(annuals, sum(annuals))
    names(Annual) <- c(substr(nye,1,4), "YTD", "Total")

    # Return Value:
    Annual
}       
   
# ------------------------------------------------------------------------------    


.netPerformancePlot <-  
    function(dates, char.dates, years, nye, P, base = 100)
{
    # A function implemented by William Chen
    
    # Description:
    #   Creates a net performance plot
    
    # Arguments:
    
    # FUNCTION:
        
    # NET PERFORMANCE PLOT:
    
    # Setup figure frame:
    Opar = par(oma = rep(0,4), mar = rep(0,4))
    
    mat <- matrix(c(1,2,3), nrow = 3, ncol = 1)
    mat <- rbind(0, cbind(0, mat, 0))
    
    layout(mat, widths = c(0, 1, 0), heights = c(lcm(0.3), lcm(0.8), 1, lcm(1)))

    # Add title: 
    plot.new()
    plot.window(xlim = c(0,1), ylim = c(0,1))
    rect(0,0,1,1,col = "grey50", border = NA)
    text(0.01,0.5, "Net Performance (rebased to 100)", font = 2, 
        col = "white", adj = 0, cex = 1.8)
    
    # Rebased to 100
    newP <- c(base, P + base)

    # limits:
    ylim.pretty <- pretty(newP)
    yLim = range(ylim.pretty)
    # extend to the end of calendar year
    shortCalendar <- 11 - rev(dates$mon)[1] 
    xLim <- c(1, length(char.dates) + shortCalendar)

    # Create empty canvas:
    opar <- par(mar = c(2,5,1,4))
    plot.new()
    plot.window(xlim = xLim, ylim = yLim, xaxs = "i", yaxs = "i")           

    # Add bottom axes:
    IDX <- match(nye, char.dates)
    temp.d <- ifelse(dates$mon[1] != 0, dates$mon[1], 12) 
    temp.y <- ifelse(temp.d == 12, as.numeric(years)[1]-1, 
        as.numeric(years)[1])           
    labs <- c(paste(temp.d, temp.y, sep = "/"), paste(12,  years, sep = "/"))
    if (shortCalendar !=0 ) labs = c(labs, paste(12, 
        as.numeric(rev(years))[1] + 1, sep = "/"))
    axis(1, at = sort(c(xLim,IDX + 1)), labels = labs, 
        cex.axis = 1, padj = 0.5)#, tck = -0.05) 

    # Add left axes:
    yseq <- seq(min(ylim.pretty), max(ylim.pretty), by = 5)
    axis(2, at = yseq, las = 1, tick = FALSE, line = -0.7)
    abline(h = yseq, col = "grey50")

    # Draw portfolio performance:
    lines(newP, col = "red", lwd = 2)
    par(opar)
        
    # Add legend and extra text:
    plot.new()
    plot.window(xlim = c(0,1), ylim = c(0,1))
    legend(0.015, 0.8, legend = "Portfolio", lty = "solid", lwd = 2, 
        col = "red", bty = "n", cex = 1.1)

    # Return Value:
    par(Opar)
    invisible()
}
        

################################################################################

        

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


###############################################################################
# FUNCTION:                     DESCRIPTION:
#  .amplExec                     Executes AMPL run file for a given project
#  .amplExample                  Optimizes mean variance portfolio example
###############################################################################


.amplExec <- 
  function(project="ampl")
  {
    # A function Implemented by Diethelm Wuertz
    
    # Description:
    #    Executes AMPL run file for a given project.
    
    # Details:
    #    Note the following files must exist:
    #    [project].mod - the AMPL model file
    #    [project].dat - the AMPL data file
    #    [project].run - the AMPL run file
    
    # FUNCTION:
    
    # Execute run file:
    command <- paste("ampl -t -vs", paste(project, "run", sep="."))
    solve <- system(command, intern=TRUE)
    
    # Print:
    cat(solve, sep="\n")
    
    # Return Value:
    invisible(solve)
  }


# -----------------------------------------------------------------------------

.amplExample <- 
  function()
  {
    # A function Implemented by Diethelm Wuertz
    
    # Description:
    #    Optimizes mean variance portfolio example.
    
    # FUNCTION:
    
    # Load Dataset
    dataSet <- data("LPP2005REC", package="timeSeries", envir=environment())
    LPP2005REC <- get(dataSet, envir=environment())
    
    # Portfolio Data:
    nAssets <- 6
    data <- 100 * LPP2005REC[, 1:nAssets]
    
    # Optimization Arguments: 
    objective <- list(dvec=rep(0, nAssets), Dmat=cov(data))
    lower <- 0
    upper <- 1
    linCons <- list(
      mat = rbind(
        budget = rep(1, times=nAssets), 
        returns = colMeans(data)),
      lower = c(
        budget = 1, 
        return = mean(data)),
      upper = c(
        budget = 1, 
        return = mean(data)))
    control <- list()
    
    # Default - AMPL Interface:
    ampl <- ramplQP(objective, lower, upper, linCons, control)
    
    # Return Value:
    ampl
  }


###############################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################
# FUNCTION:                   DESCRIPTION:
#  backtestPlot                Creates a summary of backtesting plots
#   backtestAssetsPlot         Plots assets used in a portfolio backtest   
#   backtestWeightsPlot        Plots recommended weights from a backtest
#   backtestRebalancePlot      Plots rebalanced weights of a backtest 
#   backtestPortfolioPlot      Plots benchmark and portfolio series
#   backtestDrawdownPlot       Plots the drawdown of the portfolio backtest
#   backtestReportPlot         Prints backtest report
################################################################################


backtestPlot <-
    function(object, which = "all", labels = TRUE, legend = TRUE,
    at = NULL, format = NULL, cex=0.6, font=1, family="mono")
{
    # A function implemented by Diethelm Wuertz and William Chen

    # Description:
    #   Creates a summary of backtesting plots
    
    # Arguments:
    #   object - a list as returned by the function portfolioSmoothing()
    #   which - which plots should be displayed
    #   labels - a logical flag, should automated labels added to the plot
    
    # FUNCTION:
    
    # Frame:
    if (any(which == "all"))
    par(mfrow = c(3, 2), mar = c(1.5, 4, 5, 2), oma = c(5, 1, 0, 1))
       
    # Plot:
    if(any(which == "1") || which == "all")
        backtestAssetsPlot (object, labels, legend, at, format)
    if(any(which == "2") || which == "all")
        backtestWeightsPlot (object, labels, legend, at, format)
    if(any(which == "3") || which == "all")
       backtestRebalancePlot (object, labels, legend, at, format)
    if(any(which == "4") || which == "all")
        backtestPortfolioPlot(object, labels, legend, at, format)
    if(any(which == "5") || which == "all")
        backtestDrawdownPlot(object, labels, legend, at, format)
    if(any(which == "6" )|| which == "all")
        backtestReportPlot(object, cex=cex, font=font, family=family)
        
    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------
# Plot 1:

   
backtestAssetsPlot <-
    function(object, labels=TRUE, legend=TRUE, at=NULL, format=NULL)
{
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    #   Plots assets used in a portfolio backtest
    
    # Arguments:
    #   object - a list as returned by the function portfolioSmoothing()
    #   labels - a logical flag, should automated labels added to the plot
    
    # FUNCTION:
    
    # Settings:
    data <- object$data
    benchmark <- object$benchmarkName
    assets <- object$assetsNames
    
    # Time Axis:
    if (is.null(at)) at <- paste(unique(atoms(time(data))[,1]), "12-31", sep="-")
    if (is.null(format)) Format <- "%b/%y" else Format <- format
    
    # Labels ?
    if (labels) {
        main <- "Index Series"
        xlab <- ""
        ylab <- "Cumulated log Returns"
    } else {
        main <- ""
        xlab <- ""
        ylab <- ""  
    }
    
    # Series:
    X <- data[, benchmark]
    
    # ylim - Plot Range:
    nAssets <- length(assets)
    MAX <- -1.0e99
    for (i in 1:nAssets) MAX = max(c(MAX, cumsum(data[, assets[i]])) )
    MAX <- max(MAX, cumsum(data[, benchmark]))
    MIN <- 1.0e99
    for (i in 1:nAssets) MIN <- min(MIN, cumsum(data[, assets[i]]))
    MIN <- min(MIN, cumsum(data[, benchmark]))
    rangeY <- c(MIN, MAX)
    
    # xlim - Plot Range:
    xlim <- range(time(colCumsums(data[, benchmark])))
    shift <- round(0.20 *as.integer(diff(xlim)), 0) * 24 * 60 * 60
    rangeX <- c(round(xlim[1]-shift), xlim[2])
    Days <- 1:as.integer(diff(xlim))
    Time <- as.character(xlim[1] + Days*24*60*60)
    range.tS <- timeSeries(data = matrix(rep(0, length(Time))), as.character(Time))
    
    # Limits:
    xlim <- rangeX
    ylim <- rangeY
    
    # Plot:
    plot(X, type = "n", xaxt = "n", at = at, format = Format,
        xlim = xlim, ylim = ylim, main = "", xlab = "", ylab = "") 
    grid(NA, ny = NULL)
    abline(v = as.POSIXct(at), lty = 3, col = "brown")
    
    # Add Lines:
    lines(colCumsums(data[, benchmark]), col = "black")
    lines(colCumsums(data[, benchmark]), col = "black")
    for (i in 1:nAssets) lines( colCumsums(data[, assets[i]]), col = i+1)
    
    # Asset Names:
    Benchmark <- abbreviate(benchmark, 4)
    Assets <- abbreviate(assets, 4)
    assetsList <- c(Benchmark, Assets)
    assetsTitle <- paste(Benchmark, " ~ ", 
        paste( Assets, collapse = " - ", sep = ""), sep="")
    
    # Add Title:
    if (labels) {
        title(main = main, xlab = xlab, ylab = ylab)
    }
    
    # Add Legend and Subtitle:
    if (legend) {
        mtext(assetsTitle, line = 0.5, cex = 0.7)
        legend("topleft",
            legend = assetsList,
            bty = "n",
            text.col = 1:(nAssets+1),
            cex = 0.8)
    }
   
    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------
# Plot 2:


backtestWeightsPlot <-
    function(object, labels=TRUE, legend=TRUE, at=NULL, format=NULL)
{
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    #   Plots recommended weights from a portfolio backtest
   
    # Arguments:
    #   object - a list as returned by the function portfolioSmoothing()
    #   labels - a logical flag, should automated labels added to the plot

    # FUNCTION:

    # Settings:
    data <- object$data
    weights <- object$smoothWeights
    assets <- object$assetsNames
    benchmark <- object$benchmarkName
    horizon <- getWindowsHorizon(object$backtest)
    smoothing <- getSmootherLambda(object$backtest)
    startup <- "1m"
    horizonLength = as.numeric(substr(horizon, 1, nchar(horizon)-1))
    horizonUnit = substr(horizon, nchar(horizon), nchar(horizon))
    stopifnot(horizonUnit == "m")
    
    # Time Axis:
    if (is.null(at)) at <- paste(unique(atoms(time(data))[,1]), "12-31", sep="-")
    if (is.null(format)) Format <- "%b/%y" else Format <- format
    
    # Labels ?
    if (labels) {
        main <- "Weights Recommendation"
        xlab <- ""
        ylab <- "Asset Weights %"
    } else {
        main <- ""
        xlab <- ""
        ylab <- ""
    }

    # Series:
    X <- data[, benchmark]
    nAssets <- length(assets)
    naWeights <- matrix(rep(NA, times=horizonLength*nAssets), ncol=nAssets)

    # Lmits:
    xlim <- range(data)
    ylim <- c(0, 100)
    
    # Plot:
    plot(X, type = "n", xaxt = "n", las = 2, at = at, format = Format,
        xlim = xlim, ylim = ylim, main = "", xlab = "", ylab = "") 
    grid(NA, ny = NULL)
    abline(v = as.POSIXct(at), lty = 3, col = "brown")
    
    # Add Lines:
    lines(X, col = "black")
    tS <- 100 * timeSeries(weights)
    for (i in 1:nAssets) lines(tS[, i], col = i+1)

    # Asset Names:
    Benchmark <- abbreviate(benchmark, 4)
    Assets <- abbreviate(assets, 4)
    assetsList <- c(Benchmark, Assets)
    assetsTitle <- paste(Benchmark, " ~ ", 
        paste(Assets, collapse = " - ", sep = ""), sep="")
    
    # Add Title:
    if (labels) {
        title(main = main, xlab = xlab, ylab = ylab)
        text <- paste(
            "Horizon = ", horizon,
            "| Smoothing:", smoothing,
            "| Startup:", startup,
            "| Shift 1m")
        mtext(text, line = 0.5, cex = 0.7)
    }
    
    # Add Legend:
    if (legend) {
        legend("topleft",
            legend = assetsList,
            bty = "n",
            text.col = 1:(nAssets+1),
            cex = 0.8)
    }

    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------
# Plot 3:


backtestRebalancePlot <-
    function(object, labels=TRUE, legend=TRUE, at=NULL, format=NULL)
{
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    #   Plots rebalanced weights of a backtest

    # Arguments:
    #   object - a list as returned by the function portfolioSmoothing()
    #   labels - a logical flag, should automated labels added to the plot
   
    # FUNCTION:

    # Settings:
    data <- object$data
    weights <- object$smoothWeights
    assets <- object$assetsNames
    benchmark <- object$benchmarkName
    horizon <- getWindowsHorizon(object$backtest)
    smoothing <- getSmootherLambda(object$backtest)
    startup <- "1m"
    horizonLength <- as.numeric(substr(horizon, 1, nchar(horizon)-1))
    horizonUnit <- substr(horizon, nchar(horizon), nchar(horizon))
    stopifnot(horizonUnit == "m")
    horizon <- horizonLength
    
    # Time Axis:
    if (is.null(at)) at <- paste(unique(atoms(time(data))[,1]), "12-31", sep="-")
    if (is.null(format)) Format <- "%b/%y" else Format <- format
    
    # Labels ?
    if (labels) {
        main <- "Weights Rebalance"
        xlab <- ""
        ylab <- "Weights Changes %"
    } else {
        main <- ""
        xlab <- ""
        ylab <- ""
    }
   
    # Series:
    X <- data[, benchmark]
    nAssets <- length(assets)
    naWeights <- matrix(rep(NA, times = horizon * nAssets), ncol = nAssets)
    naWeights <- rbind(naWeights, rep(NA, times = nAssets))
    diffWeights <- rbind(naWeights, diff(weights))
    absSum <- function(x) { sum(abs(x)) }
    diffWeights <- apply(diffWeights, 1, FUN = absSum)
    diffWeights <- cbind(diffWeights, rbind(naWeights, diff(weights)))
    tS <- 100 * timeSeries(diffWeights[-seq(horizon + 1),],
        charvec = rownames(diffWeights)[-seq(horizon + 1)])   
        
    # Limits:
    xlim <- range(time(data))
    ylim <- range(tS)
    
    # Plot:
    plot(X, type = "n", xaxt = "n", las = 2, at = at, format = Format,
        xlim = xlim, ylim = ylim, main = "", xlab = "", ylab = "") 
    grid(NA, ny = NULL)
    abline(v = as.POSIXct(at), lty = 3, col = "brown")
    abline(h=0, col="darkgrey")
    
    # Add Lines:
    # lines(X)
    lines(tS[, 1], type = "h", lwd = 1, col = "darkgrey")
    for (i in 2:NCOL(tS)) lines(tS[, i], col = i)
   
    # Asset Names:
    Benchmark <- abbreviate(benchmark, 4)
    Assets <- abbreviate(assets, 4)
    assetsList <- c(Benchmark, Assets)
    assetsTitle <- paste(Benchmark, " ~ ", 
        paste(Assets, collapse = " - ", sep = ""), sep="")
    
    # Add Title:
    if (labels) {
        title(main = main, xlab = xlab, ylab = ylab)
        text <- paste(
            "Horizon = ", horizon,
            "| Smoothing:", smoothing,
            "| Startup:", startup,
            "| Shift 1m")
        mtext(text, line = 0.5, cex = 0.7)
        # mText = paste("Start:", rownames(object$smoothWeights)[1])
        # mtext(mText, side = 4, line = 0, adj = 0, col = "darkgrey", cex = 0.65)
    }
    
    # Add Legend:
    if (legend) {
        legend("topleft",
            legend = assetsList,
            bty = "n",
            text.col = 1:(nAssets+1),
            cex = 0.8)
    }

    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------
# Plot 4:


backtestPortfolioPlot <-
    function(object, labels=TRUE, legend=TRUE, at=NULL, format=NULL)
{
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    #   Plots daily, benchmark and portfolio series of a portfolio backtest

    # Arguments:
    #   object - a list as returned by the function portfolioSmoothing()
    #   labels - a logical flag, should automated labels added to the plot

    # FUNCTION:

    # Settings:
    data <- object$data
    portfolioReturns <- object$portfolioReturns
    benchmarkReturns <- object$benchmarkReturns
    assets <- object$assetsNames
    benchmark <- object$benchmarkName
    horizon <- getWindowsHorizon(object$backtest)
    smoothing <- getSmootherLambda(object$backtest)
    startup <- "1m"
    offsetReturn <- object$offsetReturn
    
    # Time Axis:
    if (is.null(at)) at <- paste(unique(atoms(time(data))[,1]), "12-31", sep="-")
    if (is.null(format)) Format <- "%b/%y" else Format <- format
    
    # Labels ?
    if (labels) {
        main <- "Portfolio vs Benchmark"
        xlab <- ""
        ylab <- "Cumulated log Returns"   
    } else {
        main <- ""
        xlab <- ""
        ylab <- ""  
    }
   
    # Series:
    X <- data[, benchmark]

    # Cumulated Return Series:
    cumX <- colCumsums(X)
    cumP <- portfolioReturns + offsetReturn
    cumB <- benchmarkReturns + offsetReturn
    offsetTS <- timeSeries(offsetReturn, charvec = names(offsetReturn),
        units = "offsetReturn")
    cumP <- rbind(offsetTS, cumP)
    cumB <- rbind(offsetTS, cumB)
    MAX <- max(as.vector(series(cumP)), as.vector(series(cumB)),
        as.vector(series(cumX)))
    MIN <- min(as.vector(series(cumP)), as.vector(series(cumB)),
        as.vector(series(cumX)))
       
    # Limits:
    xlim <- c(as.POSIXct(start(X)), as.POSIXct(end(X)))
    ylim <- c(MIN, MAX)
     
    # Plot:
    plot(X, type = "n", xaxt = "n", at = at, format = Format,
        xlim = xlim, ylim = ylim, main = "", xlab = "", ylab = "") 
    grid(NA, ny = NULL)
    abline(v = as.POSIXct(at), lty = 3, col = "brown")
    abline(h=0, col="darkgrey")
    
    # Add Lines:
    lines(cumX, col = "black")
    lines(cumP-cumB, type = "h", col = "grey")
    lines(cumP, col = "red", lwd = 2)
    lines(cumB, col = "blue", lwd = 2)

    # Asset Names:
    Benchmark <- abbreviate(benchmark, 4)
    Assets <- abbreviate(assets, 4)
    assetsList <- c(Benchmark, Assets)
    assetsTitle <- paste(Benchmark, " ~ ", 
        paste(Assets, collapse = " - ", sep = ""), sep="")
    nAssets <- length(assetsList)
    
    # Add Title:
    if (labels) {
        title(main = main, xlab = xlab, ylab = ylab)
        text <- paste(
            "Horizon = ", horizon,
            "| Smoothing:", smoothing,
            "| Startup:", startup,
            "| Shift 1m")
        mtext(text, line = 0.5, cex = 0.7)
        # mText <- Type = getType(object$spec)
        # Estimator <- getEstimator(object$spec)
        # if (Type == "MV") mText = paste(mText, "|", Estimator)
        # mtext(mText, side = 4, line = 0, adj = 0, col = "darkgrey", cex = 0.7)
    }
    
    # Add Legend:
    if (legend) {
        legend("topleft",
            legend = assetsList,
            bty = "n",
            text.col = 1:(nAssets+1),
            cex = 0.8)
    }
    
    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------
# Plot 5:


backtestDrawdownPlot <- 
    function(object, labels=TRUE, legend=TRUE, at=NULL, format=NULL)
{
    # A function implemented by Diethelm Wuertz and William Chen

    # Description:
    #   Creates Backtest Portfolio Plot

    # Arguments:
    #   object - a list as returned by the function portfolioSmoothing()
    #   labels - a logical flag, should automated labels added to the plot

    # FUNCTION:
    
    # Settings:
    data <- object$data
    assets <- object$assetsNames
    benchmark <- object$benchmarkName
    horizon <- getWindowsHorizon(object$backtest)
    smoothing <- getSmootherLambda(object$backtest)
    startup <- getSmootherInitialWeights(object$backtest)
    weights <- as.timeSeries(object$smoothWeights)
    
    # Align Data:
    ## 2022-10-11 GNB: was: Data <- .align.timeSeries(data)/100
    ##   assuming that data is a 'timeSeries' objec, which seems it is,
    ##   but I was not able to verify this since there are no examples with it
    ##   and the documentation is unclear (or rather, I got lost).
    ##   Note: the timeSeries method for align() is '.align.timeSeries'
    Data <- align(data)/100
    
    # Time Axis:
    if (is.null(at)) at <- paste(unique(atoms(time(data))[,1]), "12-31", sep="-")
    if (is.null(format)) Format <- "%b/%y" else Format <- format
    
    # Labels ?
    if (labels) {
        main <- "Portfolio vs Benchmark"
        xlab <- ""
        ylab <- "Drawdown"
    } else {
        main <- ""
        xlab <- ""
        ylab <- ""
    }
    
    # Extract the Time Stamps:
    tS <- time(Data)
    tW <- time(weights)
        
    # Problem when rebalance day lands on a Weekend - 
    #   need to change the date to the nearest Monday
    if (any(isWeekend(tW))){
        weekend.tW <- tW[isWeekend(tW)]       
        # WC: check timeNdayOnOrAfter function, the nday = 2 is a Monday!???
        tW <- sort(c(tW[!isWeekend(tW)], timeNdayOnOrAfter(weekend.tW, 2)))
        # replace old times with new times
        time(weights) <- tW
    }
            
    # Extract the Updated Revalance Dates:  
    Dates <- time(weights)
    
    # Subsetting the Data:
    data <- window(Data, start(weights), end(weights))
    
    # Check whether we have data past the last balance date
    # i.e. last balance date won't take place if we don't have the return series
    if (end(data) < end(weights)){ 
        n <- length(Dates)-1 
    } else {n = length(Dates)
        Dates <- c(Dates, end(data))
    }
    
    # Calculate the portfolio returns for the given weights:
    # assume we start investing the new weights on the rebalance date
    pf <- NULL
    a <- NULL
    for (i in 1:n){
        temp <- window(data, Dates[i], Dates[i+1])[,assets]
        nr <- nrow(temp)
        if (i != n) temp = temp[-nr,]
        a <- c(a, nrow(temp))
        pf <- c(pf, pfolioReturn(temp, as.numeric(weights[i,])))
    }
    
    # Drawdown Plot Settings:
    stopifnot(length(pf) == length(rownames(data)))
    pf <- timeSeries(pf, charvec = rownames(data))
    pf.DD <- drawdowns(pf)
    benchmark.DD <- drawdowns(data[,benchmark]) 
    
    # Series:
    X <- Data[, benchmark]
    
    # Limits:
    xlim <- c(as.POSIXct(start(X)), as.POSIXct(end(X)))
    ylim <- range(c(pf.DD, benchmark.DD))
    
    # Plot:
    plot(X, type = "n", xaxt = "n", at = at, format = Format,
        xlim = xlim, ylim = ylim, main = "", xlab = "", ylab = "") 
    grid(NA, ny = NULL)
    abline(v = as.POSIXct(at), lty = 3, col = "brown")
    
    # Add Lines:
    lines(benchmark.DD,  col = "blue", lwd = 2)
    lines(pf.DD, col = "red", lwd = 2)
    
    # Asset Names:
    Benchmark <- abbreviate(benchmark, 4)
    Assets <- abbreviate(assets, 4)
    assetsList <- c(Benchmark, Assets)
    assetsTitle <- paste(Benchmark, " ~ ", 
        paste(Assets, collapse = " - ", sep = ""), sep="")
    
    # Add Title:
    if (labels) {
        title(main = main, xlab = xlab, ylab = ylab)
        text <- paste("(Max)", "Portfolio DD =", round(min(pf.DD),2),
            "|", "Benchmark DD =", round(min(benchmark.DD),2))
        mtext(text, line = 0.5, cex = 0.7)
    }
    
    # Add Legend:
    if (legend) {
        legend("bottomleft",
            legend = c("Benchmark", "Portfolio"),
            bty = "n",
            text.col = c("blue", "red"),
            cex = 0.8)
    }
       
    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------
# Plot 6 - Report:


backtestReportPlot <-
    function(object, cex=0.6, font=1, family="mono")
{    
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    #   Prints backtest report as graphical plot
    
    # Arguments:
    #   object - a list as returned by the function portfolioSmoothing()
    
    # FUNCTION:
    
    # Settings:
    CEX <- cex
    
    
    # Start Plot:
    plot.new()
    plot.window(xlim = c(0, 1), ylim = c(0, 1))
      
    # Vertical Adjustment:
    z <- -2
        
    TEXT <- paste("Strategy:", getStrategyFun(object$backtest))
    mtext(TEXT, side = 3, line =  z + 3, adj = 0, font=2, family="sans", 
          cex=CEX)
        
    TEXT <-  capture.output(round(object$stats, 2))
    mtext(TEXT[1], side = 3, line = z + +2, adj = 0, font=font, family="mono", cex=CEX)
    mtext(TEXT[2], side = 3, line = z + +1, adj = 0, font=font, family="mono", cex=CEX)
    mtext(TEXT[3], side = 3, line = z + +0, adj = 0, font=font, family="mono", cex=CEX) 
    mtext(TEXT[4], side = 3, line = z + -1, adj = 0, font=font, family="mono", cex=CEX)
    mtext(TEXT[5], side = 3, line = z + -2, adj = 0, font=font, family="mono", cex=CEX) 
        
    TEXT <- capture.output(object$spec)[c(2,3,4,5,8)]
    mtext("Portfolio Specification:", side = 3, line = z + -4, adj = 0, font=2, family="sans", cex=CEX)
        
    if (length(grep("CVaR",TEXT[2]))!=0) TEXT[2] = 
        gsub("CVaR", paste("CVaR |", getAlpha(object$spec)), TEXT[2])
       
    mtext(TEXT[2], side = 3, line = z + -5, adj = 0, font=font, family=family, cex=CEX)
    mtext(TEXT[3], side = 3, line = z + -6, adj = 0, font=font, family=family, cex=CEX)
    mtext(TEXT[4], side = 3, line = z + -7, adj = 0, font=font, family=family, cex=CEX)
    #text(TEXT[5], side = 3, line = z + -8, adj = 0, font=font, family=family, cex=CEX)
    
    TEXT <- capture.output(object$constraints)[1]      
    mtext("Constraints:", side = 3, line = z + -9, adj = 0, font=2, family="sans", cex=CEX)
    TEXT <- substr(TEXT[1], 4, 99)   
    mtext(TEXT, side = 3, line = z + -10, adj = 0, font=font, family=family, cex=CEX)
                        
    # Return Value:
    invisible()
 }


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA


################################################################################
# FUNCTION:                    DESCRIPTION:
#  weightsPlot                   Plots staggered weights along the frontier
#  weightedReturnsPlot           Plots staggered weighted returns
#  riskBudgetsPlot               Plots general risk budgets
#    covRiskBudgetsPlot            Plots covariance risk budgets
#    tailRiskBudgetsPlot           Plots copulae tail risk budgets
################################################################################


weightsPlot <-
    function(object, labels=TRUE, col=NULL, title=TRUE, box=TRUE, 
    legend=TRUE, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Plots a bar chart of weights along the frontier

    # Arguments:
    #   object - an object of class 'fPORTFOLIO'
    #   labels - should the graph be automatically labeled?
    #   col - a color palette, by default the rainbow palette
    #   title - should the graph get default title and labels?
    #   legend - should a legend be added to the plot?
    #   box - 
    #   ... - arguments passed to the function barplot()
    
    # Requires:
    #   fBasics for seqPalette()

    # FUNCTION:
    
    # Check if we have a portfolio frontier ...
    weights <- getWeights(object@portfolio)
    if (is.null(dim(weights))) 
        stop("The object is not a set of portfolios along the frontier.")

    # Use default color if not specified ...
    Title <- "Weights"
    if (is.null(col)) col <- seqPalette(getNAssets(object), "Blues")
    if (sum(c(par()$mfrow, par()$mfcol)) == 4) CEX <- 0.9 else CEX <- 0.7

    # Compute Weights:
    pos.weights <- +0.5 * (abs(weights) + weights)
    neg.weights <- -0.5 * (abs(weights) - weights)

    # Define Plot Range:
    ymax <- max(rowSums(pos.weights))
    ymin <- min(rowSums(neg.weights))
    range <- ymax - ymin
    ymax <- ymax + 0.005 * range
    ymin <- ymin - 0.005 * range
    dim <- dim(weights)
    range <- dim[1]
    xmin <- 0
    xmax <- range + 0.2 * range

    # Create Bar Plots:
    if (labels) {
        if(!legend){
        barplot(t(pos.weights), col=col, space=0, ylab="",
            ylim=c(ymin, ymax), border="grey", ...)
        } else {
            barplot(t(pos.weights), col=col, space=0, ylab="",
                xlim=c(xmin, xmax), ylim=c(ymin, ymax),
                border="grey", ...)
            legendtext <- names(getStatistics(object)$mu)
            if(is.null(legendtext)){
                for(i in 1:dim[2]){legendtext[i] = paste("Asset", i, sep = " ")}
            }
            legend("topright", legend = legendtext, bty = "n", cex=0.7,
                fill = col)
        }
        barplot(t(neg.weights), col=col, space=0, add = TRUE,
            border="grey", ...)
    } else {
        barplot(t(pos.weights), col=col, ...)
    }

    # Add Tailored Labels -  6 may be a good Number ...
    targetRisk <- getTargetRisk(object@portfolio)[, 1]
    targetReturn <- getTargetReturn(object@portfolio)[, 1]
    nSigma <- length(targetRisk)
    nLabels <- 6
    M <- c(0, ( 1:(nSigma %/% nLabels) ) ) * nLabels + 1
    # Take a reasonable number of significant digits to plot, e.g. 2 ...
    nSignifDigits = 3
    axis(3, at=M, labels=signif(targetRisk[M], nSignifDigits))
    axis(1, at=M, labels=signif(targetReturn[M], nSignifDigits))

    # Add Axis Labels and Title:
    if (title) {
        mtext("Target Risk", side=3, line=2, adj=1, cex=CEX)
        mtext("Target Return", side=1, line=2, adj=1, cex=CEX)
        mtext("Weight", side=2, line=2, adj=1, cex=CEX)
    }

    # Add Weights 0 and 1 Reference Lines
    lines(x=c(0, nSigma), c(1, 1), col="grey", lty=3)
    lines(x=c(0, nSigma), c(0, 0), col="grey", lty=3)

    # Add vertical Line at minimum risk:
    minIndex <- which.min(targetRisk)
    minRisk <- signif(min(targetRisk))
    abline(v = minIndex, col="black", lty = 1, lwd = 2)

    # Add Title:
    if (title) mtext(Title, adj=0, line=2.5, font=2, cex=CEX+0.1)

    # Complete to draw Box:
    if (box) box()
    
    # Add Rmetrics - Do not Remove!
    mtext("Rmetrics", adj=0, side=4, cex=0.7, col="darkgrey")

    # Return Value:
    invisible(list(minRisk=minRisk))
}


# ------------------------------------------------------------------------------


weightedReturnsPlot <-
    function(object, labels=TRUE, col=NULL, title=TRUE, box=TRUE, 
    legend=TRUE, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Plots weighted returns

    # Arguments:
    #   object - an object of class 'fPORTFOLIO'
    #   labels - should the graph be automatically labeled?
    #   col - a color palette, by default the rainbow palette
    #   title - should the graph get default title and labels?
    #   legend - should a legend be added to the plot?

    # FUNCTION:

    # Check if we have a portfolio frontier ...
    weights <- getWeights(object@portfolio)
    if (is.null(dim(weights)))  
        stop("The object is not a set of portfolios along the frontier.")
        
    # Use default color if not specified ...
    Title <- "Weighted Returns"
    if (is.null(col)) col=seqPalette(getNAssets(object), "Blues")
    if (sum(c(par()$mfrow, par()$mfcol)) == 4) CEX <- 0.9 else CEX <- 0.7

    # Compute Weighted Returns:
    dim <- dim(weights)
    returns <- getStatistics(object)$mu
    weightedReturns <- NULL
    for(i in 1:dim[2]){
        nextWeightedReturns <- weights[,i]*returns[i]
        weightedReturns <- cbind(weightedReturns, nextWeightedReturns)
    }
    colnames(weightedReturns) = colnames(weights)
    pos.weightedReturns <- +0.5 * (abs(weightedReturns) + weightedReturns)
    neg.weightedReturns <- -0.5 * (abs(weightedReturns) - weightedReturns)

    # Define Plot Range:
    ymax <- max(rowSums(pos.weightedReturns))
    ymin <- min(rowSums(neg.weightedReturns))
    range <- ymax - ymin
    ymax <- ymax + 0.005 * range
    ymin <- ymin - 0.005 * range
    range <- dim[1]
    xmin <- 0
    xmax <- range + 0.2 * range

    # Create Bar Plots:
    if (labels) {
        if(legend){
            barplot(t(pos.weightedReturns), space=0, ylab="",
                xlim=c(xmin, xmax), ylim=c(ymin, ymax), col=col,
                border="grey", ...)
            legendtext <- names(getStatistics(object)$mu)
            if(is.null(legendtext)){
                for(i in 1:dim[2]){legendtext[i] = paste("Asset", i, sep = " ")}
            }
            legend("topright", legend = legendtext, bty = "n", cex=0.7,
                fill = col)
        } else {
            barplot(t(pos.weightedReturns), space=0, ylab="",
                ylim=c(ymin, ymax), col=col, border="grey", ...)
        }
        barplot(t(neg.weightedReturns), space=0, add = TRUE, col=col,
            border="grey", ...)
    } else {
        barplot(t(pos.weightedReturns), col=col, ...)
    }

    # Add Tailored Labels -  6 may be a good Number ...
    targetRisk <- getTargetRisk(object@portfolio)[, 1]
    targetReturn <- getTargetReturn(object@portfolio)[, 1]
    nSigma <- length(targetRisk)
    nLabels <- 6
    M <- c(0, ( 1: (nSigma %/% nLabels) ) ) * nLabels + 1
    # Take a reasonable number of significant digits to plot, e.g. 2 ...
    nSignifDigits = 3
    axis(3, at=M, labels=signif(targetRisk[M], nSignifDigits))
    axis(1, at=M, labels=signif(targetReturn[M], nSignifDigits))

    # Add Axis Labels and Title:
    if (title) {
        mtext("Target Risk", side=3, line=2, adj=1, cex=CEX)
        mtext("Target Return", side=1, line=2, adj=1, cex=CEX)
        mtext("Weighted Return", side=2, line=2, adj=1, cex=CEX)
    }

    # Add Weights 0 and 1 Reference Lines
    lines(x=c(0, nSigma), c(1, 1), col="grey", lty=3)
    lines(x=c(0, nSigma), c(0, 0), col="grey", lty=3)

    # Add vertical Line at minimum risk:
    minIndex = which.min(targetRisk)
    minRisk = signif(min(targetRisk))
    abline(v = minIndex, col="black", lty = 1, lwd = 2)

    # Add Title:
    if (title) mtext(Title, adj=0, line=2.5, font=2, cex=CEX+0.1)

    # Complete to draw Box:
    if (box) box()

    # Add Rmetrics - Do not Remove!
    mtext("Rmetrics", adj=0, side=4, cex=0.7, col="darkgrey")
    
    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------


covRiskBudgetsPlot <-
    function(object, labels=TRUE, col=NULL, title = TRUE, box = TRUE, 
    legend = TRUE, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Plots a bar chart of covariance risk budgets

    # Arguments:
    #   object - an object of class 'fPORTFOLIO'
    #   labels - should the graph be automatically labeled?
    #   col - a color palette, by default the rainbow palette
    #   title - should the graph get default title and labels?
    #   legend - should a legend be added to the plot?

    # FUNCTION:
    
    # Settings:
    mtext <- TRUE 
      
    # Check if we have a portfolio frontier ...
    weights <- getWeights(object@portfolio)
    if (is.null(dim(weights))) 
        stop("The object is not a set of portfolios along the frontier.")
        
    # Use default color if not specified ...
    Title <- "Cov Risk Budgets"
    if (is.null(col)) col <- seqPalette(getNAssets(object), "Blues")
    if (sum(c(par()$mfrow, par()$mfcol)) == 4) CEX <- 0.9 else CEX <- 0.7

    # Compute Covariance Risk Budgets:
    budgets <- getCovRiskBudgets(object@portfolio)
    pos.budgets <- +0.5 * (abs(budgets) + budgets)
    neg.budgets <- -0.5 * (abs(budgets) - budgets)

    # Define Plot Range:
    ymax <- max(rowSums(pos.budgets))
    ymin <- min(rowSums(neg.budgets))
    range <- ymax - ymin
    ymax <- ymax + 0.005 * range
    ymin <- ymin - 0.005 * range
    dim <- dim(budgets)
    range <- dim[1]
    xmin <- 0
    xmax <- range + 0.2 * range

    # Create Bar Plots:
    if (labels) {
        if(!legend){
            barplot(t(pos.budgets), space=0, ylab="",
                ylim=c(ymin, ymax), col=col, border="grey", ...)
        } else {
            legendtext <- names(getStatistics(object)$mu)
            if(is.null(legendtext)){
                for(i in 1:dim[2]){legendtext[i] = paste("Asset", i, sep = " ")}
            }
            barplot(t(pos.budgets), space=0, ylab="", xlim=c(xmin, xmax),
                ylim=c(ymin, ymax), col=col, border="grey", ...)
            legend("topright", legend = legendtext, bty = "n", cex=0.7,
                fill = col)
        }
        barplot(t(neg.budgets), space=0, add = TRUE, col=col,
            border="grey", ...)
    } else {
        barplot(t(pos.budgets), col=col, ...)
    }

    # Add Tailored Labels -  6 may be a good Number ...
    targetRisk = getTargetRisk(object@portfolio)[, 1]
    targetReturn = getTargetReturn(object@portfolio)[, 1]
    nSigma = length(targetRisk)
    nLabels <- 6
    M = c(0, ( 1:(nSigma %/% nLabels) ) ) * nLabels + 1

    # Take a reasonable number of significant digits to plot, e.g. 2 ...
    nSignifDigits = 3
    axis(3, at = M, labels=signif(targetRisk[M], nSignifDigits))
    axis(1, at = M, labels=signif(targetReturn[M], nSignifDigits))

    # Add Axis Labels and Title:
    if(title) {
        mtext("Target Risk", side=3, line=2, adj=1, cex=CEX)
        mtext("Target Return", side=1, line=2, adj=1, cex=CEX)
        mtext("Cov Risk Budgets", side=2, line=2, adj=1, cex=CEX)
    }

    # Add Budgets 0 and 1 Reference Lines
    lines(x=c(0, nSigma), c(1, 1), col="grey", lty=3)
    lines(x=c(0, nSigma), c(0, 0), col="grey", lty=3)

    # Add vertical Line at minimum risk:
    minIndex = which.min(targetRisk)
    minRisk = signif(min(targetRisk), 3)
    abline(v = minIndex, col="black", lty = 1, lwd = 2)

    # Add Title:
    if (title) mtext(Title, adj=0, line=2.5, font=2, cex=CEX+0.1)

    # Complete to draw Box:
    if (box) box()

    # Add Rmetrics - Do not Remove!
    mtext("Rmetrics", adj=0, side=4, cex=0.7, col="darkgrey")
    
    # Return Value:
    invisible()
}


# ------------------------------------------------------------------------------


tailRiskBudgetsPlot <-
    function(object, labels=TRUE, col=NULL, title = TRUE, box = TRUE, 
    legend = TRUE, ...)
{
    # A function implemented by Diethelm Wuertz

    # Description:
    #   Plots a bar chart of tail risk budgets

    # Arguments:
    #   object - an object of class 'fPORTFOLIO'
    #   labels - should the graph be automatically labeled?
    #   col - a color palette, by default the rainbow palette
    #   title - should the graph get default title and labels?
    #   legend - should a legend be added to the plot?

    # FUNCTION:

    # Settings:
    mtext <- TRUE 
    
    # Check if we have a portfolio frontier ...
    weights <- getWeights(object@portfolio)
    if (is.null(dim(weights))) 
        stop("The object is not a set of portfolios along the frontier.")
        
    # Use default color if not specified ...
    Title <- "Tail Risk Budgets"
    if (is.null(col)) col <- seqPalette(getNAssets(object), "Blues")
    if (sum(c(par()$mfrow, par()$mfcol)) == 4) CEX <- 0.9 else CEX <- 0.7

    # Check:
    stop("The tail risk budget plot is not yet implemented")
    tailRiskMatrix <- getTailRisk(object@portfolio)

    # Compute Tail Risk Budgets:
    budgets <- getTailRiskBudgets(object@portfolio)
    budgets[is.na(budgets)] <- 0
    pos.budgets <- +0.5 * (abs(budgets) + budgets)
    neg.budgets <- -0.5 * (abs(budgets) - budgets)

    # Define Plot Range:
    ymax <- max(rowSums(pos.budgets))
    ymin <- min(rowSums(neg.budgets))
    range <- ymax - ymin
    ymax <- ymax + 0.005 * range
    ymin <- ymin - 0.005 * range
    dim <- dim(budgets)
    range <- dim[1]
    xmin <- 0
    xmax <- range + 0.2 * range

    # Create Bar Plots:
    if(!legend){
        barplot(t(pos.budgets), space=0, ylab="",
            ylim=c(ymin, ymax), col=col, border="grey", ...)
    } else {
        legendtext <- names(getStatistics(object)$mu)
        if(is.null(legendtext)){
            for(i in 1:dim[2]){legendtext[i] = paste("Asset", i, sep = " ")}
        }
        barplot(t(pos.budgets), space=0, ylab="", xlim=c(xmin, xmax),
            ylim=c(ymin, ymax), col=col, border="grey", ...)
        legend("topright", legend=legendtext, bty="n", cex=0.7,
            fill=col)
    }
    barplot(t(neg.budgets), space=0, add=TRUE, col=col,
            border="grey", ...)

    # Add Tailored Labels -  6 may be a good Number ...
    targetRisk <- getTargetRisk(object)[, 1]
    targetReturn <- getTargetReturn(object)[, 1]
    nSigma <- length(targetRisk)
    nLabels <- 6
    M <- c(0, ( 1:(nSigma %/% nLabels) ) ) * nLabels + 1
    nSignifDigits = 3
    axis(3, at=M, labels=signif(targetRisk[M], nSignifDigits))
    axis(1, at=M, labels=signif(targetReturn[M], nSignifDigits))

    # Add Axis Labels and Title:
    if (title) {
        mtext("Target Risk", side=3, line=2, adj=1, cex=CEX)
        mtext("Target Return", side=1, line=2, adj=1, cex=CEX)
        mtext("Weight", side=2, line=2, adj=1, cex=CEX)
    }

    # Add Budgets 0 and 1 Reference Lines
    lines(x=c(0, nSigma), c(1, 1), col="grey", lty=3)
    lines(x=c(0, nSigma), c(0, 0), col="grey", lty=3)

    # Add vertical Line at minimum risk:
    minIndex <- which.min(targetRisk)
    minRisk <- signif(min(targetRisk), 3)
    abline(v = minIndex, col="black", lty = 1, lwd = 2)

    # Add Title:
    if (title) mtext(Title, adj=0, line=2.5, font=2, cex=CEX+0.1)

    # Complete to draw Box:
    if (box) box()

    # Add Rmetrics - Do not Remove!
    mtext("Rmetrics", adj=0, side=4, cex=0.7, col="darkgrey")
    
    # Return Value:
    invisible()
}


################################################################################


riskBudgetsPlot <-
    function(object, FUN=c("budgetsNormalVAR","budgetsNormalES",
                           "budgetsModifiedVAR","budgetsModifiedES",
                           "budgetsSampleCOV"),
             labels=TRUE, col=NULL, title = TRUE,
             mtext = TRUE, box = TRUE, legend = TRUE, ...)
    {
        # A function implemented by Diethelm Wuertz
        # Modified by Venetia Christodoulopoulou 2013-01-22
        
        # Description:
        #   Plots a bar chart of tail risk budgets
        
        # Arguments:
        #   object - an object of class 'fPORTFOLIO'
        #   FUN - a character, the function name to compute the risk budgets, e.g.
        #       "budgetsSampleCOV", "budgetsNormalVAR", "budgetsNormalES",    
        #       "budgetsModifiedVAR", "budgetsModifiedES"  
        #   labels - a logical, should the graph be automatically labeled?
        #   col - a color palette, by default the rainbow palette
        #   title - a logical, should the graph get default title and labels?
        #   By default TRUE. 
        #   mtext - a logical, should the margin text be added?
        #   box - a logical, should a box be xdrawn around the plot?
        #   legend - a logical, should a legend be added to the plot?
        #   ... - optional arguments passed to the barplot() function.
        
        # FUNCTION:
        
        # Extract Settings from the Potfolio Object:
        data <- getSeries(object@data)
        mu <- getMu(object)
        Sigma <- getSigma(object)
        weights <- getWeights(object@portfolio)
        alpha <- getAlpha(object@spec) 
        
        # Plot Decoration:
        
        #       name<-deparse(substitute(FUN))
        FUN2<-match.arg(FUN)
        if(title)Title<-switch(FUN2,budgetsNormalVAR="Normal VAR Risk Budgets",
                               budgetsNormalES="Normal ES Risk Budgets",
                               budgetsModifiedVAR="Modified VAR Risk Budgets",
                               budgetsModifiedES="Modified ES Risk Budgets",
                               budgetsSampleCOV="Covariance Risk Budgets")
        else
            Title<-"" # empty string
        
        
        if (is.null(col)) col <- seqPalette(getNAssets(object), "Blues")
        if (sum(c(par()$mfrow, par()$mfcol)) == 4) cex=0.9 else cex=0.7
        CEX<-0.7
        
        # Compute Covariance Risk Budgets:
        FUN<-switch(FUN2,budgetsNormalVAR=budgetsNormalVAR,
                    budgetsNormalES=budgetsNormalES,
                    budgetsModifiedVAR=budgetsModifiedVAR,
                    budgetsModifiedES=budgetsModifiedES)
        
        if(FUN2 == "budgetsSampleCOV")
            budgets <- getCovRiskBudgets(object@portfolio)
        else
        {
            fun <- function(x, data, alpha, mu, Sigma) 
                FUN(data, weights=as.vector(x), alpha, mu, Sigma)$budgets
            budgets <- t(apply(weights, 1, fun, data=data, alpha=alpha, mu=mu, Sigma=Sigma))
        }
        
        pos.budgets <- +0.5 * (abs(budgets) + budgets)
        neg.budgets <- -0.5 * (abs(budgets) - budgets)
        
        # Define Plot Range:
        ymax <- max(rowSums(pos.budgets))
        ymin <- min(rowSums(neg.budgets))
        range <- ymax - ymin
        ymax <- ymax + 0.005 * range
        ymin <- ymin - 0.005 * range
        dim <- dim(budgets)
        range <- dim[1]
        xmin <- 0
        xmax <- range + 0.2 * range
        
        # Create Bar Plots:
        if (labels) {
            if(!legend){
                barplot(t(pos.budgets), space=0, ylab="",
                        ylim=c(ymin, ymax), col=col, border="grey")
            } else {
                legendtext <- names(getStatistics(object)$mu)
                if(is.null(legendtext)){
                    for(i in 1:dim[2]){legendtext[i] <- paste("Asset", i, sep = " ")}
                }
                barplot(t(pos.budgets), space=0, ylab="", xlim=c(xmin, xmax),
                        ylim=c(ymin, ymax), col=col, border="grey")
                legend("topright", legend=legendtext, bty="n", cex=0.7, fill=col)
            }
            barplot(t(neg.budgets), space=0, add=TRUE, col=col, border="grey")
        } else {
            barplot(t(pos.budgets), col=col)
        }
        
        # Add Tailored Labels:
        nLabels <- 6
        targetRisk <- getTargetRisk(object@portfolio)[, 1]
        targetReturn <- getTargetReturn(object@portfolio)[, 1]
        nSigma <- length(targetRisk)
        M <- c(0, ( 1:(nSigma %/% nLabels) ) ) * nLabels + 1
        # Take a reasonable number of significant digits to plot:
        nSignifDigits <- 3
        axis(3, at=M, labels=signif(targetRisk[M], nSignifDigits))
        axis(1, at=M, labels=signif(targetReturn[M], nSignifDigits))
        
        # Add Budgets 0 and 1 Reference Lines:
        lines(x=c(0, nSigma), c(1, 1), col="grey", lty=3)
        lines(x=c(0, nSigma), c(0, 0), col="grey", lty=3)
        
        # Add Vertical Line at Minimum Risk:
        minIndex <- which.min(targetRisk)
        minRisk <- signif(min(targetRisk), 3)
        abline(v=minIndex, col="black", lty=1, lwd=2)
        
        # Add Margin Text Info - Do not remove this line:
        mtext("Rmetrics", side=4, adj=0, col="grey", cex=CEX)
        
        # Add Title:
        if (title) mtext(Title, adj=0, line=2.5, font=2, cex=CEX+0.1)
        
        # Complete to draw Box:
        if (box) box()
        
        # Add Rmetrics - Do not Remove!
        mtext("Rmetrics", adj=0, side=4, cex=0.7, col="darkgrey")
        
        # Return Value:
        invisible(budgets)
    }



###############################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR Description. See the 
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General 
# Public License along with this library; if not, write to the 
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
# MA 02111-1307 USA


################################################################################
# FUNCTION:                     DESCRIPTION:
#  'fPFOLIODATA'                 S4 Portfolio Data Class
################################################################################


setClass("fPFOLIODATA", 
    representation(
        data = "list",
        statistics = "list",
        tailRisk = "list")  
)


################################################################################

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################
# FUNCTION:                    DESCRIPTION:
#  portfolioBacktesting         Performs a portfolio backtesting
#  portfolioSmoothing           Smoothes the weights of a portfolio backtesting
################################################################################


portfolioBacktesting <-
  function(formula, data, spec = portfolioSpec(), constraints = "LongOnly", 
           backtest = portfolioBacktest(), trace = TRUE)
  {
    # A function implemented by William Chen and Diethelm Wuertz
    
    # Description:
    #   Backtests a portfolio on rolling windows
    
    # Arguments:
    #   formula - a formula expression to select benchmark and assets
    #       from the data set
    #   data - data set of assets returns, an object of class fPFLOLIODATA 
    #       or timeSeries
    #   spec - portfolio specification, an object of class fPFLOLIOSPEC,
    #       by default as returned by the function portfolioSpec()
    #   constraints - portfolio constraints, a vector of character strings
    #   backtest - portfolio backtest specification, an object of 
    #       class fPFLOLIOBACKTEST, by default as returned by the function
    #       portfolioBacktest
    #   trace - a logical, should the backtesting be traced ?
    
    # Value:
    #   A list with the following elements 
    #   formula - the input formula 
    #   data - the input data set
    #   spec - the input portfolio specification 
    #   constraints - the input constraints 
    #   backtest - the input backtest specification  
    #   benchmarkName - the name of the benchmark returns 
    #   assetsNames - the names of the assets returns
    #   weights - the rolling weights matrix 
    #   strategyList - the rolling list of optimized portfolios 
    #   Sigma - ...
    
    # Details:
    #   Allows for user specified rolling Windows
    #   Smoothing is separated and can be user specified
    
    # Example:
    #   portfolioBacktesting(formula, data, spec, constraints, backtest)
    
    # FUNCTION:
    
    # Data:
    if (inherits(data,"fPFOLIODATA")) {
      Data <- data
      data <- getSeries(data)
    } else if (inherits(data,"timeSeries")) {
      Data <- portfolioData(data, spec)
    }
    
    # Constraints:
    if (inherits(constraints,"fPFOLIOSPEC")) {
      Constraints <- constraints
      constraints <- Constraints@stringConstraints
    } else if (inherits(constraints,"character")) {
      Constraints <- portfolioConstraints(data, spec, constraints)
    }
    
    # Formula, Benchmark and Asset Labels:
    benchmarkName = as.character(formula)[2]
    assetsNames <- strsplit(gsub(" ", "", as.character(formula)[3]), "\\+")[[1]]
    nAssets <- length(assetsNames)
    
    # Trace the Specifications and Data Info:
    if(trace) {
      cat("\nPortfolio Backtesting:\n")
      cat("\nAssets:             ", assetsNames)
      cat("\nBenchmark:          ", benchmarkName)
      cat("\nStart Series:       ", as.character(start(data)))
      cat("\nEnd Series:         ", as.character(end(data)))
      cat("\n  Type:             ", getType(spec))
      cat("\n  Cov Estimator:    ", getEstimator(spec))
      cat("\n  Solver:           ", getSolver(spec))
      cat("\nPortfolio Windows:  ", getWindowsFun(backtest))
      cat("\n  Horizon:          ", getWindowsHorizon(backtest))
      cat("\nPortfolio Strategy: ", getStrategyFun(backtest))
      cat("\nPortfolio Smoother: ", getSmootherFun(backtest))
      cat("\n  doubleSmoothing:  ", getSmootherDoubleSmoothing(backtest))
      cat("\n  Lambda:           ", getSmootherLambda(backtest))
    }
    
    # We invest in the "Strategy" or (return) efficient Portfolio:
    if(trace) {
      cat("\n\nPortfolio Optimization:")
      cat("\nOptimization Period\tTarget\tBenchmark\t Weights\n")
    }
    
    # Create Rolling Windows:
    windowsFun <- match.fun(getWindowsFun(backtest))
    rollingWindows <- windowsFun(data, backtest)
    from <- rollingWindows$from
    to <- rollingWindows$to
    
    # Roll the Portfolio:
    strategyFun <- match.fun(getStrategyFun(backtest))
    strategyList <- list()
    
    # WC: track the sigma over time:
    Sigma <- NULL
    
    for (i in 1:length(from)) 
    {
      # Optimize the Portfolio:
      pfSeries <- window(data[, assetsNames], start = from[i], end = to[i])
      bmSeries <- window(data[, benchmarkName], start = from[i], end = to[i])
      pfSeries <- portfolioData(pfSeries, spec)
      Sigma <- c(Sigma, mean(diag(getSigma(pfSeries))))
      strategy <- strategyFun(
        data = getSeries(pfSeries), 
        spec = spec, 
        constraints = constraints, 
        backtest = backtest)
      strategyList[[i]] <- strategy
      
      # Trace Optionally the Results:
      if (trace) {
        cat(as.character(from[i]), as.character(to[i]))
        spReturn <- getTargetReturn(strategy@portfolio)[[2]]
        cat("\t", round(spReturn[1], digits = 3))
        bmReturn <- mean(series(bmSeries))
        cat("\t", round(bmReturn, digits = 3))
        nAssets <- length(assetsNames)
        weights <- round(getWeights(strategy), digits = 3)
        cat("\t")
        for (i in 1:length(assetsNames)) cat("\t", weights[i])         
        cat("\t  * ", round(sum(weights), 2))
        cat("\n")
      }  
    }
    
    # Extract Portfolio Investment Weights for the current period:
    weights <- NULL
    for (i in 1:length(strategyList)) 
      weights <- rbind(weights, getWeights(strategyList[[i]]))
    rownames(weights) <- as.character(to)
    colnames(weights) <- assetsNames
    
    # Compose Result:
    ans <- list(
      formula = formula,
      data = data,
      spec = spec,
      constraints = constraints,
      backtest = backtest,
      benchmarkName = benchmarkName,
      assetsNames = assetsNames,
      weights = weights,
      strategyList = strategyList, 
      Sigma = Sigma)
    
    # Return Value:
    class(ans) <- c("portfolioBacktesting", "list")
    invisible(ans)
  }


# ------------------------------------------------------------------------------


portfolioSmoothing <-
  function(object, backtest=NULL, trace=TRUE)
  {
    # A function implemented by William Chen and Diethelm Wuertz
    
    # Description:
    #   Flexible Weights Smoother Function
    
    # Arguments:
    #   object - an object as returned by the function portfolioBacktesting()
    #   backtest - an S4 class object of 'FPFOLIOBACKTEST', the same as
    #       used in the function portfolioBacktesting() or a user modified
    #       version (obsolete)
    #   trace - a logical, should the computation be traced ?
    
    # Value:
    #   a list with the following entries
    
    # Example:
    #   data <- 100*SWX.RET; object=portfolioBacktesting(LP40~SBI+SPI+SII, data)
    #   portfolioSmoothing(object)
    
    # FUNCTION:
    
    # Obsolete Argument
    if (!is.null(backtest)) {
      warning("The backtest argument is obsolete and will be
            removed for the next release.")
    }
    
    # Backtest Settings:
    formula <- object$formula
    data <- object$data
    spec <- object$spec
    constraints <- object$constraints
    backtest <- object$backtest 
    benchmarkName <- object$benchmarkName
    assetsNames <- object$assetsNames
    weights <- object$weights
    skip <- getSmootherSkip(backtest)
    if (skip > 0) weights <- weights[-(1:skip), ]
    nAssets <- ncol(weights)
    
    # Add Smooth Weights to Backtest object:
    if (trace) print("smooth ...")
    smoother <- match.fun(getSmootherFun(backtest))
    smoothWeights <- object$smoothWeights <- smoother(weights, spec, backtest)
    
    # Compute Monthly Assets and Benchmark Returns:
    if (trace) print("aggregate ...")
    ow <- options("warn")
    options(warn = -1)
    monthlyAssets <- object$monthlyAssets <-
      applySeries(data[, assetsNames], by = "monthly", FUN = colSums)
    monthlyBenchmark <- object$monthlyBenchmark <-
      applySeries(data[, benchmarkName], by = "monthly", FUN = colSums)
    options(ow)   
    
    # Compute Offset Return of Rolling Portfolio compared to Benchmark:
    if (trace) print("offset ...")
    cumX <- colCumsums(data[, benchmarkName])
    lastX <- window(cumX, start = start(cumX), end = rownames(weights)[1] )
    offsetReturn <- as.vector(lastX[end(lastX), ])
    names(offsetReturn) <- as.character(end(lastX))
    object$offsetReturn <- offsetReturn
    
    # Backtest Return Series:
    Datum <- as.vector(rownames(smoothWeights))
    nDatum <- length(Datum)
    Portfolio = Benchmark = NULL
    for (i in 1:(nDatum-1)) {
      Portfolio <- rbind(Portfolio, as.vector((
        as.matrix(monthlyAssets[Datum[i+1], ]) %*% smoothWeights[Datum[i], ])))
      Benchmark <- rbind(Benchmark, as.vector(monthlyBenchmark[Datum[i+1], ]))
    }
    
    # Portfolio:
    P <- timeSeries(data = Portfolio, charvec = Datum[-1], units = "Portfolio")
    object$portfolioReturns <- portfolio <- colCumsums(P)
    object$P <- P
    
    # Benchmark:
    B <- timeSeries(data = Benchmark, charvec = Datum[-1], units = "Benchmark")
    object$benchmarkReturns <- benchmark <- colCumsums(B)
    object$B <- B
    
    daily <- colCumsums(data[, benchmarkName])
    Daily <- window(daily, start = start(portfolio), end = end(portfolio))
    
    portfolio <- portfolio - portfolio[1] + Daily[1]
    benchmark <- benchmark - benchmark[1] + Daily[1]
    
    # Add to backtest:
    object$portfolio <- portfolio
    object$benchmark <- benchmark
    
    # Backtest Statistics:
    P <- as.vector(P)
    B  <- as.vector(B)
    Stats <- c(sum(P, na.rm = TRUE), sum(B))
    Stats <- rbind(Stats, c(mean(P, na.rm = TRUE), mean(B)))
    Stats <- rbind(Stats, c(sd(P, na.rm = TRUE), sd(B)))
    Stats <- rbind(Stats, c(min(P, na.rm = TRUE), min(B)))
    colnames(Stats) <- c(
      "Portfolio",
      "Benchmark")
    rownames(Stats) <- c(
      "Total Return",
      "Mean Return",
      "StandardDev Return",
      "Maximum Loss")
    object$stats <- Stats
    
    # Return Value:
    class(object) <- c("portfolioSmoothing", "list")
    object
  } 


################################################################################

\name{backtest-plots}


\alias{backtestPlot}

\alias{backtestAssetsPlot}
\alias{backtestWeightsPlot}
\alias{backtestRebalancePlot}
\alias{backtestPortfolioPlot}
\alias{backtestDrawdownPlot}
\alias{backtestReportPlot}


\title{Portfolio backtesting plots}


\description{

    Creates and displays plots of cumulative assets returns, of 
    portfolio weights, of rebalanced weights, of drawdowns 
    and of a report summary for backtesting.

}


\usage{   
% We should add a generic plot function for objects of class smoothed
% portfolios. DW
%
backtestPlot(object, which="all", labels=TRUE, legend=TRUE, at=NULL, 
  format=NULL, cex=0.6, font=1, family="mono")

backtestAssetsPlot(object, labels=TRUE, legend=TRUE, at=NULL, format=NULL)
backtestWeightsPlot(object, labels=TRUE, legend=TRUE, at=NULL, format=NULL)
backtestRebalancePlot(object, labels=TRUE, legend=TRUE, at=NULL, format=NULL)
backtestPortfolioPlot(object, labels=TRUE, legend=TRUE, at=NULL, format=NULL)
backtestDrawdownPlot(object, labels=TRUE, legend=TRUE, at=NULL, format=NULL)
backtestReportPlot(object, cex=0.6, font=1, family="mono")
}


\arguments{

    \item{object}{
        a list, returned from running the function 
        \code{portfolioSmoothing}.
        }
    \item{which}{
        an integer or string value. If the argument is an integer then it
        specifies which backtest plot should be displayed. If the argument
        take the character value \code{all}, which is the default, then all 
        6 available backtest plots will be displayed.
        }
    \item{labels}{
        a logical flag, determining if the graph should be labeled
        automatically. This is the default case \code{labels=TRUE}. If 
        set to \code{FALSE} then the graph will be displayed
        undecorated and the user can it decorate by himself.
        }
    \item{legend}{
        a logical flag, determining if to the graph a legend should
        be added. This is the default case \code{labels=TRUE}. If 
        set to \code{FALSE} then the graph will be displayed
        undecorated and the user can it decorate by himself.
        }
    \item{at}{
        if NULL the time-axis ticks will be selected automatically. If
        \code{at} is a vector of \code{timeData} character formatted 
        dates then the axis ticks ar taken from this vector. 
        }
    \item{format}{
        if NULL the time-axis ticks are labeled automatically. If
        \code{format} is a POSIX format string, tthen the label
        formats are taken from this string.
        }
    \item{cex, font, family}{
        font size, font and font family specification for the report.
        }
}


\details{

    These backtest plot summarises the results obtained from portfolio 
    backtesting.
    
    The function \code{backtestAssetsPlot} displays the set of possible
    assets to construct a portfolio. 
    
    The function \code{backtestWeightsPlot} displays the recommended 
    weights for investment. 
    
    The function \code{backtestRebalancePlot} displays the weight 
    changes over time for 
    individual assets and for the portfolio.

    The function \code{backtestPortfolioPlot} displays the daily, 
    benchmark and portfolio series
    of a portfolio backtest.

    The function \code{backtestDrawdownPlot} displays the daily 
    drawdowns for the
    benchmark and the portfolio.

    The function \code{backtestReportPlot} summarises the results 
    from a portfolio backtest.

}


\references{

W\"urtz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{methods-plot}


\alias{plot-methods}
\alias{.fportfolio.plot.1}
\alias{.fportfolio.plot.2}
\alias{.fportfolio.plot.3}
\alias{.fportfolio.plot.4}
\alias{.fportfolio.plot.5}
\alias{.fportfolio.plot.6}
\alias{.fportfolio.plot.7}
\alias{.fportfolio.plot.8}

\title{plot-methods}


\description{

    Various plot-methods. In particalur, functions
    \code{.fportfolio.plot.[i]()} will:
    \enumerate{
      \item plot the efficient frontier,
      \item add minimum risk portfolio,
      \item add tangency portfolio,
      \item add risk/return of single assets,
      \item add equal weights portfolio,
      \item add two asset frontiers [0-1 PF only],
      \item add Monte Carlo portfolios, and/or
      \item add Sharpe ratio [MV PF only].
    }
}

\usage{
.fportfolio.plot.1(x)
.fportfolio.plot.2(x)
.fportfolio.plot.3(x)
.fportfolio.plot.4(x)
.fportfolio.plot.5(x)
.fportfolio.plot.6(x)
.fportfolio.plot.7(x)
.fportfolio.plot.8(x)
}



\arguments{

    \item{x}{an object of class \code{fPORTFOLIO}}

}

\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{portfolio-pfolioRisk}


\alias{pfolioRisk}

\alias{covRisk}
\alias{varRisk}
\alias{cvarRisk}


\title{portfolioRisk}


\description{

    Computes covariance and CVaR portfolio risk.

}


\usage{
covRisk(data, weights)
varRisk(data, weights, alpha = 0.05)
cvarRisk(data, weights, alpha = 0.05)
}


\arguments{

    \item{data}{
        a multivariate time series described by an S4 object of class
        \code{timeSeries}.
        }
    \item{weights}{ 
        a numeric vector of weights. 
        }
    \item{alpha}{
        a numeric value, the confidence level, by default \code{alpha=0.05},
        i.e. 5\%.
        }
}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{portfolio-getDefault}

\alias{getDefault}


\alias{getConstraints}
\alias{getControl}
\alias{getCov}
\alias{getCovRiskBudgets}
\alias{getData}
\alias{getEstimator}
\alias{getMean}
\alias{getMu}
\alias{getUnits}
\alias{getModel}
\alias{getNAssets}
\alias{getNFrontierPoints}
\alias{getObjective}
\alias{getOptim}
\alias{getOptions}
\alias{getOptimize}
\alias{getPortfolio}
\alias{getParams} 
\alias{getRiskFreeRate} 
\alias{getSeries}
\alias{getSigma}
\alias{getSolver}
\alias{getSpec}
\alias{getStatistics}
\alias{getStatus}
\alias{getAlpha}
\alias{getTailRisk}
\alias{getTailRiskBudgets} 
\alias{getTargetReturn}
\alias{getTargetRisk}
\alias{getTrace}
\alias{getType}
\alias{getWeights}


\title{Extractor Functions}


\description{
  
    Extractor functions to get information from objects
    of class fPFOLIODATA, fPFOLIOSPEC, fPFOLIODATA, fPFOLIOVAL,
    and fPORTFOLIO.
    
}
    

\usage{
getConstraints(object)
getControl(object)
getCov(object)
getCovRiskBudgets(object)
getData(object)
getEstimator(object)
getMean(object)
getMu(object)
getNAssets(object)
getNFrontierPoints(object)
getObjective(object)
getOptim(object)
getOptions(object)
getOptimize(object)
getPortfolio(object)
getParams(object)
getRiskFreeRate(object)
getSeries(object)
getSigma(object)
getSolver(object)
getSpec(object)
getStatistics(object)
getStatus(object)
getAlpha(object)
getTailRisk(object)
getTailRiskBudgets(object) 
getTargetReturn(object)
getTargetRisk(object)
getTrace(object)
getType(object)
getWeights(object)
}



\arguments{

    \item{object}{
        an object of class \code{fPFOLIODATA}, \code{fPFOLIOSPEC} or
        \code{fPORTFOLIO}.
        }

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{mathprog-QP}


\alias{rsolveQP}
 
\alias{rquadprogQP} 
\alias{quadprogQP}
\alias{quadprogQPControl}
\alias{rquadprog}

\alias{ripopQP} 
\alias{ipopQP}
\alias{ipopQPControl}

\alias{ramplQP} 
\alias{amplQP}
\alias{amplQPControl}

\alias{rkestrelQP} 
\alias{kestrelQP} 

\alias{rneosQP}
\alias{neosQP}



\alias{kestrelQPControl}
\alias{neosQPControl}


\title{Mathematical Linear Programming}


\description{

    Mathematical Quadratic Programming.

}


\usage{
rsolveQP(objective, lower=0, upper=1, linCons, 
    control=list(solver="quadprog", invoke=c("R", "AMPL", "NEOS")))

rquadprogQP(objective, lower=0, upper=1, linCons, control=list())
quadprogQP(objective=list(dvec=NULL, Dmat=NULL), 
    par.lower=NULL, par.upper=NULL, 
    eqA=NULL, eqA.bound=NULL,  
    ineqA=NULL, ineqA.lower=NULL, ineqA.upper=NULL,
    control=list())
quadprogQPControl(solver="quadprog", trace=FALSE)
rquadprog

ripopQP(objective, lower=0, upper=1, linCons, control=list())
ipopQP(objective=list(dvec=NULL, Dmat = NULL), 
    par.lower=NULL, par.upper=NULL, 
    eqA=NULL, eqA.bound=NULL,
    ineqA=NULL, ineqA.lower=NULL, ineqA.upper=NULL,
    control=list())
ipopQPControl(
    sigf=12, maxiter=400, margin=0.05, bound=10, verb=0, 
    inf=1e12, solver="ipop", trace=FALSE) 
ripop
       
ramplQP(objective, lower=0, upper=1, linCons, control=list())
amplQP(objective=list(dvec=NULL, Dmat=NULL), 
    x_L=NULL, x_U=NULL, A=NULL, b_L=NULL, b_U=NULL,
    control=list(), \dots)
amplQPControl(solver="ipopt", project="ampl", 
    inf=1e12, trace = FALSE)
       
rkestrelQP(objective, lower=0, upper=1, linCons, control=list())
kestrelQP(objective=list(dvec=NULL, Dmat=NULL), 
    x_L=NULL, x_U=NULL, A=NULL, b_L=NULL, b_U=NULL,
    control=list(), \dots)
kestrelQPControl(solver="loqo", project="kestrel", 
    inf=1e12, trace = FALSE)
        
rneosQP(objective, lower=0, upper=1, linCons, control=list())
neosQP(objective=list(dvec=NULL, Dmat=NULL), 
    x_L=NULL, x_U=NULL, A=NULL, b_L=NULL, b_U=NULL,
    control=list(), \dots)    
neosQPControl(solver="ipopt", category="nco", project="neos", 
    inf=1e12, trace=FALSE) 
}



\arguments{

    \item{objective}{
        ...
        }
    \item{lower, upper}{
        lower and upper bounds.
        }
    \item{linCons}{
        list of linear constraints: mat, lower, upper.
        }
    \item{control}{
        control list.
        }
    \item{\dots}{
        optional arguments to be passed.
        }
        
    \item{par.lower, par.upper}{...}
    \item{eqA}{...}
    \item{eqA.bound}{...}
    \item{ineqA}{...}
    \item{ineqA.lower,ineqA.upper}{...}
    
    \item{x_L,x_U}{...}
    \item{A}{...}
    \item{b_L,b_U}{...}
    
    \item{solver}{...}
    \item{category}{...}
    \item{project}{...}
    \item{inf}{...}
    \item{trace}{...}
    
    \item{sigf}{...}
    \item{maxiter}{...}
    \item{margin}{...}
    \item{bound}{...}
    \item{verb}{...}
    
}


\value{
    
    a list of class \code{solver} with the following named ebtries: 
    \code{opt}, 
    \code{solution},
    \code{objective},
    \code{status},
    \code{message}, 
    \code{solver},
    \code{version}.

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{frontier-plotControl}


\alias{frontierPlotControl}


\title{Frontier Plot Control List}


\description{

    Allows to modify plot settings for the frontier plot.

}


\usage{
frontierPlotControl(   
    # Colors:
        sharpeRatio.col   = "blue",
        minvariance.col   = "red",
        tangency.col      = "steelblue",
        cml.col           = "green",
        equalWeights.col  = "blue",
        singleAsset.col   = "topo.colors",
        twoAssets.col     = "grey",
        monteCarlo.col    = "black",
    # Point Sizes:
        minvariance.cex   = 1.25,
        tangency.cex      = 1.25,
        cml.cex           = 1.25,
        equalWeights.cex  = 1.25,
        singleAsset.cex   = 1.25,
        twoAssets.cex     = 0.01,
        monteCarlo.cex    = 0.01,
        sharpeRatio.cex   = 0.1, 
    # Limits:
        xlim              = NULL,
        ylim              = NULL,
    # MC Steps:
        mcSteps           = 5000,
    # Pie Settings:
        pieR              = NULL, 
        piePos            = NULL, 
        pieOffset         = NULL)
}


\arguments{

    \item{sharpeRatio.col}{Color setting.}
    \item{minvariance.col}{Color setting.}
    \item{tangency.col}{Color setting.}
    \item{cml.col}{Color setting.}
    \item{equalWeights.col}{Color setting.}
    \item{singleAsset.col}{Color setting.}
    \item{twoAssets.col}{Color setting.}
    \item{monteCarlo.col}{Color setting.}
    
    \item{minvariance.cex}{Font point size setting.}
    \item{tangency.cex}{Font point size setting.}
    \item{cml.cex}{Font point size setting.}
    \item{equalWeights.cex}{Font point size setting.}
    \item{singleAsset.cex}{Font point size setting.}
    \item{twoAssets.cex}{Font point size setting.}
    \item{monteCarlo.cex}{Font point size setting.}
    \item{sharpeRatio.cex}{Font point size setting.} 

    \item{xlim}{x-axis limit setting.}
    \item{ylim}{y-axis limit setting.}
    
    \item{mcSteps}{Numer of Monte Carlo steps.}

    \item{pieR}{Pie radius setting.}
    \item{piePos}{Pie position coordinates setting.}
    \item{pieOffset}{Pie offset coordinates setting.}
    
}


%\details{}



\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{risk-surfaceRisk}


\alias{markowitzHull}
\alias{feasibleGrid}


\title{Surface Risk Analytics}


\description{
  
    Functions for surface risk analytics.
    
}


\usage{
markowitzHull(data, nFrontierPoints=50)
feasibleGrid(hull, trace=FALSE)
}


\arguments{

    \item{data}{data}
    \item{hull}{hull}
    \item{nFrontierPoints}{nFrontierPoints}
    \item{trace}{trace}
}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{portfolio-riskPfolio}


\alias{riskPfolio}

\alias{pfolioVaR}
\alias{pfolioCVaR}
\alias{pfolioCVaRplus}
\alias{lambdaCVaR}
\alias{pfolioCVaRoptim}

\alias{pfolioMaxLoss}
\alias{pfolioReturn}
\alias{pfolioTargetReturn}
\alias{pfolioTargetRisk}
\alias{pfolioSigma}
\alias{pfolioHist}


\title{Risk and Related Measures for Portfolios}


\description{
  
    Computes Value-at-Risk and related measures 
    for a portfolio of assets. 

    The functions are:

    \tabular{ll}{
    \code{pfolioVaR} \tab computes Value-at-Risk for a portfolio of assets, \cr
    \code{pfolioCVaRplus} \tab computes Value-at-Risk+ for a portfolio of assets, \cr
    \code{pfolioCVaR} \tab computes Conditional Value-at-Risk for a PF of assets, \cr
    \code{lambdaCVaR} \tab computes CVaR's atomic split value lambda, \cr
    \code{pfolioCVaRoptim} \tab computes Conditional VaR from mean-CVaR optimization, \cr
    \code{pfolioMaxLoss} \tab computes Maximum Loss for a portfolio of assets, \cr
    \code{pfolioReturn} \tab computes return values of a portfolio, \cr
    \code{pfolioTargetReturn} \tab computes the target return of a portfolio, \cr
    \code{pfolioTargetRisk} \tab computes the target risk of a portfolio, \cr
    \code{pfolioHist} \tab plots a histogram of the returns of a portfolio. }
    
}
    

\usage{
pfolioVaR(x, weights = NULL, alpha = 0.05) 
pfolioCVaRplus(x, weights = NULL, alpha = 0.05) 
pfolioCVaR(x, weights = NULL, alpha = 0.05) 
lambdaCVaR(n, alpha = 0.05) 
pfolioCVaRoptim(x, weights = NULL, alpha = 0.05) 

pfolioMaxLoss(x, weights = NULL)
pfolioReturn(x, weights = NULL, geometric = FALSE) 
pfolioTargetReturn(x, weights = NULL)
pfolioTargetRisk(x, weights = NULL)
pfolioHist(x, weights = NULL, alpha = 0.05, range = NULL, details = TRUE, \dots)
}


\arguments{
  
    \item{x}{
        a 'timeSeries' object, data frame or any other rectangular
        object which can be expressed as a matrix. The first 
        dimension is the number of observations, we call it \code{n},
        and the second is the number of assets in the data set,
        we call it \code{dim}.
        }
    \item{weights}{
        usually a numeric vector which has the length of the number of 
        assets. The weights measures the normalized weights of the 
        individual assets. By default \code{NULL}, then an equally 
        weighted set of assets is assumed. 
        }
    \item{geometric}{
        a logical flag, should geometric returns be used, by default FALSE
        }
    \item{alpha}{
        a numeric value, the confidence interval, by default 0.05.
        }
    \item{details}{
        a logical value, should details be printed?
        }
    \item{n}{
        the number of observation from which the CVaR's atomic split 
        value \code{lambda=1-floor(alpha*n)/(alpha*n)} will be 
        evaluated.
        }
    \item{range}{
        a numeric vector of two elements limiting the plot range of
        the histogram. This is quite useful if one likes to compare
        several plots on the same scale. If \code{range=NULL}, the
        default value, then the range will be selected automatically.
        }
    \item{\dots}{
        optional arguments to be passet to the function \code{hist}.
        }
            
}


\details{

    The percentile measures of loss (or reward) are defined in the 
    following way: Let \eqn{f(x ,y)} be a loss functions depending 
    upon a decision vector \eqn{x = (x_1, ..., x_n )} and a random 
    vector \eqn{y = (y_1, ..., y_m)}, then 
    
    \emph{pfolioVaR} is the alpha-percentile of the loss distribution, a 
    smallest value such that the probability that losses exceed or 
    are equal to this value is greater or equal to alpha.
    
    \emph{pfolioCVaRplus} or "CVaR+" or the "upper CVaR" are the expected  
    losses strictly exceeding VaR. This is also also called "Mean 
    Excess Loss" and "Expected Shortfall".

    \emph{pfolioCVaR} is a weighted average of VaR and CVaRplus defined as
    \eqn{CVaR = lambda*VaR + (1-lambda)} CVaRplus, for 
    \eqn{0 <= lambda <= 1}.
    
    Note, CVaR is convex, but VaR and CVaRplus may be non-convex. The 
    following inequalities are valid: \eqn{VaR <= CVaR <= CVaRplus}.

}


\value{

    \code{pfolioVaR}
    \cr
    returns the value of risk, VaR,  for a portfolio of assets, a
    numeric value.
    \cr
    
    \code{pfolioCVaRplus}
    \cr
    returns the conditional value of risk plus, CVaRplus, for a 
    portfolio of assets, a numeric value.
    \cr
    
    \code{pfolioCVaR}
    \cr
    returns the conditional value of risk, CVaR, for a portfolio
    of assets, a numeric value.
    \cr
    
    \code{lambdaCVaR}
    \cr
    returns CVaR's atomic split value \code{lambda}, a numeric value.
    \cr
    
    \code{pfolioMaxLoss}
    \cr
    returns the maximum loss value of the portfolio, a numeric value.
    \cr
    
    \code{pfolioReturn}
    \cr
    returns the total portfolio return computed from the set of
    assets \code{x}, a numeric vector.
    \cr
    
    \code{pfolioTargetReturn}
    \cr
    returns the total return or target return computed from the set of
    assets \code{x} and weights \code{weights}, a numeric value.
    \cr
    
    \code{pfolioTargetRisk}
    \cr
    returns the total risk (Sigma) or target risk computed from the set 
    of assets \code{x} and \code{weights} via the formual
    \code{sqrt(weights \%*\% cov(x) \%*\% weights)}, a numeric value.
    \cr
    
    \code{pfolioHist}
    \cr
    plots a histogram of portfolio returns and adds the values
    for the VaR (blue), for the CVaRplus (red), and for the 
    maximum loss (green) to the histogram plot. The function 
    invisibly returns a list with the following elements: VaR,
    VaRplus, maxLoss, mean, and sd. If \code{details} is \code{TRUE}, 
    then the result is printed.
          
}


\references{

Uryasev S. (2000);
    \emph{Conditional Value-at-Risk (CVaR): Algorithms and Applications},
    Risk Management and Financial Engineering Lab, University of Florida

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
      
}


\keyword{math}

\name{risk-budgeting}


%\alias{pfolioReturn}
\alias{sampleCOV}
\alias{normalVaR}
\alias{modifiedVaR}
\alias{sampleVaR}
\alias{budgetsSampleCOV}
\alias{budgetsNormalVAR}
\alias{budgetsModifiedVAR}
\alias{budgetsNormalES}
\alias{budgetsModifiedES}


\title{Risk Budgeting}


\description{
  
    Functions for risk budgeting.
    
}


\usage{
%pfolioReturn(x, weights, geometric=TRUE)
sampleCOV(x)
normalVaR(x, alpha=0.05)
modifiedVaR(x, alpha=0.05)
sampleVaR(x, alpha=0.05)

budgetsSampleCOV(x, weights, mu=NULL, Sigma=NULL)

budgetsNormalVAR(x, weights, alpha=0.05, mu=NULL, Sigma=NULL)
budgetsModifiedVAR(x, weights, alpha=0.05, mu=NULL, Sigma=NULL, 
  M3=NULL, M4=NULL) 
  
budgetsNormalES(x, weights, alpha=0.05, mu=NULL, Sigma=NULL)
budgetsModifiedES(x, weights, alpha=0.05, mu=NULL, Sigma=NULL, 
  M3=NULL, M4=NULL)
}


\arguments{

    \item{x}{x}
    \item{weights}{weights}
    \item{alpha}{alpha}
    \item{mu,Sigma}{mean and covariance}
    \item{M3, M4}{M3 and M4}

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{backtest-extractors}


\alias{getWindows.fPFOLIOBACKTEST}

\alias{getWindowsFun.fPFOLIOBACKTEST}

\alias{getWindowsParams.fPFOLIOBACKTEST}

\alias{getWindowsHorizon.fPFOLIOBACKTEST}

\alias{getStrategy.fPFOLIOBACKTEST}

\alias{getStrategyFun.fPFOLIOBACKTEST}

\alias{getStrategyParams.fPFOLIOBACKTEST}

\alias{getSmoother.fPFOLIOBACKTEST}

\alias{getSmootherFun.fPFOLIOBACKTEST}

\alias{getSmootherParams.fPFOLIOBACKTEST}                       

\alias{getSmootherLambda.fPFOLIOBACKTEST}

\alias{getSmootherDoubleSmoothing.fPFOLIOBACKTEST}

\alias{getSmootherInitialWeights.fPFOLIOBACKTEST}

\alias{getSmootherSkip.fPFOLIOBACKTEST}

\alias{getMessages.fPFOLIOBACKTEST}


\title{Portfolio backtest specification extractors}


\description{
  
    Extracts information from an object of class
    fPFOLIOBACKTEST. \cr
    
    The functions are:
    
    \tabular{ll}{
    \code{getWindows} \tab Extract windows slot, \cr
       
    \code{getWindowsFun} \tab extract windows function, \cr
    
    \code{getWindowsParams} \tab extract a list of windows specific parameters, \cr
    
    \code{getWindowsHorizon} \tab extract windows horizon, \cr
 
    \code{getStrategy} \tab extract strategy slot, \cr
     
    \code{getStrategyFun} \tab extract the portfolio strategy function, \cr
    
    \code{getStrategyParams} \tab extract a list of portfolio strategy specific parameters, \cr
    
    \code{getSmoother} \tab extract the smoother slot, \cr
     
    \code{getSmootherFun} \tab Extract the Ssoother function, \cr
  
    \code{getSmootherParams} \tab extract a list of Smoothing specific parameters, \cr
 
    \code{getSmootherLambda} \tab extract the smoothing parameter Lambda, \cr
      
    \code{getSmootherDoubleSmoothing} \tab extract setting for double smoothing, \cr
       
    \code{getSmootherInitialWeights} \tab extract the initial weights to used in the smoothing, \cr
    \code{getSmootherSkip} \tab extract the number of skipped months, \cr
      
    \code{getMessages} \tab extract the message slot.}  
}
    

\usage{
\method{getWindows}{fPFOLIOBACKTEST}(object)
\method{getWindowsFun}{fPFOLIOBACKTEST}(object)
\method{getWindowsParams}{fPFOLIOBACKTEST}(object)
\method{getWindowsHorizon}{fPFOLIOBACKTEST}(object)

\method{getStrategy}{fPFOLIOBACKTEST}(object)
\method{getStrategyFun}{fPFOLIOBACKTEST}(object)
\method{getStrategyParams}{fPFOLIOBACKTEST}(object)

\method{getSmoother}{fPFOLIOBACKTEST}(object)
\method{getSmootherFun}{fPFOLIOBACKTEST}(object)
\method{getSmootherParams}{fPFOLIOBACKTEST}(object)
\method{getSmootherLambda}{fPFOLIOBACKTEST}(object)
\method{getSmootherDoubleSmoothing}{fPFOLIOBACKTEST}(object)
\method{getSmootherInitialWeights}{fPFOLIOBACKTEST}(object)
\method{getSmootherSkip}{fPFOLIOBACKTEST}(object)

\method{getMessages}{fPFOLIOBACKTEST}(object)
}


\arguments{

    \item{object}{
        an object of class \code{fPFOLIOBACKTEST} as returned by 
        function \code{portfolioBacktest}.
        }

}


\references{

W\"urtz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{portfolio-setSpec}


\alias{setSpec}

\alias{setType<-}
\alias{setOptimize<-}
\alias{setEstimator<-}
\alias{setTailRisk<-}
\alias{setParams<-}
\alias{setAlpha<-}

\alias{setWeights<-}
\alias{setTargetReturn<-}
\alias{setTargetRisk<-}
\alias{setRiskFreeRate<-}
\alias{setNFrontierPoints<-}
\alias{setStatus<-}

\alias{setSolver<-}
\alias{setObjective<-}
\alias{setTrace<-}

%\alias{maxReturn}
%\alias{minRisk}


\title{Settings for Specifications of Portfolios}


\description{
  
    Functions to set specifications for a portfolio.
    
}


\usage{
setType(spec) <- value
setOptimize(spec) <- value
setEstimator(spec) <- value
setTailRisk(spec) <- value
setParams(spec, name) <- value
setAlpha(spec) <- value

setWeights(spec) <- value
setTargetReturn(spec) <- value
setTargetRisk(spec) <- value
setRiskFreeRate(spec) <- value
setNFrontierPoints(spec) <- value
setStatus(spec) <- value

setSolver(spec) <- value
setObjective(spec) <- value
setTrace(spec) <- value

%maxReturn(x, mu)
%minRisk(x, Sigma)
}


\arguments{
        
    \item{spec}{
        an S4 object of class \code{fPFOLIOSPEC}, the specification to be 
        modified, by default the default of the function \code{portfolioSpec()}.
        }
    \item{name}{
        a character string, the name of the value to be set.
        }
    \item{value}{
        a value for that component of \code{spec} to be set.
        }

}       


\details{
    
    \tabular{ll}{
    \code{setType} \tab Sets type of portfolio optimization, \cr
    \code{setOptimize} \tab Sets what to optimize, min risk or max return, \cr
    \code{setEstimator} \tab Sets names of mean and covariance estimators, \cr
    \code{setParams} \tab Sets optional model parameters, \cr
    
    \code{setWeights} \tab Sets weights vector, \cr
    \code{setTargetReturn} \tab Sets target return value, \cr
    \code{setTargetRisk} \tab Sets target risk value, \cr
    \code{setTargetAlpha} \tab Sets CVaR target alpha value, \cr
    \code{setRiskFreeRate} \tab Sets risk-free rate value, \cr
    \code{setNFrontierPoints} \tab Sets number of frontier points, \cr
    \code{setStatus} \tab Sets status value, \cr
    
    \code{setSolver} \tab Sets the type of solver to be used, \cr
    \code{setObjective} \tab Sets objective function name to be used, \cr
    \code{setTrace} \tab Sets the logical trace flag. }
    
    %\code{maxReturn} \tab Defines the default maximum return function, \cr
    %\code{minRisk} \tab Defines the default minimum risk function. }
 
}


\value{
  
    \code{setType}\cr
    \code{setOptimize}\cr
    \code{setEstimator}\cr 
    \code{setParam}\cr
    \cr
    \emph{Model Settings:} just modify the model settings including the 
    portfolio type, the mean/covariance estimator, and optional parameters 
    of an existing portfolio structure.
    \cr
    
    \code{setWeights}\cr
    \code{setTargetReturn}\cr
    \code{setTargetRisk}\cr
    \code{setTargetAlpha}\cr
    \code{setRiskFreeRate}\cr
    \code{setNFrontierPoints}\cr
    \code{setStatus}\cr
    \cr
    \emph{Portfolio Settings:} just modify the portfolio settings including 
    predefined weights, the target return, the risk free rate, the number of 
    frontier points, and the return and risk range of an existing portfolio 
    structure.
    \cr
    
    \code{setSolver}\cr
    \code{setObjective}\cr
    \code{setTrace}\cr
    \cr
    \emph{Optim Settings:} just modifies the solver setting, i.e. the type 
    of solver to be used for portfolio optimization.

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{fPortfolio-package}


\alias{fPortfolio}


\docType{package}


\title{Portfolio Design, Optimization and Backtesting}


\description{

    The Rmetrics "fPortfolio" package is a very powerful collection
    of functions to optimize portfolios and to analyze them from
    different points of view.

}


\details{

    \tabular{ll}{
        Package:    \tab fPortfolio\cr
        Type:       \tab Package\cr
        Date:       \tab 2011\cr
        License:    \tab GPL Version 2 or later\cr
        Copyright:  \tab (c) 1999-2011 Diethelm Wuertz and Rmetrics Association\cr
        URL:        \tab \url{https://www.rmetrics.org}
    }
    
}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{fPFOLIOCON}


\alias{fPFOLIOCON}
\alias{class-fPFOLIOCON}

\alias{fPFOLIOCON-class}

\alias{show,fPFOLIOCON-method}


\title{Portfolio Constraints Handling}


\description{
  
    Creates a fPFOLIOCON object from string constraints.
      
}


\usage{
\S4method{show}{fPFOLIOCON}(object)
}


\arguments{

    \item{object}{
        an object of class \code{fPFOLIOCON} as returned by the function
        \code{portfolioData}.
        }

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{backtest-specification}


\alias{portfolioBacktest}


\title{Specification of portfolio backtesting}


\description{
  
    Specifies how the portfolio backtesting is performed.
    
}


\usage{
portfolioBacktest(
    windows = list(
        windows = "equidistWindows", 
        params = list(horizon = "12m")),
    strategy = list(
        strategy = "tangencyStrategy", 
        params = list()),
    smoother = list(
        smoother = "emaSmoother", 
        params = list(doubleSmoothing = TRUE, 
        lambda = "3m", skip = 0, 
        initialWeights = NULL)),
    messages = list())
}


\arguments{

    \item{windows}{
        a list, containing different arguments: windows, params 
        (horizon).
        }
    \item{strategy}{
        a list, containing different arguments: strategy, params.
        }
    \item{smoother}{
        a list, containing different arguments: smoother, params. 
        (doubleSmoothing, lambda, skip, initialWeights).
        }
     \item{messages}{
	    a list containing the backtesting messages.
	    }

}       


\value{
  
    returns an S4 object of class \code{"fPFOLIOBACKTEST"}.

}


\references{

W\"urtz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{fPFOLIOSPEC}


\alias{fPFOLIOSPEC}
\alias{class-fPFOLIOSPEC}

\alias{fPFOLIOSPEC-class}

\alias{show,fPFOLIOSPEC-method}


\title{Specification of Portfolios}


\description{
  
    Specifies portfolios.
    
}


\usage{       
\S4method{show}{fPFOLIOSPEC}(object)
}


\arguments{

    \item{object}{
        an S4 object of class \code{fPFOLIOSPEC}.
        }

}       


\details{

    \bold{Portfolio Specifcation Structure:}
    \cr\cr
    
    The S4 class \code{fPFOLIOSPEC} specifies the portfolio. The slots 
    are:\cr
    
    \describe{
        \item{@call}{
            a call, returning the matched function call.
            }
        \item{@model}{
            a list, setting the \code{type} of portfolio to be optimized, 
            and the mean/covariance \code{estimator} to be applied:
            \cr
            \code{type=c("MV","CVaR")} a character string denoting the type of
                portfolio, the implemented types are the
                Mean-Variance Markowitz Portfolio, \code{"MV"}, and the
                Mean-CVaR Portfolio, \code{"CVaR"}.
                \cr
            \code{estimator=c("mean","cov")} a vector of two character
                strings, the first denoting the mean estimator, and the 
                second the covariance estimator. Additional meaningful 
                selections include robust covariance estimators, e.g. 
                \code{c("mean","mcd")}, or \code{c("mean","shrink")}.
                \cr
            \code{tailRisk=list()} a list of optional tail risk information,
                currently not used.\cr
            \code{params=list()} a list of optional model parameters,
                currently not used.
            }     
        \item{@portfolio}{
            a list, settings portfolio parameters including predefined
            weights, target return, risk free rate, number of frontier
            points:
            \cr 
            \code{weights=NULL} a numeric vector specifying the portfolio 
                weights.
                \cr
            \code{targetReturn=NULL} a numeric value specifying the target 
                return. The default value sets the target return.
                \cr
            \code{targetRisk=NULL} a numeric value specifying the target 
                risk.\cr
            \code{targetAlpha=NULL} a numeric value specifying the target 
                alpha confidence level for CVaR portfolio optimization. 
                The default value sets the target return.
                \cr
            \code{riskFreeRate=0} a numeric value specifying the risk free 
                rate.
                \cr  
            \code{nFrontierPoints=50} a numeric value determining the number
                of points on the efficient frontier.
            }
        \item{@solver}{
            a list, setting the type of solver to be used for portfolio
            optimization:
            \cr 
            \code{type=c("quadprog", "lpSolve")} a character %% REMOVED: "Rdonlp2"
                string specifying the name of the solver to be used.\cr
            \code{trace=FALSE} a logical flag, should the optimization be
                traced?
            }
        \item{@title}{
            a title string, with a default project title. 
            }
        \item{@description}{
            a character string, with a default project description.
            }
    }
 
}


\value{
  
    \code{portfolioSpec}\cr
    \cr
    returns an S4 object of class \code{"fPFOLIOSPEC"}.

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{solver-family}


\alias{solveRglpk.CVAR}
\alias{solveRglpk.MAD}
\alias{solveRampl.CVAR}
\alias{solveRshortExact}
\alias{solveRquadprog}
\alias{solveRquadprog.CLA}
\alias{solveRipop}
\alias{solveRampl.MV}
\alias{solveRsocp}
%%\alias{solveRdonlp2}
\alias{solveRsolnp}


\title{LP, QP, and NLP Programming Solvers}


\description{

    Rmetrics solver interface.

}


\usage{
solveRglpk.CVAR(data, spec, constraints)
solveRglpk.MAD(data, spec, constraints)
solveRampl.CVAR(data, spec, constraints)

solveRshortExact(data, spec, constraints)
solveRquadprog(data, spec, constraints)
solveRquadprog.CLA(data, spec, constraints)
solveRipop(data, spec, constraints)
solveRampl.MV(data, spec, constraints)

solveRsocp(data, spec, constraints)

%%solveRdonlp2(data, spec, constraints)
solveRsolnp(data, spec, constraints)
}


\arguments{

    \item{data}{
        a time series or a named list, containing either a series of returns 
        or named entries 'mu' and 'Sigma' being mean and covariance matrix.
        }
    \item{spec}{
        an S4 object of class \code{fPFOLIOSPEC} as returned by the function
        \code{portfolioSpec}.
        }
    \item{constraints}{
        a character string vector, containing the constraints of the form\cr
        \code{"minW[asset]=percentage"} for box constraints resp. \cr
        \code{"maxsumW[assets]=percentage"} for sector constraints.
        }
}


\value{
    
    a list with the following named ebtries: 
    \code{solver}, 
    \code{optim},
    \code{weights},
    \code{targetReturn},
    \code{targetRisk}, 
    \code{objective},
    \code{status},
    \code{message}.

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{portfolio-covEstimator}


\alias{covEstimator}
\alias{mveEstimator}
\alias{mcdEstimator}

\alias{lpmEstimator}
\alias{slpmEstimator}

\alias{kendallEstimator}
\alias{spearmanEstimator}

\alias{covMcdEstimator}
\alias{covOGKEstimator}
\alias{shrinkEstimator}
\alias{nnveEstimator}


\title{Covariance Estimators}


\description{
  
    Functions to estimate and robustify the sample 
    mean and covariance of rectangular objects.
    
}
    

\usage{
covEstimator(x, spec = NULL, \dots)
mveEstimator(x, spec = NULL, \dots)
mcdEstimator(x, spec = NULL, \dots)

lpmEstimator(x, spec = NULL, \dots)
slpmEstimator(x, spec = NULL, \dots)

kendallEstimator(x, spec = NULL, \dots)
spearmanEstimator(x, spec = NULL, \dots)

covMcdEstimator(x, spec = NULL, \dots)
covOGKEstimator(x, spec = NULL, \dots)
shrinkEstimator(x, spec = NULL, \dots)
nnveEstimator(x, spec = NULL, \dots)

}

\arguments{
  
    \item{x}{
        an object of class \code{timeSeries}. 
        }   
    \item{spec}{
        unused, may be used to pass information from the portfolio
        specification object to the mean and covariance estimator
        function.
        }
    \item{\dots}{
        optional arguments to be passed to the underlying estimators. 
        }
        
}


\value{
  
    the functions return a list with two entries named \code{mu} and 
    \code{Sigma}.
    The first denotes the vector of column means, and the second the 
    covariance matrix. Note, that the output of this function can be
    used as data input for the portfolio functions to compute the
    efficient frontier.
  
}


\details{

    The functions are underlying the following algorithms:\cr
    
    \code{covEstimator} uses standard covariance estimation,\cr
    \code{mveEstimator} uses the function "cov.mve" from the MASS package,\cr
    \code{mcdEstimator} uses the function "cov.mcd" from the MASS package,\cr
    \code{lpmEstimator} returns lower partial moment estimator,\cr
    \code{kendallEstimator} returns Kendall's rank estimator,\cr
    \code{spearmanEstimator} returns Spearman's rankestimator,\cr
    \code{covMcdEstimator} requires "covMcd" from package robustbase,\cr
    \code{covOGKEstimator} requires "covOGK" from package robustbase,\cr
    \code{nnveEstimator} uses builtin from package covRobust,\cr
    \code{shrinkEstimator} uses builtin from package corpcor.

}
    

\references{
    
Breiman L. (1996); 
    \emph{Bagging Predictors},
    Machine Learning 24, 123--140.

Ledoit O., Wolf. M. (2003);
    \emph{ImprovedEestimation of the Covariance Matrix of Stock Returns 
    with an Application to Portfolio Selection},
    Journal of Empirical Finance 10, 503--621. 

Schaefer J., Strimmer K. (2005);  
    \emph{A Shrinkage Approach to
    Large-Scale Covariance Estimation and Implications for Functional Genomics},
    Statist. Appl. Genet. Mol. Biol. 4, 32.

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\author{

    ... for R's \code{MASS} package, \cr
    ... for R's \code{robustbase} package, \cr
    ... for R's \code{covRobust} package, \cr
    Juliane Schaefer and Korbinian Strimmer for R's \code{corpcor} package, \cr
    Diethelm Wuertz for this Rmetrics port.
    
}


\keyword{models}

\name{methods-summary}


\alias{summary-methods}


\title{summary-methods}


\description{

    summary-methods.

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{backtestStats}

\alias{backtestStats}

\alias{rollingSigma}
\alias{rollingVaR}
\alias{rollingCVaR}
\alias{rollingDaR}
\alias{rollingCDaR}
\alias{rollingRiskBudgets}


\title{Rolling portfolio backtesting statistics}


\description{

    Computes rolling statistics for backtest analysis

}


\usage{   
backtestStats(object, FUN = "rollingSigma", \dots)

rollingSigma(object)
rollingVaR(object)
rollingCVaR(object)
rollingDaR(object)
rollingCDaR(object)
}


\arguments{

    \item{object}{
        a list, returned from running the function 
        \code{portfolioSmoothing}.
        }
   \item{FUN}{
        a character string, specifying the name of the rolling statistics 
        function.
        }
   \item{\dots}{
        optional argument to be passed to the rolling statistics 
        function \code{FUN}.
        }
}


\details{
    
    The function \code{rollingSigma} calculates the portfolio risk, 
    Sigma, over time.
    
    The function \code{rollingVaR} calculates a rolling Value at Risk.
    
    The function \code{rollingCVaR} calculates a rolling Conditional 
    Value at Risk.

    The function \code{rollingDaR} calculates a rolling Drawdowns at 
    Risk.

    The function \code{rollingCDaR} calculates a rolling Conditional 
    Drawdowns at Risk.

    %The function \code{rollingRiskBudgets} calculates a rolling Risk 
    %Budget of the portfolio.

}


\references{

W\"urtz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{frontier-plot}


\alias{frontierPlot}

\alias{minvariancePoints}
\alias{cmlPoints}
\alias{cmlLines}
\alias{tangencyPoints}
\alias{tangencyLines}
\alias{equalWeightsPoints}
\alias{singleAssetPoints}
\alias{twoAssetsLines}
\alias{sharpeRatioLines}

\alias{monteCarloPoints}

\alias{tailoredFrontierPlot}


\title{Efficient Frontier Plot}


\description{

    Plots the efficient frontier of an 
    optimized portfolio and allows to add
    points and lines from specif portfolios

}


\usage{
frontierPlot(object, frontier = c("both", "lower", "upper"),
    col = c("black", "grey"), add = FALSE, labels = TRUE,
    return = c("mean", "mu"), risk = c("Cov", "Sigma", "CVaR", "VaR"),
    auto = TRUE, title = TRUE, \dots) 

minvariancePoints(object, return = c("mean", "mu"), 
    risk = c("Cov", "Sigma", "CVaR", "VaR"), auto = TRUE, \dots)
cmlPoints(object, return = c("mean", "mu"), 
    risk = c("Cov", "Sigma", "CVaR", "VaR"), auto = TRUE, \dots)
cmlLines(object, return = c("mean", "mu"), 
    risk = c("Cov", "Sigma", "CVaR", "VaR"), auto = TRUE, \dots)
tangencyPoints(object, return = c("mean", "mu"), 
    risk = c("Cov", "Sigma", "CVaR", "VaR"), auto = TRUE, \dots)
tangencyLines(object, return = c("mean", "mu"), 
    risk = c("Cov", "Sigma", "CVaR", "VaR"), auto = TRUE, \dots)
equalWeightsPoints(object, return = c("mean", "mu"), 
    risk = c("Cov", "Sigma", "CVaR", "VaR"), auto = TRUE, \dots)
singleAssetPoints(object, return = c("mean", "mu"), 
    risk = c("Cov", "Sigma", "CVaR", "VaR"), auto = TRUE, \dots)
twoAssetsLines(object, return = c("mean", "mu"), 
    risk = c("Cov", "Sigma", "CVaR", "VaR"), auto = TRUE, \dots)
sharpeRatioLines(object, return = c("mean", "mu"), 
    risk = c("Cov", "Sigma", "CVaR", "VaR"), auto = TRUE, \dots)

monteCarloPoints(object, mcSteps = 5000, return = c("mean", "mu"), 
    risk = c("Cov", "Sigma", "CVaR", "VaR"), auto = TRUE, \dots)

tailoredFrontierPlot(object,
    return = c("mean", "mu"), risk = c("Cov", "Sigma", "CVaR", "VaR"),
    mText = NULL, col = NULL, xlim = NULL, ylim = NULL, 
    twoAssets = FALSE, sharpeRatio = TRUE, title = TRUE, \dots)      
}


\arguments{

    \item{object}{
        an S4 object of class \code{fPORTFOLIO}, containing slots call, data,
        specification, constraints, portfolio, title, description.
        }
    \item{frontier}{
        a character string, determining which part of the frontier should be
        extracted. \code{"both"} stands for the full hyperbola, \code{"lower"}
        for all points below the minimum variance return and \code{"upper"}
        for the actual efficient frontier, by default "both".
        }
    \item{col}{
        a character string vector, setting the color. For \code{frontierPlot} 
        it is a two dimensional a vector; first entry is the upper part of the
        frontier,\cr second entry the lower, by default "black" and "grey".\cr
        For the other functions the argument defines the color representation,
        by default sets the default color is the rainbow palette.
        }          
    \item{add}{
        a logical value, determining whether the frontier should be added to 
        an existing plot, by default FALSE.
        }
    \item{return}{
        a character string denoting which type of return should be
        plotted. Allowed values for the
        return are either \code{"mean"}, or \code{"mu"}.
        }
    \item{risk}{
        a character string denoting which type of risk should be
        plotted. Allowed values for the
        risk measure are either \code{"cov"}, \code{"sigma"},  
        \code{"VaR"}, or \code{"CVaR"}.
        }
    \item{auto}{
        a logical flag denoting if the type of return and risk to be
        plotted should be selected automatically, by default TRUE.
        }
    \item{labels}{
        a logical flag, should the plot be automatically labeled and
        decorated? By default \code{TRUE}.
        }
    \item{title}{
        a logical flag, should the plot obtain a default main title and
        x- and y-labels? By default \code{TRUE}.
        }
    \item{mcSteps}{
        an integer value, the number of Monte Carlo steps.
        }
    \item{xlim, ylim}{
        two numeric vectors with two elelemts , the plot range. 
        If set to NULL the values for the plot ranges are determined 
        automatically.
        }
    \item{mText}{
        a character string, representing a marginal text string. 
        If set to NULL the value is taken from the title of the
        input frontier argument.
        }
    \item{twoAssets}{
        a logical flag, if TRUE, then the two assets frontier lines
        will be drawn.
        }
    \item{sharpeRatio}{
        a logical flag, if TRUE, then the Sharpe ratio will be added to
        the plot.
        }
    \item{\dots}{
        optional arguments to be passed.
        }

}


\details{

    \tabular{ll}{
    \code{frontierPlot} \tab Plots efficient frontier, \cr
    \code{minvariancePoints} \tab Adds minimum variance point, \cr
    \code{cmlPoints} \tab Adds market portfolio, \cr
    \code{cmlLines} \tab  Adds capital market Line, \cr
    \code{tangencyPoints} \tab Adds tangency portfolio point, \cr
    \code{tangencyLines} \tab Adds tangency line, \cr
    \code{equalWeightsPoints} \tab Adds point of equal weights portfolio, \cr
    \code{singleAssetPoints} \tab Adds points of single asset portfolios, \cr
    \code{twoAssetsLines} \tab Adds EF for all combinations of two assets, \cr
    \code{sharpeRatioLines} \tab Adds Sharpe ratio line, \cr
    \code{monteCarloPoints} \tab Adds randomly produced feasible portfolios, \cr
    \code{tailoredFrontierPlot} \tab an example for a tailored plot. }

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{fPORTFOLIO}


\alias{fPORTFOLIO}
\alias{class-fPORTFOLIO}
\alias{fPORTFOLIO-class}
   
\alias{plot.fPORTFOLIO}
\alias{summary.fPORTFOLIO}


\title{Portfolio Class}


\description{
  
    A collection and description of functions 
    allowing to gain information about optimal 
    portfolios. Generally, optimization is done 
    via three arguments, data, specification of 
    the portfolio, and constraints, while function
    portfolioFrontier has two additional 
    arguments for title and description. 
    
}
    

\usage{
\method{plot}{fPORTFOLIO}(x, which = "ask", control = list(), \dots)

\method{summary}{fPORTFOLIO}(object, \dots)
}


\arguments{

    \item{control}{
        a list, defining the plotting parameters. The list modifies amongst
        others the color, e.g. \code{minvariance.col}, type of point, e.g.
        \code{tangency.pch}, or the dimension of the point, e.g. \code{cml.cex},
        see Notes for a complete list of control parameters.
        }
    \item{which}{
        which of the plots should be displayed? \code{which} can be either a 
        character string, \code{"all"} (displays all plots) or \code{"ask"} 
        (interactively asks which one to display), or a vector of integer
        values displaying the corresponding plot. Default value is 
        \code{"ask"}.
        }
    \item{object, x}{
        an S4 object of class \code{fPORTFOLIO}.
        }
    \item{\dots}{
        optional arguments to be passed.\cr
        }

}


\details{

    \bold{Portfolio Class:}
    \cr\cr
    
    This S4 class contains all information about the portfolio. Basically 
    these are risk measure, mean and covariance estimation, target return, 
    risk free rate, number of frontier points, ranges for calculation, see 
    the "Value" section  for a detailed description of the slots. 

}


\value{
    
    \code{portfolioFrontier()}\cr
    returns an S4 object of class \code{"fPORTFOLIO"}, with the following 
    slots:

    \item{@call}{
        a call, returning the matched function call.
        }
    \item{@data}{
        a list with two named elements, \code{series} holding the time series
        data if available, otherwise NA, and \code{statistics}, itself a
        named list with two named elements \code{mu} and \code{Sigma}
        holding the vector of means and the matrix of covariances.
        }  
    \item{@description}{
        a character string, allowing for a brief project description.
        }        
    \item{@portfolio}{
        a list, containing parameter specifications for the portfolio:\cr 
        \code{weights} a numeric vector specifying the portfolio 
            weights,\cr
        \code{targetReturn} a numeric value specifying the target 
            return,\cr
        \code{targetRisk} a numeric value specifying the target 
            risk,\cr  
        \code{targetMean} a numeric value specifying the target 
            return determinated with function mean(),\cr
        \code{targetStdev} a numeric value specifying the target risk in 
            standart deviation as risk measure.
        }
    \item{@specification}{
        a list with one named element \code{spec} which represents an object 
        of class \code{fPFOLIOSPEC}, including all information about
        the portfolio specifications, see \code{PortfolioSpec}
        for further details.
        }
    \item{@title}{
        a title string.
        }
    
    \code{feasiblePortfolio}\cr
    \code{cmlPortfolio}\cr
    \code{tangencyPortfolio}\cr
    \code{minvariancePortfolio}\cr
    \code{efficientPortfolio}\cr
    return an S4 object of class \code{fPORTFOLIO} having information only 
    about one portfolio.
    
}


\section{Control Parameters}{

    In the following all elements of argument control from functions
    \code{plot}, \code{weightsSlider}, \code{frontierSlider} are listed. 

    \describe{
    \item{sliderResolution}{
        [weightsSlider, frontierSlider] - 
        a numeric, determining the numbers of slider points, by default 
        nFrontierPoints/10.
        } 
    \item{sliderFlag}{
        [weightsSlider, frontierSlider] - 
        a character string, denoting the slidertype, by default "frontier"
        for \code{frontierSlider} and "weights" for \code{weightsSlider}.
        } 
    \item{sharpeRatio.col}{
        [plot, frontierSlider] - 
        a character string, defining color of the Sharpe
        ratio plot, by default "black".
        }
    \item{minvariance.col}{
        a character string, defining color of the minimum variance
        portfolio, by default "red".
        }
    \item{tangency.col}{
        a character string, defining color of the tangency
        portfolio, by default "steelblue".
        }
    \item{cml.col}{
        [plot, frontierSlider] - 
        a character string, defining  color of the market
        portfolio and the capital market line, by default "green".
        }
    \item{equalWeights.col}{
        [plot, frontierSlider] - 
        a character string, defining the color of the
        equal weights portfolio, by default "blue".
        }
    \item{runningPoint.col}{
        [weightsSlider] - 
        a character string, defining color of the
        point indicating the current portfolio, by default "red".
        }
    \item{singleAsset.col}{
        a character string vector, defining color of the
        single asset portfolios. The vector must have length the number 
        of assets, by default \code{rainbow}.
         }
    \item{twoAssets.col}{
        [plot, frontierSlider] - 
        a character string, defining color of the
         two assets efficient frontier, by default "grey".
         }
    \item{monteCarlo.col}{
        [plot, frontierSlider] - 
        a character string, defining color of the
        Monte Carlo portfolios, by default "black".
         }     
    \item{minvariance.pch}{
        a number, defining symbol used for the minimum
        variance portfolio. See \code{\link{points}} for description.
        Default symbol is 17.
        }
    \item{tangency.pch}{    
        a number, defining symbol used for the tangency portfolio.
        See \code{\link{points}} for description.
        Default symbol is 17.
        }
    \item{cml.pch}{
        [plot, frontierSlider] - 
        a number, defining symbol used for the market portfolio.
        See \code{\link{points}} for description.
        Default symbol is 17.
        }
    \item{equalWeights.pch}{
        [plot, frontierSlider] - 
        a number, defining symbol used for the equal weights portfolio.
        See \code{\link{points}} for description.
        Default symbol is 15.
        }        
    \item{singleAsset.pch}{
        a number, defining symbol used for the single asset portfolios.
        See \code{\link{points}} for description.
        Default symbol is 18.
        }           
    \item{sharpeRatio.cex}{
        [plot, frontierSlider] - 
        a number, determining size (percentage) of the
        Sharpe ratio plot, by default 0.1.
        }
    \item{minvariance.cex}{
        a number, determining size (percentage) of the
        minimum variance portfolio symbol, by default 1.
        }
    \item{tangency.cex}{
        a number, determining size (percentage) of the
        tangency portfolio symbol, by default 1.25.
        }
    \item{cml.cex}{
        [plot, frontierSlider] - 
        a number, determining size (percentage) of the
        market portfolio symbol, by default 1.25.
        }
    \item{equalWeights.cex}{
        [plot, frontierSlider] - 
        a number, determining size (percentage) of the
        equal weights portfolio symbol, by default 0.8.
        }
    \item{runningPoint.cex}{
        [weightsSlider] - 
        a number, determining size (percentage) of the
        point indicating the current portfolio
        equal weights portfolio symbol, by default 0.8.
        }
    \item{singleAsset.cex}{
        a number, determining size (percentage) of the
        singel asset portfolio symbols, by default 0.8.
        }
    \item{twoAssets.cex}{
        [plot, frontierSlider] - 
        a number, determining size (percentage) of the
        two assets efficient frontier plot, by default 0.01.
        }
    \item{monteCarlo.cex}{
        [plot, frontierSlider] - 
        a number, determining size (percentage) of the
        Monte Carol portfolio symbols, by default 0.01.
        }         
    \item{monteCarlo.cex}{
        [plot, frontierSlider] - 
        a number, determining size (percentage) of the
        Monte Carol portfolio symbols, by default 0.01.
        }
    \item{mcSteps}{
        [plot] - 
        a number, determining number of Monte Carol portfolio, by default 5000.
        }
    \item{pieR}{
        [plot, frontierSlider] - 
        a vector, containing factors for shrinking and stretching the x- and
        y-axis, by default NULL, i.e. c(1, 1) is used. Default pie size is 
        1/15 of the plot range.
        }
    \item{piePos}{
        [plot, frontierSlider] - 
        a number, determining the weight on the efficient frontier,
        which is illustrated by the pie. Default is tangency portfolio
        }
    \item{pieOffset}{
        [plot, frontierSlider] - 
        a vector, containing the pie's x- and y-axis offset from the efficient
        frontier. Default is NULL, i.e. the pie is set one default radius left
        of the efficient frontier.
        }
    \item{xlim}{
        [weightsSlider, frontierSlider] - 
        a vector, containing x-axis plot limits of the efficient frontier.
        Default setting is maximum of frontier range or single assets 
        portfolios.
        }
    \item{ylim}{
        [weightsSlider, frontierSlider] - 
        a vector, containing y-axis plot limits of the efficient frontier.
        Default setting is maximum of frontier range or single assets 
        portfolios. 
        } 
    } % end describe  
                   
}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{fPFOLIODATA}


\alias{fPFOLIODATA}
\alias{class-fPFOLIODATA}

\alias{fPFOLIODATA-class}

\alias{show,fPFOLIODATA-method}

\alias{portfolioData}


\title{Portfolio Data Handling}


\description{
  
    Creates a fPFOLIODATA object with data set and 
    statistical measures.
      
}
    

\usage{
portfolioData(data, spec = portfolioSpec())

\S4method{show}{fPFOLIODATA}(object)
}


\arguments{

    \item{data}{
        [portfolioStatistics] - \cr
        a time series or a named list, containing either a series of returns or
        named entries 'mu' and 'Sigma' being mean and covariance matrix.
        }
    \item{object}{
        [show] - \cr
        an object of class \code{fPFOLIODATA} as returned by the function
        \code{portfolioData}.
        }
    \item{spec}{
        an S4 object of class \code{fPFOLIOSPEC}, the specification to be 
        modified, by default the default of the function \code{portfolioSpec()}.
        }

}


\details{

    \bold{Dutch Portfolio Data Set:}
    \cr\cr
    This data represents seven stocks from the Dutch AEX index, Netherlands blue
    chips. The data is a list of the covariance matrix and the return means and
    is based on daily returns over a period from January 1990 till end of
    October 2003. Companies representing the data are Elsevier, Fortis,
    Getronics, Heineken, Philips, Shell and Unilever.
    \cr 
    
    
    \bold{US Portfolio Data Set:}
    \cr\cr
    The data inherits eight assets being indexes, commodities and bonds.
    The data is a time series of yearly returns from December 1973 till 
    December 1994. Assets are TBills3m, LongBonds, SP500, Wilshire5000, 
    NASDAQComp, LehmanBonds, EAFE, Gold.
    \cr
    
    
    \bold{Simulated Mean-Cov Data Set:}
    \cr\cr   
    This data is taken from chapter 1.3.2 in Scherer, M., Martin, R.D. (2005);
    \emph{Introduction To Modern Portfolio Optimization with NuOPT, S-PLUS and
    S+Bayes}, Springer, Berlin. It is a list of covariance matrix and the return
    means of imaginary assets. It is an example set for learning about 
    optimization.
    \cr
    
    
    \bold{World Index Returns Data Set:}
    \cr\cr   
    This data set is contributed by D. Locher (2007);
    It is a timeSeries object of four world
    index return data sets including Asia, Eastern Europe, Far East and 
    Latin America.

}


\value{
  
    \code{portfolioStatistics}\cr
    returns a named list of estimated mean \code{$mu} and covariance 
    \code{$Sigma} statistics, from a multivariate time series of assets.
    \cr
    
    \code{portfolioData}\cr
    returns a named list of the time series \code{$series} and the 
    portfolio \code{$statistics} as returned by the function
    \code{portfolioStatistics}.
  
}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{solver-ampl}
 

\alias{amplModelOpen}       
\alias{amplModelAdd}    
\alias{amplModelShow}       

\alias{amplDataOpen}  
\alias{amplDataAdd}      
\alias{amplDataAddValue}   
\alias{amplDataAddVector}  
\alias{amplDataAddMatrix}  
\alias{amplDataSemicolon}  
\alias{amplDataShow}        

\alias{amplRunOpen}         
\alias{amplRunAdd}         
\alias{amplRunShow}         

\alias{amplOutShow}        


\title{AMPL Interface}


\description{
  
    R/AMPL Interface functions.
    
}
    

\usage{
amplModelOpen(project)  
amplModelAdd(model, project)      
amplModelShow(project)       

amplDataOpen(project)     
amplDataAdd(name, data, type, project) 
amplDataAddValue(data, value, project)  
amplDataAddVector(data, vector, project)  
amplDataAddMatrix(data, matrix, project) 
amplDataSemicolon(project)  
amplDataShow(project)        

amplRunOpen(project)         
amplRunAdd(run, project)          
amplRunShow(project)        

amplOutShow(project)      
}


\arguments{
  
    \item{project}{
        a character string, the AMPL project name. 
        }   
    \item{model}{...}
    \item{data}{...}
    \item{run}{...}
    \item{type}{...}
    \item{name}{...}
    \item{value}{...}
    \item{vector}{...}
    \item{matrix}{...}
        
}


\value{
  
    returns AMPL files.
    
}


\author{

    Diethelm Wuertz.
    
}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{optim}

\name{solve-environment}
 

\alias{Data}

\alias{portfolioObjective}
\alias{portfolioReturn}
\alias{portfolioRisk}


\title{Nonlinear Objective Presettings}


\description{
  
    Prests variables for Data, portfolioObjective, portfolioReturn,
    and portfolioRisk in the case of NL math programming of 
    portfolios.
    
}
    

\usage{
Data

portfolioObjective(weights)
portfolioReturn(weights)
portfolioRisk(weights)
}


\arguments{
  
    \item{weights}{
        a vector of portfolio weights 
        }
    
}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{optim}

\name{weights-piePlot}


\alias{weightsPie}
\alias{weightedReturnsPie}
\alias{covRiskBudgetsPie}
\alias{tailRiskBudgetsPie}


\title{Portfolio Pie Plots}


\description{

    Displays pie plots of weights, weighted Returns, covariance 
    and tail risk budgets for a portfolio.

}


\usage{
weightsPie(object, pos = NULL, labels = TRUE, col = NULL, 
    box = TRUE, legend = TRUE, radius = 0.8, \dots)
    
weightedReturnsPie(object, pos = NULL, labels = TRUE, col = NULL, 
    box = TRUE, legend = TRUE, radius = 0.8, \dots)
    
covRiskBudgetsPie(object, pos = NULL, labels = TRUE, col = NULL, 
    box = TRUE, legend = TRUE, radius = 0.8, \dots)
    
tailRiskBudgetsPie(object, pos = NULL, labels = TRUE, col = NULL, 
    box = TRUE, legend = TRUE, radius = 0.8, \dots)
}


\arguments{

    \item{object}{
        an S4 object of class \code{fPORTFOLIO}, as returned by one of
        the portfolio functions, e.g. \code{efficientPortfolio} or
        \code{portfolioFrontier}.
        }
    \item{pos}{
        NULL or an integer value. If NULL it is assumend that we consider
        a single portfolio like for example a tengency portfolio. However,
        if the \code{object} describes a whole frontier then \code{pos}
        has to be the number of that point from the frontier which we
        want to display. The frontier points are numbered from one up
        to the value give by the number of frontier points, which can
        be retrieved by calling \code{getNFrontierPoints}.
        }
    \item{labels}{
        a logical flag, determining if the graph should be labeled
        automatically, which is the default case \code{labels=TRUE}. If 
        set to \code{FALSE} then the graph will be displayed
        undecorated and the user can it decorate by himself. 
        Evenmore, if \code{labels} takes the value of a string vector, 
        then the names of the assets from the porftolio \code{object} 
        will be ignored, and the labels will be taken from the specified 
        string vector.
        }
    \item{col}{
        a character string vector, defined from a color palette. The 
        default setting uses the "Blues" \code{seqPalette} palette.
        }
    \item{box}{
        a logical flag, determining whether a boxed frame should be plotted
        around the pie, by default the value is set to \code{TRUE}.
        }
    \item{legend}{
        a logical flag, determining if a legend should be added to the 
        plot. The default setting shows the legend.
        }
    \item{radius}{
        a numeric value, determining the radius of the pie. The default
        value is 0.8.
        }
    \item{\dots}{
        arguments to be passed.
        }
}


\details{

    The pie plots allow for different views on the results obtained
    from a feasible or an optimized portfolio.
    
    The function \code{weightsPie} displays the weights composition
    of a portfolio. 
    
    The function \code{weightedReturnsPie} displays the investment, i.e.
    the weighted returns of a portfolio. 
    
    The function \code{covRiskBudgetsPie} displays the covariance risk
    budgets of a portfolio.

    The function \code{taikRiskBudgetsPie} displays the copulae tail
    risk budgets of a portfolio. Note, this is only possible if in the
    portfolio specificsation a copulae tail risk is defined.

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{nlminb2Control}

\alias{nlminb2Control}

\title{Control variables for Rnlminb2}


\description{
Collection of Control Variables}


\usage{
nlminb2Control(eval.max = 500, iter.max = 400, trace = 0, abs.tol = 1e-20, 
    rel.tol = 1e-10, x.tol = 1.5e-8, step.min = 2.2e-14, scale = 1,
    R = 1.0, beta = 0.01, steps.max = 10, steps.tol = 1e-6)
}


\arguments{
    \item{eval.max}{
        an integer value. Maximum number of evaluations of the objective 
        function allowed. Defaults to 500.}
  
    \item{iter.max}{
        an integer value. Maximum number of iterations allowed. Defaults 
        to 400.}

    \item{trace}{
        an integer value. The value of the objective function and the 
        parameters is printed every trace'th iteration. Defaults to 0 
        which indicates no trace information is to be printed.}
        
    \item{abs.tol}{
        a numeric value. Absolute tolerance. Defaults to 1e-20.}

    \item{rel.tol}{
        a numeric value. Relative tolerance. Defaults to 1e-10.}

    \item{x.tol}{
        a numeric value. X tolerance. Defaults to 1.5e-8.}

    \item{step.min}{
        a numeric value. Minimum step size. Defaults to 2.2e-14.}
        
    \item{scale}{
        See PORT documentation (or leave alone).}

    \item{R}{
        a numeric value. The multiplier and devisor for the barrier and
        penalty function terms. Defaults to 1.0}

    \item{beta}{
        a numeric value. The value by which R is lowered in each iteration 
        step. Defaults to 0.01.}

    \item{steps.max}{
        an integer value. The maximum number of iteration steps in 
        which the penalty and barrier terms are lowered. Defaults to 10.}

    \item{steps.tol}{
        a numeric value. The penalty and barrier tolerance. Defaults
        to 1e-6.}

}


\keyword{optimize}

\name{frontier-points}


\alias{frontierPoints}


\title{Get Frontier Points}


\description{
  
    Extracts the risk and return coordinates of
    the efficient frontier.
    
}
    

\usage{
frontierPoints(object, frontier = c("both", "lower", "upper"), 
    return = c("mean", "mu"), risk = c("Cov", "Sigma", "CVaR", "VaR"),
    auto = TRUE)
}



\arguments{

    \item{object}{
        an object of class \code{fPORTFOLIO}.
        }
    \item{frontier}{
        a character string denoting which part of the
        efficient portfolio should be extractacted.
        }
    \item{return}{
        character strings denoting which return measure
        should be plotted. Allowed values for the
        return are either \code{"mean"}, or \code{"mu"}.
        }
    \item{risk}{
        character strings denoting which risk measure
        should be plotted. Allowed values for the
        risk measure are either \code{"cov"}, \code{"sigma"},  
        \code{"VaR"}, or \code{"CVaR"}.
        }
    \item{auto}{
        a logical flag. If \code{auto} is \code{TRUE}, the 
        default setting, then the risk willbe identified
        automatically from the object.
        }

}


\details{

    The automated risk detection, \code{auto=TRUE} takes the 
    following decision:

    \preformatted{
    if (auto) {
        Type = getType(object)
        Estimator = getEstimator(object)
        if (Type == "MV") risk = "cov"
        if (Type == "MV" & Estimator != "covEstimator") risk = "sigma"
        if (Type == "QLPM") risk = "sigma"
        if (Type == "CVaR") risk = "CVaR" 
    }
    }
    
}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{methods-show}


\alias{show-methods}
\alias{show,fPORTFOLIO-method}


\title{Portfolio Print Methods}


\description{

    show-methods.

}

\usage{
\S4method{show}{fPORTFOLIO}(object)
}


\arguments{

    \item{object}{
        an S4 object of class \code{fPORTFOLIO}.
        }

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{portfolio-efficientPortfolio}


\alias{efficientPortfolio}
\alias{maxratioPortfolio}
\alias{tangencyPortfolio}
\alias{minriskPortfolio}
\alias{minvariancePortfolio}
\alias{maxreturnPortfolio}


\title{Efficient Portfolios}


\description{
  
    Returns efficient portfolios.
    
}
    

\usage{
efficientPortfolio(data, spec = portfolioSpec(), constraints = "LongOnly")

maxratioPortfolio(data, spec = portfolioSpec(), constraints = "LongOnly")
tangencyPortfolio(data, spec = portfolioSpec(), constraints = "LongOnly")

minriskPortfolio(data, spec = portfolioSpec(), constraints = "LongOnly")
minvariancePortfolio(data, spec = portfolioSpec(), constraints = "LongOnly")

maxreturnPortfolio(data, spec = portfolioSpec(), constraints = "LongOnly")
}



\arguments{

    \item{constraints}{
        a character string vector, containing the constraints of the form\cr
        \code{"minW[asset]=percentage"} for box constraints resp. \cr
        \code{"maxsumW[assets]=percentage"} for sector constraints.
        }
    \item{data}{
        a multivariate time series described by an S4 object of class
        \code{timeSeries}. If your timeSerie is not a \code{timeSeries}
        object, consult the generic function \code{as.timeSeries} to 
        convert your time series.
        }
    \item{spec}{
        an S4 object of class \code{fPFOLIOSPEC} as returned by the function
        \code{portfolioSpec}.
        }

}


\details{
    
    \bold{Efficient Portfolio:}
    
    
    An efficient portfolio is a portfolio which lies on the efficient 
    frontier. 
    The \code{efficientPortfolio} function returns the properties of 
    the efficient portfolio as an S4 object of class \code{fPORTFOLIO}.

    
    \bold{Minumum Risk or Tangency Portfolio:}
    
    The function \code{tangencyPortfolio} returns the portfolio with 
    the highest return/risk ratio on the efficient frontier. For the
    Markowitz portfolio this is the same as the Sharpe ratio. To find 
    this point on the frontier the return/risk ratio calculated from 
    the target return and target risk returned by the function 
    \code{efficientPortfolio}. 

    
    \bold{Global minimum risk or Minimum Variance Portfolio:}
    
    The function \code{minvariancePortfolio} returns the portfolio 
    with the minimal risk on the efficient frontier. To find the 
    minimal risk point the target risk returned by the function
    \code{efficientPortfolio} is minimized.
    
    
    \bold{Maximum Return Portfolio:}
    
    The function \code{maxreturnPortfolio} returns the portfolio 
    with the maximal return for a fixed target risk.

}


\value{
    
    returns an S4 object of class \code{"fPORTFOLIO"}.
    
}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{weightsPlot}


\alias{weightsPlot}

\alias{weightedReturnsPlot}
\alias{covRiskBudgetsPlot}
\alias{tailRiskBudgetsPlot}
\alias{riskBudgetsPlot}


\title{Portfolio Weights Bar Plots}


\description{

    Displays plots of weights, investments, covariance 
    and tail risk budgets.

}


\usage{   
weightsPlot(object, labels = TRUE, col = NULL, title = TRUE, 
    box = TRUE, legend = TRUE, ...)
    
weightedReturnsPlot(object, labels = TRUE, col = NULL, title = TRUE, 
    box = TRUE, legend = TRUE, ...)
    
covRiskBudgetsPlot(object, labels = TRUE, col = NULL, title = TRUE, 
    box = TRUE, legend = TRUE, ...)
    
tailRiskBudgetsPlot(object, labels = TRUE, col = NULL, title = TRUE, 
    box = TRUE, legend = TRUE, ...)
    
riskBudgetsPlot(object, FUN=c("budgetsNormalVAR","budgetsNormalES",
    "budgetsModifiedVAR","budgetsModifiedES", "budgetsSampleCOV"),
    labels = TRUE, col = NULL, title = TRUE, mtext = TRUE, box = TRUE, 
    legend = TRUE, ...)
             
}


\arguments{

    \item{object}{
        an S4 object of class \code{fPORTFOLIO}, as returned by one of
        the portfolio functions, e.g. \code{efficientPortfolio} or
        \code{portfolioFrontier}.
        }
    \item{labels}{
        a logical flag, determining if the the graph should be labeled
        automatically, which is the default case \code{labels=TRUE}. If 
        set to \code{FALSE} then the graph will be displayed
        undecorated and the user can it decorate by himself.
        }
    \item{col}{
        a character string vector, defined from a color palette. The 
        default setting uses the "Blues" \code{seqPalette} palette.
        }
    \item{title}{
        a logical flag. Should automatically a title and axis labels be
        added to the plot. 
        }   
    \item{box}{
        a logical flag, determining whether a boxed frame should be plotted
        around the pie, by default the value is set to \code{TRUE}.
        }
    \item{legend}{
        a logical value, determining if the the graph should be labeled
        automatically, shich is the default case \code{labels=TRUE}. If 
        set to \code{FALSE} then the graph will be displayed
        undecorated and the user can it decorate by himself. 
        Evenmore, if \code{labels} takes the value of a string vector, 
        then the names of the assets from the porftolio \code{object} 
        will be ignored, and the labels will be taken from the specified 
        string vector.
        }
    \item{\dots}{
        additional arguments passed to the function \code{barplot}. Only 
        active if \code{labels=FALSE}. 
        }
    \item{FUN}{FUN}
    \item{mtext}{mtext}
    

}


\details{

    These barplots plots allow for different views on the results obtained
    from a feasible or an optimized portfolio.
    
    The function \code{weightsPlot} displays the weights composition
    along the frontier of a portfolio. 
    
    The function \code{weightedReturnsPlot} displays the investment
    composition, i.e. the weighted returns along the frontier of a portfolio. 
    
    The function \code{covRiskBudgetsPlot} displays the covariance risk
    budgets composition along the frontier of a portfolio.

    The function \code{tailRiskBudgetsPlot} displays the copulae tail
    risk budgets composition along the frontier of a portfolio. Note, 
    this is only possible if in the portfolio specificsation a copulae 
    tail risk is defined.

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{mathprog-LP}


\alias{rsolveLP} 

\alias{rglpkLP} 
\alias{glpkLP}
\alias{glpkLPControl}

\alias{rsymphonyLP}
\alias{symphonyLP} 
\alias{symphonyLPControl}

\alias{ramplLP} 
\alias{amplLP}
\alias{amplLPControl}

\alias{rneosLP}
\alias{neosLP}
\alias{neosLPControl} 


\title{Mathematical Linear Programming}


\description{

    Mathematical Linear Programming.

}


\usage{
rsolveLP(objective, lower=0, upper=1, linCons, 
    control=list(solver="glpk", invoke=c("R", "AMPL", "NEOS")))

rglpkLP(objective, lower=0, upper=1, linCons, control=list())
glpkLP
glpkLPControl(solver = "glpk", project="r", trace=FALSE) 

rsymphonyLP(objective, lower=0, upper=1, linCons, control=list())
symphonyLP
symphonyLPControl(solver="symphony", project="r", trace=FALSE) 

ramplLP(objective, lower = 0, upper = 1, linCons, control=list()) 
amplLP(objective, x_L=NULL, x_U=NULL, A=NULL, b_L=NULL, b_U=NULL,
    control=list())
amplLPControl(solver="ipopt", project="ampl", inf=1e12, trace=FALSE) 
    
rneosLP(objective, lower = 0, upper = 1, linCons, control=list()) 
neosLP(objective, x_L=NULL, x_U=NULL, A=NULL, b_L=NULL, b_U=NULL,
    control=list())
neosLPControl(solver="ipopt", category="lp", project="neos", 
    inf=1e12, trace=FALSE) 
}


\arguments{

    \item{objective}{
        a numeric vector.
        }
    \item{lower, upper}{
        lower and upper bounds.
        }
    \item{linCons}{
        list of linear constraints: mat, lower, upper.
        }
    \item{control}{
        control list.
        }
        
    \item{x_L, x_U}{
        lower and upper box bounds.
        }
    \item{A}{
        linear constraints matrix.
        }
    \item{b_L, b_U}{
        lower and upper linear constraints bounds.
        }
        
    \item{solver}{
        a character string, the solver name.
        }
    \item{category}{
        a character string, the NEOS category name.
        }
    \item{project}{
        a character string, the AMPL project name.
        }
    \item{inf}{
        a numeric value, the maximum value used for bounds.
        }
    \item{trace}{
        a logical flag, if TRUE the optimization will be traced.
        }
    
}


\value{
    
    a list of class \code{solver} with the following named ebtries: 
    \code{opt}, 
    \code{solution},
    \code{objective},
    \code{status},
    \code{message}, 
    \code{solver},
    \code{version}.

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{monitor-stability}


\alias{stabilityAnalytics}
\alias{turnsAnalytics}
\alias{drawdownsAnalytics}
\alias{garchAnalytics}
\alias{riskmetricsAnalytics}
\alias{bcpAnalytics}
\alias{pcoutAnalytics}
\alias{addRainbow}
\alias{waveletSpectrum}
\alias{parAnalytics}


\title{Monitoring Stability}


\description{
  
    Functions for time series aggregation, converting a time series
    from a daily to a monthly or weekly base.
    
}


\usage{
stabilityAnalytics(index, method=c("turns", "drawdowns", "garch", 
    "riskmetrics", "bcp", "pcout"), \dots)
    
turnsAnalytics(index, spar=0.5, main=NULL, 
    trace=TRUE, doplot=TRUE, at=pretty(index), format="\%m/\%y")
drawdownsAnalytics(index, spar=0.5, main=NULL, 
    trace=TRUE, doplot=TRUE, at=pretty(index), format="\%m/\%y")
garchAnalytics(index, spar = 0.5, main=NULL, 
    trace=TRUE, doplot=TRUE, at=pretty(index), format="\%m/\%y")    
riskmetricsAnalytics(index, spar=0.5, lambda=0.9, main=NULL, 
    trace=TRUE, doplot=TRUE, at=pretty(index), format="\%m/\%y")
bcpAnalytics(index, spar=0.5, FUN=returns, method=c("prob", "mean", "var"),
    main=NULL, trace=TRUE, doplot=TRUE, at=pretty(index), format="\%m/\%y")    
pcoutAnalytics(index, spar=0.5, main=NULL, trace=TRUE, doplot=TRUE, 
    at=pretty(index), format="\%m/\%y", strong=TRUE, k=2, cs=0.25, outbound=0.25)     
    
addRainbow(analytics, palette=rainbow, a=0.3, b=0.8, K=100)    

waveletSpectrum(index, spar=0.5, main=NULL, trace=TRUE, doplot=TRUE, 
  at=pretty(index), format="\%m/\%y")
  
parAnalytics()
}


\arguments{

    \item{index}{an object of class 'timeSeries'}
    \item{method}{name of selected analytics}
    \item{analytics}{analytics object}
    \item{\dots}{optional arguments}
    
    \item{spar}{0.5}
    \item{main}{""}
    \item{trace}{TRUE}
    \item{doplot}{TRUE}
    \item{at}{pretty()}
    \item{format}{"\%m/\%y"}
    
    \item{lambda}{riskmetricsAnalytics}
    \item{bcp}{bcpAnalytics}
    \item{FUN,strong,k,cs,outbound}{pcoutAnalytics}
    \item{palette,a,b,K}{addRainbow}

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{fPFOLIOVAL}


\alias{fPFOLIOVAL}
\alias{class-fPFOLIOVAL}

\alias{fPFOLIOVAL-class}

\alias{show,fPFOLIOVAL-method}


\title{Values of Portfolio Frontiers}


\description{
  
    Specifies portfolio Optimized Values.
    
}


\usage{       
\S4method{show}{fPFOLIOVAL}(object)
}


\arguments{

    \item{object}{
        an S4 object of class \code{fPFOLIOVAL}.
        }

}       



\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{portfolio-constraints}


\alias{portfolioConstraints}

\alias{minWConstraints}
\alias{maxWConstraints}

\alias{eqsumWConstraints}
\alias{minsumWConstraints}
\alias{maxsumWConstraints}

\alias{minBConstraints}
\alias{maxBConstraints}

\alias{listFConstraints}
\alias{minFConstraints}
\alias{maxFConstraints}

\alias{minBuyinConstraints}
\alias{maxBuyinConstraints}
 
\alias{nCardConstraints}
\alias{minCardConstraints}  
\alias{maxCardConstraints} 


\title{Portfolio Constraints}


\description{
  
    Computes portfolio constraints given constraints strings.
    
}    

\usage{
portfolioConstraints(data, spec=portfolioSpec(), constraints="LongOnly", \dots)

minWConstraints(data, spec=portfolioSpec(), constraints="LongOnly")
maxWConstraints(data, spec=portfolioSpec(), constraints="LongOnly")

eqsumWConstraints(data, spec=portfolioSpec(), constraints="LongOnly")
minsumWConstraints(data, spec=portfolioSpec(), constraints="LongOnly")
maxsumWConstraints(data, spec=portfolioSpec(), constraints="LongOnly")

minBConstraints(data, spec=portfolioSpec(), constraints="LongOnly")
maxBConstraints(data, spec=portfolioSpec(), constraints="LongOnly")

listFConstraints(data, spec=portfolioSpec(), constraints="LongOnly")
minFConstraints(data, spec=portfolioSpec(), constraints="LongOnly")
maxFConstraints(data, spec=portfolioSpec(), constraints="LongOnly")

minBuyinConstraints(data, spec=portfolioSpec(), constraints="LongOnly")
maxBuyinConstraints(data, spec=portfolioSpec(), constraints="LongOnly")

nCardConstraints(data, spec=portfolioSpec(), constraints="LongOnly")
minCardConstraints(data, spec=portfolioSpec(), constraints="LongOnly")
maxCardConstraints(data, spec=portfolioSpec(), constraints="LongOnly")
}


\arguments{

    \item{constraints}{
        a character value or character vector, containing the constraint
        strings. Setting constraints is described in the details section 
        }
    \item{data}{
        a list, having a statistics named list, having named entries 'mu' and
        'Sigma', containing the information of the statistics\cr
        }
    \item{spec}{
        an S4 object of class \code{fPFOLIOSPEC} as returned by the function
        \code{portfolioSpec}.
        }
    \item{\dots}{
        %%arguments passed to the function \code{.setRdonlp2Constraints}.
        For internal use only.
        }

}


\details{

    \bold{How to define constraints?}
    \cr\cr
    Constraints are defined by a character string or a vector of character
    strings.
    
    
    \emph{Summary Constraints: NULL, "LongOnly", "Short"}
    
    There are three special cases, the settings \code{constraints=NULL},
    \code{constraints="Short"}, and \code{constraints="LongOnly"}. Note,
    that these three constraint settings are not allowed to be combined 
    with more general constraint definitions.
    
    \code{NULL}: This selection defines the default value and is equivalent
    to the \code{"LongOnly"} case, see below.
    
    \code{"Short"}: This selection defines the case of unlimited short selling.
    i.e. each weight may range between \code{-Inf} and \code{Inf}. 
    Consequently, there are no group constraints. Risk budget constraints
    are not included in the portfolio optimization.
    
    \code{"LongOnly"}: This selection is the same as the default setting.
    Each weight may range between \code{0} ans \code{1}. No group
    constraints and risk budget constraints will be included in the 
    portfolio optimization.
    
    
    \emph{Lower and Upper Bounds: minW and maxW}
    
    \emph{Group Constraints: eqsumW, minsumW and maxsumW}
    
    Lower and upper bounded portfolios may be specified by a vector of
    character strings which describe executable code, setting values to
    to vectors \code{minW}, \code{maxW}, \code{minsumW}, and \code{maxsumW}.
    The individual string elements of the vector have the following form:
    
    \describe{
        \item{box constraints}{
            \code{"minW[Asset(s)]=Value(s)"}, and/or \cr
            \code{"maxW[Asset(s)]=Value(s)"}.
        }
        \item{sector constraints}{
            \code{"minsumW[Asset(s)]=Value(s)"}, and/or \cr
            \code{"maxsumW[Asset(s)]=Value(s)"}. 
        }
    }
    
    \code{Asset(s)} is an index of one or more assets, and \code{value} 
    a numeric value or vector assigning the desired value. Note, if the 
    \code{values} range between zero and one, then we have a long only 
    portfolio allowing for box and group constraints of the weights. If 
    the values are set to negative values, and values larger than one, 
    then (constrained) short selling will be allowed. 
    
    
    \emph{Risk Budget Constrained Portfolios:}
    
    By default, risk budgets are not included in the portfolio optimization.
    Covariance risk budgets have to be added explicitely, and have the 
    following form:
    
    \describe{
        \item{box constraints}{
            \code{"minB[Asset(s)]=Value(s)"}, and/or \cr
            \code{"minB[Asset(s)]=Value(s)"}.
        }
    }
    
    Again, \code{Asset(s)} is an index of one or more assets, and \code{value} 
    a numeric value or vector with numbers ranging between zero and one,
    assigning the desired risk budgets. 
    
    Note, risk budget constraints will enforce diversification at the 
    expense of return generation. The resulting portfolios will thus lie 
    below the unconstrained efficient frontier.
    
    \emph{Non-Linear Constraints: listF, minF, maxF}

}


\value{
    
    an object of class S4.

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{nlminb2}


\alias{nlminb2}


\title{Constrained nonlinear minimization}
    
    
\description{

    Solve constrained nonlinear minimization 
    problem with nonlinear constraints using 
    a penalty and barrier approach.
    
}


\usage{
nlminb2(start, objective, eqFun = NULL, leqFun = NULL,  
    lower = -Inf, upper = Inf, gradient = NULL, hessian = NULL, 
    control = list(), env = .GlobalEnv)
}


\arguments{

    \item{start}{
        a numeric vector, initial values for the parameters to be 
        optimized.}
  
    \item{objective}{
        function to be minimized. Must return a scalar value (possibly 
        NA/Inf). The first argument to objective is the vector of 
        parameters to be optimized, whose initial values are supplied 
        through \code{start}. Further arguments (fixed during the course 
        of the optimization) to objective may be specified as well.
        see \code{env}. }
  
    \item{eqFun}{
        a list of functions describing equal constraints.}
  
    \item{leqFun}{
        a list of functions describing less equal constraints.}
  
    \item{lower, upper}{
        two vectors of lower and upper bounds, replicated to be as long 
        as \code{start}. If unspecified, all parameters are assumed to 
        be unconstrained.}
 
    \item{gradient}{
        an optional function that takes the same arguments as \code{objective} 
        and evaluates the gradient of \code{objective} at its first argument. 
        Must return a vector as long as \code{start}.}

    \item{hessian}{
        an optional function that takes the same arguments as \code{objective} 
        and evaluates the hessian of \code{objective} at its first argument. 
        Must return a square matrix of order \code{length(start)}. Only the 
        lower triangle is used.}
 
    \item{control}{
        a list of control parameters. See below for details.}

    \item{env}{
        the environment in which objective, constraint, control
        functions are evaluated.}
 
}


\value{

    A list with following elements:
  
    \item{par}{
        a numeric vector, the best set of parameters found.}
    
    \item{objective}{
        a numeric value, the value of \code{objective} corresponding 
        to \code{par}.}
  
    \item{convergence}{
        an integer code, 0 indicates successful convergence.}
  
    \item{message}{
        a character string giving any additional information returned 
        by the optimizer, or NULL. For details, see PORT documentation.}

    \item{iterations}{
        am integer value, the number of iterations performed.}

    \item{evaluations}{
        an integer value, the number of objective function and gradient 
        function evaluations.}
        
}


\author{ 

    For the R port of \code{nlminb} Douglas Bates and Deepayan Sarkar,
    for the R/Rmetrics port of \code{nlminb2} Diethelm Wuertz,
    for the PORT library netlib.bell-labs.com.
    
}


\references{

Paul A. Jensen & Jonathan F. Bard, 
Operations Research Models and Methods, 2001
Appendix A, Algorithms for Constrained Optimization,
\url{https://www.me.utexas.edu/~jensen/ORMM/supplements/index.html}.

PORT Library,
\url{https://netlib.org/port/}.

}


\keyword{optimize}

\name{risk-ternaryMap}


\alias{ternaryMap}
\alias{ternaryFrontier}
\alias{riskMap}
\alias{maxddMap}
\alias{ternaryWeights}
\alias{ternaryCoord}
\alias{ternaryPoints}


\title{Creates and Plots a Ternary Map}


\description{
  
    Functions for craeting and plotting ternary maps.
    
}


\usage{
ternaryMap(data, FUN=NULL, ..., 
  locator=FALSE, N=41, palette=topo.colors, nlevels=11) 
ternaryFrontier(data, locator=FALSE)  

riskMap(data, weights)
maxddMap(data, weights)

ternaryWeights(n=21)
ternaryCoord(weights)
ternaryPoints(weights, \dots)
}



\arguments{

    \item{data}{data}
    \item{weights}{weights}
    
    \item{FUN, locator, N, palette, nlevels}{ternaryMap}
    \item{n}{n}
    \item{\dots}{optional arguments}

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{fPFOLIOBACKTEST}


\alias{fPFOLIOBACKTEST}
\alias{class-fPFOLIOBACKTEST}

\alias{fPFOLIOBACKTEST-class}

\alias{show,fPFOLIOBACKTEST-method}


\title{Portfolio backtesting specifications}


\description{
  
    Specifies portfolio backtesting objects.
    
}


\usage{       
\S4method{show}{fPFOLIOBACKTEST}(object)
}


\arguments{

    \item{object}{
        an S4 object of class \code{fPFOLIOBACKTEST}.
        }

}       

\details{

    \bold{Portfolio Backtest Specification:}
    \cr\cr
    
    The S4 class \code{fPFOLIOBACKTEST} specifies portfolio backtesting. 
    The slots are:\cr
    
    \describe{
        \item{@windows}{
            a list, setting the \code{windows} function that defines the 
            rolling windows, 
            and the set of window specific parameters \code{params}.  
            E.g The window horizon
            is set as a parameter \code{horizon = "24m"}
            }
        \item{@strategy}{
            a list, setting the portfolio \code{strategy} to implement 
            during the backtest, and any strategy specific parameters 
            are found in \code{params}.
            }
        \item{@smoother}{
            a list, specifying the smoothing style, given as a 
            \code{smoother} function, and any smoother specific parameters 
            are stored in the list \code{params}.
            }
        \item{@messages}{
            a list, any messages collected during the backtest}
            }
 
}


\value{
  
    \code{portfolioBacktest} returns an S4 object of class 
    \code{"fPFOLIOBACKTEST"}.

}


\references{

W\"urtz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{portfolio-portfolioFrontier}


\alias{portfolioFrontier}


\title{Efficient Portfolio Frontier}


\description{
  
    Compoutes the efficient portfolio frontier.
    
}
    

\usage{
portfolioFrontier(data, spec = portfolioSpec(), constraints = "LongOnly",
    include.mvl = TRUE, title = NULL, description = NULL)
}



\arguments{

    \item{constraints}{
        a character string vector, containing the constraints of the form\cr
        \code{"minW[asset]=percentage"} for box constraints resp. \cr
        \code{"maxsumW[assets]=percentage"} for sector constraints.
        }
    \item{data}{
        a multivariate time series described by an S4 object of class
        \code{timeSeries}. If your timeSerie is not a \code{timeSeries}
        object, consult the generic function \code{as.timeSeries} to 
        convert your time series.
        }
    \item{description}{ 
        a character string which allows for a brief description. 
        }
    \item{include.mvl}{ 
        a logical flag, should the minimum variance locus be added to the 
        plot? 
        }
    \item{spec}{
        an S4 object of class \code{fPFOLIOSPEC} as returned by the function
        \code{portfolioSpec}.
        }
    \item{title}{
        a character string which allows for a project title.
        } 
}


\details{
    
    \bold{Portfolio Frontier:}
    \cr\cr   
    

    The function \code{portfolioFrontier} calculates the whole efficient 
    frontier. The portfolio information consists of five arguments: data, 
    specifications, constraints, title and description. 
    
    The range of the frontier is determined from the range of the asset
    returns, and the number of equidistant points in the returns, is
    calculated from the number of frontier points hold in the specifrication 
    structure. To extract or to modify the number of frontier points
    use the functions \code{getNFrontierPoints} and \code{setNFrontierPoints}.
   
    
    The \code{frontierPortfolio} function returns the properties of 
    the the efficient frontier as an S4 object of class \code{fPORTFOLIO}.

}


\value{
    
    \code{portfolioFrontier} function returns an S4 object of class 
    \code{"fPORTFOLIO"}.
    
}



\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{backtest-constructors}


\alias{setBacktest}

\alias{setWindowsFun<-} 
\alias{setWindowsParams<-}

\alias{setWindowsHorizon<-} 

\alias{setStrategyFun<-}
\alias{setStrategyParams<-} 
\alias{setSmootherFun<-}
\alias{setSmootherParams<-} 
\alias{setSmootherLambda<-} 
\alias{setSmootherDoubleSmoothing<-} 

\alias{setSmootherInitialWeights<-}
\alias{setSmootherSkip<-} 


\title{Specification of backtesting portfolios}


\description{
  
    Functions to set specifications for portfolio backtesting.
    \cr
    
    The functions are:
    
    \tabular{ll}{
    \code{setWindowsFun} \tab Sets Windows function, \cr
    \code{setWindowsParams} \tab Sets additional parameters for rolling windows function, \cr
      
    \code{setWindowsHorizon} \tab Sets Windows horizon, \cr
    
    \code{setStrategyFun} \tab Sets the portfolio Strategy function, \cr
    \code{setStrategyParams} \tab Sets additional parameters for Strategy function, \cr
    
    \code{setSmootherFun} \tab Sets the Smoother function, \cr
    \code{setSmootherParams} \tab Sets additional parameters for Smoother function, \cr
     
    \code{setSmootherLambda} \tab Sets the smoothing parameter Lambda, \cr
     
    \code{setSmootherDoubleSmoothing} \tab Sets setting for double smoothing, \cr
      
    \code{setSmootherInitialWeights} \tab Sets the initial weights to used in the smoothing, \cr
    \code{setSmootherSkip} \tab Sets the number of skipped months. }
    
}


\usage{
setWindowsFun(backtest) <- value
setWindowsParams(backtest) <- value
setWindowsHorizon(backtest) <- value

setStrategyFun(backtest) <- value
setStrategyParams(backtest) <- value

setSmootherFun(backtest) <- value
setSmootherParams(backtest) <- value
setSmootherLambda(backtest) <- value
setSmootherDoubleSmoothing(backtest) <- value
setSmootherInitialWeights(backtest) <- value
setSmootherSkip(backtest) <- value
}


\arguments{

    \item{backtest}{
        an S4 object of class \code{fPFOLIOBACKTEST}, the specification 
        to be modified, by default the default of the function 
        \code{portfolioBacktest()}.
        }
    \item{value}{
        a value for that component of \code{backtest} to be set.  
        Note for setting Params value is a list.
        }

}       


\details{
    
    The function \code{portfolioBacktest()} allows to set the values 
    for the specification structure from scratch. 
    
    To modify individual settings one can use the set functions.
 
}


\references{

W\"urtz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{weights-Slider}


\alias{weightsSlider}     


\title{Portfolio Weights Slider}


\description{
  
    Interactive portfolio weights plot.
    
}
    

\usage{     
weightsSlider(object, control = list(), \dots)
}



\arguments{

    \item{control}{
        a list, defining the plotting parameters. The list modifies amongst
        others the color, e.g. \code{minvariance.col}, type of point, e.g.
        \code{tangency.pch}, or the dimension of the point, e.g. \code{cml.cex},
         see Notes for a complete list of control parameters.
        }
    \item{object}{
         an S4 object of class \code{fPORTFOLIO}.
        }
    \item{\dots}{
        optional arguments to be passed.
        }

}


\details{
  
    The slider has illustrative objectives. The function expects an S4 
    object of class \code{fPORTFOLIO}.
    
    The weights slider gives an overview of the weights 
    on the efficient frontier. Three weight plots \code{weightsPlot}, 
    \code{piePlot} and the not stacked weights and a frontier plot with the
    single assets, the tangency portfolio and a legend are provided. In the 
    two weights plots the vertical line indicates the current portfolio and a 
    dotted one indicates the minimum variance portfolio. The number in the 
    pie plot stands for the asset and the sign shows whether this asset is 
    short or long. In all plots colors represent the same asset.

}


\value{
    
    Creates interactive plots.
    
}


\section{Control Parameters}{

    In the following all elements of argument control from functions
    \code{plot}, \code{weightsSlider}, \code{frontierSlider} are listed. 

    \describe{
    \item{sliderResolution}{
        a numeric, determining the numbers of slider points, by default 
        nFrontierPoints/10.
        } 
    \item{sliderFlag}{
        a character string, denoting the slidertype, by default "frontier"
        for \code{frontierSlider} and "weights" for \code{weightsSlider}.
        } 
    \item{sharpeRatio.col}{ 
        a character string, defining color of the Sharpe
        ratio plot, by default "black".
        }
    \item{minvariance.col}{
        a character string, defining color of the minimum variance
        portfolio, by default "red".
        }
    \item{tangency.col}{
        a character string, defining color of the tangency
        portfolio, by default "steelblue".
        }
    \item{cml.col}{
        a character string, defining  color of the market
        portfolio and the capital market line, by default "green".
        }
    \item{equalWeights.col}{
        a character string, defining the color of the
        equal weights portfolio, by default "blue".
        }
    \item{runningPoint.col}{
        a character string, defining color of the
        point indicating the current portfolio, by default "red".
        }
    \item{singleAsset.col}{
        a character string vector, defining color of the
        single asset portfolios. The vector must have length the number 
        of assets, by default \code{rainbow}.
         }
    \item{twoAssets.col}{
        a character string, defining color of the
         two assets efficient frontier, by default "grey".
         }
    \item{monteCarlo.col}{
        a character string, defining color of the
        Monte Carlo portfolios, by default "black".
         }     
    \item{minvariance.pch}{
        a number, defining symbol used for the minimum
        variance portfolio. See \code{\link{points}} for description.
        Default symbol is 17.
        }
    \item{tangency.pch}{    
        a number, defining symbol used for the tangency portfolio.
        See \code{\link{points}} for description.
        Default symbol is 17.
        }
    \item{cml.pch}{
        a number, defining symbol used for the market portfolio.
        See \code{\link{points}} for description.
        Default symbol is 17.
        }
    \item{equalWeights.pch}{
        a number, defining symbol used for the equal weights portfolio.
        See \code{\link{points}} for description.
        Default symbol is 15.
        }        
    \item{singleAsset.pch}{
        a number, defining symbol used for the single asset portfolios.
        See \code{\link{points}} for description.
        Default symbol is 18.
        }           
    \item{sharpeRatio.cex}{
        a number, determining size (percentage) of the
        Sharpe ratio plot, by default 0.1.
        }
    \item{minvariance.cex}{
        a number, determining size (percentage) of the
        minimum variance portfolio symbol, by default 1.
        }
    \item{tangency.cex}{
        a number, determining size (percentage) of the
        tangency portfolio symbol, by default 1.25.
        }
    \item{cml.cex}{
        a number, determining size (percentage) of the
        market portfolio symbol, by default 1.25.
        }
    \item{equalWeights.cex}{
        a number, determining size (percentage) of the
        equal weights portfolio symbol, by default 0.8.
        }
    \item{runningPoint.cex}{
        a number, determining size (percentage) of the
        point indicating the current portfolio
        equal weights portfolio symbol, by default 0.8.
        }
    \item{singleAsset.cex}{
        a number, determining size (percentage) of the
        singel asset portfolio symbols, by default 0.8.
        }
    \item{twoAssets.cex}{ 
        a number, determining size (percentage) of the
        two assets efficient frontier plot, by default 0.01.
        }
    \item{monteCarlo.cex}{ 
        a number, determining size (percentage) of the
        Monte Carol portfolio symbols, by default 0.01.
        }         
    \item{monteCarlo.cex}{
        a number, determining size (percentage) of the
        Monte Carol portfolio symbols, by default 0.01.
        }
    \item{mcSteps}{
        a number, determining number of Monte Carol portfolio, 
        by default 5000.
        }
    \item{pieR}{
        a vector, containing factors for shrinking and stretching the x- and
        y-axis, by default NULL, i.e. c(1, 1) is used. Default pie size is 
        1/15 of the plot range.
        }
    \item{piePos}{
        a number, determining the weight on the efficient frontier,
        which is illustrated by the pie. Default is tangency portfolio
        }
    \item{pieOffset}{
        a vector, containing the pie's x- and y-axis offset from the efficient
        frontier. Default is NULL, i.e. the pie is set one default radius left
        of the efficient frontier.
        }
    \item{xlim}{
        a vector, containing x-axis plot limits of the efficient frontier.
        Default setting is maximum of frontier range or single assets 
        portfolios.
        }
    \item{ylim}{
        a vector, containing y-axis plot limits of the efficient frontier.
        Default setting is maximum of frontier range or single assets 
        portfolios. 
        } 
    }                 
}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{backtest-performance}


\alias{netPerformance}


\title{Portfolio backtesting net performance}


\description{

    Displays plot of rebased portfolio performance and summary 
    statistics.
    
}


\usage{   
netPerformance(object, format = "\%Y-\%m-\%d")
}


\arguments{

    \item{object}{
        a list, returned from running the function \code{portfolioSmoothing}.
        }
    \item{format}{
        a character string of the date format used
       }
    
}


\value{

    A plot of rebased portfolio returns and tables summarising portfolio 
    performance over time.
    
}


\note{

    This function will become obsolete by functions provided in the
    upcoming \code{fPortfolioPerformance} package.

}


\references{

W\"urtz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{backtest-getMethods}

\alias{getWindows}
\alias{getWindowsFun}
\alias{getWindowsParams}
\alias{getWindowsHorizon}
\alias{getStrategy}
\alias{getStrategyFun}
\alias{getStrategyParams}
\alias{getSmoother}
\alias{getSmootherFun}
\alias{getSmootherParams}						
\alias{getSmootherLambda}
\alias{getSmootherDoubleSmoothing}
\alias{getSmootherInitialWeights}
\alias{getSmootherSkip}
%\alias{getMessages} Already defined in fPortfolio


\title{Portfolio Backtest Extractors}


\description{
  
    Extractor functions to get information from objects
    of class fPFOLIOBACKTEST.
    
}
    

\arguments{

    \item{object}{
        an object of class \code{fPFOLIOBACKTEST} as returned by 
        function \code{portfolioBacktest}.
        }

}


\references{

W\"urtz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\examples{
## portfolioBacktest Specification -
   backtestSpec = portfolioBacktest()
   backtestSpec
    
## Extract Windows Information -  	
   getWindows(backtestSpec) 
   getWindowsFun(backtestSpec) 
   getWindowsParams(backtestSpec) 
   getWindowsHorizon(backtestSpec) 
   
## Extract Strategy Information - 
   getStrategy(backtestSpec) 
   getStrategyFun(backtestSpec) 
   getStrategyParams(backtestSpec) 
   
## Extract Smoother Information - 
   getSmoother(backtestSpec) 
   getSmootherFun(backtestSpec) 
   getSmootherParams(backtestSpec) 						
   getSmootherLambda(backtestSpec) 
   getSmootherDoubleSmoothing(backtestSpec) 
   getSmootherInitialWeights(backtestSpec) 
   getSmootherSkip(backtestSpec) 
}


\keyword{models}

\name{portfolio-getVal}


\alias{getAlpha.fPFOLIOVAL}
\alias{getCovRiskBudgets.fPFOLIOVAL}
\alias{getNFrontierPoints.fPFOLIOVAL}
\alias{getPortfolio.fPFOLIOVAL}
\alias{getRiskFreeRate.fPFOLIOVAL}
\alias{getStatus.fPFOLIOVAL}
\alias{getTargetReturn.fPFOLIOVAL}
\alias{getTargetRisk.fPFOLIOVAL}
\alias{getWeights.fPFOLIOVAL}


\title{PortfolioVal Extractor Functions}


\description{
  
    Extracts information from an object of class fPFOLIOVAL. 
    
}
    

\usage{
\method{getAlpha}{fPFOLIOVAL}(object)
\method{getCovRiskBudgets}{fPFOLIOVAL}(object)
\method{getNFrontierPoints}{fPFOLIOVAL}(object)
\method{getPortfolio}{fPFOLIOVAL}(object)
\method{getRiskFreeRate}{fPFOLIOVAL}(object)
\method{getStatus}{fPFOLIOVAL}(object)
\method{getTargetReturn}{fPFOLIOVAL}(object)
\method{getTargetRisk}{fPFOLIOVAL}(object)
\method{getWeights}{fPFOLIOVAL}(object)
}


\arguments{

    \item{object}{
        an object of class \code{fPFOLIODATA}.
        }

}


% \details{

% }


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{backtest-portfolio}


\alias{portfolioBacktesting}
\alias{portfolioSmoothing}


\title{Portfolio backtesting}


\description{
  
    Tests a portfolio by a rolling backtest.
    
}
    

\usage{  
portfolioBacktesting(formula, data, spec = portfolioSpec(), 
    constraints = "LongOnly", backtest = portfolioBacktest(), 
    trace = TRUE)

portfolioSmoothing(object, backtest, trace = TRUE)    
}


\arguments{
    \item{formula}{
        a formula describing the benchmark and assets used for backtesting
        in the form \code{backtest ~ assetA + ... + assetZ}. Here,
        \code{backtest} and \code{asset*} are column names of the
        \code{data} set.
        }
    \item{data}{
        an object of class \code{timeSeries}.
        }
    \item{spec}{
        an S4 object of class \code{fPFOLIOSPEC} as returned by the function
        \code{portfolioSpec}.
        }         
    \item{constraints}{
        a character string value or vector defining the constraints, for
        details we refer to \code{portfolioConstraints}.
        }      
    \item{backtest}{
        an S4 object of class \code{fPFOLIOBACKTEST} as returned by the 
        function \code{portfolioBacktest}.
        }  
    \item{object}{
        a list as returned by the function \code{portfolioBacktesting}.
        }
    \item{trace}{
        a logical flag, by default TRUE. Should the backtersting be traced?
        }
}


\references{

W\"urtz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{portfolio-feasiblePortfolio}


\alias{feasiblePortfolio}


\title{Feasible Portfolios}


\description{
  
    Returns properties of a feasible portfolio.
    
}
    

\usage{
feasiblePortfolio(data, spec = portfolioSpec(), constraints = "LongOnly")
}



\arguments{

    \item{constraints}{
        a character string vector, containing the constraints of the form\cr
        \code{"minW[asset]=percentage"} for box constraints resp. \cr
        \code{"maxsumW[assets]=percentage"} for sector constraints.
        }
    \item{data}{
        a multivariate time series described by an S4 object of class
        \code{timeSeries}. If your timeSerie is not a \code{timeSeries}
        object, consult the generic function \code{as.timeSeries} to 
        convert your time series.
        }
    \item{spec}{
        an S4 object of class \code{fPFOLIOSPEC} as returned by the function
        \code{portfolioSpec}.
        }

}


\details{  
    
    A feasible portfolio is a portfolio with given weights which
    lies inside the feasible region of portfolios.
       
    The function requires three arguments: \code{data}, \code{spec} 
    (specifications), and \code{constraints}, see above. Be sure that
    the specification structure \code{"spec"} has defined a weights
    vector which is different from \code{"NULL"}.  To assign values
    to the weights in the specification structure, use the function 
    \code{setWeights}. 
    
    The \code{feasiblePortfolio} function returns the properties of 
    the feasible portfolio as an S4 object of class \code{fPORTFOLIO}.

}


\value{
    
    \code{feasiblePortfolio} function returns an S4 object of class 
    \code{"fPORTFOLIO"}.
    
}



\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{weights-linePlot}


\alias{weightsLinePlot}
\alias{weightedReturnsLinePlot}
\alias{covRiskBudgetsLinePlot}

%\alias{tailRiskBudgetsLinePlot}


\title{Portfolio Weights Line Plots}


\description{

    Displays line plots of weights, weighted returns, covariance 
    and tail risk budgets.

}


\usage{   
weightsLinePlot(object, labels = TRUE, col = NULL, title = TRUE, 
    box = TRUE, legend = TRUE, ...)
    
weightedReturnsLinePlot(object, labels = TRUE, col = NULL, title = TRUE, 
    box = TRUE, legend = TRUE, ...)
    
covRiskBudgetsLinePlot(object, labels = TRUE, col = NULL, title = TRUE, 
    box = TRUE, legend = TRUE, ...)
    
% NYI   
%tailRiskBudgetsLinePlot(object, labels = TRUE, col = NULL, title = TRUE, 
%    box = TRUE, legend = TRUE, ...)
}


\arguments{

    \item{object}{
        an S4 object of class \code{fPORTFOLIO}, as returned by one of
        the portfolio functions, e.g. \code{efficientPortfolio} or
        \code{portfolioFrontier}.
        }
    \item{labels}{
        a logical flag, determining if the the graph should be labeled
        automatically, which is the default case \code{labels=TRUE}. If 
        set to \code{FALSE} then the graph will be displayed
        undecorated and the user can it decorate by himself.
        }
    \item{col}{
        a character string vector, defined from a color palette. The 
        default setting uses the "Blues" \code{seqPalette} palette.
        }
    \item{title}{
        a logical flag. Should automatically a title and axis labels be
        added to the plot. 
        }
    \item{box}{
        a logical flag, determining whether a boxed frame should be plotted
        around the pie, by default the value is set to \code{TRUE}.
        }
    \item{legend}{
        a logical value, determining if the the graph should be labeled
        automatically, shich is the default case \code{labels=TRUE}. If 
        set to \code{FALSE} then the graph will be displayed
        undecorated and the user can it decorate by himself. 
        Evenmore, if \code{labels} takes the value of a string vector, 
        then the names of the assets from the porftolio \code{object} 
        will be ignored, and the labels will be taken from the specified 
        string vector.
        }
    \item{\dots}{
        additional arguments passed to the function \code{barplot}. Only 
        active if \code{labels=FALSE}. 
        }
    

}


\details{

    These line plots allow for different views on the results obtained
    from a feasible or an optimized portfolio.
    
    The function \code{weightsPlot} displays the weights composition
    along the frontier of a portfolio. 
    
    The function \code{weightedReturnsPlot} displays the investment
    composition, i.e. the weighted returns along the frontier of a portfolio. 
    
    The function \code{covRiskBudgetsPlot} displays the covariance risk
    budgets composition along the frontier of a portfolio.

    % NYI
    %The function \code{tailRiskBudgetsPlot} displays the copulae tail
    %risk budgets composition along the frontier of a portfolio. Note, 
    %this is only possible if in the portfolio specification a copulae 
    %tail risk is defined.

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{backtest-functions}


\alias{equidistWindows}
\alias{tangencyStrategy}
\alias{emaSmoother}


\title{User defined functions to perform portfolio backtesting}


\description{
  
    Default windows, strategy and smoothing functions used for portfolio 
    backtesting.
    
}
    

\usage{
equidistWindows(data, backtest = portfolioBacktest())

tangencyStrategy(data, spec = portfolioSpec(), constraints = "LongOnly", 
    backtest = portfolioBacktest())
    
emaSmoother(weights, spec, backtest)
}



\arguments{


    \item{data}{
        a multivariate time series described by an S4 object of class
        \code{timeSeries}. If your timeSerie is not a \code{timeSeries}
        object, consult the generic function \code{as.timeSeries} to 
        convert your time series.
        }
    \item{backtest}{
        an S4 object of class \code{fPFOLIOBACKTEST} as returned by the 
        function \code{portfolioBacktest}.
        }  
    \item{spec}{
        an S4 object of class \code{fPFOLIOSPEC} as returned by the function
        \code{portfolioSpec}.
        }
    \item{constraints}{
        a character string vector, containing the constraints of the form\cr
        \code{"minW[asset]=percentage"} for box constraints resp. \cr
        \code{"maxsumW[assets]=percentage"} for sector constraints.
        }
    \item{weights}{
	    a numeric vector, containing the portfolio weights of an asset
	}

}


\details{
    
    \bold{equidistWindows:}\cr   
    
    Defines equal distant rolling windows.
       
    The function requires two arguments: \code{data} and 
    \code{backtest}, see above.  To assign the horizon 
    value to the backtest specification structure, use the function 
    \code{setWindowsHorizon}. 

    
    \bold{tangencyStrategy:}\cr   
    
    A pre-defined tangency portfolio strategy.
       
    The function requires four arguments: \code{data}, \code{spec}, 
    \code{constraints} and \code{backtest}, see above.

    
    \bold{emaSmoother:}\cr   
    
    A pre-defined weights smoother (EMA) for portfolio backtesting.
       
    The function requires three arguments: \code{weights}, \code{spec} 
    and \code{backtest}, see above.  To assign initial starting weights, 
    smoothing parameter (lambda) or whether to perform double smoothing 
    to the backtest specification structure, use the functions 
    \code{setSmootherInitialWeights}, \code{setSmootherLambda}
    and \code{setSmootherDoubleSmoothing}, respectively.
    

}


\value{
    
    \code{equidistWindows}\cr
    function returns the "from" and "to" dates of the rolling window 
    in a list form.

    \code{tangencyStrategy}\cr
    function returns a S4 object of class \code{"fPORTFOLIO"}.
 
    \code{emaSmoother}\cr
    function returns a numeric vector of smoothed weights.
    
}



\references{

W\"urtz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}



\keyword{models}

\name{data-sets}


\alias{dataSets}

\alias{GCCINDEX.DF}
\alias{SPISECTOR.DF}
\alias{SWX.DF}
\alias{LPP2005.RET.DF}
\alias{SMALLCAP.RET.DF}

\alias{GCCINDEX}
\alias{SPISECTOR}
\alias{SWX}
\alias{LPP2005}
\alias{SMALLCAP}

\alias{GCCINDEX.RET}
\alias{SPISECTOR.RET}
\alias{SWX.RET}
\alias{LPP2005.RET}
\alias{SMALLCAP.RET}

\alias{ECON85}
\alias{ECON85LONG}


\title{Assets Data Sets}


\description{
  
    Example data sets for portfolio optimization.
    
}
    

\usage{
ECON85
ECON85LONG

GCCINDEX
SPISECTOR
SWX
LPP2005
SMALLCAP

GCCINDEX.RET
SPISECTOR.RET
SWX.RET
LPP2005.RET
SMALLCAP.RET
}


%\details{
%}


\value{
  
    an object of class \code{"timeSeries"}.
  
}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{datasets}

\name{portfolio-portfolioSpec}


\alias{portfolioSpec}


\title{Specification of Portfolios}


\description{
  
    Specifies a portfolio from scratch.
    
}


\usage{
portfolioSpec(
    model = list(
        type = "MV", optimize = "minRisk",           
        estimator = "covEstimator", tailRisk = list(),
        params = list(alpha = 0.05)),
    portfolio = list(
        weights = NULL, targetReturn = NULL,
        targetRisk = NULL, riskFreeRate = 0, nFrontierPoints = 50,
        status = NA),
    optim = list(
        solver = "solveRquadprog", 
        objective = c("portfolioObjective", "portfolioReturn", "portfolioRisk"),
        options = list(meq = 2), control = list(), trace = FALSE),
    messages = list(
        messages = FALSE, note = ""),
    ampl = list(
        ampl = FALSE, project = "ampl", solver = "ipopt",
        protocol = FALSE, trace = FALSE)
    )
}   
    
   

\arguments{


    \item{model}{
        a list, containing different arguments: type, estimator, params.
        See these arguments for further explanation.
        }
    \item{portfolio}{
        a list, containing different arguments: weights, targetReturn,
        riskFreeRate, nFrontierPoints. See these
        arguments for further explanation.
        }
    \item{optim}{
        a list with four entries, a character string \code{solver} denoting 
        the type of the solver to be used, a \code{params} list to pass further
        arguments to the objective function to optimize, a \code{control} list
        for all control settings of the solver, and a logical flag, \code{trace}
        denoting if the optimization should be traced.
        }
    \item{messages}{
        a list, for optional messages.
        }
    \item{ampl}{
        a list, controls settings for the R/AMPL interface.
        }
}       


\details{

    To optimize a portfolio of assets we first have to specify it. All
    settings which specify a portfolio of assets are respresented by a
    S4 class named \code{fPFOLIOSPEC}.
        
    \preformatted{
    setClass("fPFOLIOSPEC", 
      representation(
        model = "list",
        portfolio = "list",
        optim = "list") )
    }
        
    An object of class \code{fPFOLIOSPEC} has three slots, named
    \code{@model}, \code{@portfolio}, and \code{@optim}. The first 
    slot \code{@model} holds the model information, the second
    slot \code{@portfolio} the portfolio information, and the last 
    slot \code{@optim} the information about the solver used for 
    optimization.
    
    The default settings are as follows:
        
    \preformatted{
    model = list(
        type = "MV",
        optimize = "minRisk",    
        estimator = "covEstimator",     
        tailRisk = list(),               
        params = list(alpha = 0.05, a = 2)),
    portfolio = list(
        weights = NULL, 
        targetReturn = NULL, 
        targetRisk = NULL,
        riskFreeRate = 0, 
        nFrontierPoints = 50,
        status = NA),
    optim = list(
        solver = "solveRquadprog",    
        objective = NULL,  
        parames = list(),
        control = list(meq = 2),   
        trace = FALSE)
    }
        
        
    \bold{Model Slot:}
    
    
    \emph{Type of Model:}\cr
    The list entry \code{type} from the \code{@model} slot describes 
    the type of the desired portfolio. The current implementation 
    supports three types of portfolios. This may be 
    a Markowitz mean -- variance portfolio named \code{"MV"}, 
    a mean -- lower partial moment portfolio named \code{"LPM"}, or 
    a mean -- CVaR condititional value-at-risk portfolio named \code{"CVaR"}. 
    One can use the function \code{getType} to retrieve the current
    setting and the function \code{setType} to modify this selection.
     
    
    \emph{What to optimize?}\cr
    The list entry \code{optimize} from the \code{@model} slot describes 
    what should be optimized. Two choices are psssible. Either
    
    \preformatted{
        \code{"minRisk"}
        } 
        
    which minimizes the risk if the target returns is given, or 
    
    \preformatted{
        \code{"maxReturn"}
        } 
        
    which maximizes the return if the target risk is given.
    One can use the function \code{getOptimize} to retrieve the current
    setting and the function \code{setOptimize} to modify this selection.
    
    
    \emph{How to estimate mean and covariance?}\cr
    The list entry \code{estimator} from the \code{@model} slot requests 
    for a string that denotes the function name of the covariance 
    estimator which should be used for the estimation of risk. 
    
    In Markowitz' mean-variance portfolio model, \code{type="MV"}, the 
    default function 
    
    \preformatted{
        \code{"covEstimator"} 
        }
        
    is used which computes the standard column means 
    of the multivariate assets data series and the standard covariance 
    matrix. Alternative robust estimators include 
    
    \preformatted{
        \code{"covMcdEstimator"}
        \code{"covOGKEstimator"}
        \code{"mveEstimator"}
        \code{"nnveEstimator"}
        \code{"mcdEstimator"}
        }
        
    In addition a shrinkage covariance estimator named
        
    \preformatted{
        \code{"shrinkEstimator"},
        }
        
    and a bagged covariance estimator named
     
    \preformatted{   
        \code{"baggedEstimator"} 
        }
    
    are also available.
    Note, the experienced user can add his own function to estimate in any
    alternative way the mean and the covariance of the multivariate assets
    data series. In this case (s)he has to write a function, e.g. named
     
    \preformatted{   
        \code{myEstimator=function(x,spec=NULL,...)}
        }
        
    where \code{x} is a multivariate time series, \code{spec} optionally 
    the portfolio specification, if rquired, and \code{...} additional
    arguments passed to the users code. Note, \code{myEstimator} must
    a return a named list, with at least the following two entries
    \code{\$mu} and \code{\$Sigma}, which represent estimators for the
    mean and covariance, respectively. 
    
    In the case of the Mean -- Lower-Partial-Moment portfolio, 
    \code{type="LPM"} we make use of the equivalence to Markowitz' 
    mean-variance portfolio with a modified covariance estimator, i.e. 
    
    \preformatted{
        \code{"lpmEstimator"},
        }
        
    Note, in this case the setting of \code{type="LPM"} 
    changes the covariance estimator function name 
    from any selection previously made to the function automatically
    to \code{"lpmEstimator"} which returns the LPM mean and covariance 
    estimates.   

    One can use the function \code{getEstimator} to retrieve the current
    setting and the function \code{setEstimator} to modify this selection.
     
     
    \emph{Tail Risk List:}\cr
    The list entry \code{tailRisk} from the \code{@model} slot is an empty 
    list. It can be used to add tail risk budget constrains to the 
    optimization. In this case a square matrix of the size of the 
    number of assets is expected as list entry, which contains bivariate 
    tail risk measures, i.e. the tail dependence coefficients estaimated 
    via a copulae approach.   
    Use the function \code{setType} to modify this selection.
    
    
    The list entry \code{parameters} from the \code{@model} slot is a 
    list with additional parameters used in different situations. It can 
    be ebhanced by the user if needed. 
    By default it contains the exponent \code{a=2}, the parameter needed 
        for "LPM" portfolio optimization,
    and it contains the \code{targetAlpha=0.05}, the confidence level
        for "CVaR" portfolio optimization.
    Use the function \code{setParams} to modify this selection.
    
    
    \bold{Portfolio Slot:}
    
    
    The values \code{weights}, \code{targetReturn}, and \code{targetRisk}
    from the \code{portfolio} slot have to be considered in common. By
    default all three are set to \code{NULL}. If this is the case, then 
    it is assumed that an equal weight portfolio should be calculated.
    If only one of the three values is different from \code{NULL} then
    the following procedure will be startet. 
    If the weights are specified then it is assumed that a feasible 
        portfolio should be considered.
    If the target return is fixed then it is assumed that the efficient 
        portfolio with the minimal risk will be considered. 
    And finally if the risk is fixed, then the return should be maximized.
    Use the functions \code{setWeights}, \code{setTargetReturn}, and
    \code{setTargetRisk} to modify this selection.Note, the change in of
    the three functions will influence the settings of the other two.
    
    
    The \code{riskFreeRate=0} is also stored in the \code{portfolio} slot.
    Its value defaults to zero. It can be changed by the user.
    Use the function \code{setRiskFreeRate} to modify this selection.
    
    
    The number of frontier points reqauired by the calculation of
    the \code{portfolioFrontier} is obtained from the value of
    \code{nFrontierPoints=50} hold in the \code{portfolio} slot.
    Its value defaults to 50. It can be changed by the user.
    Use the function \code{setNFrontierPoints} to modify this selection.
    
    
    The final \code{status} of portfolio optimization is returned 
    and stored in the \code{portfolio} slot. Before optimization the
    value is unset to \code{NA}, after optimization a value of 
    \code{status=0} means a successful termination. For other
    values we recommend to inspect the help page of the selected
    solver, the name of the solver can be returned by the function
    \code{getSolver}.
    Use the function \code{setSolver} to reset the value to \code{NA}
    if it should be required.
    
    
    \bold{Optim Slot:}
    
    
    The name of the default solver used for optimization can be retrieved
    calling the function \code{getSolver}. The default value for the
    value \code{solver} in the specification is set to \code{NULL} 
    which means that the best solver availalbe will be autoselected 
    and used. Before optimization the user can change the setting
    to another solver. Be aware, that a possible personal change will 
    be overwritten by the function \code{setType}, so call 
    \code{setSolver} after setting the type of the portfolio.
    
    
    The logical flag \code{trace} in the slot \code{optim} allows
    to trace optionally the portfolio optimization process. By 
    default this will not be the case since the default value is 
    \code{trace=FALSE}. Use the fanction \code{setTrace} to modify
    the selection.
        
    
    \bold{Retrieving and Modifying Specification Settings:}
    
             
    Information about the current portfolio specification can be retrieved 
    by \code{"get"} functions. These include:
    
    \tabular{ll}{
    \code{getType} \tab Extracts portfolio type from specification, \cr
    \code{getOptimize} \tab Extracts what to optimize from specification, \cr
    \code{getEstimator} \tab Extracts type of covariance estimator, \cr
    \code{getTailRisk} \tab Extracts list of tail dependency risk matrixes, \cr
    \code{getParams} \tab Extracts parameters from specification, \cr
    \code{getWeights} \tab Extracts weights from a portfolio object, \cr
    \code{getTargetReturn} \tab Extracts target return from specification, \cr
    \code{getTargetRisk} \tab Extracts target riks from specification, \cr
    \code{getAlpha} \tab Extracts target VaR-alpha specification, \cr
    \code{getRiskFreeRate} \tab Extracts risk free rate from specification, \cr
    \code{getNFrontierPoints} \tab Extracts number of frontier points, \cr 
    \code{getStatus} \tab Extracts the status of optimization, \cr 
    \code{getSolver} \tab Extracts solver from specification, \cr
    \code{getTrace} \tab Extracts solver's trace flag. }
    
    For details we refer to \code{link{getSpec}}.
    
    To modify the setting from a portfolio specification use the 
    \code{"set"} functions:
    
    \tabular{ll}{
    \code{setType} \tab Sets type of portfolio optimization, \cr
    \code{setOptimize} \tab Sets what to optimize, min risk or max return, \cr
    \code{setEstimator} \tab Sets names of mean and covariance estimators, \cr
    \code{setParams} \tab Sets optional model parameters, \cr
    \code{setWeights} \tab Sets weights vector, \cr
    \code{setTargetReturn} \tab Sets target return value, \cr
    \code{setTargetRisk} \tab Sets target risk value, \cr
    \code{setTargetAlpha} \tab Sets CVaR target alpha value, \cr
    \code{setRiskFreeRate} \tab Sets risk-free rate value, \cr
    \code{setNFrontierPoints} \tab Sets number of frontier points, \cr
    \code{setStatus} \tab Sets status value, \cr
    \code{setSolver} \tab Sets the type of solver to be used, \cr
    \code{setTrace} \tab Sets the logical trace flag. }
    
    
    For details we refer to \code{link{setSpec}}.
      
    \bold{Printing Specification Settings:}
    
    There is a generic print function to print information from
    specification. What is printed depends on the values of the
    settings. For example \code{print(portfolioSpec())} returns
    the type of portfolio, the name of the covariance estimator,
    the portfolios risk free rate, and the desired solver.
    
}


\value{
  
    \code{portfolioSpec}\cr
    \cr
    returns an S4 object of class \code{"fPFOLIOSPEC"}.

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{portfolio-getData}


%%\alias{getData} %%

\alias{getData.fPFOLIODATA}
\alias{getSeries.fPFOLIODATA}
\alias{getNAssets.fPFOLIODATA}
\alias{getUnits.fPFOLIODATA}

\alias{getStatistics.fPFOLIODATA}
\alias{getMean.fPFOLIODATA}
\alias{getCov.fPFOLIODATA}
\alias{getMu.fPFOLIODATA}
\alias{getSigma.fPFOLIODATA}
\alias{getEstimator.fPFOLIODATA}

\alias{getTailRisk.fPFOLIODATA}


\title{Portfolio Data Extractor Functions}


\description{
  
    Extracts information from an object of class fPFOLIODATA. 
    
}
    

\usage{
\method{getData}{fPFOLIODATA}(object)
\method{getSeries}{fPFOLIODATA}(object)
\method{getNAssets}{fPFOLIODATA}(object)
\method{getUnits}{fPFOLIODATA}(x)

\method{getStatistics}{fPFOLIODATA}(object)
\method{getMean}{fPFOLIODATA}(object)
\method{getCov}{fPFOLIODATA}(object)
\method{getMu}{fPFOLIODATA}(object)
\method{getSigma}{fPFOLIODATA}(object)
\method{getEstimator}{fPFOLIODATA}(object)

\method{getTailRisk}{fPFOLIODATA}(object)
}


\arguments{

    \item{object}{
        an object of class \code{fPFOLIODATA}.
        }
    \item{x}{
        an object of class \code{fPFOLIODATA}.
        }

}


\details{

    \tabular{ll}{
    \code{getData} \tab Extracts data slot, \cr
    \code{getSeries} \tab Extracts assets series, \cr
    \code{getNAssets} \tab Extracts number of assets, \cr
    \code{getUnits} \tab Extracts names of assets, \cr
    
    \code{getStatistics} \tab Extracts statistics slot, \cr
    \code{getMean} \tab Extracs mean vector, \cr
    \code{getCov} \tab Extracs covariance matrix, \cr
    \code{getMu} \tab Extracs mu vector, \cr
    \code{getSigma} \tab Extracs Sigma matrix, \cr
    \code{getEstimator} \tab Extracs Sigma matrix, \cr
    
    \code{getTailRisk} \tab Extracts tail risk slot. }  
}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{portfolio-rollingPortfolios}


\alias{portfolioRolling}
\alias{rollingPortfolio}

\alias{rollingWindows}

\alias{rollingCmlPortfolio}
\alias{rollingTangencyPortfolio}
\alias{rollingMinvariancePortfolio}

\alias{rollingPortfolioFrontier}


\title{Rolling Portfolio}


\description{
  
    A collection and description of functions 
    allowing to roll a portfolio optimization
    over time.
    
    The functions are:
    
    \tabular{ll}{
    \code{rollingWindows} \tab Returns a list of rolling window frames, \cr
    \code{rollingCmlPortfolio} \tab Rolls a CML portfolio, \cr
    \code{rollingTangencyPortfolio} \tab Rolls a tangency portfolio, \cr
    \code{rollingMinvariancePortfolio} \tab Rolls a minimum risk portfolio, \cr
    \code{rollingPortfolioFrontier} \tab returns an efficient portfolio}
    
}
    

\usage{
rollingWindows(x, period = "12m", by = "1m")

rollingCmlPortfolio(data, spec, constraints, from, to, action = NULL, 
    title = NULL, description = NULL, \dots)
rollingTangencyPortfolio(data, spec, constraints, from, to, action = NULL, 
    title = NULL, description = NULL, \dots)
rollingMinvariancePortfolio(data, spec, constraints, from, to, action = NULL, 
    title = NULL, description = NULL, \dots)
    
rollingPortfolioFrontier(data, spec, constraints, from, to, action = NULL, 
    title = NULL, description = NULL, \dots)
}



\arguments{

    \item{action}{
        a  character string naming a user defined function. This function
        is optionally applied after each rolling step.
        }
    \item{by}{
        a character string, by default \code{"1m"}, which denotes 1
        month. The shift by which the portfolio is rolled.
        }
    \item{constraints}{
        a character string vector, containing the constraints of the form\cr
        \code{"minW[asset]=percentage"} for box constraints resp. \cr
        \code{"maxsumW[assets]=percentage"} for sector constraints.
        }
    \item{data}{
        a list, having a statistics named list, having named entries 'mu' and 
        'Sigma', containing the information of the statistics.
        }
    \item{description}{
        a character string, allowing for a brief project description,
        by default NULL, i.e. Date and User.
        }
    \item{from, to}{
        a vector of S4 \code{timeDate} objects which denote the starting
        and ending dates for the investigation.
        }
    \item{period}{
        a character string, by default \code{"12m"}, which denotes 12
        months. The period over which the portfolio is rolled.
        }
    \item{spec}{
        an S4 object of class \code{fPFOLIOSPEC}.
        }
    \item{title}{
        a character string, containing the title for the object, by 
        default NULL.
        }
    \item{x}{
        an S4 object of class \code{timeSeries} from which the rolling
        window frames will be created. The length of these frames is given
        by the argument \code{period} and they are shifted by the value
        specified by the argument \code{by}.
        }
    \item{\dots}{
        optional arguments to be passed.\cr
        }   

}


\details{

    \bold{RollingWindows:}
    The function \code{rollingWindows} constructs from a 'timeSeries'
    object windows frames of given length \code{period} and shift
    \code{by}. ...
    \cr
    
    
    \bold{Rolling Portfolios:}
    \cr\cr
    The functions \code{rolling*Portfolio} ...
    \cr
    
    
    \bold{Rolling Frontier:}
    \cr\cr   
    The function \code{rollingPortfolioFrontier} ...

}


\value{
    
    \code{rollingwindows()}\cr
    returns ...
    \cr
    
    \code{rollingCmlPortfolio}\cr
    \code{rollingTangencyPortfolio}\cr
    \code{rollingMinvariancePortfolio}\cr
    return ...
    \cr
    
    \code{rollingPortfolioFrontier}\cr
    returns ...
    \cr
    
}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{utils-methods}


\alias{print.solver}


\title{Print Method for Solvers}


\description{

    S3 print method for mathematical programming solvers.

}


\usage{
\method{print}{solver}(x, \dots)
}


\arguments{

    \item{x}{x}
    \item{\dots}{optional arguments}

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{portfolio-getPortfolio}



\alias{getData.fPORTFOLIO}
\alias{getSeries.fPORTFOLIO}
\alias{getNAssets.fPORTFOLIO}
\alias{getUnits.fPORTFOLIO}
\alias{getStatistics.fPORTFOLIO}
\alias{getMean.fPORTFOLIO}
\alias{getCov.fPORTFOLIO}
\alias{getMu.fPORTFOLIO}
\alias{getSigma.fPORTFOLIO}
\alias{getEstimator.fPORTFOLIO}

\alias{getSpec.fPORTFOLIO}
\alias{getModel.fPORTFOLIO}
\alias{getType.fPORTFOLIO}
\alias{getOptimize.fPORTFOLIO}
\alias{getEstimator.fPORTFOLIO}
\alias{getTailRisk.fPORTFOLIO}
\alias{getParams.fPORTFOLIO}
\alias{getOptim.fPORTFOLIO}
\alias{getSolver.fPORTFOLIO}
\alias{getTrace.fPORTFOLIO}

\alias{getConstraints.fPORTFOLIO}
\alias{getConstraintsTypes}

\alias{getPortfolio.fPORTFOLIO}
\alias{getWeights.fPORTFOLIO}
\alias{getTargetReturn.fPORTFOLIO}
\alias{getTargetRisk.fPORTFOLIO}
\alias{getAlpha.fPORTFOLIO}
\alias{getRiskFreeRate.fPORTFOLIO}
\alias{getNFrontierPoints.fPORTFOLIO}
\alias{getStatus.fPORTFOLIO}

\alias{getCovRiskBudgets.fPORTFOLIO}
\alias{getTailRiskBudgets.fPORTFOLIO}

\alias{getA.fPORTFOLIO}
\alias{getControl.fPORTFOLIO}
\alias{getObjective.fPORTFOLIO}
\alias{getOptions.fPORTFOLIO}


\title{Portfolio Class Extractors}


\description{
  
    A collection and description of functions 
    allowing to get information about an object
    of class fPORTFOLIO. 
    \cr
    
    The functions are:
    
    \tabular{ll}{
    \code{getData} \tab Extracts ..., \cr
    \code{getSeries} \tab Extracts ..., \cr
    \code{getStatistics} \tab Extracts ..., \cr
    \code{getNAssets} \tab Extracts ..., \cr
    \code{getSpec} \tab Extracts ..., \cr
    \code{getType} \tab Extracts ..., \cr
    \code{getEstimator} \tab Extracts ..., \cr
    \code{getParams} \tab Extracts ..., \cr
    \code{getSolver} \tab Extracts ..., \cr
    \code{getTrace} \tab Extracts ..., \cr
    \code{getConstraints} \tab Extracts ..., \cr
    \code{getPortfolio} \tab Extracts ..., \cr
    \code{getWeights} \tab Extracts ..., \cr
    \code{getTargetReturn} \tab Extracts ..., \cr
    \code{getTargetRisk} \tab Extracts ..., \cr
    \code{getAlpha} \tab Extracts ..., \cr
    \code{getRiskFreeRate} \tab Extracts ..., \cr
    \code{getNFrontierPoints} \tab Extracts ..., \cr
    \code{getStatus} \tab Extracts ..., \cr
    \code{getCovRiskBudgets} \tab Extracts ..., \cr
    \code{getTailRiskBudgets} \tab Extracts ... . }
    
}
    

\usage{
\method{getData}{fPORTFOLIO}(object)
\method{getSeries}{fPORTFOLIO}(object)
\method{getNAssets}{fPORTFOLIO}(object)
\method{getUnits}{fPORTFOLIO}(x)
\method{getStatistics}{fPORTFOLIO}(object)
\method{getMean}{fPORTFOLIO}(object)
\method{getCov}{fPORTFOLIO}(object)
\method{getMu}{fPORTFOLIO}(object)
\method{getSigma}{fPORTFOLIO}(object)
\method{getEstimator}{fPORTFOLIO}(object)

\method{getSpec}{fPORTFOLIO}(object)
\method{getModel}{fPORTFOLIO}(object)
\method{getType}{fPORTFOLIO}(object)
\method{getOptimize}{fPORTFOLIO}(object)
\method{getEstimator}{fPORTFOLIO}(object)
\method{getTailRisk}{fPORTFOLIO}(object)
\method{getParams}{fPORTFOLIO}(object)
\method{getOptim}{fPORTFOLIO}(object)
\method{getSolver}{fPORTFOLIO}(object)
\method{getTrace}{fPORTFOLIO}(object)

\method{getConstraints}{fPORTFOLIO}(object)

\method{getPortfolio}{fPORTFOLIO}(object)
\method{getWeights}{fPORTFOLIO}(object)
\method{getTargetReturn}{fPORTFOLIO}(object)
\method{getTargetRisk}{fPORTFOLIO}(object)
\method{getAlpha}{fPORTFOLIO}(object)
\method{getRiskFreeRate}{fPORTFOLIO}(object)
\method{getNFrontierPoints}{fPORTFOLIO}(object)
\method{getStatus}{fPORTFOLIO}(object)

\method{getCovRiskBudgets}{fPORTFOLIO}(object)
\method{getTailRiskBudgets}{fPORTFOLIO}(object)


\method{getA}{fPORTFOLIO}(object)
\method{getControl}{fPORTFOLIO}(object)
\method{getObjective}{fPORTFOLIO}(object)
\method{getOptions}{fPORTFOLIO}(object)

}



\arguments{

    \item{object}{
        an object of class \code{fPORTFOLIO}, containing slots call, data,
        specification, constraints, portfolio, title, description.
        }
    \item{x}{
        an object of class \code{fPORTFOLIO}, containing slots call, data,
        specification, constraints, portfolio, title, description.
        }
}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{mathprog-NLP}


%\alias{rdonlp2NLP} 
%\alias{donlp2NLP} 
%\alias{donlp2NLPControl} 
%\alias{rdonlp2}

\alias{rsolnpNLP} 
\alias{solnpNLP} 
\alias{solnpNLPControl} 

\alias{rnlminb2NLP} 
\alias{nlminb2NLP} 
\alias{nlminb2NLPControl} 
\alias{rnlminb2}

\alias{ramplNLP} 
\alias{amplNLP} 
\alias{amplNLPControl} 


\title{Mathematical Non-Linear Programming}


\description{

    Mathematical Non-Linear Programming.

}


\usage{
%% rdonlp2NLP(start, objective, 
%%     lower=0, upper=1, linCons, funCons, control=list())   
%% donlp2NLP(start, objective, 
%%     par.lower=NULL, par.upper=NULL,
%%     eqA=NULL, eqA.bound=NULL,
%%     ineqA=NULL, ineqA.lower=NULL, ineqA.upper=NULL,
%%     eqFun=list(), eqFun.bound=NULL,
%%     ineqFun=list(), ineqFun.lower=NULL, ineqFun.upper=NULL,
%%     control=list())    
%% donlp2NLPControl(
%%     iterma=4000, nstep=20, fnscale=1, report=FALSE, rep.freq=1, 
%%     tau0=1, tau=0.1, del0=1, epsx=1e-05, delmin=0.1 * del0, 
%%     epsdif=1e-08, nreset.multiplier=1, difftype=3, epsfcn=1e-16, 
%%     taubnd=1, hessian=FALSE, te0=TRUE, te1=FALSE, te2=FALSE, 
%%     te3=FALSE, silent=TRUE, intakt=TRUE) 
%% rdonlp2

rsolnpNLP(start, objective, 
    lower=0, upper=1, linCons, funCons, control=list())   
solnpNLP(start, objective, 
    par.lower=NULL, par.upper=NULL,
    eqA=NULL, eqA.bound=NULL,
    ineqA=NULL, ineqA.lower=NULL, ineqA.upper=NULL,
    eqFun=list(), eqFun.bound=NULL,
    ineqFun=list(), ineqFun.lower=NULL, ineqFun.upper=NULL,
    control=list())    
solnpNLPControl(
    rho=1, outer.iter=400, inner.iter=800, delta=1e-07, tol=1e-08, trace=0)
      
rnlminb2NLP(start, objective, 
    lower=0, upper=1, linCons, funCons, control=list())   
nlminb2NLP(start, objective, 
    par.lower=NULL, par.upper=NULL,
    eqA=NULL, eqA.bound=NULL,
    ineqA=NULL, ineqA.lower=NULL, ineqA.upper=NULL,
    eqFun=list(), eqFun.bound=NULL,
    ineqFun=list(), ineqFun.lower=NULL, ineqFun.upper=NULL,
    control=list())    
nlminb2NLPControl(
    eval.max=500, iter.max=400, trace=0, abs.tol=1e-20, rel.tol=1e-10, 
    x.tol=1.5e-08, step.min=2.2e-14, scale=1, R=1, beta.tol=1e-20)  
rnlminb2
      
ramplNLP(start, objective, 
    lower=0, upper=1, amplCons, control=list(), ...)   
amplNLP()    
amplNLPControl(
    solver="minos", project="ampl", trace=FALSE)
}


\arguments{

    \item{start}{
        a numeric vector, the start values.
        }
    \item{objective}{
        a function object, the function to be optimized.
        }
    \item{lower, upper}{
        lower and upper bounds.
        }
    \item{linCons}{
        list of linear constraints: mat, lower, upper.
        }
    \item{funCons}{
        list of function constraints.
        }
    \item{amplCons}{
        AMPL constraints.
        }
    \item{control}{
        control list.
        }
    \item{\dots}{
        optional arguments to be passed.
        }
        
    \item{par.lower, par.upper}{...}
    \item{eqA}{...}
    \item{eqA.bound}{...}
    \item{ineqA}{...}
    \item{ineqA.lower,ineqA.upper}{...}
    \item{eqFun}{...}
    \item{eqFun.bound}{...}
    \item{ineqFun}{...}
    \item{ineqFun.lower,ineqFun.upper}{...}
    
    %\item{x_L,x_U}{...}
    %\item{A}{...}
    %\item{b_L,b_U}{...}
    
    %\item{solver}{...}
    %\item{category}{...}
    %\item{project}{...}
    %\item{inf}{...}
    %\item{trace}{...}

    %% donlp2 items:
    %% \item{iterma}{4000}
    %% \item{nstep}{20}
    %% \item{fnscale}{1}
    %% \item{report}{FALSE}
    %% \item{rep.freq}{1}
    %% \item{tau0}{1}
    %% \item{tau}{0.1}
    %% \item{del0}{1}
    %% \item{epsx}{1e-5}
    %% \item{delmin}{0.1 * del0}
    %% \item{epsdif}{1e-8}
    %% \item{nreset.multiplier}{1}
    %% \item{difftype}{3}
    %% \item{epsfcn}{1e-16}
    %% \item{taubnd}{1}
    %% \item{hessian}{FALSE}
    %% \item{te0}{TRUE}
    %% \item{te1}{FALSE}
    %% \item{te2}{FALSE}
    %% \item{te3}{FALSE}
    %% \item{silent}{TRUE}
    %% \item{intakt}{TRUE}
    
    \item{rho}{1}
    \item{outer.iter}{400}
    \item{inner.iter}{800}
    \item{delta}{1.0e-7}
    \item{tol}{1.0e-8}
    %\item{trace}{0}
    
    \item{eval.max}{500}
    \item{iter.max}{400}
    \item{trace}{0}
    \item{abs.tol}{1e-20}
    \item{rel.tol}{1e-10}
    \item{x.tol}{1.5e-08}
    \item{step.min}{2.2e-14}
    \item{scale}{1}
    \item{R}{1}
    \item{beta.tol}{1e-20}
    
    \item{solver}{solver name}
    \item{project}{project name}    
  
}
  

\value{
    
    a list of class \code{solver} with the following named ebtries: 
    \code{opt}, 
    \code{solution},
    \code{objective},
    \code{status},
    \code{message}, 
    \code{solver},
    \code{version}.

}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}


\keyword{models}

\name{portfolio-getSpec}


%%\alias{getSpec} %%conflicts with getDefault

\alias{getModel.fPFOLIOSPEC}
\alias{getType.fPFOLIOSPEC}  
\alias{getOptimize.fPFOLIOSPEC}        
\alias{getEstimator.fPFOLIOSPEC} 
\alias{getTailRisk.fPFOLIOSPEC}      
\alias{getParams.fPFOLIOSPEC} 
\alias{getAlpha.fPFOLIOSPEC}  
\alias{getA.fPFOLIOSPEC}      
\alias{getA}
\alias{getMessages}
 
\alias{getPortfolio.fPFOLIOSPEC}        
\alias{getWeights.fPFOLIOSPEC}         
\alias{getTargetReturn.fPFOLIOSPEC}    
\alias{getTargetRisk.fPFOLIOSPEC}       
\alias{getRiskFreeRate.fPFOLIOSPEC}    
\alias{getNFrontierPoints.fPFOLIOSPEC} 
\alias{getStatus.fPFOLIOSPEC}

\alias{getOptim.fPFOLIOSPEC}
\alias{getSolver.fPFOLIOSPEC}   
\alias{getObjective.fPFOLIOSPEC} 
\alias{getOptions.fPFOLIOSPEC}       
\alias{getControl.fPFOLIOSPEC}        
\alias{getTrace.fPFOLIOSPEC}           

\alias{getMessages.fPFOLIOSPEC}

\title{Portfolio Specification Extractor Functions}


\description{
  
    Extracts information from an object of class fPFOLIOSPEC. 
    
}
    

\usage{
\method{getModel}{fPFOLIOSPEC}(object)
\method{getType}{fPFOLIOSPEC}(object)
\method{getOptimize}{fPFOLIOSPEC}(object)
\method{getEstimator}{fPFOLIOSPEC}(object)
\method{getTailRisk}{fPFOLIOSPEC}(object)
\method{getParams}{fPFOLIOSPEC}(object)

\method{getPortfolio}{fPFOLIOSPEC}(object)
\method{getWeights}{fPFOLIOSPEC}(object)
\method{getTargetReturn}{fPFOLIOSPEC}(object)
\method{getTargetRisk}{fPFOLIOSPEC}(object)
\method{getAlpha}{fPFOLIOSPEC}(object)
\method{getRiskFreeRate}{fPFOLIOSPEC}(object)
\method{getNFrontierPoints}{fPFOLIOSPEC}(object)
\method{getStatus}{fPFOLIOSPEC}(object)

\method{getOptim}{fPFOLIOSPEC}(object)
\method{getSolver}{fPFOLIOSPEC}(object)
\method{getObjective}{fPFOLIOSPEC}(object)
\method{getOptions}{fPFOLIOSPEC}(object)
\method{getControl}{fPFOLIOSPEC}(object)
\method{getTrace}{fPFOLIOSPEC}(object)

\method{getMessages}{fPFOLIOSPEC}(object)
}


\arguments{

    \item{object}{
        an object of class \code{fPFOLIOSPEC}.
        }

}


\details{

    \tabular{ll}{
    \code{getType} \tab Extracts portfolio type from specification, \cr
    \code{getOptimize} \tab Extracts what to optimize from specification, \cr
    \code{getEstimator} \tab Extracts type of covariance estimator, \cr
    \code{getTailRisk} \tab Extracts list of tail dependency risk matrixes, \cr
    \code{getParams} \tab Extracts parameters from specification, \cr
    \code{getWeights} \tab Extracts weights from a portfolio object, \cr
    \code{getTargetReturn} \tab Extracts target return from specification, \cr
    \code{getTargetRisk} \tab Extracts target riks from specification, \cr
    \code{getAlpha} \tab Extracts target VaR-alpha specification, \cr
    \code{getRiskFreeRate} \tab Extracts risk free rate from specification, \cr
    \code{getNFrontierPoints} \tab Extracts number of frontier points, \cr 
    \code{getStatus} \tab Extracts the status of optimization, \cr 
    \code{getSolver} \tab Extracts solver from specification, \cr
    \code{getobjective} \tab Extracts name of objective function, \cr
    \code{getTrace} \tab Extracts solver's trace flag. }
    
}


\references{

Wuertz, D., Chalabi, Y., Chen W., Ellis A. (2009);
    \emph{Portfolio Optimization with R/Rmetrics}, 
    Rmetrics eBook, Rmetrics Association and Finance Online, Zurich.
    
}
   
   
\keyword{models}
